@@ -32,22 +32,18 @@ header = """\
 Command line tools using Python
 ===============================
 
-.. contents:: Page contents
-   :local:
-   :depth: 1
-
 """
 
 command_rst = """
 
-.. _gen_%s:
+.. _{0}:
 
-%s
-%s
+{0}
+{1}
 
 .. rst-class:: callout
 
-%s
+{2}
 
 """
 
@@ -66,7 +62,7 @@ def generate_commands_rst(app=None):
         for fname in glob.glob(op.join(command_path, 'mne_*.py'))])
     iterator = sphinx_compatibility.status_iterator(
         fnames, 'generating MNE command help ... ', length=len(fnames))
-    with open(out_fname, 'w') as f:
+    with open(out_fname, 'w', encoding='utf8') as f:
         f.write(header)
         for fname in iterator:
             cmd_name = fname[:-3]
@@ -100,10 +96,9 @@ def generate_commands_rst(app=None):
                 output.insert(ii + 3, '.. rst-class:: field-list cmd-list')
                 output.insert(ii + 4, '')
             output = '\n'.join(output)
-            f.write(command_rst % (cmd_name,
-                                   cmd_name.replace('mne_', 'mne '),
-                                   '=' * len(cmd_name),
-                                   output))
+            cmd_name_space = cmd_name.replace('mne_', 'mne ')
+            f.write(command_rst.format(
+                cmd_name_space, '=' * len(cmd_name_space), output))
     _replace_md5(out_fname)
 
 
