@@ -9,7 +9,6 @@
 from copy import deepcopy
 
 import numpy as np
-from scipy import linalg
 
 from ..cov import Covariance, make_ad_hoc_cov
 from ..forward.forward import is_fixed_orient, _restrict_forward_to_src_sel
@@ -17,18 +16,15 @@ from ..io.proj import make_projector, Projection
 from ..minimum_norm.inverse import _get_vertno, _prepare_forward
 from ..source_space import label_src_vertno_sel
 from ..utils import (verbose, check_fname, _reg_pinv, _check_option, logger,
-                     _pl, _svd_lwork, _repeated_svd, _repeated_pinv2,
-                     _inv_lwork, _repeated_inv, _eig_lwork, _repeated_eig,
-                     LinAlgError, _check_src_normal)
+                     _pl, _check_src_normal, check_version, _sym_mat_pow, warn)
 from ..time_frequency.csd import CrossSpectralDensity
 
 from ..externals.h5io import read_hdf5, write_hdf5
 
 
-def _check_proj_match(info, filters):
+def _check_proj_match(proj, filters):
     """Check whether SSP projections in data and spatial filter match."""
-    proj_data, _, _ = make_projector(info['projs'],
-                                     filters['ch_names'])
+    proj_data, _, _ = make_projector(proj, filters['ch_names'])
     if not np.allclose(proj_data, filters['proj'],
                        atol=np.finfo(float).eps, rtol=1e-13):
         raise ValueError('The SSP projections present in the data '
@@ -97,98 +93,56 @@ def _prepare_beamformer_input(info, forward, label=None, pick_ori=None,
         orient_std = np.ones(gain.shape[1])
 
     # Get the projector
-    proj, ncomp, _ = make_projector(
+    proj, _, _ = make_projector(
         info_picked['projs'], info_picked['ch_names'])
     return (is_free_ori, info_picked, proj, vertno, gain, whitener, nn,
             orient_std)
 
 
-def _normalized_weights(Wk, Gk, Cm_inv_sq, reduce_rank, nn, sk,
-                        svd_lwork, inv_lwork, eig_lwork):
-    """Compute the normalized weights in max-power orientation.
+def _reduce_leadfield_rank(G):
+    """Reduce the rank of the leadfield."""
+    # decompose lead field
+    u, s, v = np.linalg.svd(G, full_matrices=False)
 
-    Uses Eq. 4.47 from [1]_.
+    # backproject, omitting one direction (equivalent to setting the smallest
+    # singular value to zero)
+    G = np.matmul(u[:, :, :-1], s[:, :-1, np.newaxis] * v[:, :-1, :])
 
-    Parameters
-    ----------
-    Wk : ndarray, shape (3, n_channels)
-        The set of un-normalized filters at a single source point.
-    Gk : ndarray, shape (n_channels, 3)
-        The leadfield at a single source point.
-    Cm_inv_sq : nsarray, snape (n_channels, n_channels)
-        The squared inverse covariance matrix.
-    reduce_rank : bool
-        Whether to reduce the rank of the filter by one.
-    nn : ndarray, shape (3,)
-        The source normal.
-    sk : ndarray, shape (3,)
-        The source prior.
-    svd_lwork : int
-        The svd lwork value.
-    inv_lwork : int
-        The inv lwork value.
-    eig_lwork : int
-        The eig lwork value.
+    return G
 
-    Returns
-    -------
-    Wk : ndarray, shape (n_dipoles, n_channels)
-        The normalized beamformer filters at the source point in the direction
-        of max power.
 
-    References
-    ----------
-    .. [1] Sekihara & Nagarajan. Adaptive spatial filters for electromagnetic
-           brain imaging (2008) Springer Science & Business Media
-    """
-    norm_inv = np.dot(Gk.T, np.dot(Cm_inv_sq, Gk))
-    if reduce_rank:
-        # Use pseudo inverse computation setting smallest
-        # component to zero if the leadfield is not full rank
-        norm = _reg_pinv(norm_inv, rank=norm_inv.shape[0] - 1,
-                         svd_lwork=svd_lwork)[0]
+def _sym_inv_sm(x, reduce_rank, inversion, sk):
+    """Symmetric inversion with single- or matrix-style inversion."""
+    if x.shape[1:] == (1, 1):
+        with np.errstate(divide='ignore', invalid='ignore'):
+            x_inv = 1. / x
+        x_inv[~np.isfinite(x_inv)] = 1.
     else:
-        # Use straight inverse with full rank leadfield
-        try:
-            norm = _repeated_inv(norm_inv, inv_lwork)
-        except LinAlgError:
-            raise ValueError(
-                'Singular matrix detected when estimating spatial filters. '
-                'Consider reducing the rank of the forward operator by using '
-                'reduce_rank=True.'
-            )
-    # Reapply source covariance after inversion
-    norm *= sk
-    norm *= sk[:, np.newaxis]
-    power = np.dot(norm, np.dot(Wk, Gk))
-
-    # Determine orientation of max power
-    assert power.dtype in (np.float64, np.complex128)  # LCMV, DICS
-    eig_vals, eig_vecs = _repeated_eig(power, eig_lwork)
-    if not np.iscomplexobj(power) and np.iscomplexobj(eig_vecs):
-        raise ValueError('The eigenspectrum of the leadfield at this voxel is '
-                         'complex. Consider reducing the rank of the '
-                         'leadfield by using reduce_rank=True.')
-
-    idx_max = eig_vals.argmax()
-    max_power_ori = eig_vecs[:, idx_max]
-
-    # set the (otherwise arbitrary) sign to match the normal
-    sign = np.sign(np.dot(max_power_ori, nn)) or 1
-    max_power_ori *= sign
-
-    # Compute the filter in the orientation of max power
-    Wk[:] = np.dot(max_power_ori, Wk)
-    Gk = np.dot(Gk, max_power_ori)
-    denom = np.dot(Gk.T, np.dot(Cm_inv_sq, Gk))
-    denom = np.sqrt(denom)
-    Wk /= denom
-
-    return Wk
+        assert x.shape[1:] == (3, 3)
+        if inversion == 'matrix':
+            x_inv = _sym_mat_pow(x, -1, reduce_rank=reduce_rank)
+            # Reapply source covariance after inversion
+            x_inv *= sk[:, :, np.newaxis]
+            x_inv *= sk[:, np.newaxis, :]
+        else:
+            # Invert for each dipole separately using plain division
+            diags = np.diagonal(x, axis1=1, axis2=2)
+            assert not reduce_rank   # guaranteed earlier
+            with np.errstate(divide='ignore'):
+                diags = 1. / diags
+            # set the diagonal of each 3x3
+            x_inv = np.zeros_like(x)
+            for k in range(x.shape[0]):
+                this = diags[k]
+                # Reapply source covariance after inversion
+                this *= (sk[k] * sk[k])
+                x_inv[k].flat[::4] = this
+    return x_inv
 
 
 def _compute_beamformer(G, Cm, reg, n_orient, weight_norm, pick_ori,
-                        reduce_rank, rank, inversion, nn, orient_std):
+                        reduce_rank, rank, inversion, nn, orient_std,
+                        whitener):
     """Compute a spatial beamformer filter (LCMV or DICS).
 
     For more detailed information on the parameters, see the docstrings of
@@ -218,104 +172,194 @@ def _compute_beamformer(G, Cm, reg, n_orient, weight_norm, pick_ori,
         The source normals.
     orient_std : ndarray, shape (n_dipoles,)
         The std of the orientation prior used in weighting the lead fields.
+    whitener : ndarray, shape (n_channels, n_channels)
+        The whitener.
 
     Returns
     -------
     W : ndarray, shape (n_dipoles, n_channels)
         The beamformer filter weights.
     """
+    _check_option('weight_norm', weight_norm,
+                  ['unit-noise-gain-invariant', 'unit-noise-gain',
+                   'nai', None])
+
+    # Whiten the data covariance
+    Cm = whitener @ Cm @ whitener.T.conj()
+    # Restore to properly Hermitian as large whitening coefs can have bad
+    # rounding error
+    Cm[:] = (Cm + Cm.T.conj()) / 2.
+
+    assert Cm.shape == (G.shape[0],) * 2
+    s, _ = np.linalg.eigh(Cm)
+    if not (s >= -s.max() * 1e-7).all():
+        # This shouldn't ever happen, but just in case
+        warn('data covariance does not appear to be positive semidefinite, '
+             'results will likely be incorrect')
     # Tikhonov regularization using reg parameter to control for
     # trade-off between spatial resolution and noise sensitivity
     # eq. 25 in Gross and Ioannides, 1999 Phys. Med. Biol. 44 2081
     Cm_inv, loading_factor, rank = _reg_pinv(Cm, reg, rank)
-    Cm_inv_sq = Cm_inv.dot(Cm_inv)
 
-    # Compute spatial filters
-    W = np.dot(G.T, Cm_inv)
     assert orient_std.shape == (G.shape[1],)
     n_sources = G.shape[1] // n_orient
     assert nn.shape == (n_sources, 3)
 
     logger.info('Computing beamformer filters for %d source%s'
                 % (n_sources, _pl(n_sources)))
-    svd_lwork = _svd_lwork((3, 3), Cm.dtype)  # for real or complex
-    real_svd_lwork = _svd_lwork((3, 3))  # for one that will always be real
-    eig_lwork = _eig_lwork((3, 3), Cm.dtype)
-    inv_lwork = _inv_lwork((3, 3), Cm.dtype)
-    for k in range(n_sources):
-        this_sl = slice(n_orient * k, n_orient * k + n_orient)
-        Wk, Gk, sk = W[this_sl], G[:, this_sl], orient_std[this_sl]
+    n_channels = G.shape[0]
+    assert n_orient in (3, 1)
+    Gk = np.reshape(G.T, (n_sources, n_orient, n_channels)).transpose(0, 2, 1)
+    assert Gk.shape == (n_sources, n_channels, n_orient)
+    sk = np.reshape(orient_std, (n_sources, n_orient))
+    del G, orient_std
+    pinv_kwargs = dict()
+    if check_version('numpy', '1.17'):
+        pinv_kwargs['hermitian'] = True
 
-        if (inversion == 'matrix' and pick_ori == 'max-power' and
-                weight_norm in ['unit-noise-gain', 'nai']):
-            # In this case, take a shortcut to compute the filter
-            Wk[:] = _normalized_weights(
-                Wk, Gk, Cm_inv_sq, reduce_rank, nn[k], sk,
-                svd_lwork, inv_lwork, eig_lwork)
+    _check_option('reduce_rank', reduce_rank, (True, False))
+
+    # inversion of the denominator
+    _check_option('inversion', inversion, ('matrix', 'single'))
+    if inversion == 'single' and n_orient > 1 and pick_ori == 'vector' and \
+            weight_norm == 'unit-noise-gain-invariant':
+        raise ValueError(
+            'Cannot use pick_ori="vector" with inversion="single" and '
+            'weight_norm="unit-noise-gain-invariant"')
+    if reduce_rank and inversion == 'single':
+        raise ValueError('reduce_rank cannot be used with inversion="single"; '
+                         'consider using inversion="matrix" if you have a '
+                         'rank-deficient forward model (i.e., from a sphere '
+                         'model with MEG channels), otherwise consider using '
+                         'reduce_rank=False')
+    if n_orient > 1:
+        _, Gk_s, _ = np.linalg.svd(Gk, full_matrices=False)
+        assert Gk_s.shape == (n_sources, n_orient)
+        if not reduce_rank and (Gk_s[:, 0] > 1e6 * Gk_s[:, 2]).any():
+            raise ValueError(
+                'Singular matrix detected when estimating spatial filters. '
+                'Consider reducing the rank of the forward operator by using '
+                'reduce_rank=True.')
+        del Gk_s
+
+    #
+    # 1. Reduce rank of the lead field
+    #
+    if reduce_rank:
+        Gk = _reduce_leadfield_rank(Gk)
+
+    def _compute_bf_terms(Gk, Cm_inv):
+        bf_numer = np.matmul(Gk.swapaxes(-2, -1).conj(), Cm_inv)
+        bf_denom = np.matmul(bf_numer, Gk)
+        return bf_numer, bf_denom
+
+    #
+    # 2. Reorient lead field in direction of max power or normal
+    #
+    if pick_ori == 'max-power':
+        assert n_orient == 3
+        _, bf_denom = _compute_bf_terms(Gk, Cm_inv)
+        if weight_norm is None:
+            ori_numer = np.eye(n_orient)[np.newaxis]
+            ori_denom = bf_denom
         else:
-            # Compute power at the source
-            Ck = np.dot(Wk, Gk)
+            # compute power, cf Sekihara & Nagarajan 2008, eq. 4.47
+            ori_numer = bf_denom
+            # Cm_inv should be Hermitian so no need for .T.conj()
+            ori_denom = np.matmul(
+                np.matmul(Gk.swapaxes(-2, -1).conj(), Cm_inv @ Cm_inv), Gk)
+        ori_denom_inv = _sym_inv_sm(ori_denom, reduce_rank, inversion, sk)
+        ori_pick = np.matmul(ori_denom_inv, ori_numer)
+        assert ori_pick.shape == (n_sources, n_orient, n_orient)
 
-            # Normalize the spatial filters
-            if Wk.ndim == 2 and len(Wk) > 1:
-                # Free source orientation
-                if inversion == 'single':
-                    # Invert for each dipole separately using plain division
-                    with np.errstate(divide='ignore'):
-                        norm = np.diag(1. / np.diag(Ck))
-                elif inversion == 'matrix':
-                    # Invert for all dipoles simultaneously using matrix
-                    # inversion.
-                    assert Ck.shape == (3, 3)
-                    norm = _repeated_pinv2(Ck, svd_lwork)
-                # Reapply source covariance after inversion
-                norm *= sk
-                norm *= sk[:, np.newaxis]
-            else:
-                assert Ck.shape == (1, 1)
-                # Fixed source orientation
-                norm = np.eye(1) if Ck[0, 0] == 0. else 1. / Ck
-            Wk[:] = np.dot(norm, Wk)
+        # pick eigenvector that corresponds to maximum eigenvalue:
+        eig_vals, eig_vecs = np.linalg.eig(ori_pick.real)  # not Hermitian!
+        # sort eigenvectors by eigenvalues for picking:
+        order = np.argsort(np.abs(eig_vals), axis=-1)
+        # eig_vals = np.take_along_axis(eig_vals, order, axis=-1)
+        max_power_ori = eig_vecs[np.arange(len(eig_vecs)), :, order[:, -1]]
+        assert max_power_ori.shape == (n_sources, n_orient)
 
-            if pick_ori == 'max-power':
-                # Compute the power
-                if inversion == 'single' and weight_norm is not None:
-                    # First make the filters unit gain, then apply them to the
-                    # cov matrix to compute power.
-                    Wk_norm = Wk / np.sqrt(np.sum(Wk ** 2, axis=1,
-                                                  keepdims=True))
-                    power = Wk_norm.dot(Cm).dot(Wk_norm.T)
-                elif weight_norm is None:
-                    # Compute power by applying the spatial filters to
-                    # the cov matrix.
-                    power = Wk.dot(Cm).dot(Wk.T)
+        # set the (otherwise arbitrary) sign to match the normal
+        signs = np.sign(np.sum(max_power_ori * nn, axis=1, keepdims=True))
+        signs[signs == 0] = 1.
+        max_power_ori *= signs
 
-                # Compute the direction of max power
-                u, s, _ = _repeated_svd(power.real, real_svd_lwork)
-                max_power_ori = u[:, 0]
+        # Compute the lead field for the optimal orientation,
+        # and adjust numer/denom
+        Gk = np.matmul(Gk, max_power_ori[..., np.newaxis])
+        n_orient = 1
+    else:
+        max_power_ori = None
+        if pick_ori == 'normal':
+            Gk = Gk[..., 2:3]
+            n_orient = 1
 
-                # set the (otherwise arbitrary) sign to match the normal
-                sign = np.sign(np.dot(nn[k], max_power_ori)) or 1  # avoid 0
-                max_power_ori *= sign
+    #
+    # 3. Compute numerator and denominator of beamformer formula (unit-gain)
+    #
 
-                # Re-compute the filter in the direction of max power
-                Wk[:] = max_power_ori.dot(Wk)
+    bf_numer, bf_denom = _compute_bf_terms(Gk, Cm_inv)
+    assert bf_denom.shape == (n_sources,) + (n_orient,) * 2
+    assert bf_numer.shape == (n_sources, n_orient, n_channels)
+    del Gk  # lead field has been adjusted and should not be used anymore
 
-    if pick_ori == 'normal':
-        W = W[2::3]
-    elif pick_ori == 'max-power':
-        W = W[0::3]
+    #
+    # 4. Invert the denominator
+    #
 
-    # Re-scale the filter weights according to the selected weight
-    # normalization scheme
-    if weight_norm in ['unit-noise-gain', 'nai']:
-        if pick_ori in [None, 'vector'] and n_orient > 1:
-            # Rescale each set of 3 filters
-            W = W.reshape(-1, 3, W.shape[1])
-            noise_norm = np.sqrt(np.sum(W ** 2, axis=(1, 2), keepdims=True))
+    # Here W is W_ug, i.e.:
+    # G.T @ Cm_inv / (G.T @ Cm_inv @ G)
+    bf_denom_inv = _sym_inv_sm(bf_denom, reduce_rank, inversion, sk)
+    assert bf_denom_inv.shape == (n_sources, n_orient, n_orient)
+    W = np.matmul(bf_denom_inv, bf_numer)
+    assert W.shape == (n_sources, n_orient, n_channels)
+    del bf_denom_inv, sk
+
+    #
+    # 5. Re-scale filter weights according to the selected weight_norm
+    #
+
+    # Weight normalization is done by computing, for each source::
+    #
+    #     W_ung = W_ug / sqrt(W_ug @ W_ug.T)
+    #
+    # with W_ung referring to the unit-noise-gain (weight normalized) filter
+    # and W_ug referring to the above-calculated unit-gain filter stored in W.
+
+    if weight_norm is not None:
+        # Three different ways to calculate the normalization factors here.
+        # Only matters when in vector mode, as otherwise n_orient == 1 and
+        # they are all equivalent. Sekihara 2008 says to use
+        #
+        # In MNE < 0.21, we just used the Frobenius matrix norm:
+        #
+        #    noise_norm = np.linalg.norm(W, axis=(1, 2), keepdims=True)
+        #    assert noise_norm.shape == (n_sources, 1, 1)
+        #    W /= noise_norm
+        #
+        # Sekihara 2008 says to use sqrt(diag(W_ug @ W_ug.T)), which is not
+        # rotation invariant:
+        if weight_norm in ('unit-noise-gain', 'nai'):
+            noise_norm = np.matmul(W, W.swapaxes(-2, -1).conj()).real
+            noise_norm = np.reshape(  # np.diag operation over last two axes
+                noise_norm, (n_sources, -1, 1))[:, ::n_orient + 1]
+            np.sqrt(noise_norm, out=noise_norm)
+            noise_norm[noise_norm == 0] = np.inf
+            assert noise_norm.shape == (n_sources, n_orient, 1)
+            W /= noise_norm
         else:
-            # Rescale each filter separately
-            noise_norm = np.sqrt(np.sum(W ** 2, axis=1, keepdims=True))
+            assert weight_norm == 'unit-noise-gain-invariant'
+            # Here we use sqrtm. The shortcut:
+            #
+            #    use = W
+            #
+            # ... does not match the direct route (it is rotated!), so we'll
+            # use the direct one to match FieldTrip:
+            use = bf_numer
+            inner = np.matmul(use, use.swapaxes(-2, -1).conj())
+            W = np.matmul(_sym_mat_pow(inner, -0.5), use)
+            noise_norm = 1.
 
         if weight_norm == 'nai':
             # Estimate noise level based on covariance matrix, taking the
@@ -332,23 +376,17 @@ def _compute_beamformer(G, Cm, reg, n_orient, weight_norm, pick_ori,
                         'matrix or using regularization.')
                 noise = loading_factor
             else:
-                noise, _ = linalg.eigh(Cm)
+                noise, _ = np.linalg.eigh(Cm)
                 noise = noise[-rank]
                 noise = max(noise, loading_factor)
-            noise_norm *= np.sqrt(noise)
-
-        # Apply the normalization
-        if np.all(noise_norm == 0.):
-            noise_norm_inv = 0.  # avoid division by 0
-        else:
-            noise_norm_inv = 1 / noise_norm
-        W *= noise_norm_inv
-        W = W.reshape(-1, W.shape[-1])
+            W /= np.sqrt(noise)
 
+    W = W.reshape(n_sources * n_orient, n_channels)
     logger.info('Filter computation complete')
-    return W
+    return W, max_power_ori
 
 
+# TODO: Eventually we can @jit() this to make it faster
 def _compute_power(Cm, W, n_orient):
     """Use beamformer filters to compute source power.
 
@@ -369,12 +407,7 @@ def _compute_power(Cm, W, n_orient):
     source_power = np.zeros(n_sources)
     for k in range(n_sources):
         Wk = W[n_orient * k: n_orient * k + n_orient]
-        power = Wk.dot(Cm).dot(Wk.T)
-
-        if n_orient > 1:  # Pool the orientations
-            source_power[k] = np.abs(power.trace())
-        else:
-            source_power[k] = np.abs(power)
+        source_power[k] = np.trace(Wk @ Cm @ Wk.conj().T).real
 
     return source_power
 
@@ -404,7 +437,7 @@ class Beamformer(dict):
             subject = 'unknown'
         else:
             subject = '"%s"' % (self['subject'],)
-        out = ('<Beamformer  |  %s, subject %s, %s vert, %s ch'
+        out = ('<Beamformer | %s, subject %s, %s vert, %s ch'
                % (self['kind'], subject, n_verts, n_channels))
         if self['pick_ori'] is not None:
             out += ', %s ori' % (self['pick_ori'],)
@@ -426,8 +459,7 @@ class Beamformer(dict):
         fname : str
             The filename to use to write the HDF5 data.
             Should end in ``'-lcmv.h5'`` or ``'-dics.h5'``.
-        overwrite : bool
-            If True, overwrite the file (if it exists).
+        %(overwrite)s
         %(verbose)s
         """
         ending = '-%s.h5' % (self['kind'].lower(),)
@@ -475,3 +507,15 @@ def read_beamformer(fname):
                   for arg in ('data', 'names', 'bads', 'projs', 'nfree', 'eig',
                               'eigvec', 'method', 'loglik')])
     return Beamformer(beamformer)
+
+
+def _proj_whiten_data(M, proj, filters):
+    if filters.get('is_ssp', True):
+        # check whether data and filter projs match
+        _check_proj_match(proj, filters)
+        if filters['whitener'] is None:
+            M = np.dot(filters['proj'], M)
+
+    if filters['whitener'] is not None:
+        M = np.dot(filters['whitener'], M)
+    return M
