@@ -8,27 +8,32 @@
 # License: BSD (3-clause)
 import numpy as np
 
+from ..channels import equalize_channels
+from ..io.pick import pick_info, pick_channels
 from ..utils import (logger, verbose, warn, _check_one_ch_type,
                      _check_channels_spatial_filter, _check_rank,
-                     _check_option)
+                     _check_option, _validate_type, deprecated)
 from ..forward import _subject_from_forward
-from ..minimum_norm.inverse import combine_xyz, _check_reference
+from ..minimum_norm.inverse import combine_xyz, _check_reference, _check_depth
+from ..rank import compute_rank
 from ..source_estimate import _make_stc, _get_src_type
 from ..time_frequency import csd_fourier, csd_multitaper, csd_morlet
-from ._compute_beamformer import (_check_proj_match, _prepare_beamformer_input,
+from ._compute_beamformer import (_prepare_beamformer_input,
                                   _compute_beamformer, _check_src_type,
-                                  Beamformer, _compute_power)
+                                  Beamformer, _compute_power,
+                                  _proj_whiten_data)
 
 
 @verbose
-def make_dics(info, forward, csd, reg=0.05, label=None, pick_ori=None,
-              rank=None, inversion='single', weight_norm=None,
-              normalize_fwd=True, real_filter=False, reduce_rank=False,
-              verbose=None):
+def make_dics(info, forward, csd, reg=0.05, noise_csd=None, label=None,
+              pick_ori=None, rank=None, weight_norm=None,
+              reduce_rank=False, depth=1., real_filter=None,
+              inversion='matrix', verbose=None):
     """Compute a Dynamic Imaging of Coherent Sources (DICS) spatial filter.
 
     This is a beamformer filter that can be used to estimate the source power
-    at a specific frequency range [1]_. It does this by constructing a spatial
+    at a specific frequency range :footcite:`GrossEtAl2001`. It does this by
+    constructing a spatial
     filter for each source point.  The computation of these filters is very
     similar to those of the LCMV beamformer (:func:`make_lcmv`), but instead of
     operating on a covariance matrix, the CSD matrix is used. When applying
@@ -48,58 +53,36 @@ def make_dics(info, forward, csd, reg=0.05, label=None, pick_ori=None,
     reg : float
         The regularization to apply to the cross-spectral density before
         computing the inverse.
+    noise_csd : instance of CrossSpectralDensity | None
+        Noise cross-spectral density (CSD) matrices. If provided, whitening
+        will be done. The noise CSDs need to have been computed for the same
+        frequencies as the data CSDs. Providing noise CSDs is mandatory if you
+        mix sensor types, e.g. gradiometers with magnetometers or EEG with
+        MEG.
+
+        .. versionadded:: 0.20
     label : Label | None
         Restricts the solution to a given label.
-    pick_ori : None | 'normal' | 'max-power'
-        The source orientation to compute the filter for:
-
-            ``None`` :
-                orientations are pooled (Default)
-            'normal' :
-                filters are computed for the orientation tangential to the
-                cortical surface
-            'max-power' :
-                filters are computer for the orientation that maximizes
-                spectral power.
-
-    rank : None | int | 'full'
-        This controls the effective rank of the covariance matrix when
-        computing the inverse. The rank can be set explicitly by specifying an
-        integer value. If ``None``, the rank will be automatically estimated.
-        Since applying regularization will always make the covariance matrix
-        full rank, the rank is estimated before regularization in this case. If
-        'full', the rank will be estimated after regularization and hence
-        will mean using the full rank, unless ``reg=0`` is used.
-        The default is None.
+    %(bf_pick_ori)s
+    %(rank_None)s
 
         .. versionadded:: 0.17
-    inversion : 'single' | 'matrix'
-        This determines how the beamformer deals with source spaces in "free"
-        orientation. Such source spaces define three orthogonal dipoles at each
-        source point. When ``inversion='single'``, each dipole is considered
-        as an individual source and the corresponding spatial filter is
-        computed for each dipole separately. When ``inversion='matrix'``, all
-        three dipoles at a source vertex are considered as a group and the
-        spatial filters are computed jointly using a matrix inversion. While
-        ``inversion='single'`` is more stable, ``inversion='matrix'`` is more
-        precise. See section 5 of [5]_.  Defaults to 'single'.
-    weight_norm : 'unit-noise-gain' | 'nai' | None
-        If 'unit-noise-gain', the unit-noise gain minimum variance beamformer
-        will be computed (Borgiotti-Kaplan beamformer) [2]_,
-        If 'nai', the Neural Activity Index [4]_ will be computed.
+    %(weight_norm)s
+
         Defaults to ``None``, in which case no normalization is performed.
-    normalize_fwd : bool
-        Whether to normalize the forward solution. Defaults to ``True``. Note
-        that this normalization is not required when weight normalization
-        (``weight_norm``) is used.
+    %(reduce_rank)s
+    %(depth)s
     real_filter : bool
         If ``True``, take only the real part of the cross-spectral-density
-        matrices to compute real filters. Defaults to ``False``.
-    reduce_rank : bool
-        If ``True``, the rank of the forward operator will be reduced by 1 for
-        each spatial location, prior to inversion. This may be necessary when
-        you use a single sphere model for MEG and ``mode='vertex'``.
-        Defaults to ``False``.
+        matrices to compute real filters.
+
+        .. versionchanged:: 0.23
+            Version 0.23 deprecated ``False`` as default for ``real_filter``.
+            With version 0.24, ``True`` will be the new default.
+    %(bf_inversion)s
+
+        .. versionchanged:: 0.21
+           Default changed to ``'matrix'``.
     %(verbose)s
 
     Returns
@@ -108,6 +91,8 @@ def make_dics(info, forward, csd, reg=0.05, label=None, pick_ori=None,
         Dictionary containing filter weights from DICS beamformer.
         Contains the following keys:
 
+            'kind' : str
+                The type of beamformer, in this case 'DICS'.
             'weights' : ndarray, shape (n_frequencies, n_weights)
                 For each frequency, the filter weights of the beamformer.
             'csd' : instance of CrossSpectralDensity
@@ -119,20 +104,31 @@ def make_dics(info, forward, csd, reg=0.05, label=None, pick_ori=None,
                 Projections used to compute the beamformer.
             'vertices' : list of ndarray
                 Vertices for which the filter weights were computed.
+            'n_sources' : int
+                Number of source location for which the filter weight were
+                computed.
+            'subject' : str
+                The subject ID.
+            'pick-ori' : None | 'max-power' | 'normal' | 'vector'
+                The orientation in which the beamformer filters were computed.
             'inversion' : 'single' | 'matrix'
                 Whether the spatial filters were computed for each dipole
                 separately or jointly for all dipoles at each vertex using a
                 matrix inversion.
             'weight_norm' : None | 'unit-noise-gain'
                 The normalization of the weights.
-            'normalize_fwd' : bool
-                Whether the forward solution was normalized
-            'n_orient' : int
-                Number of source orientations defined in the forward model.
-            'subject' : str
-                The subject ID.
             'src_type' : str
                 Type of source space.
+            'is_free_ori' : bool
+                Whether the filter was computed in a fixed direction
+                (pick_ori='max-power', pick_ori='normal') or not.
+            'whitener' : None | ndarray, shape (n_channels, n_channels)
+                Whitening matrix, provided if whitening was applied to the
+                covariance matrix and leadfield during computation of the
+                beamformer weights.
+            'max-power-ori' : ndarray, shape (n_sources, 3) | None
+                When pick_ori='max-power', this fields contains the estimated
+                direction of maximum power at each source location.
 
     See Also
     --------
@@ -140,8 +136,8 @@ def make_dics(info, forward, csd, reg=0.05, label=None, pick_ori=None,
 
     Notes
     -----
-    The original reference is [1]_. See [5]_ for a tutorial style paper on the
-    topic.
+    The original reference is :footcite:`GrossEtAl2001`. See
+    :footcite:`vanVlietEtAl2018` for a tutorial style paper on the topic.
 
     The DICS beamformer is very similar to the LCMV (:func:`make_lcmv`)
     beamformer and many of the parameters are shared. However,
@@ -151,88 +147,77 @@ def make_dics(info, forward, csd, reg=0.05, label=None, pick_ori=None,
     searching), and it remains to be seen how functionally interchangeable they
     could be.
 
-    The default setting reproduce the DICS beamformer as described in [5]_::
+    The default setting reproduce the DICS beamformer as described in
+    :footcite:`vanVlietEtAl2018`::
 
-        inversion='single', weight_norm=None, normalize_fwd=True
+        inversion='single', weight_norm=None, depth=1.
 
     To use the :func:`make_lcmv` defaults, use::
 
-        inversion='matrix', weight_norm='unit-gain', normalize_fwd=False
+        inversion='matrix', weight_norm='unit-noise-gain-invariant', depth=None
 
     For more information about ``real_filter``, see the
-    supplemental information from [3]_.
+    supplemental information from :footcite:`HippEtAl2011`.
 
     References
     ----------
-    .. [1] Gross et al. (2001) Dynamic imaging of coherent sources: Studying
-           neural interactions in the human brain. PNAS vol. 98 (2)
-           pp. 694-699. https://doi.org/10.1073/pnas.98.2.694
-    .. [2] Sekihara & Nagarajan. Adaptive spatial filters for electromagnetic
-           brain imaging (2008) Springer Science & Business Media
-    .. [3] Hipp JF, Engel AK, Siegel M (2011) Oscillatory Synchronization
-           in Large-Scale Cortical Networks Predicts Perception.
-           Neuron (2011) vol 69 pp. 387-396.
-           https://doi.org/10.1016/j.neuron.2010.12.027
-    .. [4] Van Veen et al. Localization of brain electrical activity via
-           linearly constrained minimum variance spatial filtering.
-           Biomedical Engineering (1997) vol. 44 (9) pp. 867--880
-    .. [5] van Vliet, et al. (2018) Analysis of functional connectivity and
-           oscillatory power using DICS: from raw MEG data to group-level
-           statistics in Python. bioRxiv, 245530.
-           https://doi.org/10.1101/245530
+    .. footbibliography::
     """  # noqa: E501
     rank = _check_rank(rank)
     _check_option('pick_ori', pick_ori, [None, 'normal', 'max-power'])
     _check_option('inversion', inversion, ['single', 'matrix'])
-    _check_option('weight_norm', weight_norm, ['unit-noise-gain', 'nai', None])
-
-    # Leadfield rank and optional rank reduction
-    # (to deal with problems with complex eigenvalues within the computation
-    # of the optimal orientation when using pinv if the leadfield was only
-    # rank 2 (e.g., with the spherical headmodel of the phantom data),
-    # see gh-4568 and gh-4628.
-    if reduce_rank and not (pick_ori == 'max-power' and inversion == 'matrix'):
-        raise NotImplementedError(
-            'The computation of spatial filters with rank reduction using '
-            'reduce_rank=True is only implemented with pick_ori=="max-power" '
-            'and inversion="matrix".'
-        )
+    _validate_type(weight_norm, (str, None), 'weight_norm')
 
     frequencies = [np.mean(freq_bin) for freq_bin in csd.frequencies]
     n_freqs = len(frequencies)
-    n_orient = forward['sol']['ncol'] // forward['nsource']
 
-    # Determine how to normalize the leadfield
-    if normalize_fwd:
-        if inversion == 'single':
-            if weight_norm == 'unit-noise-gain':
-                raise ValueError('The computation of a unit-noise-gain '
-                                 'beamformer with inversion="single" is not '
-                                 'stable with depth normalization, set  '
-                                 'normalize_fwd to False.')
-            combine_xyz = False
-        else:
-            combine_xyz = 'fro'
-        exp = 1.  # turn on depth weighting with exponent 1
-    else:
-        exp = None  # turn off depth weighting entirely
-        combine_xyz = False
+    _, _, allow_mismatch = _check_one_ch_type('dics', info, forward, csd,
+                                              noise_csd)
+    # remove bads so that equalize_channels only keeps all good
+    info = pick_info(info, pick_channels(info['ch_names'], [], info['bads']))
+    info, forward, csd = equalize_channels([info, forward, csd])
 
-    _check_one_ch_type('dics', info, forward)
+    if real_filter is None:
+        depr_message = ('The current default of real_filter=False is '
+                        'deprecated and will be changed to real_filter=True '
+                        'in version 0.24. Set real_filter explicitly to '
+                        'avoid this warning.')
+        warn(depr_message, DeprecationWarning)
+        real_filter = False
 
-    # pick info, get gain matrix, etc.
-    _, info, proj, vertices, G, _, nn, orient_std = _prepare_beamformer_input(
-        info, forward, label, pick_ori,
-        combine_xyz=combine_xyz, exp=exp)
-    subject = _subject_from_forward(forward)
-    src_type = _get_src_type(forward['src'], vertices)
-    del forward
+    csd, noise_csd = _prepare_noise_csd(csd, noise_csd, real_filter)
+
+    depth = _check_depth(depth, 'depth_sparse')
+    if inversion == 'single':
+        depth['combine_xyz'] = False
+
+    is_free_ori, info, proj, vertices, G, whitener, nn, orient_std = \
+        _prepare_beamformer_input(
+            info, forward, label, pick_ori, noise_cov=noise_csd, rank=rank,
+            pca=False, **depth)
+
+    # Compute ranks
+    csd_int_rank = []
+    if not allow_mismatch:
+        noise_rank = compute_rank(noise_csd, info=info, rank=rank)
+    for i in range(len(frequencies)):
+        csd_rank = compute_rank(csd.get_data(index=i, as_cov=True),
+                                info=info, rank=rank)
+        if not allow_mismatch:
+            for key in csd_rank:
+                if key not in noise_rank or csd_rank[key] != noise_rank[key]:
+                    raise ValueError('%s data rank (%s) did not match the '
+                                     'noise rank (%s)'
+                                     % (key, csd_rank[key],
+                                        noise_rank.get(key, None)))
+        csd_int_rank.append(sum(csd_rank.values()))
+
+    del noise_csd
     ch_names = list(info['ch_names'])
 
-    csd_picks = [csd.ch_names.index(ch) for ch in ch_names]
-
     logger.info('Computing DICS spatial filters...')
     Ws = []
+    max_oris = []
     for i, freq in enumerate(frequencies):
         if n_freqs > 1:
             logger.info('    computing DICS spatial filter at %sHz (%d/%d)' %
@@ -240,30 +225,53 @@ def make_dics(info, forward, csd, reg=0.05, label=None, pick_ori=None,
 
         Cm = csd.get_data(index=i)
 
+        # XXX: Weird that real_filter happens *before* whitening, which could
+        # make things complex again...?
         if real_filter:
             Cm = Cm.real
 
-        # Ensure the CSD is in the same order as the leadfield
-        Cm = Cm[csd_picks, :][:, csd_picks]
-
         # compute spatial filter
-        W = _compute_beamformer(G, Cm, reg, n_orient, weight_norm, pick_ori,
-                                reduce_rank, rank=rank, inversion=inversion,
-                                nn=nn, orient_std=orient_std)
+        n_orient = 3 if is_free_ori else 1
+        W, max_power_ori = _compute_beamformer(
+            G, Cm, reg, n_orient, weight_norm, pick_ori, reduce_rank,
+            rank=csd_int_rank[i], inversion=inversion, nn=nn,
+            orient_std=orient_std, whitener=whitener)
         Ws.append(W)
+        max_oris.append(max_power_ori)
 
     Ws = np.array(Ws)
+    if pick_ori == 'max-power':
+        max_oris = np.array(max_oris)
+    else:
+        max_oris = None
+
+    src_type = _get_src_type(forward['src'], vertices)
+    subject = _subject_from_forward(forward)
+    is_free_ori = is_free_ori if pick_ori in [None, 'vector'] else False
+    n_sources = np.sum([len(v) for v in vertices])
 
     filters = Beamformer(
         kind='DICS', weights=Ws, csd=csd, ch_names=ch_names, proj=proj,
-        vertices=vertices, subject=subject, pick_ori=pick_ori,
-        inversion=inversion, weight_norm=weight_norm,
-        normalize_fwd=bool(normalize_fwd), src_type=src_type,
-        n_orient=n_orient if pick_ori is None else 1)
+        vertices=vertices, n_sources=n_sources, subject=subject,
+        pick_ori=pick_ori, inversion=inversion, weight_norm=weight_norm,
+        src_type=src_type, is_free_ori=is_free_ori, whitener=whitener,
+        max_power_ori=max_oris)
 
     return filters
 
 
+def _prepare_noise_csd(csd, noise_csd, real_filter):
+    if noise_csd is not None:
+        csd, noise_csd = equalize_channels([csd, noise_csd])
+        # Use the same noise CSD for all frequencies
+        if len(noise_csd.frequencies) > 1:
+            noise_csd = noise_csd.mean()
+        noise_csd = noise_csd.get_data(as_cov=True)
+        if real_filter:
+            noise_csd['data'] = noise_csd['data'].real
+    return csd, noise_csd
+
+
 def _apply_dics(data, filters, info, tmin):
     """Apply DICS spatial filter to data for source reconstruction."""
     if isinstance(data, np.ndarray) and data.ndim == 2:
@@ -284,16 +292,14 @@ def _apply_dics(data, filters, info, tmin):
             logger.info("Processing epoch : %d" % (i + 1))
 
         # Apply SSPs
-        if info['projs']:
-            _check_proj_match(info, filters)
-            M = np.dot(filters['proj'], M)
+        M = _proj_whiten_data(M, info['projs'], filters)
 
         stcs = []
         for W in Ws:
             # project to source space using beamformer weights
             sol = np.dot(W, M)
 
-            if filters['n_orient'] > 1:
+            if filters['is_free_ori']:
                 logger.info('combining the current components...')
                 sol = combine_xyz(sol)
 
@@ -431,7 +437,7 @@ def apply_dics_csd(csd, filters, verbose=None):
 
     Apply a previously computed DICS beamformer to a cross-spectral density
     (CSD) object to estimate source power in time and frequency windows
-    specified in the CSD object [1]_.
+    specified in the CSD object :footcite:`GrossEtAl2001`.
 
     Parameters
     ----------
@@ -455,14 +461,14 @@ def apply_dics_csd(csd, filters, verbose=None):
 
     References
     ----------
-    .. [1] Gross et al. Dynamic imaging of coherent sources: Studying neural
-           interactions in the human brain. PNAS (2001) vol. 98 (2) pp. 694-699
+    .. footbibliography::
     """  # noqa: E501
     ch_names = filters['ch_names']
     vertices = filters['vertices']
-    n_orient = filters['n_orient']
+    n_orient = 3 if filters['is_free_ori'] else 1
     subject = filters['subject']
-    n_sources = np.sum([len(v) for v in vertices])
+    whitener = filters['whitener']
+    n_sources = filters['n_sources']
 
     # If CSD is summed over multiple frequencies, take the average frequency
     frequencies = [np.mean(dfreq) for dfreq in csd.frequencies]
@@ -483,6 +489,9 @@ def apply_dics_csd(csd, filters, verbose=None):
         Cm = Cm[csd_picks, :][:, csd_picks]
         W = filters['weights'][i]
 
+        # Whiten the CSD
+        Cm = np.dot(whitener, np.dot(Cm, whitener.conj().T))
+
         source_power[:, i] = _compute_power(Cm, W, n_orient)
 
     logger.info('[done]')
@@ -496,21 +505,24 @@ def apply_dics_csd(csd, filters, verbose=None):
             frequencies)
 
 
+@deprecated(
+    'tf_dics is deprecated and will be removed in 0.24, use LCMV with '
+    'covariances matrices computed on band-passed data or DICS instead.')
 @verbose
 def tf_dics(epochs, forward, noise_csds, tmin, tmax, tstep, win_lengths,
             subtract_evoked=False, mode='fourier', freq_bins=None,
             frequencies=None, n_ffts=None, mt_bandwidths=None,
             mt_adaptive=False, mt_low_bias=True, cwt_n_cycles=7, decim=1,
             reg=0.05, label=None, pick_ori=None, rank=None, inversion='single',
-            weight_norm=None, normalize_fwd=True, real_filter=False,
+            weight_norm=None, depth=1., real_filter=False,
             reduce_rank=False, verbose=None):
     """5D time-frequency beamforming based on DICS.
 
     Calculate source power in time-frequency windows using a spatial filter
     based on the Dynamic Imaging of Coherent Sources (DICS) beamforming
-    approach [1]_. For each time window and frequency bin combination,
-    cross-spectral density (CSD) is computed and used to create a DICS
-    beamformer spatial filter.
+    approach :footcite:`DalalEtAl2008`. For each time window and frequency bin
+    combination, cross-spectral density (CSD) is computed and used to create a
+    DICS beamformer spatial filter.
 
     Parameters
     ----------
@@ -563,7 +575,7 @@ def tf_dics(epochs, forward, noise_csds, tmin, tmax, tstep, win_lengths,
     mt_low_bias : bool
         Only use tapers with more than 90%% spectral concentration within
         bandwidth. Only used in 'multitaper' mode. Defaults to True.
-    cwt_n_cycles: float | list of float | None
+    cwt_n_cycles : float | list of float | None
         Number of cycles to use when constructing Morlet wavelets. Fixed number
         or one per frequency. Defaults to 7.
         Only used in 'cwt_morlet' mode.
@@ -617,20 +629,13 @@ def tf_dics(epochs, forward, noise_csds, tmin, tmax, tstep, win_lengths,
     weight_norm : None | 'unit-noise-gain'
         How to normalize the beamformer weights. None means no normalization is
         performed.  If 'unit-noise-gain', the unit-noise gain minimum variance
-        beamformer will be computed (Borgiotti-Kaplan beamformer) [2]_.
-        Defaults to ``None``.
-    normalize_fwd : bool
-        Whether to normalize the forward solution. Defaults to ``True``. Note
-        that this normalization is not required when weight normalization
-        (``weight_norm``) is used.
+        beamformer will be computed (Borgiotti-Kaplan beamformer)
+        :footcite:`SekiharaNagarajan2008`. Defaults to ``None``.
+    %(depth)s
     real_filter : bool
         If ``True``, take only the real part of the cross-spectral-density
         matrices to compute real filters. Defaults to ``False``.
-    reduce_rank : bool
-        If ``True``, the rank of the forward operator will be reduced by 1 for
-        each spatial location, prior to inversion. This may be necessary when
-        you use a single sphere model for MEG and ``mode='vertex'``.
-        Defaults to ``False``.
+    %(reduce_rank)s
     %(verbose)s
 
     Returns
@@ -641,20 +646,17 @@ def tf_dics(epochs, forward, noise_csds, tmin, tmax, tstep, win_lengths,
 
     Notes
     -----
-    Dalal et al. [1]_ used a synthetic aperture magnetometry beamformer (SAM)
-    in each time-frequency window instead of DICS.
+    Dalal et al. :footcite:`DalalEtAl2008` used a synthetic aperture
+    magnetometry beamformer (SAM) in each time-frequency window instead of
+    DICS.
 
     An alternative to using noise CSDs is to normalize the forward solution
-    (``normalize_fwd``) or the beamformer weights (``weight_norm``). In
+    (``depth``) or the beamformer weights (``weight_norm``). In
     this case, ``noise_csds`` may be set to ``None``.
 
     References
     ----------
-    .. [1] Dalal et al. Five-dimensional neuroimaging: Localization of the
-           time-frequency dynamics of cortical activity.
-           NeuroImage (2008) vol. 40 (4) pp. 1686-1700
-    .. [2] Sekihara & Nagarajan. Adaptive spatial filters for electromagnetic
-           brain imaging (2008) Springer Science & Business Media
+    .. footbibliography::
     """
     _check_reference(epochs)
     rank = _check_rank(rank)
@@ -689,6 +691,10 @@ def tf_dics(epochs, forward, noise_csds, tmin, tmax, tstep, win_lengths,
         raise ValueError('When using multitaper mode and specifying '
                          'multitaper transform bandwidth, one value must be '
                          'provided per frequency bin')
+    if isinstance(cwt_n_cycles, (int, float)):
+        # create a list out of single values to match n_freq_bins
+        n_cyc = cwt_n_cycles
+        cwt_n_cycles = [n_cyc] * n_freq_bins
 
     # Multiplying by 1e3 to avoid numerical issues, e.g. 0.3 // 0.05 == 5
     n_time_steps = int(((tmax - tmin) * 1e3) // (tstep * 1e3))
@@ -713,6 +719,7 @@ def tf_dics(epochs, forward, noise_csds, tmin, tmax, tstep, win_lengths,
             freq_bin = frequencies[i_freq]
             fmin = np.min(freq_bin)
             fmax = np.max(freq_bin)
+            n_cycles = cwt_n_cycles[i_freq]
         else:
             fmin, fmax = freq_bins[i_freq]
             if n_ffts is None:
@@ -763,7 +770,7 @@ def tf_dics(epochs, forward, noise_csds, tmin, tmax, tstep, win_lengths,
                 elif mode == 'cwt_morlet':
                     csd = csd_morlet(
                         epochs, frequencies=freq_bin, tmin=win_tmin,
-                        tmax=win_tmax, n_cycles=cwt_n_cycles, decim=decim,
+                        tmax=win_tmax, n_cycles=n_cycles, decim=decim,
                         verbose=False)
                 else:
                     raise ValueError('Invalid mode, choose either '
@@ -778,8 +785,7 @@ def tf_dics(epochs, forward, noise_csds, tmin, tmax, tstep, win_lengths,
                 filters = make_dics(epochs.info, forward, csd, reg=reg,
                                     label=label, pick_ori=pick_ori,
                                     rank=rank, inversion=inversion,
-                                    weight_norm=weight_norm,
-                                    normalize_fwd=normalize_fwd,
+                                    weight_norm=weight_norm, depth=depth,
                                     reduce_rank=reduce_rank,
                                     real_filter=real_filter, verbose=False)
                 stc, _ = apply_dics_csd(csd, filters, verbose=False)
