@@ -15,9 +15,15 @@ from mne.datasets import testing
 from mne.beamformer import (make_dics, apply_dics, apply_dics_epochs,
                             apply_dics_csd, tf_dics, read_beamformer,
                             Beamformer)
+from mne.beamformer._compute_beamformer import _prepare_beamformer_input
+from mne.beamformer._dics import _prepare_noise_csd
 from mne.time_frequency import csd_morlet
-from mne.utils import run_tests_if_main, object_diff, requires_h5py
+from mne.utils import object_diff, requires_h5py, catch_logging
 from mne.proj import compute_proj_evoked, make_projector
+from mne.surface import _compute_nearest
+from mne.beamformer.tests.test_lcmv import _assert_weight_norm
+from mne.time_frequency import CrossSpectralDensity
+from mne.time_frequency.csd import _sym_mat_to_vector
 
 data_path = testing.data_path(download=False)
 fname_raw = op.join(data_path, 'MEG', 'sample', 'sample_audvis_trunc_raw.fif')
@@ -29,8 +35,6 @@ fname_event = op.join(data_path, 'MEG', 'sample',
                       'sample_audvis_trunc_raw-eve.fif')
 
 subjects_dir = op.join(data_path, 'subjects')
-fname_label = op.join(subjects_dir, 'sample', 'label', 'aparc',
-                      'rostralmiddlefrontal-lh.label')
 
 
 @pytest.fixture(scope='module', params=[testing._pytest_param()])
@@ -44,14 +48,12 @@ def _load_forward():
     fwd_fixed = mne.convert_forward_solution(fwd_free, force_fixed=True,
                                              use_cps=False)
     fwd_vol = mne.read_forward_solution(fname_fwd_vol)
-    label = mne.read_label(fname_label)
-
-    return fwd_free, fwd_surf, fwd_fixed, fwd_vol, label
+    return fwd_free, fwd_surf, fwd_fixed, fwd_vol
 
 
-def _simulate_data(fwd):
+def _simulate_data(fwd, idx):  # Somewhere on the frontal lobe by default
     """Simulate an oscillator on the cortex."""
-    source_vertno = 146374  # Somewhere on the frontal lobe
+    source_vertno = fwd['src'][0]['vertno'][idx]
 
     sfreq = 50.  # Hz.
     times = np.arange(10 * sfreq) / sfreq  # 10 seconds of data
@@ -91,64 +93,114 @@ def _simulate_data(fwd):
     evoked = epochs.average()
 
     # Compute the cross-spectral density matrix
-    csd = csd_morlet(epochs, frequencies=[10, 20], n_cycles=[5, 10], decim=10)
+    csd = csd_morlet(epochs, frequencies=[10, 20], n_cycles=[5, 10], decim=5)
 
-    return epochs, evoked, csd, source_vertno
+    labels = mne.read_labels_from_annot(
+        'sample', hemi='lh', subjects_dir=subjects_dir)
+    label = [
+        label for label in labels if np.in1d(source_vertno, label.vertices)[0]]
+    assert len(label) == 1
+    label = label[0]
+    vertices = np.intersect1d(label.vertices, fwd['src'][0]['vertno'])
+    source_ind = vertices.tolist().index(source_vertno)
+    assert vertices[source_ind] == source_vertno
+    return epochs, evoked, csd, source_vertno, label, vertices, source_ind
 
 
-def _test_weight_norm(filters, norm=1):
-    """Test weight normalization."""
-    for ws in filters['weights']:
-        ws = ws.reshape(-1, filters['n_orient'], ws.shape[1])
-        for w in ws:
-            assert_allclose(np.trace(w.dot(w.T)), norm)
+idx_param = pytest.mark.parametrize('idx', [
+    0,
+    pytest.param(100, marks=pytest.mark.slowtest),
+    200,
+    pytest.param(233, marks=pytest.mark.slowtest),
+])
 
 
+def _rand_csd(rng, info):
+    scales = mne.make_ad_hoc_cov(info).data
+    n = scales.size
+    # Some random complex correlation structure (with channel scalings)
+    data = rng.randn(n, n) + 1j * rng.randn(n, n)
+    data = data @ data.conj().T
+    data *= scales
+    data *= scales[:, np.newaxis]
+    data.flat[::n + 1] = scales
+    return data
+
+
+def _make_rand_csd(info, csd):
+    rng = np.random.RandomState(0)
+    data = _rand_csd(rng, info)
+    # now we need to have the same null space as the data csd
+    s, u = np.linalg.eigh(csd.get_data(csd.frequencies[0]))
+    mask = np.abs(s) >= s[-1] * 1e-7
+    rank = mask.sum()
+    assert rank == len(data) == len(info['ch_names'])
+    noise_csd = CrossSpectralDensity(
+        _sym_mat_to_vector(data), info['ch_names'], 0., csd.n_fft)
+    return noise_csd, rank
+
+
+@pytest.mark.filterwarnings('ignore:.*real_filter.*will be changed.*:'
+                            'DeprecationWarning')
 @pytest.mark.slowtest
 @testing.requires_testing_data
 @requires_h5py
-def test_make_dics(tmpdir, _load_forward):
+@idx_param
+@pytest.mark.parametrize('whiten', [
+    pytest.param(False, marks=pytest.mark.slowtest),
+    True,
+])
+def test_make_dics(tmpdir, _load_forward, idx, whiten):
     """Test making DICS beamformer filters."""
     # We only test proper handling of parameters here. Testing the results is
     # done in test_apply_dics_timeseries and test_apply_dics_csd.
 
-    fwd_free, fwd_surf, fwd_fixed, fwd_vol, label = _load_forward
-    epochs, _, csd, _ = _simulate_data(fwd_fixed)
-    with pytest.raises(RuntimeError, match='several sensor types'):
+    fwd_free, fwd_surf, fwd_fixed, fwd_vol = _load_forward
+    epochs, _, csd, _, label, vertices, source_ind = \
+        _simulate_data(fwd_fixed, idx)
+    with pytest.raises(ValueError, match='several sensor types'):
         make_dics(epochs.info, fwd_surf, csd, label=label, pick_ori=None)
-    epochs.pick_types(meg='grad')
+    if whiten:
+        noise_csd, rank = _make_rand_csd(epochs.info, csd)
+        assert rank == len(epochs.info['ch_names']) == 62
+    else:
+        noise_csd = None
+        epochs.pick_types(meg='grad')
 
     with pytest.raises(ValueError, match="Invalid value for the 'pick_ori'"):
-        make_dics(epochs.info, fwd_fixed, csd, pick_ori="notexistent")
+        make_dics(epochs.info, fwd_fixed, csd, pick_ori="notexistent",
+                  noise_csd=noise_csd)
     with pytest.raises(ValueError, match='rank, if str'):
-        make_dics(epochs.info, fwd_fixed, csd, rank='foo')
+        make_dics(epochs.info, fwd_fixed, csd, rank='foo', noise_csd=noise_csd)
     with pytest.raises(TypeError, match='rank must be'):
-        make_dics(epochs.info, fwd_fixed, csd, rank=1.)
+        make_dics(epochs.info, fwd_fixed, csd, rank=1., noise_csd=noise_csd)
 
     # Test if fixed forward operator is detected when picking normal
     # orientation
     with pytest.raises(ValueError, match='forward operator with free ori'):
-        make_dics(epochs.info, fwd_fixed, csd, pick_ori="normal")
+        make_dics(epochs.info, fwd_fixed, csd, pick_ori="normal",
+                  noise_csd=noise_csd)
 
     # Test if non-surface oriented forward operator is detected when picking
     # normal orientation
     with pytest.raises(ValueError, match='oriented in surface coordinates'):
-        make_dics(epochs.info, fwd_free, csd, pick_ori="normal")
+        make_dics(epochs.info, fwd_free, csd, pick_ori="normal",
+                  noise_csd=noise_csd)
 
     # Test if volume forward operator is detected when picking normal
     # orientation
     with pytest.raises(ValueError, match='oriented in surface coordinates'):
-        make_dics(epochs.info, fwd_vol, csd, pick_ori="normal")
+        make_dics(epochs.info, fwd_vol, csd, pick_ori="normal",
+                  noise_csd=noise_csd)
 
     # Test invalid combinations of parameters
-    with pytest.raises(NotImplementedError, match='implemented with pick_ori'):
-        make_dics(epochs.info, fwd_free, csd, reduce_rank=True, pick_ori=None)
-    with pytest.raises(NotImplementedError, match='implemented with pick_ori'):
-        make_dics(epochs.info, fwd_free, csd, reduce_rank=True,
-                  pick_ori='max-power', inversion='single')
-    with pytest.raises(ValueError, match='not stable with depth'):
-        make_dics(epochs.info, fwd_free, csd, weight_norm='unit-noise-gain',
-                  inversion='single', normalize_fwd=True)
+    with pytest.raises(ValueError, match='reduce_rank cannot be used with'):
+        make_dics(epochs.info, fwd_free, csd, inversion='single',
+                  reduce_rank=True, noise_csd=noise_csd)
+    # TODO: Restore this?
+    # with pytest.raises(ValueError, match='not stable with depth'):
+    #     make_dics(epochs.info, fwd_free, csd, weight_norm='unit-noise-gain',
+    #               inversion='single', depth=None)
 
     # Sanity checks on the returned filters
     n_freq = len(csd.frequencies)
@@ -158,83 +210,111 @@ def test_make_dics(tmpdir, _load_forward):
 
     n_channels = len(epochs.ch_names)
     # Test return values
+    weight_norm = 'unit-noise-gain'
+    inversion = 'single'
     filters = make_dics(epochs.info, fwd_surf, csd, label=label, pick_ori=None,
-                        weight_norm='unit-noise-gain', normalize_fwd=False)
+                        weight_norm=weight_norm, depth=None,
+                        noise_csd=noise_csd, inversion=inversion)
     assert filters['weights'].shape == (n_freq, n_verts * n_orient, n_channels)
     assert np.iscomplexobj(filters['weights'])
-    assert filters['csd'] == csd
+    assert filters['csd'].ch_names == epochs.ch_names
+    assert isinstance(filters['csd'], CrossSpectralDensity)
     assert filters['ch_names'] == epochs.ch_names
     assert_array_equal(filters['proj'], np.eye(n_channels))
     assert_array_equal(filters['vertices'][0], vertices)
     assert_array_equal(filters['vertices'][1], [])  # Label was on the LH
-    assert filters['subject'] == fwd_free['src'][0]['subject_his_id']
+    assert filters['subject'] == fwd_free['src']._subject
     assert filters['pick_ori'] is None
-    assert filters['n_orient'] == n_orient
-    assert filters['inversion'] == 'single'
-    assert not filters['normalize_fwd']
-    assert filters['weight_norm'] == 'unit-noise-gain'
+    assert filters['is_free_ori']
+    assert filters['inversion'] == inversion
+    assert filters['weight_norm'] == weight_norm
     assert 'DICS' in repr(filters)
     assert 'subject "sample"' in repr(filters)
-    assert '13' in repr(filters)
+    assert str(len(vertices)) in repr(filters)
     assert str(n_channels) in repr(filters)
     assert 'rank' not in repr(filters)
-    _test_weight_norm(filters)
+    _, noise_cov = _prepare_noise_csd(csd, noise_csd, real_filter=False)
+    _, _, _, _, G, _, _, _ = _prepare_beamformer_input(
+        epochs.info, fwd_surf, label, 'vector', combine_xyz=False, exp=None,
+        noise_cov=noise_cov)
+    G.shape = (n_channels, n_verts, n_orient)
+    G = G.transpose(1, 2, 0).conj()  # verts, orient, ch
+    _assert_weight_norm(filters, G)
+
+    inversion = 'matrix'
+    filters = make_dics(epochs.info, fwd_surf, csd, label=label, pick_ori=None,
+                        weight_norm=weight_norm, depth=None,
+                        noise_csd=noise_csd, inversion=inversion)
+    _assert_weight_norm(filters, G)
+
+    weight_norm = 'unit-noise-gain-invariant'
+    inversion = 'single'
+    filters = make_dics(epochs.info, fwd_surf, csd, label=label, pick_ori=None,
+                        weight_norm=weight_norm, depth=None,
+                        noise_csd=noise_csd, inversion=inversion)
+    _assert_weight_norm(filters, G)
 
     # Test picking orientations. Also test weight norming under these different
     # conditions.
+    weight_norm = 'unit-noise-gain'
     filters = make_dics(epochs.info, fwd_surf, csd, label=label,
-                        pick_ori='normal', weight_norm='unit-noise-gain',
-                        normalize_fwd=False)
+                        pick_ori='normal', weight_norm=weight_norm,
+                        depth=None, noise_csd=noise_csd, inversion=inversion)
     n_orient = 1
     assert filters['weights'].shape == (n_freq, n_verts * n_orient, n_channels)
-    assert filters['n_orient'] == n_orient
-    _test_weight_norm(filters)
+    assert not filters['is_free_ori']
+    _assert_weight_norm(filters, G)
 
     filters = make_dics(epochs.info, fwd_surf, csd, label=label,
-                        pick_ori='max-power', weight_norm='unit-noise-gain',
-                        normalize_fwd=False)
+                        pick_ori='max-power', weight_norm=weight_norm,
+                        depth=None, noise_csd=noise_csd, inversion=inversion)
     n_orient = 1
     assert filters['weights'].shape == (n_freq, n_verts * n_orient, n_channels)
-    assert filters['n_orient'] == n_orient
-    _test_weight_norm(filters)
+    assert not filters['is_free_ori']
+    _assert_weight_norm(filters, G)
 
     # From here on, only work on a single frequency
     csd = csd[0]
 
     # Test using a real-valued filter
     filters = make_dics(epochs.info, fwd_surf, csd, label=label,
-                        pick_ori='normal', real_filter=True)
+                        pick_ori='normal', real_filter=True,
+                        noise_csd=noise_csd)
     assert not np.iscomplexobj(filters['weights'])
 
     # Test forward normalization. When inversion='single', the power of a
     # unit-noise CSD should be 1, even without weight normalization.
-    csd_noise = csd.copy()
-    inds = np.triu_indices(csd.n_channels)
-    # Using [:, :] syntax for in-place broadcasting
-    csd_noise._data[:, :] = np.eye(csd.n_channels)[inds][:, np.newaxis]
-    filters = make_dics(epochs.info, fwd_surf, csd_noise, label=label,
-                        weight_norm=None, normalize_fwd=True)
-    w = filters['weights'][0][:3]
-    assert_allclose(np.diag(w.dot(w.T)), 1.0, rtol=1e-6, atol=0)
+    if not whiten:
+        csd_noise = csd.copy()
+        inds = np.triu_indices(csd.n_channels)
+        # Using [:, :] syntax for in-place broadcasting
+        csd_noise._data[:, :] = np.eye(csd.n_channels)[inds][:, np.newaxis]
+        filters = make_dics(epochs.info, fwd_surf, csd_noise, label=label,
+                            weight_norm=None, depth=1., noise_csd=noise_csd,
+                            inversion='single')
+        w = filters['weights'][0][:3]
+        assert_allclose(np.diag(w.dot(w.conjugate().T)), 1.0, rtol=1e-6,
+                        atol=0)
 
     # Test turning off both forward and weight normalization
     filters = make_dics(epochs.info, fwd_surf, csd, label=label,
-                        weight_norm=None, normalize_fwd=False)
+                        weight_norm=None, depth=None, noise_csd=noise_csd)
     w = filters['weights'][0][:3]
-    assert not np.allclose(np.diag(w.dot(w.T)), 1.0, rtol=1e-2, atol=0)
+    assert not np.allclose(np.diag(w.dot(w.conjugate().T)), 1.0,
+                           rtol=1e-2, atol=0)
 
     # Test neural-activity-index weight normalization. It should be a scaled
     # version of the unit-noise-gain beamformer.
     filters_nai = make_dics(
         epochs.info, fwd_surf, csd, label=label, pick_ori='max-power',
-        weight_norm='nai', normalize_fwd=False)
+        weight_norm='nai', depth=None, noise_csd=noise_csd)
     w_nai = filters_nai['weights'][0]
     filters_ung = make_dics(
         epochs.info, fwd_surf, csd, label=label, pick_ori='max-power',
-        weight_norm='unit-noise-gain', normalize_fwd=False)
+        weight_norm='unit-noise-gain', depth=None, noise_csd=noise_csd)
     w_ung = filters_ung['weights'][0]
-    assert np.allclose(np.corrcoef(np.abs(w_nai).ravel(),
-                                   np.abs(w_ung).ravel()), 1)
+    assert_allclose(np.corrcoef(np.abs(w_nai).ravel(),
+                                np.abs(w_ung).ravel()), 1, atol=1e-7)
 
     # Test whether spatial filter contains src_type
     assert 'src_type' in filters
@@ -245,64 +325,73 @@ def test_make_dics(tmpdir, _load_forward):
     assert isinstance(filters, Beamformer)
     assert isinstance(filters_read, Beamformer)
     for key in ['tmin', 'tmax']:  # deal with strictness of object_diff
-        setattr(filters['csd'], key, np.float(getattr(filters['csd'], key)))
+        setattr(filters['csd'], key, np.float64(getattr(filters['csd'], key)))
     assert object_diff(filters, filters_read) == ''
 
 
-def test_apply_dics_csd(_load_forward):
+def _fwd_dist(power, fwd, vertices, source_ind, tidx=1):
+    idx = np.argmax(power.data[:, tidx])
+    rr_got = fwd['src'][0]['rr'][vertices[idx]]
+    rr_want = fwd['src'][0]['rr'][vertices[source_ind]]
+    return np.linalg.norm(rr_got - rr_want)
+
+
+@pytest.mark.filterwarnings('ignore:.*real_filter.*will be changed.*:'
+                            'DeprecationWarning')
+@idx_param
+@pytest.mark.parametrize('inversion, weight_norm', [
+    ('single', None),
+    ('matrix', 'unit-noise-gain'),
+])
+def test_apply_dics_csd(_load_forward, idx, inversion, weight_norm):
     """Test applying a DICS beamformer to a CSD matrix."""
-    fwd_free, fwd_surf, fwd_fixed, _, label = _load_forward
-    epochs, _, csd, source_vertno = _simulate_data(fwd_fixed)
-    vertices = np.intersect1d(label.vertices, fwd_free['src'][0]['vertno'])
-    source_ind = vertices.tolist().index(source_vertno)
+    fwd_free, fwd_surf, fwd_fixed, _ = _load_forward
+    epochs, _, csd, source_vertno, label, vertices, source_ind = \
+        _simulate_data(fwd_fixed, idx)
     reg = 1  # Lots of regularization for our toy dataset
 
-    with pytest.raises(RuntimeError, match='several sensor types'):
+    with pytest.raises(ValueError, match='several sensor types'):
         make_dics(epochs.info, fwd_free, csd)
     epochs.pick_types(meg='grad')
 
     # Try different types of forward models
     assert label.hemi == 'lh'
-    assert vertices[source_ind] == source_vertno
-    rr_want = fwd_free['src'][0]['rr'][source_vertno]
     for fwd in [fwd_free, fwd_surf, fwd_fixed]:
         filters = make_dics(epochs.info, fwd, csd, label=label, reg=reg,
-                            inversion='single')
+                            inversion=inversion, weight_norm=weight_norm)
         power, f = apply_dics_csd(csd, filters)
         assert f == [10, 20]
 
         # Did we find the true source at 20 Hz?
-        idx = np.argmax(power.data[:, 1])
-        rr_got = fwd_free['src'][0]['rr'][vertices[idx]]
-        dist = np.linalg.norm(rr_got - rr_want)
+        dist = _fwd_dist(power, fwd_free, vertices, source_ind)
         assert dist == 0.
 
         # Is the signal stronger at 20 Hz than 10?
         assert power.data[source_ind, 1] > power.data[source_ind, 0]
 
 
+@pytest.mark.filterwarnings('ignore:.*real_filter.*will be changed.*:'
+                            'DeprecationWarning')
 @pytest.mark.parametrize('pick_ori', [None, 'normal', 'max-power'])
 @pytest.mark.parametrize('inversion', ['single', 'matrix'])
-def test_apply_dics_ori_inv(_load_forward, pick_ori, inversion):
-    """Testpicking different orientations and inversion modes."""
-    fwd_free, fwd_surf, fwd_fixed, fwd_vol, label = _load_forward
-    epochs, _, csd, source_vertno = _simulate_data(fwd_fixed)
-    epochs.pick_types('grad')
-    vertices = np.intersect1d(label.vertices, fwd_free['src'][0]['vertno'])
-    source_ind = vertices.tolist().index(source_vertno)
-    rr_want = fwd_free['src'][0]['rr'][source_vertno]
+@idx_param
+def test_apply_dics_ori_inv(_load_forward, pick_ori, inversion, idx):
+    """Test picking different orientations and inversion modes."""
+    fwd_free, fwd_surf, fwd_fixed, fwd_vol = _load_forward
+    epochs, _, csd, source_vertno, label, vertices, source_ind = \
+        _simulate_data(fwd_fixed, idx)
+    epochs.pick_types(meg='grad')
 
     reg_ = 5 if inversion == 'matrix' else 1
     filters = make_dics(epochs.info, fwd_surf, csd, label=label,
                         reg=reg_, pick_ori=pick_ori,
-                        inversion=inversion, normalize_fwd=False,
+                        inversion=inversion, depth=None,
                         weight_norm='unit-noise-gain')
     power, f = apply_dics_csd(csd, filters)
     assert f == [10, 20]
-    idx = np.argmax(power.data[:, 1])
-    rr_got = fwd_free['src'][0]['rr'][vertices[idx]]
-    dist = np.linalg.norm(rr_got - rr_want)
-    assert dist <= (0.03 if inversion == 'matrix' else 0.)
+    dist = _fwd_dist(power, fwd_surf, vertices, source_ind)
+    # This is 0. for unit-noise-gain-invariant:
+    assert dist <= (0.02 if inversion == 'matrix' else 0.)
     assert power.data[source_ind, 1] > power.data[source_ind, 0]
 
     # Test unit-noise-gain weighting
@@ -310,34 +399,42 @@ def test_apply_dics_ori_inv(_load_forward, pick_ori, inversion):
     inds = np.triu_indices(csd.n_channels)
     csd_noise._data[...] = np.eye(csd.n_channels)[inds][:, np.newaxis]
     noise_power, f = apply_dics_csd(csd_noise, filters)
-    assert np.allclose(noise_power.data, 1)
+    want_norm = 3 if pick_ori is None else 1.
+    assert_allclose(noise_power.data, want_norm, atol=1e-7)
 
     # Test filter with forward normalization instead of weight
     # normalization
     filters = make_dics(epochs.info, fwd_surf, csd, label=label,
                         reg=reg_, pick_ori=pick_ori,
                         inversion=inversion, weight_norm=None,
-                        normalize_fwd=True)
+                        depth=1.)
     power, f = apply_dics_csd(csd, filters)
     assert f == [10, 20]
-    idx = np.argmax(power.data[:, 1])
-    rr_got = fwd_free['src'][0]['rr'][vertices[idx]]
-    dist = np.linalg.norm(rr_got - rr_want)
-    assert dist <= (0.035 if inversion == 'matrix' else 0.)
+    dist = _fwd_dist(power, fwd_surf, vertices, source_ind)
+    mat_tol = {0: 0.055, 100: 0.20, 200: 0.015, 233: 0.035}[idx]
+    max_ = (mat_tol if inversion == 'matrix' else 0.)
+    assert 0 <= dist <= max_
     assert power.data[source_ind, 1] > power.data[source_ind, 0]
 
 
-def test_real(_load_forward):
+def _nearest_vol_ind(fwd_vol, fwd, vertices, source_ind):
+    return _compute_nearest(
+        fwd_vol['source_rr'],
+        fwd['src'][0]['rr'][vertices][source_ind][np.newaxis])[0]
+
+
+@pytest.mark.filterwarnings('ignore:.*real_filter.*will be changed.*:'
+                            'DeprecationWarning')
+@idx_param
+def test_real(_load_forward, idx):
     """Test using a real-valued filter."""
-    fwd_free, fwd_surf, fwd_fixed, fwd_vol, label = _load_forward
-    epochs, _, csd, source_vertno = _simulate_data(fwd_fixed)
-    vertices = np.intersect1d(label.vertices, fwd_free['src'][0]['vertno'])
-    source_ind = vertices.tolist().index(source_vertno)
-    rr_want = fwd_free['src'][0]['rr'][source_vertno]
-    epochs.pick_types('grad')
+    fwd_free, fwd_surf, fwd_fixed, fwd_vol = _load_forward
+    epochs, _, csd, source_vertno, label, vertices, source_ind = \
+        _simulate_data(fwd_fixed, idx)
+    epochs.pick_types(meg='grad')
     reg = 1  # Lots of regularization for our toy dataset
     filters_real = make_dics(epochs.info, fwd_surf, csd, label=label, reg=reg,
-                             real_filter=True)
+                             real_filter=True, inversion='single')
     # Also test here that no warings are thrown - implemented to check whether
     # src should not be None warning occurs:
     with pytest.warns(None) as w:
@@ -345,7 +442,8 @@ def test_real(_load_forward):
     assert len(w) == 0
 
     assert f == [10, 20]
-    assert np.argmax(power.data[:, 1]) == source_ind
+    dist = _fwd_dist(power, fwd_surf, vertices, source_ind)
+    assert dist == 0
     assert power.data[source_ind, 1] > power.data[source_ind, 0]
 
     # Test rank reduction
@@ -354,18 +452,20 @@ def test_real(_load_forward):
                              reduce_rank=True)
     power, f = apply_dics_csd(csd, filters_real)
     assert f == [10, 20]
-    idx = np.argmax(power.data[:, 1])
-    rr_got = fwd_free['src'][0]['rr'][vertices[idx]]
-    dist = np.linalg.norm(rr_got - rr_want)
-    assert dist <= 0.02
+    dist = _fwd_dist(power, fwd_surf, vertices, source_ind)
+    assert dist == 0
     assert power.data[source_ind, 1] > power.data[source_ind, 0]
 
     # Test computing source power on a volume source space
-    filters_vol = make_dics(epochs.info, fwd_vol, csd, reg=reg)
+    filters_vol = make_dics(epochs.info, fwd_vol, csd, reg=reg,
+                            inversion='single')
     power, f = apply_dics_csd(csd, filters_vol)
-    vol_source_ind = 3851  # FIXME: not make this hardcoded
+    vol_source_ind = _nearest_vol_ind(fwd_vol, fwd_surf, vertices, source_ind)
     assert f == [10, 20]
-    assert np.argmax(power.data[:, 1]) == vol_source_ind
+    dist = _fwd_dist(
+        power, fwd_vol, fwd_vol['src'][0]['vertno'], vol_source_ind)
+    vol_tols = {100: 0.008, 200: 0.008}
+    assert dist <= vol_tols.get(idx, 0.)
     assert power.data[vol_source_ind, 1] > power.data[vol_source_ind, 0]
 
     # check whether a filters object without src_type throws expected warning
@@ -375,17 +475,19 @@ def test_real(_load_forward):
         apply_dics_csd(csd, filters_vol)
 
 
+@pytest.mark.filterwarnings('ignore:.*real_filter.*will be changed.*:'
+                            'DeprecationWarning')
 @pytest.mark.filterwarnings("ignore:The use of several sensor types with the"
                             ":RuntimeWarning")
-def test_apply_dics_timeseries(_load_forward):
+@idx_param
+def test_apply_dics_timeseries(_load_forward, idx):
     """Test DICS applied to timeseries data."""
-    fwd_free, fwd_surf, fwd_fixed, fwd_vol, label = _load_forward
-    epochs, evoked, csd, source_vertno = _simulate_data(fwd_fixed)
-    vertices = np.intersect1d(label.vertices, fwd_free['src'][0]['vertno'])
-    source_ind = vertices.tolist().index(source_vertno)
+    fwd_free, fwd_surf, fwd_fixed, fwd_vol = _load_forward
+    epochs, evoked, csd, source_vertno, label, vertices, source_ind = \
+        _simulate_data(fwd_fixed, idx)
     reg = 5  # Lots of regularization for our toy dataset
 
-    with pytest.raises(RuntimeError, match='several sensor types'):
+    with pytest.raises(ValueError, match='several sensor types'):
         make_dics(evoked.info, fwd_surf, csd)
     evoked.pick_types(meg='grad')
 
@@ -406,7 +508,8 @@ def test_apply_dics_timeseries(_load_forward):
 
     # From now on, only apply filters with a single frequency (20 Hz).
     csd20 = csd.pick_frequency(20)
-    filters = make_dics(evoked.info, fwd_surf, csd20, label=label, reg=reg)
+    filters = make_dics(evoked.info, fwd_surf, csd20, label=label, reg=reg,
+                        inversion='single')
 
     # Sanity checks on the resulting STC after applying DICS on epochs.
     # Also test here that no warnings are thrown - implemented to check whether
@@ -423,12 +526,14 @@ def test_apply_dics_timeseries(_load_forward):
 
     # Did we find the source?
     stc = (stcs[0] ** 2).mean()
-    assert np.argmax(stc.data) == source_ind
+    dist = _fwd_dist(stc, fwd_surf, vertices, source_ind, tidx=0)
+    assert dist == 0
 
     # Apply filters to evoked
     stc = apply_dics(evoked, filters)
     stc = (stc ** 2).mean()
-    assert np.argmax(stc.data) == source_ind
+    dist = _fwd_dist(stc, fwd_surf, vertices, source_ind, tidx=0)
+    assert dist == 0
 
     # Test if wrong channel selection is detected in application of filter
     evoked_ch = cp.deepcopy(evoked)
@@ -459,10 +564,17 @@ def test_apply_dics_timeseries(_load_forward):
     assert_array_equal(stcs[0].data, next(stcs_gen).data)
 
     # Test computing timecourses on a volume source space
-    filters_vol = make_dics(evoked.info, fwd_vol, csd20, reg=reg)
+    filters_vol = make_dics(evoked.info, fwd_vol, csd20, reg=reg,
+                            inversion='single')
     stc = apply_dics(evoked, filters_vol)
     stc = (stc ** 2).mean()
-    assert np.argmax(stc.data) == 3851  # TODO: don't make this hard coded
+    assert stc.data.shape[1] == 1
+    vol_source_ind = _nearest_vol_ind(fwd_vol, fwd_surf, vertices, source_ind)
+    dist = _fwd_dist(stc, fwd_vol, fwd_vol['src'][0]['vertno'], vol_source_ind,
+                     tidx=0)
+    vol_tols = {100: 0.008, 200: 0.015}
+    vol_tol = vol_tols.get(idx, 0.)
+    assert dist <= vol_tol
 
     # check whether a filters object without src_type throws expected warning
     del filters_vol['src_type']  # emulate 0.16 behaviour to cause warning
@@ -472,12 +584,15 @@ def test_apply_dics_timeseries(_load_forward):
 
 @pytest.mark.slowtest
 @testing.requires_testing_data
+@pytest.mark.filterwarnings('ignore:.*tf_dics is dep.*:DeprecationWarning')
+@pytest.mark.filterwarnings('ignore:.*real_filter.*will be changed.*:'
+                            'DeprecationWarning')
 def test_tf_dics(_load_forward):
     """Test 5D time-frequency beamforming based on DICS."""
-    fwd_free, fwd_surf, fwd_fixed, _, label = _load_forward
-    epochs, _, _, source_vertno = _simulate_data(fwd_fixed)
-    vertices = np.intersect1d(label.vertices, fwd_free['src'][0]['vertno'])
-    source_ind = vertices.tolist().index(source_vertno)
+    fwd_free, fwd_surf, fwd_fixed, _ = _load_forward
+    # idx isn't really used so let's just simulate one
+    epochs, _, _, source_vertno, label, vertices, source_ind = \
+        _simulate_data(fwd_fixed, idx=0)
     reg = 1  # Lots of regularization for our toy dataset
 
     tmin = 0
@@ -487,7 +602,7 @@ def test_tf_dics(_load_forward):
     frequencies = [10, 20]
     freq_bins = [(8, 12), (18, 22)]
 
-    with pytest.raises(RuntimeError, match='several sensor types'):
+    with pytest.raises(ValueError, match='several sensor types'):
         stcs = tf_dics(epochs, fwd_surf, None, tmin, tmax, tstep, win_lengths,
                        freq_bins=freq_bins, frequencies=frequencies,
                        decim=10, reg=reg, label=label)
@@ -499,8 +614,10 @@ def test_tf_dics(_load_forward):
                        decim=10, reg=reg, label=label)
 
         # Did we find the true source at 20 Hz?
-        assert np.argmax(stcs[1].data[:, 0]) == source_ind
-        assert np.argmax(stcs[1].data[:, 1]) == source_ind
+        dist = _fwd_dist(stcs[1], fwd_surf, vertices, source_ind, tidx=0)
+        assert dist == 0
+        dist = _fwd_dist(stcs[1], fwd_surf, vertices, source_ind, tidx=1)
+        assert dist == 0
 
         # 20 Hz power should decrease over time
         assert stcs[1].data[source_ind, 0] > stcs[1].data[source_ind, 1]
@@ -516,7 +633,8 @@ def test_tf_dics(_load_forward):
                          tmin=time_window[0], tmax=time_window[1], decim=10)
         csd = csd.sum()
         csd._data /= csd.n_fft
-        filters = make_dics(epochs.info, fwd_surf, csd, reg=reg, label=label)
+        filters = make_dics(epochs.info, fwd_surf, csd, reg=reg, label=label,
+                            inversion='single')
         stc_source_power, _ = apply_dics_csd(csd, filters)
         source_power.append(stc_source_power.data)
 
@@ -528,7 +646,7 @@ def test_tf_dics(_load_forward):
     # no effect.
     stcs = tf_dics(epochs, fwd_surf, None, tmin, tmax, tstep, win_lengths,
                    mode='cwt_morlet', frequencies=frequencies, decim=10,
-                   reg=reg, label=label, normalize_fwd=False,
+                   reg=reg, label=label, depth=None,
                    weight_norm='unit-noise-gain')
     noise_csd = csd.copy()
     inds = np.triu_indices(csd.n_channels)
@@ -539,10 +657,10 @@ def test_tf_dics(_load_forward):
     stcs_norm = tf_dics(epochs, fwd_surf, noise_csds, tmin, tmax, tstep,
                         win_lengths, mode='cwt_morlet',
                         frequencies=frequencies, decim=10, reg=reg,
-                        label=label, normalize_fwd=False,
+                        label=label, depth=None,
                         weight_norm='unit-noise-gain')
-    assert_allclose(stcs_norm[0].data, stcs[0].data, atol=0)
-    assert_allclose(stcs_norm[1].data, stcs[1].data, atol=0)
+    assert_allclose(3 * stcs_norm[0].data, stcs[0].data, atol=0)
+    assert_allclose(3 * stcs_norm[1].data, stcs[1].data, atol=0)
 
     # Test invalid parameter combinations
     with pytest.raises(ValueError, match='fourier.*freq_bins" parameter'):
@@ -578,6 +696,15 @@ def test_tf_dics(_load_forward):
                 win_lengths=win_lengths, freq_bins=freq_bins,
                 mode='multitaper', mt_bandwidths=[20])
 
+    # Test if 'cwt_morlet' mode works with both fixed cycle numbers and lists
+    # of cycle numbers
+    tf_dics(epochs, fwd_surf, None, tmin, tmax, tstep,
+            win_lengths, frequencies=frequencies, mode='cwt_morlet',
+            cwt_n_cycles=7)
+    tf_dics(epochs, fwd_surf, None, tmin, tmax, tstep,
+            win_lengths, frequencies=frequencies, mode='cwt_morlet',
+            cwt_n_cycles=[5., 7.])
+
     # Test if subtracting evoked responses yields NaN's, since we only have one
     # epoch. Suppress division warnings.
     assert len(epochs) == 1, len(epochs)
@@ -588,4 +715,108 @@ def test_tf_dics(_load_forward):
     assert np.all(np.isnan(stcs[0].data))
 
 
-run_tests_if_main()
+def _cov_as_csd(cov, info):
+    rng = np.random.RandomState(0)
+    assert cov['data'].ndim == 2
+    assert len(cov['data']) == len(cov['names'])
+    # we need to make this have at least some complex structure
+    data = cov['data'] + 1e-1 * _rand_csd(rng, info)
+    assert data.dtype == np.complex128
+    return CrossSpectralDensity(_sym_mat_to_vector(data), cov['names'], 0., 16)
+
+
+# Just test free ori here (assume fixed is same as LCMV if these are)
+# Changes here should be synced with test_lcmv.py
+@pytest.mark.slowtest
+@pytest.mark.parametrize(
+    'reg, pick_ori, weight_norm, use_cov, depth, lower, upper, real_filter', [
+        (0.05, None, 'unit-noise-gain-invariant', False, None, 26, 28, False),
+        (0.05, None, 'unit-noise-gain-invariant', True, None, 40, 42, False),
+        (0.05, None, 'unit-noise-gain-invariant', True, None, 40, 42, True),
+        (0.05, None, 'unit-noise-gain', False, None, 13, 14, False),
+        (0.05, None, 'unit-noise-gain', True, None, 35, 37, False),
+        (0.05, None, 'nai', True, None, 35, 37, False),
+        (0.05, None, None, True, None, 12, 14, False),
+        (0.05, None, None, True, 0.8, 39, 43, False),
+        (0.05, 'max-power', 'unit-noise-gain-invariant', False, None, 17, 20,
+         False),
+        (0.05, 'max-power', 'unit-noise-gain', False, None, 17, 20, False),
+        (0.05, 'max-power', 'unit-noise-gain', False, None, 17, 20, True),
+        (0.05, 'max-power', 'nai', True, None, 21, 24, False),
+        (0.05, 'max-power', None, True, None, 7, 10, False),
+        (0.05, 'max-power', None, True, 0.8, 15, 18, False),
+        # skip most no-reg tests, assume others are equal to LCMV if these are
+        (0.00, None, None, True, None, 21, 32, False),
+        (0.00, 'max-power', None, True, None, 13, 19, False),
+    ])
+def test_localization_bias_free(bias_params_free, reg, pick_ori, weight_norm,
+                                use_cov, depth, lower, upper, real_filter):
+    """Test localization bias for free-orientation DICS."""
+    evoked, fwd, noise_cov, data_cov, want = bias_params_free
+    noise_csd = _cov_as_csd(noise_cov, evoked.info)
+    data_csd = _cov_as_csd(data_cov, evoked.info)
+    del noise_cov, data_cov
+    if not use_cov:
+        evoked.pick_types(meg='grad')
+        noise_csd = None
+    loc = apply_dics(evoked, make_dics(
+        evoked.info, fwd, data_csd, reg, noise_csd, pick_ori=pick_ori,
+        weight_norm=weight_norm, depth=depth, real_filter=real_filter)).data
+    loc = np.linalg.norm(loc, axis=1) if pick_ori == 'vector' else np.abs(loc)
+    # Compute the percentage of sources for which there is no loc bias:
+    perc = (want == np.argmax(loc, axis=0)).mean() * 100
+    assert lower <= perc <= upper
+
+
+@pytest.mark.filterwarnings('ignore:.*real_filter.*will be changed.*:'
+                            'DeprecationWarning')
+@testing.requires_testing_data
+@idx_param
+@pytest.mark.parametrize('whiten', (False, True))
+def test_make_dics_rank(_load_forward, idx, whiten):
+    """Test making DICS beamformer filters with rank param."""
+    _, fwd_surf, fwd_fixed, _ = _load_forward
+    epochs, _, csd, _, label, _, _ = _simulate_data(fwd_fixed, idx)
+    if whiten:
+        noise_csd, want_rank = _make_rand_csd(epochs.info, csd)
+        kind = 'mag + grad'
+    else:
+        noise_csd = None
+        epochs.pick_types(meg='grad')
+        want_rank = len(epochs.ch_names)
+        assert want_rank == 41
+        kind = 'grad'
+
+    with catch_logging() as log:
+        filters = make_dics(
+            epochs.info, fwd_surf, csd, label=label, noise_csd=noise_csd,
+            verbose=True)
+    log = log.getvalue()
+    assert f'Estimated rank ({kind}): {want_rank}' in log, log
+    stc, _ = apply_dics_csd(csd, filters)
+    other_rank = want_rank - 1  # shouldn't make a huge difference
+    use_rank = dict(meg=other_rank)
+    if not whiten:
+        # XXX it's a bug that our rank functions don't treat "meg"
+        # properly here...
+        use_rank['grad'] = use_rank.pop('meg')
+    with catch_logging() as log:
+        filters_2 = make_dics(
+            epochs.info, fwd_surf, csd, label=label, noise_csd=noise_csd,
+            rank=use_rank, verbose=True)
+    log = log.getvalue()
+    assert f'Computing rank from covariance with rank={use_rank}' in log, log
+    stc_2, _ = apply_dics_csd(csd, filters_2)
+    corr = np.corrcoef(stc_2.data.ravel(), stc.data.ravel())[0, 1]
+    assert 0.8 < corr < 0.99999
+
+    # degenerate conditions
+    if whiten:
+        # make rank deficient
+        data = noise_csd.get_data(0.)
+        data[0] = data[:0] = 0
+        noise_csd._data[:, 0] = _sym_mat_to_vector(data)
+        with pytest.raises(ValueError, match='meg data rank.*the noise rank'):
+            filters = make_dics(
+                epochs.info, fwd_surf, csd, label=label, noise_csd=noise_csd,
+                verbose=True)
