@@ -1,10 +1,11 @@
-# Authors: Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
+# Authors: Alexandre Gramfort <alexandre.gramfort@inria.fr>
 #          Denis Engemann <denis.engemann@gmail.com>
 #          Martin Luessi <mluessi@nmr.mgh.harvard.edu>
 #          Eric Larson <larson.eric.d@gmail.com>
 #          Marijn van Vliet <w.m.vanvliet@gmail.com>
 #          Jona Sassenhagen <jona.sassenhagen@gmail.com>
 #          Teon Brooks <teon.brooks@gmail.com>
+#          Robert Luke <mail@robertluke.net>
 #
 # License: Simplified BSD
 
@@ -16,12 +17,11 @@ import os.path as op
 import numpy as np
 
 from ..transforms import _pol_to_cart, _cart_to_sph
-from ..bem import fit_sphere_to_headshape
-from ..io.pick import pick_types
+from ..io.pick import pick_types, _picks_to_idx, _FNIRS_CH_TYPES_SPLIT
 from ..io.constants import FIFF
 from ..io.meas_info import Info
-from ..utils import _clean_names, warn, _check_ch_locs
-from ..externals.six.moves import map
+from ..utils import (_clean_names, warn, _check_ch_locs, fill_doc,
+                     _check_option, _check_sphere, logger)
 from .channels import _get_ch_info
 
 
@@ -36,7 +36,8 @@ class Layout(object):
     box : tuple of length 4
         The box dimension (x_min, x_max, y_min, y_max).
     pos : array, shape=(n_channels, 4)
-        The positions of the channels in 2d (x, y, width, height).
+        The unit-normalized positions of the channels in 2d
+        (x, y, width, height).
     names : list
         The channel names.
     ids : list
@@ -77,8 +78,9 @@ class Layout(object):
                              '.lout or .lay.')
 
         for ii in range(x.shape[0]):
-            out_str += ('%03d %8.2f %8.2f %8.2f %8.2f %s\n' % (self.ids[ii],
-                        x[ii], y[ii], width[ii], height[ii], self.names[ii]))
+            out_str += ('%03d %8.2f %8.2f %8.2f %8.2f %s\n'
+                        % (self.ids[ii], x[ii], y[ii],
+                           width[ii], height[ii], self.names[ii]))
 
         f = open(fname, 'w')
         f.write(out_str)
@@ -89,20 +91,21 @@ class Layout(object):
         return '<Layout | %s - Channels: %s ...>' % (self.kind,
                                                      ', '.join(self.names[:3]))
 
-    def plot(self, picks=None, show=True):
+    @fill_doc
+    def plot(self, picks=None, show_axes=False, show=True):
         """Plot the sensor positions.
 
         Parameters
         ----------
-        picks : array-like
-            Indices of the channels to show. If None (default), all the
-            channels are shown.
+        %(picks_nostr)s
+        show_axes : bool
+            Show layout axes if True. Defaults to False.
         show : bool
             Show figure if True. Defaults to True.
 
         Returns
         -------
-        fig : instance of matplotlib figure
+        fig : instance of matplotlib.figure.Figure
             Figure containing the sensor topography.
 
         Notes
@@ -110,7 +113,7 @@ class Layout(object):
         .. versionadded:: 0.12.0
         """
         from ..viz.topomap import plot_layout
-        return plot_layout(self, picks=picks, show=show)
+        return plot_layout(self, picks=picks, show_axes=show_axes, show=show)
 
 
 def _read_lout(fname):
@@ -126,7 +129,7 @@ def _read_lout(fname):
                 name = chkind + ' ' + nb
             else:
                 cid, x, y, dx, dy, name = splits
-            pos.append(np.array([x, y, dx, dy], dtype=np.float))
+            pos.append(np.array([x, y, dx, dy], dtype=np.float64))
             names.append(name)
             ids.append(int(cid))
 
@@ -147,7 +150,7 @@ def _read_lay(fname):
                 name = chkind + ' ' + nb
             else:
                 cid, x, y, dx, dy, name = splits
-            pos.append(np.array([x, y, dx, dy], dtype=np.float))
+            pos.append(np.array([x, y, dx, dy], dtype=np.float64))
             names.append(name)
             ids.append(int(cid))
 
@@ -214,7 +217,8 @@ def read_layout(kind, path=None, scale=True):
     return Layout(box=box, pos=pos, names=names, kind=kind, ids=ids)
 
 
-def make_eeg_layout(info, radius=0.5, width=None, height=None, exclude='bads'):
+def make_eeg_layout(info, radius=0.5, width=None, height=None, exclude='bads',
+                    csd=False):
     """Create .lout file from EEG electrode digitization.
 
     Parameters
@@ -228,10 +232,12 @@ def make_eeg_layout(info, radius=0.5, width=None, height=None, exclude='bads'):
         this will be the maximum width possible without axes overlapping.
     height : float | None
         Height of sensor axes as a fraction of main figure height. By default,
-        this will be the maximum height possible withough axes overlapping.
-    exclude : list of string | str
+        this will be the maximum height possible without axes overlapping.
+    exclude : list of str | str
         List of channels to exclude. If empty do not exclude any.
         If 'bads', exclude channels in info['bads'] (default).
+    csd : bool
+        Whether the channels contain current-source-density-transformed data.
 
     Returns
     -------
@@ -249,15 +255,17 @@ def make_eeg_layout(info, radius=0.5, width=None, height=None, exclude='bads'):
     if height is not None and not (0 <= height <= 1.0):
         raise ValueError('The height parameter should be between 0 and 1.')
 
-    picks = pick_types(info, meg=False, eeg=True, ref_meg=False,
-                       exclude=exclude)
-    loc2d = _auto_topomap_coords(info, picks)
+    pick_kwargs = dict(meg=False, eeg=True, ref_meg=False, exclude=exclude)
+    if csd:
+        pick_kwargs.update(csd=True, eeg=False)
+    picks = pick_types(info, **pick_kwargs)
+    loc2d = _find_topomap_coords(info, picks)
     names = [info['chs'][i]['ch_name'] for i in picks]
 
-    # Scale [x, y] to [-0.5, 0.5]
-    loc2d_min = np.min(loc2d, axis=0)
-    loc2d_max = np.max(loc2d, axis=0)
-    loc2d = (loc2d - (loc2d_max + loc2d_min) / 2.) / (loc2d_max - loc2d_min)
+    # Scale [x, y] to be in the range [-0.5, 0.5]
+    # Don't mess with the origin or aspect ratio
+    scale = np.maximum(-np.min(loc2d, axis=0), np.max(loc2d, axis=0)).max() * 2
+    loc2d /= scale
 
     # If no width or height specified, calculate the maximum value possible
     # without axes overlapping.
@@ -289,6 +297,7 @@ def make_eeg_layout(info, radius=0.5, width=None, height=None, exclude='bads'):
     return layout
 
 
+@fill_doc
 def make_grid_layout(info, picks=None, n_col=None):
     """Generate .lout file for custom data, i.e., ICA sources.
 
@@ -297,9 +306,7 @@ def make_grid_layout(info, picks=None, n_col=None):
     info : instance of Info | None
         Measurement info (e.g., raw.info). If None, default names will be
         employed.
-    picks : array-like of int | None
-        The indices of the channels to be included. If None, al misc channels
-        will be included.
+    %(picks_base)s all good misc channels.
     n_col : int | None
         Number of columns to generate. If None, a square grid will be produced.
 
@@ -312,8 +319,7 @@ def make_grid_layout(info, picks=None, n_col=None):
     --------
     make_eeg_layout, generate_2d_layout
     """
-    if picks is None:
-        picks = pick_types(info, misc=True, ref_meg=False, exclude='bads')
+    picks = _picks_to_idx(info, picks, 'misc')
 
     names = [info['chs'][k]['ch_name'] for k in picks]
 
@@ -374,9 +380,9 @@ def find_layout(info, ch_type=None, exclude='bads'):
     ch_type : {'mag', 'grad', 'meg', 'eeg'} | None
         The channel type for selecting single channel layouts.
         Defaults to None. Note, this argument will only be considered for
-        VectorView type layout. Use `meg` to force using the full layout
+        VectorView type layout. Use ``'meg'`` to force using the full layout
         in situations where the info does only contain one sensor type.
-    exclude : list of string | str
+    exclude : list of str | str
         List of channels to exclude. If empty do not exclude any.
         If 'bads', exclude channels in info['bads'] (default).
 
@@ -385,15 +391,13 @@ def find_layout(info, ch_type=None, exclude='bads'):
     layout : Layout instance | None
         None if layout not found.
     """
-    our_types = ' or '.join(['`None`', '`mag`', '`grad`', '`meg`'])
-    if ch_type not in (None, 'meg', 'mag', 'grad', 'eeg'):
-        raise ValueError('Invalid channel type (%s) requested '
-                         '`ch_type` must be %s' % (ch_type, our_types))
+    _check_option('ch_type', ch_type, [None, 'mag', 'grad', 'meg', 'eeg',
+                                       'csd'])
 
     (has_vv_mag, has_vv_grad, is_old_vv, has_4D_mag, ctf_other_types,
      has_CTF_grad, n_kit_grads, has_any_meg, has_eeg_coils,
      has_eeg_coils_and_meg, has_eeg_coils_only,
-     has_neuromag_122_grad) = _get_ch_info(info)
+     has_neuromag_122_grad, has_csd_coils) = _get_ch_info(info)
     has_vv_meg = has_vv_mag and has_vv_grad
     has_vv_only_mag = has_vv_mag and not has_vv_grad
     has_vv_only_grad = has_vv_grad and not has_vv_mag
@@ -403,6 +407,7 @@ def find_layout(info, ch_type=None, exclude='bads'):
     if ch_type == "eeg" and not has_eeg_coils:
         raise RuntimeError('No EEG channels present. Cannot find EEG layout.')
 
+    layout_name = None
     if ((has_vv_meg and ch_type is None) or
             (any([has_vv_mag, has_vv_grad]) and ch_type == 'meg')):
         layout_name = 'Vectorview-all'
@@ -421,17 +426,22 @@ def find_layout(info, ch_type=None, exclude='bads'):
             raise RuntimeError('Cannot make EEG layout, no measurement info '
                                'was passed to `find_layout`')
         return make_eeg_layout(info, exclude=exclude)
+    elif has_csd_coils and ch_type in [None, 'csd']:
+        return make_eeg_layout(info, exclude=exclude, csd=True)
     elif has_4D_mag:
         layout_name = 'magnesWH3600'
     elif has_CTF_grad:
         layout_name = 'CTF-275'
     elif n_kit_grads > 0:
         layout_name = _find_kit_layout(info, n_kit_grads)
-    else:
-        xy = _auto_topomap_coords(info, picks=range(info['nchan']),
-                                  ignore_overlap=True, to_sphere=False)
-        return generate_2d_layout(xy, ch_names=info['ch_names'], name='custom',
-                                  normalize=False)
+
+    # If no known layout is found, fall back on automatic layout
+    if layout_name is None:
+        picks = _picks_to_idx(info, 'data', exclude=(), with_ref_meg=False)
+        ch_names = [info['ch_names'][pick] for pick in picks]
+        xy = _find_topomap_coords(info, picks=picks, ignore_overlap=True)
+        return generate_2d_layout(xy, ch_names=ch_names, name='custom',
+                                  normalize=True)
 
     layout = read_layout(layout_name)
     if not is_old_vv:
@@ -462,23 +472,17 @@ def _find_kit_layout(info, n_grads):
 
     Returns
     -------
-    kit_layout : str
-        One of 'KIT-AD', 'KIT-157', 'KIT-160', or 'KIT-UMD'.
+    kit_layout : str | None
+        String naming the detected KIT layout or ``None`` if layout is missing.
     """
     if info['kit_system_id'] is not None:
         # avoid circular import
         from ..io.kit.constants import KIT_LAYOUT
-
-        if info['kit_system_id'] in KIT_LAYOUT:
-            kit_layout = KIT_LAYOUT[info['kit_system_id']]
-            if kit_layout is not None:
-                return kit_layout
-        raise NotImplementedError("The layout for the KIT system with ID %i "
-                                  "is missing. Please contact the developers "
-                                  "about adding it." % info['kit_system_id'])
+        return KIT_LAYOUT.get(info['kit_system_id'])
     elif n_grads == 160:
         return 'KIT-160'
-
+    elif n_grads == 125:
+        return 'KIT-125'
     elif n_grads > 157:
         return 'KIT-AD'
 
@@ -591,39 +595,43 @@ def _box_size(points, width=None, height=None, padding=0.0):
     return width, height
 
 
-def _find_topomap_coords(info, picks, layout=None):
+def _find_topomap_coords(info, picks, layout=None, ignore_overlap=False,
+                         to_sphere=True, sphere=None):
     """Guess the E/MEG layout and return appropriate topomap coordinates.
 
     Parameters
     ----------
     info : instance of Info
         Measurement info.
-    picks : list of int
-        Channel indices to generate topomap coords for.
+    picks : str | list | slice | None
+        None will choose all channels.
     layout : None | instance of Layout
         Enforce using a specific layout. With None, a new map is generated
         and a layout is chosen based on the channels in the picks
         parameter.
+    sphere : array-like | str
+        Definition of the head sphere.
 
     Returns
     -------
     coords : array, shape = (n_chs, 2)
         2 dimensional coordinates for each sensor for a topomap plot.
     """
-    if len(picks) == 0:
-        raise ValueError("Need more than 0 channels.")
+    picks = _picks_to_idx(info, picks, 'all', exclude=(), allow_empty=False)
 
     if layout is not None:
         chs = [info['chs'][i] for i in picks]
         pos = [layout.pos[layout.names.index(ch['ch_name'])] for ch in chs]
         pos = np.asarray(pos)
     else:
-        pos = _auto_topomap_coords(info, picks)
+        pos = _auto_topomap_coords(
+            info, picks, ignore_overlap=ignore_overlap, to_sphere=to_sphere,
+            sphere=sphere)
 
     return pos
 
 
-def _auto_topomap_coords(info, picks, ignore_overlap=False, to_sphere=True):
+def _auto_topomap_coords(info, picks, ignore_overlap, to_sphere, sphere):
     """Make a 2 dimensional sensor map from sensor positions in an info dict.
 
     The default is to use the electrode locations. The fallback option is to
@@ -634,14 +642,16 @@ def _auto_topomap_coords(info, picks, ignore_overlap=False, to_sphere=True):
     ----------
     info : instance of Info
         The measurement info.
-    picks : list of int
-        The channel indices to generate topomap coords for.
+    picks : list | str | slice | None
+        None will pick all channels.
     ignore_overlap : bool
         Whether to ignore overlapping positions in the layout. If False and
         positions overlap, an error is thrown.
     to_sphere : bool
         If True, the radial distance of spherical coordinates is ignored, in
-        effect fitting the xyz-coordinates to a sphere. Defaults to True.
+        effect fitting the xyz-coordinates to a sphere.
+    sphere : array-like | str
+        The head sphere definition.
 
     Returns
     -------
@@ -649,7 +659,10 @@ def _auto_topomap_coords(info, picks, ignore_overlap=False, to_sphere=True):
         An array of positions of the 2 dimensional map.
     """
     from scipy.spatial.distance import pdist, squareform
+    sphere = _check_sphere(sphere, info)
+    logger.debug(f'Generating coords using: {sphere}')
 
+    picks = _picks_to_idx(info, picks, 'all', exclude=(), allow_empty=False)
     chs = [info['chs'][i] for i in picks]
 
     # Use channel locations if available
@@ -691,18 +704,17 @@ def _auto_topomap_coords(info, picks, ignore_overlap=False, to_sphere=True):
                              "doesn't match the number of EEG channels "
                              "(%d)" % (len(locs3d), len(eeg_ch_names)))
 
-        # Center digitization points on head origin
-        dig_kinds = (FIFF.FIFFV_POINT_CARDINAL,
-                     FIFF.FIFFV_POINT_EEG,
-                     FIFF.FIFFV_POINT_EXTRA)
-        _, origin_head, _ = fit_sphere_to_headshape(info, dig_kinds, units='m')
-        locs3d -= origin_head
+        # We no longer center digitization points on head origin, as we work
+        # in head coordinates always
 
         # Match the digitization points with the requested
         # channels.
         eeg_ch_locs = dict(zip(eeg_ch_names, locs3d))
         locs3d = np.array([eeg_ch_locs[ch['ch_name']] for ch in chs])
 
+    # Sometimes we can get nans
+    locs3d[~np.isfinite(locs3d)] = 0.
+
     # Duplicate points cause all kinds of trouble during visualization
     dist = pdist(locs3d)
     if len(locs3d) > 1 and np.min(dist) < 1e-10 and not ignore_overlap:
@@ -716,9 +728,17 @@ def _auto_topomap_coords(info, picks, ignore_overlap=False, to_sphere=True):
                          ', '.join(problematic_electrodes))
 
     if to_sphere:
+        # translate to sphere origin, transform/flatten Z, translate back
+        locs3d -= sphere[:3]
         # use spherical (theta, pol) as (r, theta) for polar->cartesian
-        return _pol_to_cart(_cart_to_sph(locs3d)[:, 1:][:, ::-1])
-    return _pol_to_cart(_cart_to_sph(locs3d))
+        cart_coords = _cart_to_sph(locs3d)
+        out = _pol_to_cart(cart_coords[:, 1:][:, ::-1])
+        # scale from radians to mm
+        out *= cart_coords[:, [0]] / (np.pi / 2.)
+        out += sphere[:2]
+    else:
+        out = _pol_to_cart(_cart_to_sph(locs3d))
+    return out
 
 
 def _topo_to_sphere(pos, eegs):
@@ -786,8 +806,7 @@ def _pair_grad_sensors(info, layout=None, topomap_coords=True, exclude='bads',
     pairs = defaultdict(list)
     grad_picks = pick_types(info, meg='grad', ref_meg=False, exclude=exclude)
 
-    (_, has_vv_grad, _, _, _, _, _, _, _, _, _, has_neuromag_122_grad) = \
-        _get_ch_info(info)
+    _, has_vv_grad, *_, has_neuromag_122_grad, _ = _get_ch_info(info)
 
     for i in grad_picks:
         ch = info['chs'][i]
@@ -877,6 +896,35 @@ def _pair_grad_sensors_ch_names_neuromag122(ch_names):
     return grad_chs
 
 
+def _merge_ch_data(data, ch_type, names, method='rms'):
+    """Merge data from channel pairs.
+
+    Parameters
+    ----------
+    data : array, shape = (n_channels, ..., n_times)
+        Data for channels, ordered in pairs.
+    ch_type : str
+        Channel type.
+    names : list
+        List of channel names.
+    method : str
+        Can be 'rms' or 'mean'.
+
+    Returns
+    -------
+    data : array, shape = (n_channels / 2, ..., n_times)
+        The root mean square or mean for each pair.
+    names : list
+        List of channel names.
+    """
+    if ch_type == 'grad':
+        data = _merge_grad_data(data, method)
+    else:
+        assert ch_type in _FNIRS_CH_TYPES_SPLIT
+        data, names = _merge_nirs_data(data, names)
+    return data, names
+
+
 def _merge_grad_data(data, method='rms'):
     """Merge data from channel pairs using the RMS or mean.
 
@@ -898,10 +946,47 @@ def _merge_grad_data(data, method='rms'):
     elif method == 'rms':
         data = np.sqrt(np.sum(data ** 2, axis=1) / 2)
     else:
-        raise ValueError('method must be "rms" or "mean, got %s.' % method)
+        raise ValueError('method must be "rms" or "mean", got %s.' % method)
     return data.reshape(data.shape[:1] + orig_shape[1:])
 
 
+def _merge_nirs_data(data, merged_names):
+    """Merge data from multiple nirs channel using the mean.
+
+    Channel names that have an x in them will be merged. The first channel in
+    the name is replaced with the mean of all listed channels. The other
+    channels are removed.
+
+    Parameters
+    ----------
+    data : array, shape = (n_channels, ..., n_times)
+        Data for channels.
+    merged_names : list
+        List of strings containing the channel names. Channels that are to be
+        merged contain an x between them.
+
+    Returns
+    -------
+    data : array
+        Data for channels with requested channels merged. Channels used in the
+        merge are removed from the array.
+    """
+    to_remove = np.empty(0, dtype=np.int32)
+    for idx, ch in enumerate(merged_names):
+        if 'x' in ch:
+            indices = np.empty(0, dtype=np.int32)
+            channels = ch.split("x")
+            for sub_ch in channels[1:]:
+                indices = np.append(indices, merged_names.index(sub_ch))
+            data[idx] = np.mean(data[np.append(idx, indices)], axis=0)
+            to_remove = np.append(to_remove, indices)
+    to_remove = np.unique(to_remove)
+    for rem in sorted(to_remove, reverse=True):
+        del merged_names[rem]
+        data = np.delete(data, rem, 0)
+    return data, merged_names
+
+
 def generate_2d_layout(xy, w=.07, h=.05, pad=.02, ch_names=None,
                        ch_indices=None, name='ecog', bg_image=None,
                        normalize=True):
@@ -914,12 +999,12 @@ def generate_2d_layout(xy, w=.07, h=.05, pad=.02, ch_names=None,
 
     Parameters
     ----------
-    xy : ndarray (N x 2)
+    xy : ndarray, shape (N, 2)
         The xy coordinates of sensor locations.
     w : float
-        The width of each sensor's axis (between 0 and 1)
+        The width of each sensor's axis (between 0 and 1).
     h : float
-        The height of each sensor's axis (between 0 and 1)
+        The height of each sensor's axis (between 0 and 1).
     pad : float
         Portion of the box to reserve for padding. The value can range between
         0.0 (boxes will touch, default) to 1.0 (boxes consist of only padding).
@@ -929,7 +1014,7 @@ def generate_2d_layout(xy, w=.07, h=.05, pad=.02, ch_names=None,
     ch_indices : list
         Index of each channel - must be a collection of unique integers,
         one index per channel.
-    name : string
+    name : str
         The name of this layout type.
     bg_image : str | ndarray
         The image over which sensor axes will be plotted. Either a path to an
@@ -958,7 +1043,7 @@ def generate_2d_layout(xy, w=.07, h=.05, pad=.02, ch_names=None,
     if ch_indices is None:
         ch_indices = np.arange(xy.shape[0])
     if ch_names is None:
-        ch_names = ['{0}'.format(i) for i in ch_indices]
+        ch_names = ['{}'.format(i) for i in ch_indices]
 
     if len(ch_names) != len(ch_indices):
         raise ValueError('# channel names and indices must be equal')
