@@ -15,14 +15,13 @@ import pytest
 import matplotlib.pyplot as plt
 
 from mne.channels import (make_eeg_layout, make_grid_layout, read_layout,
-                          find_layout)
-from mne.channels.layout import (_box_size, _auto_topomap_coords,
+                          find_layout, HEAD_SIZE_DEFAULT)
+from mne.channels.layout import (_box_size, _find_topomap_coords,
                                  generate_2d_layout)
 from mne.utils import run_tests_if_main
 from mne import pick_types, pick_info
 from mne.io import read_raw_kit, _empty_info, read_info
 from mne.io.constants import FIFF
-from mne.bem import fit_sphere_to_headshape
 from mne.utils import _TempDir
 
 io_dir = op.join(op.dirname(__file__), '..', '..', 'io')
@@ -40,10 +39,10 @@ def _get_test_info():
     loc = np.array([0., 0., 0., 1., 0., 0., 0., 1., 0., 0., 0., 1.],
                    dtype=np.float32)
     test_info['chs'] = [
-        {'cal': 1, 'ch_name': 'ICA 001', 'coil_type': 0, 'coord_Frame': 0,
+        {'cal': 1, 'ch_name': 'ICA 001', 'coil_type': 0, 'coord_frame': 0,
          'kind': 502, 'loc': loc.copy(), 'logno': 1, 'range': 1.0, 'scanno': 1,
          'unit': -1, 'unit_mul': 0},
-        {'cal': 1, 'ch_name': 'ICA 002', 'coil_type': 0, 'coord_Frame': 0,
+        {'cal': 1, 'ch_name': 'ICA 002', 'coil_type': 0, 'coord_frame': 0,
          'kind': 502, 'loc': loc.copy(), 'logno': 2, 'range': 1.0, 'scanno': 2,
          'unit': -1, 'unit_mul': 0},
         {'cal': 0.002142000012099743, 'ch_name': 'EOG 061', 'coil_type': 1,
@@ -77,7 +76,7 @@ def test_io_layout_lay():
     assert layout.names == layout_read.names
 
 
-def test_auto_topomap_coords():
+def test_find_topomap_coords():
     """Test mapping of coordinates in 3D space to 2D."""
     info = read_info(fif_fname)
     picks = pick_types(info, meg=False, eeg=True, eog=False, stim=False)
@@ -86,51 +85,57 @@ def test_auto_topomap_coords():
     # with the EEG channels
     del info['dig'][85]
 
-    # Remove head origin from channel locations, so mapping with digitization
-    # points yields the same result
-    dig_kinds = (FIFF.FIFFV_POINT_CARDINAL,
-                 FIFF.FIFFV_POINT_EEG,
-                 FIFF.FIFFV_POINT_EXTRA)
-    _, origin_head, _ = fit_sphere_to_headshape(info, dig_kinds, units='m')
-    for ch in info['chs']:
-        ch['loc'][:3] -= origin_head
-
     # Use channel locations
-    l0 = _auto_topomap_coords(info, picks)
+    kwargs = dict(ignore_overlap=False, to_sphere=True,
+                  sphere=HEAD_SIZE_DEFAULT)
+    l0 = _find_topomap_coords(info, picks, **kwargs)
 
     # Remove electrode position information, use digitization points from now
     # on.
     for ch in info['chs']:
         ch['loc'].fill(np.nan)
 
-    l1 = _auto_topomap_coords(info, picks)
+    l1 = _find_topomap_coords(info, picks, **kwargs)
     assert_allclose(l1, l0, atol=1e-3)
 
+    for z_pt in ((HEAD_SIZE_DEFAULT, 0., 0.),
+                 (0., HEAD_SIZE_DEFAULT, 0.)):
+        info['dig'][-1]['r'] = z_pt
+        l1 = _find_topomap_coords(info, picks, **kwargs)
+        assert_allclose(l1[-1], z_pt[:2], err_msg='Z=0 point moved', atol=1e-6)
+
     # Test plotting mag topomap without channel locations: it should fail
     mag_picks = pick_types(info, meg='mag')
-    pytest.raises(ValueError, _auto_topomap_coords, info, mag_picks)
+    with pytest.raises(ValueError, match='Cannot determine location'):
+        _find_topomap_coords(info, mag_picks, **kwargs)
 
     # Test function with too many EEG digitization points: it should fail
     info['dig'].append({'r': [1, 2, 3], 'kind': FIFF.FIFFV_POINT_EEG})
-    pytest.raises(ValueError, _auto_topomap_coords, info, picks)
+    with pytest.raises(ValueError, match='Number of EEG digitization points'):
+        _find_topomap_coords(info, picks, **kwargs)
 
     # Test function with too little EEG digitization points: it should fail
     info['dig'] = info['dig'][:-2]
-    pytest.raises(ValueError, _auto_topomap_coords, info, picks)
+    with pytest.raises(ValueError, match='Number of EEG digitization points'):
+        _find_topomap_coords(info, picks, **kwargs)
 
     # Electrode positions must be unique
     info['dig'].append(info['dig'][-1])
-    pytest.raises(ValueError, _auto_topomap_coords, info, picks)
+    with pytest.raises(ValueError, match='overlapping positions'):
+        _find_topomap_coords(info, picks, **kwargs)
 
     # Test function without EEG digitization points: it should fail
     info['dig'] = [d for d in info['dig'] if d['kind'] != FIFF.FIFFV_POINT_EEG]
-    pytest.raises(RuntimeError, _auto_topomap_coords, info, picks)
+    with pytest.raises(RuntimeError, match='Did not find any digitization'):
+        _find_topomap_coords(info, picks, **kwargs)
 
     # Test function without any digitization points, it should fail
     info['dig'] = None
-    pytest.raises(RuntimeError, _auto_topomap_coords, info, picks)
+    with pytest.raises(RuntimeError, match='No digitization points found'):
+        _find_topomap_coords(info, picks, **kwargs)
     info['dig'] = []
-    pytest.raises(RuntimeError, _auto_topomap_coords, info, picks)
+    with pytest.raises(RuntimeError, match='No digitization points found'):
+        _find_topomap_coords(info, picks, **kwargs)
 
 
 def test_make_eeg_layout():
@@ -251,9 +256,15 @@ def test_find_layout():
     lout = find_layout(raw_kit.info)
     assert_equal(lout.kind, 'KIT-157')
 
-    raw_kit.info['bads'] = ['MEG  13', 'MEG  14', 'MEG  15', 'MEG  16']
+    raw_kit.info['bads'] = ['MEG 013', 'MEG 014', 'MEG 015', 'MEG 016']
+    raw_kit.info._check_consistency()
     lout = find_layout(raw_kit.info)
     assert_equal(lout.kind, 'KIT-157')
+    # fallback for missing IDs
+    for val in (35, 52, 54, 1001):
+        raw_kit.info['kit_system_id'] = val
+        lout = find_layout(raw_kit.info)
+        assert lout.kind == 'custom'
 
     raw_umd = read_raw_kit(fname_kit_umd)
     lout = find_layout(raw_umd.info)
