@@ -21,13 +21,15 @@ import numpy as np
 from .io import read_fiducials, write_fiducials, read_info
 from .io.constants import FIFF
 from .label import read_label, Label
-from .source_space import (add_source_space_distances, read_source_spaces,
-                           write_source_spaces, _get_mri_header, _read_talxfm)
+from .source_space import (add_source_space_distances, read_source_spaces,  # noqa: E501,F401
+                           write_source_spaces, _read_mri_info,
+                           get_mni_fiducials)
 from .surface import read_surface, write_surface, _normalize_vectors
 from .bem import read_bem_surfaces, write_bem_surfaces
 from .transforms import (rotation, rotation3d, scaling, translation, Transform,
                          _read_fs_xfm, _write_fs_xfm, invert_transform,
-                         combine_transforms, apply_trans)
+                         combine_transforms, _quat_to_euler,
+                         _fit_matched_points)
 from .utils import (get_config, get_subjects_dir, logger, pformat, verbose,
                     warn, has_nibabel)
 from .viz._3d import _fiducial_coords
@@ -289,6 +291,9 @@ def _trans_from_params(param_info, params):
     return trans
 
 
+_ALLOW_ANALITICAL = True
+
+
 # XXX this function should be moved out of coreg as used elsewhere
 def fit_matched_points(src_pts, tgt_pts, rotate=True, translate=True,
                        scale=False, tol=None, x0=None, out='trans',
@@ -334,28 +339,59 @@ def fit_matched_points(src_pts, tgt_pts, rotate=True, translate=True,
         A single tuple containing the rotation, translation, and scaling
         parameters in that order (as applicable).
     """
-    # XXX eventually this should be refactored with the cHPI fitting code,
-    # which use fmin_cobyla with constraints
-    from scipy.optimize import leastsq
     src_pts = np.atleast_2d(src_pts)
     tgt_pts = np.atleast_2d(tgt_pts)
     if src_pts.shape != tgt_pts.shape:
         raise ValueError("src_pts and tgt_pts must have same shape (got "
                          "{}, {})".format(src_pts.shape, tgt_pts.shape))
     if weights is not None:
-        weights = np.array(weights, float)
+        weights = np.asarray(weights, src_pts.dtype)
         if weights.ndim != 1 or weights.size not in (src_pts.shape[0], 1):
             raise ValueError("weights (shape=%s) must be None or have shape "
                              "(%s,)" % (weights.shape, src_pts.shape[0],))
         weights = weights[:, np.newaxis]
 
-    rotate = bool(rotate)
-    translate = bool(translate)
-    scale = int(scale)
-    if translate:
+    param_info = (bool(rotate), bool(translate), int(scale))
+    del rotate, translate, scale
+
+    # very common use case, rigid transformation (maybe with one scale factor,
+    # with or without weighted errors)
+    if param_info in ((True, True, 0), (True, True, 1)) and _ALLOW_ANALITICAL:
+        src_pts = np.asarray(src_pts, float)
+        tgt_pts = np.asarray(tgt_pts, float)
+        x, s = _fit_matched_points(
+            src_pts, tgt_pts, weights, bool(param_info[2]))
+        x[:3] = _quat_to_euler(x[:3])
+        x = np.concatenate((x, [s])) if param_info[2] else x
+    else:
+        x = _generic_fit(src_pts, tgt_pts, param_info, weights, x0)
+
+    # re-create the final transformation matrix
+    if (tol is not None) or (out == 'trans'):
+        trans = _trans_from_params(param_info, x)
+
+    # assess the error of the solution
+    if tol is not None:
+        src_pts = np.hstack((src_pts, np.ones((len(src_pts), 1))))
+        est_pts = np.dot(src_pts, trans.T)[:, :3]
+        err = np.sqrt(np.sum((est_pts - tgt_pts) ** 2, axis=1))
+        if np.any(err > tol):
+            raise RuntimeError("Error exceeds tolerance. Error = %r" % err)
+
+    if out == 'params':
+        return x
+    elif out == 'trans':
+        return trans
+    else:
+        raise ValueError("Invalid out parameter: %r. Needs to be 'params' or "
+                         "'trans'." % out)
+
+
+def _generic_fit(src_pts, tgt_pts, param_info, weights, x0):
+    from scipy.optimize import leastsq
+    if param_info[1]:  # translate
         src_pts = np.hstack((src_pts, np.ones((len(src_pts), 1))))
 
-    param_info = (rotate, translate, scale)
     if param_info == (True, False, 0):
         def error(x):
             rx, ry, rz = x
@@ -410,27 +446,7 @@ def fit_matched_points(src_pts, tgt_pts, rotate=True, translate=True,
             "rotate=%r, translate=%r, scale=%r" % param_info)
 
     x, _, _, _, _ = leastsq(error, x0, full_output=True)
-
-    # re-create the final transformation matrix
-    if (tol is not None) or (out == 'trans'):
-        trans = _trans_from_params(param_info, x)
-
-    # assess the error of the solution
-    if tol is not None:
-        if not translate:
-            src_pts = np.hstack((src_pts, np.ones((len(src_pts), 1))))
-        est_pts = np.dot(src_pts, trans.T)[:, :3]
-        err = np.sqrt(np.sum((est_pts - tgt_pts) ** 2, axis=1))
-        if np.any(err > tol):
-            raise RuntimeError("Error exceeds tolerance. Error = %r" % err)
-
-    if out == 'params':
-        return x
-    elif out == 'trans':
-        return trans
-    else:
-        raise ValueError("Invalid out parameter: %r. Needs to be 'params' or "
-                         "'trans'." % out)
+    return x
 
 
 def _find_label_paths(subject='fsaverage', pattern=None, subjects_dir=None):
@@ -908,8 +924,8 @@ def scale_mri(subject_from, subject_to, scale, overwrite=False,
 
     See Also
     --------
-    scale_labels : add labels to a scaled MRI
-    scale_source_space : add a source space to a scaled MRI
+    scale_labels : Add labels to a scaled MRI.
+    scale_source_space : Add a source space to a scaled MRI.
     """
     subjects_dir = get_subjects_dir(subjects_dir, raise_error=True)
     paths = _find_mri_paths(subject_from, skip_fiducials, subjects_dir)
@@ -1076,10 +1092,10 @@ def scale_source_space(subject_to, src_name, subject_from=None, scale=None,
         ss['subject_his_id'] = subject_to
         ss['rr'] *= scale
         # additional tags for volume source spaces
-        if 'vox_mri_t' in ss:
+        for key in ('vox_mri_t', 'src_mri_t'):
             # maintain transform to original MRI volume ss['mri_volume_name']
-            ss['vox_mri_t']['trans'][:3, :3] /= scale
-            ss['src_mri_t']['trans'][:3, :3] /= scale
+            if key in ss:
+                ss[key]['trans'][:3] *= scale[:, np.newaxis]
         # distances and patch info
         if uniform:
             if ss['dist'] is not None:
@@ -1092,10 +1108,13 @@ def scale_source_space(subject_to, src_name, subject_from=None, scale=None,
             _normalize_vectors(ss['nn'])
             if ss['dist'] is not None:
                 add_dist = True
+                dist_limit = float(np.abs(sss[0]['dist_limit']))
+            elif ss['nearest'] is not None:
+                add_dist = True
+                dist_limit = 0
 
     if add_dist:
         logger.info("Recomputing distances, this might take a while")
-        dist_limit = float(np.abs(sss[0]['dist_limit']))
         add_source_space_distances(sss, dist_limit, n_jobs)
 
     write_source_spaces(dst, sss)
@@ -1174,25 +1193,18 @@ def _scale_xfm(subject_to, xfm_fname, mri_name, subject_from, scale,
     #
     xfm, kind = _read_fs_xfm(fname_from)
     assert kind == 'MNI Transform File', kind
+    _, _, F_mri_ras, _, _ = _read_mri_info(mri_name, units='mm')
     F_ras_mni = Transform('ras', 'mni_tal', xfm)
-    hdr = _get_mri_header(mri_name)
-    F_vox_ras = Transform('mri_voxel', 'ras', hdr.get_vox2ras())
-    F_vox_mri = Transform('mri_voxel', 'mri', hdr.get_vox2ras_tkr())
-    F_mri_ras = combine_transforms(
-        invert_transform(F_vox_mri), F_vox_ras, 'mri', 'ras')
-    del F_vox_ras, F_vox_mri, hdr, xfm
+    del xfm
 
     #
     # Get the necessary transforms of the "to" subject
     #
     mri_name = op.join(mri_dirname.format(
         subjects_dir=subjects_dir, subject=subject_to), op.basename(mri_name))
-    hdr = _get_mri_header(mri_name)
-    T_vox_ras = Transform('mri_voxel', 'ras', hdr.get_vox2ras())
-    T_vox_mri = Transform('mri_voxel', 'mri', hdr.get_vox2ras_tkr())
-    T_ras_mri = combine_transforms(
-        invert_transform(T_vox_ras), T_vox_mri, 'ras', 'mri')
-    del mri_name, hdr, T_vox_ras, T_vox_mri
+    _, _, T_mri_ras, _, _ = _read_mri_info(mri_name, units='mm')
+    T_ras_mri = invert_transform(T_mri_ras)
+    del mri_name, T_mri_ras
 
     # Finally we construct as above:
     #
@@ -1207,56 +1219,3 @@ def _scale_xfm(subject_to, xfm_fname, mri_name, subject_from, scale,
                 F_mri_ras, 'ras', 'ras'),
             F_ras_mni, 'ras', 'mni_tal')
     _write_fs_xfm(fname_to, T_ras_mni['trans'], kind)
-
-
-def get_mni_fiducials(subject, subjects_dir=None):
-    """Estimate fiducials for a subject.
-
-    Parameters
-    ----------
-    subject : str
-        Name of the mri subject
-    subjects_dir : None | str
-        Override the SUBJECTS_DIR environment variable
-        (sys.environ['SUBJECTS_DIR'])
-
-    Returns
-    -------
-    fids_mri : list
-        List of estimated fiducials (each point in a dict)
-
-    Notes
-    -----
-    This takes the ``fsaverage-fiducials.fif`` file included with MNE—which
-    contain the LPA, nasion, and RPA for the ``fsaverage`` subject—and
-    transforms them to the given FreeSurfer subject's MRI space.
-    The MRI of ``fsaverage`` is already in MNI Talairach space, so applying
-    the inverse of the given subject's MNI Talairach affine transformation
-    (``$SUBJECTS_DIR/$SUBJECT/mri/transforms/talairach.xfm``) is used
-    to estimate the subject's fiducial locations.
-
-    For more details about the coordinate systems and transformations involved,
-    see https://surfer.nmr.mgh.harvard.edu/fswiki/CoordinateSystems and
-    :ref:`plot_source_alignment`.
-    """
-    # Eventually we might want to allow using the MNI Talairach with-skull
-    # transformation rather than the standard brain-based MNI Talaranch
-    # transformation, and/or project the points onto the head surface
-    # (if available).
-    fname_fids_fs = os.path.join(os.path.dirname(__file__), 'data',
-                                 'fsaverage', 'fsaverage-fiducials.fif')
-
-    # Read fsaverage fiducials file and subject Talairach.
-    fids, coord_frame = read_fiducials(fname_fids_fs)
-    assert coord_frame == FIFF.FIFFV_COORD_MRI
-    mni_mri_t = invert_transform(_read_talxfm(subject, subjects_dir))
-
-    # Convert to mm since this is Freesurfer's unit.
-    lnr = np.array([f['r'] for f in fids]) * 1000.
-    assert lnr.shape == (3, 3)
-
-    # Apply transformation, to fsaverage (MNI) fiducials, convert back to m
-    lnr = apply_trans(mni_mri_t, lnr) / 1000.
-    for ii in range(3):
-        fids[ii]['r'] = lnr[ii]
-    return fids
