@@ -14,6 +14,7 @@ from . import mtrf
 from . import sample
 from . import somato
 from . import multimodal
+from . import fnirs_motor
 from . import opm
 from . import spm_face
 from . import testing
@@ -21,14 +22,20 @@ from . import _fake
 from . import phantom_4dbti
 from . import sleep_physionet
 from . import limo
+from . import refmeg_noise
+from . import ssvep
+from . import erp_core
+from . import epilepsy_ecog
 from .utils import (_download_all_example_data, fetch_hcp_mmp_parcellation,
                     fetch_aparc_sub_parcellation)
 from ._fsaverage.base import fetch_fsaverage
+from ._infant.base import fetch_infant_template
 
 __all__ = [
     '_download_all_example_data', '_fake', 'brainstorm', 'eegbci',
-    'fetch_aparc_sub_parcellation', 'fetch_fsaverage',
+    'fetch_aparc_sub_parcellation', 'fetch_fsaverage', 'fetch_infant_template',
     'fetch_hcp_mmp_parcellation', 'fieldtrip_cmc', 'hf_sef', 'kiloword',
     'misc', 'mtrf', 'multimodal', 'opm', 'phantom_4dbti', 'sample',
-    'sleep_physionet', 'somato', 'spm_face', 'testing', 'visual_92_categories',
+    'sleep_physionet', 'somato', 'spm_face', 'ssvep', 'testing',
+    'visual_92_categories', 'limo', 'erp_core', 'epilepsy_ecog'
 ]
