@@ -6,7 +6,7 @@ from functools import partial
 
 from ...utils import verbose
 from ..utils import (has_dataset, _data_path, _get_version, _version_doc,
-                     _data_path_doc)
+                     _data_path_doc_accept)
 
 has_brainstorm_data = partial(has_dataset, name='brainstorm.bst_resting')
 
@@ -20,15 +20,15 @@ URL: http://neuroimage.usc.edu/brainstorm/DatasetResting
 
 @verbose
 def data_path(path=None, force_update=False, update_path=True, download=True,
-              verbose=None):  # noqa: D103
+              *, accept=False, verbose=None):  # noqa: D103
     return _data_path(path=path, force_update=force_update,
                       update_path=update_path, name='brainstorm',
-                      download=download, archive_name='bst_resting.tar.gz')
+                      download=download, archive_name='bst_resting.tar.gz',
+                      accept=accept)
 
 
-_data_path_doc = _data_path_doc.format(name='brainstorm',
-                                       conf='MNE_DATASETS_BRAINSTORM_DATA'
-                                            '_PATH')
+_data_path_doc = _data_path_doc_accept.format(
+    name='brainstorm', conf='MNE_DATASETS_BRAINSTORM_DATA_PATH')
 _data_path_doc = _data_path_doc.replace('brainstorm dataset',
                                         'brainstorm (bst_resting) dataset')
 data_path.__doc__ = _data_path_doc
