@@ -7,10 +7,9 @@ from ..utils import _data_path, _get_version, _version_doc
 @verbose
 def data_path(path=None, force_update=False, update_path=True, download=True,
               verbose=None):
-    """
-    Get path to local copy of the kiloword dataset.
+    """Get path to local copy of the kiloword dataset.
 
-    This is the dataset from [1]_.
+    This is the dataset from :footcite:`DufauEtAl2015`.
 
     Parameters
     ----------
@@ -27,6 +26,11 @@ def data_path(path=None, force_update=False, update_path=True, download=True,
     update_path : bool | None
         If True, set the MNE_DATASETS_KILOWORD_PATH in mne-python
         config to the given path. If None, the user is prompted.
+    download : bool
+        If False and the kiloword dataset has not been downloaded yet,
+        it will not be downloaded and the path will be returned as
+        '' (empty string). This is mostly used for debugging purposes
+        and can be safely ignored by most users.
     %(verbose)s
 
     Returns
@@ -37,9 +41,7 @@ def data_path(path=None, force_update=False, update_path=True, download=True,
 
     References
     ----------
-    .. [1] Dufau, S., Grainger, J., Midgley, KJ., Holcomb, PJ. A thousand
-       words are worth a picture: Snapshots of printed-word processing in an
-       event-related potential megastudy. Psychological science, 2015
+    .. footbibliography::
     """
     return _data_path(path=path, force_update=force_update,
                       update_path=update_path, name='kiloword',
