@@ -1,25 +1,27 @@
 import os
 from os import path as op
+import re
 import shutil
 import zipfile
 import sys
 
 import pytest
 
-from mne import datasets
-from mne.datasets import testing
+from mne import datasets, read_labels_from_annot, write_labels_to_annot
+from mne.datasets import testing, fetch_infant_template
+from mne.datasets._infant import base as infant_base
 from mne.datasets._fsaverage.base import _set_montage_coreg_path
 from mne.datasets.utils import _manifest_check_download
 
-from mne.utils import (run_tests_if_main, requires_good_network, modified_env,
+from mne.utils import (requires_good_network,
                        get_subjects_dir, ArgvSetter, _pl, use_log_level,
-                       catch_logging)
+                       catch_logging, hashfunc)
 
 
 subjects_dir = op.join(testing.data_path(download=False), 'subjects')
 
 
-def test_datasets_basic(tmpdir):
+def test_datasets_basic(tmpdir, monkeypatch):
     """Test simple dataset functions."""
     # XXX 'hf_sef' and 'misc' do not conform to these standards
     for dname in ('sample', 'somato', 'spm_face', 'testing', 'opm',
@@ -43,27 +45,75 @@ def test_datasets_basic(tmpdir):
     tempdir = str(tmpdir)
     # don't let it read from the config file to get the directory,
     # force it to look for the default
-    with modified_env(**{'_MNE_FAKE_HOME_DIR': tempdir, 'SUBJECTS_DIR': None}):
-        assert (datasets.utils._get_path(None, 'foo', 'bar') ==
-                op.join(tempdir, 'mne_data'))
-        assert get_subjects_dir(None) is None
-        _set_montage_coreg_path()
-        sd = get_subjects_dir()
-        assert sd.endswith('MNE-fsaverage-data')
-
-
-def _fake_fetch_file(url, destination, print_destination=False):
-    with open(destination, 'w') as fid:
-        fid.write(url)
+    monkeypatch.setenv('_MNE_FAKE_HOME_DIR', tempdir)
+    monkeypatch.delenv('SUBJECTS_DIR', raising=False)
+    assert (datasets.utils._get_path(None, 'foo', 'bar') ==
+            op.join(tempdir, 'mne_data'))
+    assert get_subjects_dir(None) is None
+    _set_montage_coreg_path()
+    sd = get_subjects_dir()
+    assert sd.endswith('MNE-fsaverage-data')
+    monkeypatch.setenv('MNE_DATA', str(tmpdir.join('foo')))
+    with pytest.raises(FileNotFoundError, match='as specified by MNE_DAT'):
+        testing.data_path(download=False)
 
 
 @requires_good_network
-def test_downloads(tmpdir):
-    """Test dataset URL handling."""
+def test_downloads(tmpdir, monkeypatch, capsys):
+    """Test dataset URL and version handling."""
     # Try actually downloading a dataset
-    path = datasets._fake.data_path(path=str(tmpdir), update_path=False)
+    kwargs = dict(path=str(tmpdir), verbose=True)
+    path = datasets._fake.data_path(update_path=False, **kwargs)
+    out, _ = capsys.readouterr()
+    assert 'Downloading' in out
+    assert op.isdir(path)
     assert op.isfile(op.join(path, 'bar'))
+    assert not datasets.utils.has_dataset('fake')  # not in the desired path
     assert datasets._fake.get_version() is None
+    assert datasets.utils._get_version('fake') is None
+    monkeypatch.setenv('_MNE_FAKE_HOME_DIR', str(tmpdir))
+    with pytest.warns(RuntimeWarning, match='non-standard config'):
+        new_path = datasets._fake.data_path(update_path=True, **kwargs)
+    assert path == new_path
+    out, _ = capsys.readouterr()
+    assert 'Downloading' not in out
+    # No version: shown as existing but unknown version
+    assert datasets.utils.has_dataset('fake')
+    # XXX logic bug, should be "unknown"
+    assert datasets._fake.get_version() == '0.7'
+    # With a version but no required one: shown as existing and gives version
+    fname = tmpdir / 'foo' / 'version.txt'
+    with open(fname, 'w') as fid:
+        fid.write('0.1')
+    assert datasets.utils.has_dataset('fake')
+    assert datasets._fake.get_version() == '0.1'
+    datasets._fake.data_path(download=False, **kwargs)
+    out, _ = capsys.readouterr()
+    assert 'out of date' not in out
+    # With the required version: shown as existing with the required version
+    monkeypatch.setattr(datasets.utils, '_FAKE_VERSION', '0.1')
+    assert datasets.utils.has_dataset('fake')
+    assert datasets._fake.get_version() == '0.1'
+    datasets._fake.data_path(download=False, **kwargs)
+    out, _ = capsys.readouterr()
+    assert 'out of date' not in out
+    monkeypatch.setattr(datasets.utils, '_FAKE_VERSION', '0.2')
+    # With an older version:
+    # 1. Marked as not actually being present
+    assert not datasets.utils.has_dataset('fake')
+    # 2. Will try to update when `data_path` gets called, with logged message
+    want_msg = 'Correctly trying to download newer version'
+
+    def _error_download(url, full_name, print_destination, hash_, hash_type):
+        assert 'foo.tgz' in url
+        assert str(tmpdir) in full_name
+        raise RuntimeError(want_msg)
+
+    monkeypatch.setattr(datasets.utils, '_fetch_file', _error_download)
+    with pytest.raises(RuntimeError, match=want_msg):
+        datasets._fake.data_path(**kwargs)
+    out, _ = capsys.readouterr()
+    assert re.match(r'.* 0\.1 .*out of date.* 0\.2.*', out, re.MULTILINE), out
 
 
 @pytest.mark.slowtest
@@ -89,6 +139,18 @@ def test_fetch_parcellations(tmpdir):
     for hemi in ('lh', 'rh'):
         assert op.isfile(op.join(this_subjects_dir, 'fsaverage', 'label',
                                  '%s.aparc_sub.annot' % hemi))
+    # test our annot round-trips here
+    kwargs = dict(subject='fsaverage', hemi='both', sort=False,
+                  subjects_dir=this_subjects_dir)
+    labels = read_labels_from_annot(parc='HCPMMP1', **kwargs)
+    write_labels_to_annot(
+        labels, parc='HCPMMP1_round',
+        table_name='./left.fsaverage164.label.gii', **kwargs)
+    orig = op.join(this_subjects_dir, 'fsaverage', 'label', 'lh.HCPMMP1.annot')
+    first = hashfunc(orig)
+    new = orig[:-6] + '_round.annot'
+    second = hashfunc(new)
+    assert first == second
 
 
 _zip_fnames = ['foo/foo.txt', 'foo/bar.txt', 'foo/baz.txt']
@@ -141,4 +203,17 @@ def test_manifest_check_download(tmpdir, n_have, monkeypatch):
         assert op.isfile(op.join(destination, fname))
 
 
-run_tests_if_main()
+def _fake_mcd(manifest_path, destination, url, hash_):
+    name = url.split('/')[-1].split('.')[0]
+    assert name in manifest_path
+    assert name in destination
+    assert name in url
+    assert len(hash_) == 32
+
+
+def test_infant(tmpdir, monkeypatch):
+    """Test fetch_infant_template."""
+    monkeypatch.setattr(infant_base, '_manifest_check_download', _fake_mcd)
+    fetch_infant_template('12mo', subjects_dir=tmpdir)
+    with pytest.raises(ValueError, match='Invalid value for'):
+        fetch_infant_template('0mo', subjects_dir=tmpdir)
