@@ -6,7 +6,8 @@ import numpy as np
 from numpy.testing import assert_array_equal, assert_equal
 import pytest
 
-from mne.utils import requires_version
+from mne.utils import requires_sklearn
+from mne.fixes import _get_args
 from mne.decoding.search_light import SlidingEstimator, GeneralizingEstimator
 from mne.decoding.transformer import Vectorizer
 
@@ -23,7 +24,7 @@ def make_data():
     return X, y
 
 
-@requires_version('sklearn', '0.17')
+@requires_sklearn
 def test_search_light():
     """Test SlidingEstimator."""
     from sklearn.linear_model import Ridge, LogisticRegression
@@ -81,7 +82,16 @@ def test_search_light():
     sl = SlidingEstimator(logreg, scoring='roc_auc')
     y = np.arange(len(X)) % 3
     sl.fit(X, y)
-    pytest.raises(ValueError, sl.score, X, y)
+    with pytest.raises(ValueError, match='for two-class'):
+        sl.score(X, y)
+    # But check that valid ones should work with new enough sklearn
+    if 'multi_class' in _get_args(roc_auc_score):
+        scoring = make_scorer(
+            roc_auc_score, needs_proba=True, multi_class='ovo')
+        sl = SlidingEstimator(logreg, scoring=scoring)
+        sl.fit(X, y)
+        sl.score(X, y)  # smoke test
+
     # -- 2 class problem not in [0, 1]
     y = np.arange(len(X)) % 2 + 1
     sl.fit(X, y)
@@ -161,7 +171,7 @@ def test_search_light():
         assert (isinstance(pipe.estimators_[0], BaggingClassifier))
 
 
-@requires_version('sklearn', '0.17')
+@requires_sklearn
 def test_generalization_light():
     """Test GeneralizingEstimator."""
     from sklearn.pipeline import make_pipeline
@@ -248,7 +258,7 @@ def test_generalization_light():
     assert_array_equal(y_preds[0], y_preds[1])
 
 
-@requires_version('sklearn', '0.19')  # 0.18 does not raise when it should
+@requires_sklearn
 def test_cross_val_predict():
     """Test cross_val_predict with predict_proba."""
     from sklearn.linear_model import LinearRegression
