@@ -13,7 +13,7 @@ from .base import BaseEstimator
 from .. import pick_types
 from ..filter import filter_data, _triage_filter_params
 from ..time_frequency.psd import psd_array_multitaper
-from ..utils import check_version, fill_doc, _check_option
+from ..utils import fill_doc, _check_option, _validate_type
 from ..io.pick import (pick_info, _pick_data_channels, _picks_by_type,
                        _picks_to_idx)
 from ..cov import _check_scalings_user
@@ -56,7 +56,6 @@ def _sklearn_reshape_apply(func, return_result, X, *args, **kwargs):
     """Reshape epochs and apply function."""
     if not isinstance(X, np.ndarray):
         raise ValueError("data should be an np.ndarray, got %s." % type(X))
-    X = np.atleast_3d(X)
     orig_shape = X.shape
     X = np.reshape(X.transpose(0, 2, 1), (-1, orig_shape[1]))
     X = func(X, *args, **kwargs)
@@ -81,7 +80,7 @@ class Scaler(TransformerMixin, BaseEstimator):
     info : instance of Info | None
         The measurement info. Only necessary if ``scalings`` is a dict or
         None.
-    scalings : dict, string, default None.
+    scalings : dict, str, default None
         Scaling method to be applied to data channel wise.
 
         * if scalings is None (default), scales mag by 1e15, grad by 1e13,
@@ -95,10 +94,10 @@ class Scaler(TransformerMixin, BaseEstimator):
           :class:`sklearn.preprocessing.StandardScaler`
           is used.
 
-    with_mean : boolean, default True
+    with_mean : bool, default True
         If True, center the data using mean (or median) before scaling.
         Ignored for channel-type scaling.
-    with_std : boolean, default True
+    with_std : bool, default True
         If True, scale the data to unit variance (``scalings='mean'``),
         quantile range (``scalings='median``), or using channel type
         if ``scalings`` is a dict or None).
@@ -123,13 +122,12 @@ class Scaler(TransformerMixin, BaseEstimator):
             self._scaler = _ConstantScaler(info, scalings, self.with_std)
         elif scalings == 'mean':
             from sklearn.preprocessing import StandardScaler
-            self._scaler = StandardScaler(self.with_mean, self.with_std)
+            self._scaler = StandardScaler(
+                with_mean=self.with_mean, with_std=self.with_std)
         else:  # scalings == 'median':
-            if not check_version('sklearn', '0.17'):
-                raise ValueError("median requires version 0.17 of "
-                                 "sklearn library")
             from sklearn.preprocessing import RobustScaler
-            self._scaler = RobustScaler(self.with_mean, self.with_std)
+            self._scaler = RobustScaler(
+                with_centering=self.with_mean, with_scaling=self.with_std)
 
     def fit(self, epochs_data, y=None):
         """Standardize data across channels.
@@ -144,8 +142,12 @@ class Scaler(TransformerMixin, BaseEstimator):
         Returns
         -------
         self : instance of Scaler
-            Returns the modified instance.
+            The modified instance.
         """
+        _validate_type(epochs_data, np.ndarray, 'epochs_data')
+        if epochs_data.ndim == 2:
+            epochs_data = epochs_data[..., np.newaxis]
+        assert epochs_data.ndim == 3, epochs_data.shape
         _sklearn_reshape_apply(self._scaler.fit, False, epochs_data, y=y)
         return self
 
@@ -154,7 +156,7 @@ class Scaler(TransformerMixin, BaseEstimator):
 
         Parameters
         ----------
-        epochs_data : array, shape (n_epochs, n_channels, n_times)
+        epochs_data : array, shape (n_epochs, n_channels[, n_times])
             The data.
 
         Returns
@@ -167,6 +169,12 @@ class Scaler(TransformerMixin, BaseEstimator):
         This function makes a copy of the data before the operations and the
         memory usage may be large with big data.
         """
+        _validate_type(epochs_data, np.ndarray, 'epochs_data')
+        if epochs_data.ndim == 2:  # can happen with SlidingEstimator
+            if self.info is not None:
+                assert len(self.info['ch_names']) == epochs_data.shape[1]
+            epochs_data = epochs_data[..., np.newaxis]
+        assert epochs_data.ndim == 3, epochs_data.shape
         return _sklearn_reshape_apply(self._scaler.transform, True,
                                       epochs_data)
 
@@ -214,6 +222,7 @@ class Scaler(TransformerMixin, BaseEstimator):
         This function makes a copy of the data before the operations and the
         memory usage may be large with big data.
         """
+        assert epochs_data.ndim == 3, epochs_data.shape
         return _sklearn_reshape_apply(self._scaler.inverse_transform, True,
                                       epochs_data)
 
@@ -224,15 +233,15 @@ class Vectorizer(TransformerMixin):
     This class reshapes an n-dimensional array into an n_samples * n_features
     array, usable by the estimators and transformers of scikit-learn.
 
-    Examples
-    --------
-    clf = make_pipeline(SpatialFilter(), _XdawnTransformer(), Vectorizer(),
-                        LogisticRegression())
-
     Attributes
     ----------
     features_shape_ : tuple
          Stores the original shape of data.
+
+    Examples
+    --------
+    clf = make_pipeline(SpatialFilter(), _XdawnTransformer(), Vectorizer(),
+                        LogisticRegression())
     """
 
     def fit(self, X, y=None):
@@ -314,10 +323,10 @@ class Vectorizer(TransformerMixin):
             dimension is of length n_samples.
         """
         X = np.asarray(X)
-        if X.ndim != 2:
-            raise ValueError("X should be of 2 dimensions but given has %s "
-                             "dimension(s)" % X.ndim)
-        return X.reshape((len(X),) + self.features_shape_)
+        if X.ndim not in (2, 3):
+            raise ValueError("X should be of 2 or 3 dimensions but has shape "
+                             "%s" % (X.shape,))
+        return X.reshape(X.shape[:-1] + self.features_shape_)
 
 
 @fill_doc
@@ -374,12 +383,12 @@ class PSDEstimator(TransformerMixin):
         epochs_data : array, shape (n_epochs, n_channels, n_times)
             The data.
         y : array, shape (n_epochs,)
-            The label for each epoch
+            The label for each epoch.
 
         Returns
         -------
         self : instance of PSDEstimator
-            returns the modified instance
+            The modified instance.
         """
         if not isinstance(epochs_data, np.ndarray):
             raise ValueError("epochs_data should be of type ndarray (got %s)."
@@ -393,7 +402,7 @@ class PSDEstimator(TransformerMixin):
         Parameters
         ----------
         epochs_data : array, shape (n_epochs, n_channels, n_times)
-            The data
+            The data.
 
         Returns
         -------
@@ -493,7 +502,7 @@ class FilterEstimator(TransformerMixin):
         Returns
         -------
         self : instance of FilterEstimator
-            Returns the modified instance
+            The modified instance.
         """
         if not isinstance(epochs_data, np.ndarray):
             raise ValueError("epochs_data should be of type ndarray (got %s)."
@@ -539,7 +548,7 @@ class FilterEstimator(TransformerMixin):
         Returns
         -------
         X : array, shape (n_epochs, n_channels, n_times)
-            The data after filtering
+            The data after filtering.
         """
         if not isinstance(epochs_data, np.ndarray):
             raise ValueError("epochs_data should be of type ndarray (got %s)."
@@ -796,7 +805,7 @@ class TemporalFilter(TransformerMixin):
         Returns
         -------
         self : instance of TemporalFilter
-            Returns the modified instance.
+            The modified instance.
         """  # noqa: E501
         return self
 
