@@ -3,6 +3,7 @@
 #
 # License: BSD (3-clause)
 
+from datetime import datetime, timezone, timedelta
 import json
 import sys
 import tempfile
@@ -10,10 +11,6 @@ from shutil import rmtree
 from os import path as op
 
 import numpy as np
-try:
-    from scipy import sparse
-except ImportError:
-    sparse = None
 
 # Adapted from six
 PY3 = sys.version_info[0] == 3
@@ -24,6 +21,14 @@ special_chars = {'{FWDSLASH}': '/'}
 tab_str = '----'
 
 
+def _import_sparse():
+    try:
+        from scipy import sparse
+    except ImportError:
+        sparse = None
+    return sparse
+
+
 ##############################################################################
 # WRITING
 
@@ -61,7 +66,7 @@ def _create_pandas_dataset(fname, root, key, title, data):
 
 def write_hdf5(fname, data, overwrite=False, compression=4,
                title='h5io', slash='error', use_json=False):
-    """Write python object to HDF5 format using h5py
+    """Write python object to HDF5 format using h5py.
 
     Parameters
     ----------
@@ -69,7 +74,9 @@ def write_hdf5(fname, data, overwrite=False, compression=4,
         Filename to use.
     data : object
         Object to write. Can be of any of these types:
-            {ndarray, dict, list, tuple, int, float, str}
+
+            {ndarray, dict, list, tuple, int, float, str, Datetime}
+
         Note that dict objects must only have ``str`` keys. It is recommended
         to use ndarrays where possible, as it is handled most efficiently.
     overwrite : True | False | 'update'
@@ -123,6 +130,7 @@ def write_hdf5(fname, data, overwrite=False, compression=4,
 def _triage_write(key, value, root, comp_kw, where,
                   cleanup_data, slash='error', title=None,
                   use_json=False):
+    sparse = _import_sparse()
     if key != title and '/' in key:
         if slash == 'error':
             raise ValueError('Found a key with "/", '
@@ -135,7 +143,7 @@ def _triage_write(key, value, root, comp_kw, where,
             raise ValueError("slash must be one of ['error', 'replace'")
 
     if use_json and isinstance(value, (list, dict)) and \
-            json_compatible(value, slash=slash):
+            _json_compatible(value, slash=slash):
         value = np.frombuffer(json.dumps(value).encode('utf-8'), np.uint8)
         _create_titled_dataset(root, key, 'json', value, comp_kw)
     elif isinstance(value, dict):
@@ -161,6 +169,10 @@ def _triage_write(key, value, root, comp_kw, where,
         else:  # isinstance(value, float):
             title = 'float'
         _create_titled_dataset(root, key, title, np.atleast_1d(value))
+    elif isinstance(value, datetime):
+        title = 'datetime'
+        value = np.frombuffer(value.isoformat().encode('utf-8'), np.uint8)
+        _create_titled_dataset(root, key, title, value)
     elif isinstance(value, (np.integer, np.floating, np.bool_)):
         title = 'np_{0}'.format(value.__class__.__name__)
         _create_titled_dataset(root, key, title, np.atleast_1d(value))
@@ -267,6 +279,7 @@ def _triage_read(node, slash='ignore'):
     if slash not in ['ignore', 'replace']:
         raise ValueError("slash must be one of 'replace', 'ignore'")
     h5py = _check_h5py()
+    sparse = _import_sparse()
     type_str = node.attrs['TITLE']
     if isinstance(type_str, bytes):
         type_str = type_str.decode()
@@ -325,6 +338,9 @@ def _triage_read(node, slash='ignore'):
     elif type_str in ('int', 'float'):
         cast = int if type_str == 'int' else float
         data = cast(np.array(node)[0])
+    elif type_str == 'datetime':
+        data = text_type(np.array(node).tobytes().decode('utf-8'))
+        data = fromisoformat(data)
     elif type_str.startswith('np_'):
         np_type = type_str.split('_')[1]
         cast = getattr(np, np_type)
@@ -332,9 +348,9 @@ def _triage_read(node, slash='ignore'):
     elif type_str in ('unicode', 'ascii', 'str'):  # 'str' for backward compat
         decoder = 'utf-8' if type_str == 'unicode' else 'ASCII'
         cast = text_type if type_str == 'unicode' else str
-        data = cast(np.array(node).tostring().decode(decoder))
+        data = cast(np.array(node).tobytes().decode(decoder))
     elif type_str == 'json':
-        node_unicode = str(np.array(node).tostring().decode('utf-8'))
+        node_unicode = str(np.array(node).tobytes().decode('utf-8'))
         data = json.loads(node_unicode)
     elif type_str == 'None':
         data = None
@@ -372,7 +388,7 @@ def object_diff(a, b, pre=''):
     diffs : str
         A string representation of the differences.
     """
-
+    sparse = _import_sparse()
     try:
         from pandas import DataFrame, Series
     except ImportError:
@@ -479,7 +495,7 @@ def _list_file_contents(h5file):
             desc = 'Text: %s'
             decoder = 'utf-8' if type_str == 'unicode' else 'ASCII'
             cast = text_type if type_str == 'unicode' else str
-            data = cast(np.array(data).tostring().decode(decoder))
+            data = cast(np.array(data).tobytes().decode(decoder))
             desc_val = data[:10] + '...' if len(data) > 10 else data
         else:
             desc = 'Items: %s'
@@ -513,19 +529,20 @@ def list_file_contents(h5file):
         _list_file_contents(h5file)
 
 
-def json_compatible(obj, slash='error'):
+def _json_compatible(obj, slash='error'):
     if isinstance(obj, (string_types, int, float, bool, type(None))):
         return True
     elif isinstance(obj, list):
-        return all([json_compatible(item) for item in obj])
+        return all([_json_compatible(item) for item in obj])
     elif isinstance(obj, dict):
         _check_keys_in_dict(obj, slash=slash)
-        return all([json_compatible(item) for item in obj.values()])
+        return all([_json_compatible(item) for item in obj.values()])
     else:
         return False
 
 
 def _check_keys_in_dict(obj, slash='error'):
+    repl = list()
     for key in obj.keys():
         if '/' in key:
             key_prev = key
@@ -536,9 +553,11 @@ def _check_keys_in_dict(obj, slash='error'):
                 # Auto-replace keys with proper values
                 for key_spec, val_spec in special_chars.items():
                     key = key.replace(val_spec, key_spec)
-                obj[key] = obj.pop(key_prev)
+                repl.append((key, key_prev))
             else:
                 raise ValueError("slash must be one of ['error', 'replace'")
+    for key, key_prev in repl:
+        obj[key] = obj.pop(key_prev)
 
 
 ##############################################################################
@@ -595,3 +614,133 @@ def multiarray_load(index, array_merged):
         i_prev = i
     array_restore.append(array_merged[i_prev:])
     return np.array(array_restore)
+
+
+###############################################################################
+# BACKPORTS
+
+try:
+    fromisoformat = datetime.fromisoformat
+except AttributeError:  # Python < 3.7
+    # Code adapted from CPython
+    # https://github.com/python/cpython/blob/master/Lib/datetime.py
+
+    def _parse_hh_mm_ss_ff(tstr):
+        # Parses things of the form HH[:MM[:SS[.fff[fff]]]]
+        len_str = len(tstr)
+
+        time_comps = [0, 0, 0, 0]
+        pos = 0
+        for comp in range(0, 3):
+            if (len_str - pos) < 2:
+                raise ValueError('Incomplete time component')
+
+            time_comps[comp] = int(tstr[pos:pos + 2])
+
+            pos += 2
+            next_char = tstr[pos:pos + 1]
+
+            if not next_char or comp >= 2:
+                break
+
+            if next_char != ':':
+                raise ValueError('Invalid time separator: %c' % next_char)
+
+            pos += 1
+
+        if pos < len_str:
+            if tstr[pos] != '.':
+                raise ValueError('Invalid microsecond component')
+            else:
+                pos += 1
+
+                len_remainder = len_str - pos
+                if len_remainder not in (3, 6):
+                    raise ValueError('Invalid microsecond component')
+
+                time_comps[3] = int(tstr[pos:])
+                if len_remainder == 3:
+                    time_comps[3] *= 1000
+
+        return time_comps
+
+    def fromisoformat(date_string):
+        """Construct a datetime from the output of datetime.isoformat()."""
+        if not isinstance(date_string, str):
+            raise TypeError('fromisoformat: argument must be str')
+
+        # Split this at the separator
+        dstr = date_string[0:10]
+        tstr = date_string[11:]
+
+        try:
+            date_components = _parse_isoformat_date(dstr)
+        except ValueError:
+            raise ValueError(
+                'Invalid isoformat string: {!r}'.format(date_string))
+
+        if tstr:
+            try:
+                time_components = _parse_isoformat_time(tstr)
+            except ValueError:
+                raise ValueError(
+                    'Invalid isoformat string: {!r}'.format(date_string))
+        else:
+            time_components = [0, 0, 0, 0, None]
+
+        return datetime(*(date_components + time_components))
+
+    def _parse_isoformat_date(dtstr):
+        # It is assumed that this function will only be called with a
+        # string of length exactly 10, and (though this is not used) ASCII-only
+        year = int(dtstr[0:4])
+        if dtstr[4] != '-':
+            raise ValueError('Invalid date separator: %s' % dtstr[4])
+
+        month = int(dtstr[5:7])
+
+        if dtstr[7] != '-':
+            raise ValueError('Invalid date separator')
+
+        day = int(dtstr[8:10])
+
+        return [year, month, day]
+
+    def _parse_isoformat_time(tstr):
+        # Format supported is HH[:MM[:SS[.fff[fff]]]][+HH:MM[:SS[.ffffff]]]
+        len_str = len(tstr)
+        if len_str < 2:
+            raise ValueError('Isoformat time too short')
+
+        # This is equivalent to re.search('[+-]', tstr), but faster
+        tz_pos = (tstr.find('-') + 1 or tstr.find('+') + 1)
+        timestr = tstr[:tz_pos - 1] if tz_pos > 0 else tstr
+
+        time_comps = _parse_hh_mm_ss_ff(timestr)
+
+        tzi = None
+        if tz_pos > 0:
+            tzstr = tstr[tz_pos:]
+
+            # Valid time zone strings are:
+            # HH:MM               len: 5
+            # HH:MM:SS            len: 8
+            # HH:MM:SS.ffffff     len: 15
+
+            if len(tzstr) not in (5, 8, 15):
+                raise ValueError('Malformed time zone string')
+
+            tz_comps = _parse_hh_mm_ss_ff(tzstr)
+            if all(x == 0 for x in tz_comps):
+                tzi = timezone.utc
+            else:
+                tzsign = -1 if tstr[tz_pos - 1] == '-' else 1
+
+                td = timedelta(hours=tz_comps[0], minutes=tz_comps[1],
+                               seconds=tz_comps[2], microseconds=tz_comps[3])
+
+                tzi = timezone(tzsign * td)
+
+        time_comps.append(tzi)
+
+        return time_comps
