@@ -1,10 +1,13 @@
-# Authors: Alexandre Gramfort <alexandre.gramfort@inria.fr>
-#          Matti Hamalainen <msh@nmr.mgh.harvard.edu>
+# Authors: Matti Hämäläinen <msh@nmr.mgh.harvard.edu>
+#          Alexandre Gramfort <alexandre.gramfort@inria.fr>
 #          Martin Luessi <mluessi@nmr.mgh.harvard.edu>
 #          Eric Larson <larsoner@uw.edu>
 #
 # License: BSD (3-clause)
 
+# The computations in this code were primarily derived from Matti Hämäläinen's
+# C code.
+
 from copy import deepcopy
 from contextlib import contextmanager
 import os
@@ -22,7 +25,7 @@ from ..transforms import (_ensure_trans, transform_surface_to, apply_trans,
 from ..utils import logger, verbose, warn, _pl
 from ..parallel import check_n_jobs
 from ..source_space import (_ensure_src, _filter_source_spaces,
-                            _make_discrete_source_space, SourceSpaces)
+                            _make_discrete_source_space, _complete_vol_src)
 from ..source_estimate import VolSourceEstimate
 from ..surface import _normalize_vectors
 from ..bem import read_bem_solution, _bem_find_surface, ConductorModel
@@ -454,7 +457,7 @@ def _prepare_for_forward(src, mri_head_t, info, bem, mindist, n_jobs,
     megcoils, compcoils, megnames, meg_info = [], [], [], []
     eegels, eegnames = [], []
 
-    if meg and len(pick_types(info, ref_meg=False, exclude=[])) > 0:
+    if meg and len(pick_types(info, meg=True, ref_meg=False, exclude=[])) > 0:
         megcoils, compcoils, megnames, meg_info = \
             _prep_meg_channels(info, ignore_ref=ignore_ref)
     if eeg and len(pick_types(info, meg=False, eeg=True, ref_meg=False,
@@ -673,8 +676,8 @@ def make_forward_dipole(dipole, bem, info, trans=None, n_jobs=1, verbose=None):
     # NB information about dipole orientation enters here, then no more
     sources = dict(rr=pos, nn=ori)
     # Dipole objects must be in the head frame
-    sp = _make_discrete_source_space(sources, coord_frame='head')
-    src = SourceSpaces([sp])  # dict with working_dir, command_line not nec
+    src = _complete_vol_src(
+        [_make_discrete_source_space(sources, coord_frame='head')])
 
     # Forward operator created for channels in info (use pick_info to restrict)
     # Use defaults for most params, including min_dist
@@ -687,7 +690,7 @@ def make_forward_dipole(dipole, bem, info, trans=None, n_jobs=1, verbose=None):
     # Check for omissions due to proximity to inner skull in
     # make_forward_solution, which will result in an exception
     if fwd['src'][0]['nuse'] != len(pos):
-        inuse = fwd['src'][0]['inuse'].astype(np.bool)
+        inuse = fwd['src'][0]['inuse'].astype(bool)
         head = ('The following dipoles are outside the inner skull boundary')
         msg = len(head) * '#' + '\n' + head + '\n'
         for (t, pos) in zip(times[np.logical_not(inuse)],
@@ -725,14 +728,14 @@ def make_forward_dipole(dipole, bem, info, trans=None, n_jobs=1, verbose=None):
         row += len(amp)
 
     if tstep > 0:
-        stc = VolSourceEstimate(data, vertices=fwd['src'][0]['vertno'],
+        stc = VolSourceEstimate(data, vertices=[fwd['src'][0]['vertno']],
                                 tmin=timepoints[0],
                                 tstep=tstep, subject=None)
     else:  # Must return a list of stc, one for each time point
         stc = []
         for col, tp in enumerate(timepoints):
             stc += [VolSourceEstimate(data[:, col][:, np.newaxis],
-                                      vertices=fwd['src'][0]['vertno'],
+                                      vertices=[fwd['src'][0]['vertno']],
                                       tmin=tp, tstep=0.001, subject=None)]
     return fwd, stc
 
