@@ -10,7 +10,7 @@ from mne.datasets import testing
 from mne import (read_forward_solution, apply_forward, apply_forward_raw,
                  average_forward_solutions, write_forward_solution,
                  convert_forward_solution, SourceEstimate, pick_types_forward,
-                 read_evokeds)
+                 read_evokeds, VectorSourceEstimate)
 from mne.io import read_info
 from mne.label import read_label
 from mne.utils import (requires_mne, run_subprocess,
@@ -18,6 +18,7 @@ from mne.utils import (requires_mne, run_subprocess,
 from mne.forward import (restrict_forward_to_stc, restrict_forward_to_label,
                          Forward, is_fixed_orient, compute_orient_prior,
                          compute_depth_prior)
+from mne.channels import equalize_channels
 
 data_path = testing.data_path(download=False)
 fname_meeg = op.join(data_path, 'MEG', 'sample',
@@ -29,21 +30,22 @@ fname_evoked = op.join(op.dirname(__file__), '..', '..', 'io', 'tests',
                        'data', 'test-ave.fif')
 
 
-def compare_forwards(f1, f2):
+def assert_forward_allclose(f1, f2, rtol=1e-7):
     """Compare two potentially converted forward solutions."""
-    assert_allclose(f1['sol']['data'], f2['sol']['data'])
-    assert_equal(f1['sol']['ncol'], f2['sol']['ncol'])
-    assert_equal(f1['sol']['ncol'], f1['sol']['data'].shape[1])
-    assert_allclose(f1['source_nn'], f2['source_nn'])
+    assert_allclose(f1['sol']['data'], f2['sol']['data'], rtol=rtol)
+    assert f1['sol']['ncol'] == f2['sol']['ncol']
+    assert f1['sol']['ncol'] == f1['sol']['data'].shape[1]
+    assert_allclose(f1['source_nn'], f2['source_nn'], rtol=rtol)
     if f1['sol_grad'] is not None:
         assert (f2['sol_grad'] is not None)
         assert_allclose(f1['sol_grad']['data'], f2['sol_grad']['data'])
-        assert_equal(f1['sol_grad']['ncol'], f2['sol_grad']['ncol'])
-        assert_equal(f1['sol_grad']['ncol'], f1['sol_grad']['data'].shape[1])
+        assert f1['sol_grad']['ncol'] == f2['sol_grad']['ncol']
+        assert f1['sol_grad']['ncol'] == f1['sol_grad']['data'].shape[1]
     else:
         assert (f2['sol_grad'] is None)
-    assert_equal(f1['source_ori'], f2['source_ori'])
-    assert_equal(f1['surf_ori'], f2['surf_ori'])
+    assert f1['source_ori'] == f2['source_ori']
+    assert f1['surf_ori'] == f2['surf_ori']
+    assert f1['src'][0]['coord_frame'] == f1['src'][0]['coord_frame']
 
 
 @testing.requires_testing_data
@@ -61,7 +63,7 @@ def test_convert_forward():
     fwd_new = convert_forward_solution(fwd_surf, surf_ori=False)
     assert (repr(fwd_new))
     assert (isinstance(fwd_new, Forward))
-    compare_forwards(fwd, fwd_new)
+    assert_forward_allclose(fwd, fwd_new)
     del fwd_new
     gc.collect()
 
@@ -78,7 +80,7 @@ def test_convert_forward():
                                        force_fixed=False)
     assert (repr(fwd_new))
     assert (isinstance(fwd_new, Forward))
-    compare_forwards(fwd, fwd_new)
+    assert_forward_allclose(fwd, fwd_new)
     del fwd, fwd_new, fwd_fixed
     gc.collect()
 
@@ -123,7 +125,7 @@ def test_io_forward(tmpdir):
     assert (repr(fwd_read))
     assert (isinstance(fwd_read, Forward))
     assert (is_fixed_orient(fwd_read))
-    compare_forwards(fwd, fwd_read)
+    assert_forward_allclose(fwd, fwd_read)
 
     fwd = convert_forward_solution(fwd, surf_ori=True, force_fixed=True,
                                    use_cps=True)
@@ -142,7 +144,7 @@ def test_io_forward(tmpdir):
     assert (repr(fwd_read))
     assert (isinstance(fwd_read, Forward))
     assert (is_fixed_orient(fwd_read))
-    compare_forwards(fwd, fwd_read)
+    assert_forward_allclose(fwd, fwd_read)
 
     fwd = read_forward_solution(fname_meeg_grad)
     fwd = convert_forward_solution(fwd, surf_ori=True, force_fixed=True,
@@ -162,7 +164,7 @@ def test_io_forward(tmpdir):
     assert (repr(fwd_read))
     assert (isinstance(fwd_read, Forward))
     assert (is_fixed_orient(fwd_read))
-    compare_forwards(fwd, fwd_read)
+    assert_forward_allclose(fwd, fwd_read)
 
     # test warnings on bad filenames
     fwd = read_forward_solution(fname_meeg_grad)
@@ -175,7 +177,7 @@ def test_io_forward(tmpdir):
     fwd = read_forward_solution(fname_meeg)
     write_forward_solution(fname_temp, fwd, overwrite=True)
     fwd_read = read_forward_solution(fname_temp)
-    compare_forwards(fwd, fwd_read)
+    assert_forward_allclose(fwd, fwd_read)
 
 
 @testing.requires_testing_data
@@ -213,6 +215,15 @@ def test_apply_forward():
     assert_array_almost_equal(times[0], t_start)
     assert_array_almost_equal(times[-1], t_start + (n_times - 1) / sfreq)
 
+    # vector
+    stc_vec = VectorSourceEstimate(
+        fwd['source_nn'][:, :, np.newaxis] * stc.data[:, np.newaxis],
+        stc.vertices, stc.tmin, stc.tstep)
+    with pytest.warns(RuntimeWarning, match='very large'):
+        evoked_2 = apply_forward(fwd, stc_vec, evoked.info)
+    assert np.abs(evoked_2.data).mean() > 1e-5
+    assert_allclose(evoked.data, evoked_2.data, atol=1e-10)
+
     # Raw
     with pytest.warns(RuntimeWarning, match='only .* positive values'):
         raw_proj = apply_forward_raw(fwd, stc, evoked.info, start=start,
@@ -279,7 +290,7 @@ def test_restrict_forward_to_stc(tmpdir):
     fwd_out_read = read_forward_solution(fname_copy)
     fwd_out_read = convert_forward_solution(fwd_out_read, surf_ori=True,
                                             force_fixed=False)
-    compare_forwards(fwd_out, fwd_out_read)
+    assert_forward_allclose(fwd_out, fwd_out_read)
 
 
 @testing.requires_testing_data
@@ -345,7 +356,7 @@ def test_restrict_forward_to_label(tmpdir):
     fname_copy = tmpdir.join('copy-fwd.fif')
     write_forward_solution(fname_copy, fwd_out, overwrite=True)
     fwd_out_read = read_forward_solution(fname_copy)
-    compare_forwards(fwd_out, fwd_out_read)
+    assert_forward_allclose(fwd_out, fwd_out_read)
 
 
 @testing.requires_testing_data
@@ -388,7 +399,7 @@ def test_average_forward_solution(tmpdir):
     # with gradient
     fwd = read_forward_solution(fname_meeg_grad)
     fwd_ave = average_forward_solutions([fwd, fwd])
-    compare_forwards(fwd, fwd_ave)
+    assert_forward_allclose(fwd, fwd_ave)
 
 
 @testing.requires_testing_data
@@ -408,9 +419,7 @@ def test_priors():
     with pytest.raises(ValueError, match='noise_cov must be a Covariance'):
         compute_depth_prior(fwd, info, limit_depth_chs='whiten')
     fwd_fixed = convert_forward_solution(fwd, force_fixed=True)
-    with pytest.deprecated_call():
-        depth_prior = compute_depth_prior(
-            fwd_fixed['sol']['data'], info, is_fixed_ori=True)
+    depth_prior = compute_depth_prior(fwd_fixed, info=info)
     assert depth_prior.shape == (n_sources,)
     # Orientation prior
     orient_prior = compute_orient_prior(fwd, 1.)
@@ -428,4 +437,15 @@ def test_priors():
         compute_orient_prior(fwd_fixed, 0.5)
 
 
+@testing.requires_testing_data
+def test_equalize_channels():
+    """Test equalization of channels for instances of Forward."""
+    fwd1 = read_forward_solution(fname_meeg)
+    fwd1.pick_channels(['EEG 001', 'EEG 002', 'EEG 003'])
+    fwd2 = fwd1.copy().pick_channels(['EEG 002', 'EEG 001'], ordered=True)
+    fwd1, fwd2 = equalize_channels([fwd1, fwd2])
+    assert fwd1.ch_names == ['EEG 001', 'EEG 002']
+    assert fwd2.ch_names == ['EEG 001', 'EEG 002']
+
+
 run_tests_if_main()
