@@ -33,27 +33,27 @@ class RawArray(BaseRaw):
         .. versionadded:: 0.18
     %(verbose)s
 
-    Notes
-    -----
-    Proper units of measure:
-    * V: eeg, eog, seeg, emg, ecg, bio, ecog
-    * T: mag
-    * T/m: grad
-    * M: hbo, hbr
-    * Am: dipole
-    * AU: misc
-
     See Also
     --------
     mne.EpochsArray
     mne.EvokedArray
     mne.create_info
+
+    Notes
+    -----
+    Proper units of measure:
+    * V: eeg, eog, seeg, dbs, emg, ecg, bio, ecog
+    * T: mag
+    * T/m: grad
+    * M: hbo, hbr
+    * Am: dipole
+    * AU: misc
     """
 
     @verbose
     def __init__(self, data, info, first_samp=0, copy='auto',
                  verbose=None):  # noqa: D102
-        _validate_type(info, "info")
+        _validate_type(info, 'info', 'info')
         _check_option('copy', copy, ('data', 'info', 'both', 'auto', None))
         dtype = np.complex128 if np.any(np.iscomplex(data)) else np.float64
         orig_data = data
