@@ -1,15 +1,17 @@
 """Conversion tool from CTF to FIF."""
 
-# Author: Eric Larson <larson.eric.d<gmail.com>
+# Authors: Matti Hämäläinen <msh@nmr.mgh.harvard.edu>
+#          Eric Larson <larsoner@uw.edu>
 #
 # License: BSD (3-clause)
 
 import os
-import os.path as op
 
 import numpy as np
 
-from ...utils import verbose, logger, _clean_names, fill_doc, _check_option
+from .._digitization import _format_dig_points
+from ...utils import (verbose, logger, _clean_names, fill_doc, _check_option,
+                      _check_fname)
 
 from ..base import BaseRaw
 from ..utils import _mult_cal_one, _blk_read_lims
@@ -22,8 +24,6 @@ from .info import _compose_meas_info, _read_bad_chans, _annotate_bad_segments
 from .constants import CTF
 from .markers import _read_annotations_ctf_call
 
-from ..._digitization.base import _format_dig_points
-
 
 @fill_doc
 def read_raw_ctf(directory, system_clock='truncate', preload=False,
@@ -90,11 +90,11 @@ class RawCTF(BaseRaw):
     def __init__(self, directory, system_clock='truncate', preload=False,
                  verbose=None, clean_names=False):  # noqa: D102
         # adapted from mne_ctf2fiff.c
-        if not isinstance(directory, str) or \
-                not directory.endswith('.ds'):
-            raise TypeError('directory must be a directory ending with ".ds"')
-        if not op.isdir(directory):
-            raise ValueError('directory does not exist: "%s"' % directory)
+        directory = _check_fname(directory, 'read', True, 'directory',
+                                 need_dir=True)
+        if not directory.endswith('.ds'):
+            raise TypeError('directory must be a directory ending with ".ds", '
+                            f'got {directory}')
         _check_option('system_clock', system_clock, ['ignore', 'truncate'])
         logger.info('ds directory : %s' % directory)
         res4 = _read_res4(directory)  # Read the magical res4 file
@@ -141,11 +141,13 @@ class RawCTF(BaseRaw):
 
         # Add bad segments as Annotations (correct for start time)
         start_time = -res4['pre_trig_pts'] / float(info['sfreq'])
-        annot = _annotate_bad_segments(directory, start_time)
+        annot = _annotate_bad_segments(directory, start_time,
+                                       info['meas_date'])
         marker_annot = _read_annotations_ctf_call(
             directory=directory,
             total_offset=(res4['pre_trig_pts'] / res4['sfreq']),
             trial_duration=(res4['nsamp'] / res4['sfreq']),
+            meas_date=info['meas_date']
         )
         annot = marker_annot if annot is None else annot + marker_annot
         self.set_annotations(annot)
@@ -153,7 +155,6 @@ class RawCTF(BaseRaw):
         if clean_names:
             self._clean_names()
 
-    @verbose
     def _read_segment_file(self, data, idx, fi, start, stop, cals, mult):
         """Read a chunk of raw data."""
         si = self._raw_extras[fi]
