@@ -8,28 +8,30 @@
 #
 # License: BSD (3-clause)
 
+from functools import partial
 import os.path as op
 import inspect
 
 import numpy as np
 from numpy.testing import (assert_array_almost_equal, assert_array_equal,
-                           assert_equal)
+                           assert_equal, assert_allclose)
 from scipy.io import loadmat
 
 import pytest
 
 from mne import pick_types, Annotations
 from mne.datasets import testing
-from mne.utils import run_tests_if_main, requires_pandas, _TempDir
-from mne.io import read_raw_edf, read_raw_bdf
+from mne.fixes import nullcontext
+from mne.utils import requires_pandas
+from mne.io import read_raw_edf, read_raw_bdf, read_raw_fif, edf, read_raw_gdf
 from mne.io.tests.test_raw import _test_raw_reader
-from mne.io.edf.edf import _get_edf_default_event_id
-from mne.io.edf.edf import _read_annotations_edf
-from mne.io.edf.edf import _read_ch
-from mne.io.pick import channel_indices_by_type
+from mne.io.edf.edf import (_get_edf_default_event_id, _read_annotations_edf,
+                            _read_ch, _parse_prefilter_string, _edf_str,
+                            _read_edf_header, _read_header)
+from mne.io.pick import channel_indices_by_type, get_channel_type_constants
 from mne.annotations import events_from_annotations, read_annotations
-from mne.io.meas_info import _kind_dict as _KIND_DICT
 
+td_mark = testing._pytest_mark()
 
 FILE = inspect.getfile(inspect.currentframe())
 data_dir = op.join(op.dirname(op.abspath(FILE)), 'data')
@@ -51,9 +53,11 @@ edf_overlap_annot_path = op.join(data_path, 'EDF',
                                  'test_edf_overlapping_annotations.edf')
 edf_reduced = op.join(data_path, 'EDF', 'test_reduced.edf')
 bdf_stim_channel_path = op.join(data_path, 'BDF', 'test_bdf_stim_channel.bdf')
-
+bdf_multiple_annotations_path = op.join(data_path, 'BDF',
+                                        'multiple_annotation_chans.bdf')
 test_generator_bdf = op.join(data_path, 'BDF', 'test_generator_2.bdf')
 test_generator_edf = op.join(data_path, 'EDF', 'test_generator_2.edf')
+edf_annot_sub_s_path = op.join(data_path, 'EDF', 'subsecond_starttime.edf')
 
 eog = ['REOG', 'LEOG', 'IEOG']
 misc = ['EXG1', 'EXG5', 'EXG8', 'M1', 'M2']
@@ -66,18 +70,39 @@ def test_orig_units():
     # Test original units
     orig_units = raw._orig_units
     assert len(orig_units) == len(raw.ch_names)
-    assert orig_units['A1'] == u'µV'  # formerly 'uV' edit by _check_orig_units
+    assert orig_units['A1'] == 'µV'  # formerly 'uV' edit by _check_orig_units
+
+
+def test_subject_info(tmpdir):
+    """Test exposure of original channel units."""
+    raw = read_raw_edf(edf_path)
+    assert raw.info['subject_info'] is None  # XXX this is arguably a bug
+    edf_info = raw._raw_extras[0]
+    assert edf_info['subject_info'] is not None
+    want = {'id': 'X', 'sex': 'X', 'birthday': 'X', 'name': 'X'}
+    for key, val in want.items():
+        assert edf_info['subject_info'][key] == val, key
+    fname = tmpdir.join('test_raw.fif')
+    raw.save(fname)
+    raw = read_raw_fif(fname)
+    assert raw.info['subject_info'] is None  # XXX should eventually round-trip
 
 
 def test_bdf_data():
     """Test reading raw bdf files."""
+    # XXX BDF data for these is around 0.01 when it should be in the uV range,
+    # probably some bug
+    test_scaling = False
     raw_py = _test_raw_reader(read_raw_bdf, input_fname=bdf_path,
                               eog=eog, misc=misc,
-                              exclude=['M2', 'IEOG'])
+                              exclude=['M2', 'IEOG'],
+                              test_scaling=test_scaling,
+                              )
     assert len(raw_py.ch_names) == 71
     raw_py = _test_raw_reader(read_raw_bdf, input_fname=bdf_path,
                               montage='biosemi64', eog=eog, misc=misc,
-                              exclude=['M2', 'IEOG'])
+                              exclude=['M2', 'IEOG'],
+                              test_scaling=test_scaling)
     assert len(raw_py.ch_names) == 71
     assert 'RawEDF' in repr(raw_py)
     picks = pick_types(raw_py.info, meg=False, eeg=True, exclude='bads')
@@ -104,36 +129,52 @@ def test_bdf_crop_save_stim_channel(tmpdir):
 
 
 @testing.requires_testing_data
-def test_edf_reduced():
-    """Test EDF with various sampling rates."""
-    _test_raw_reader(read_raw_edf, input_fname=edf_reduced, verbose='error')
+@pytest.mark.parametrize('fname', [
+    edf_reduced,
+    edf_overlap_annot_path,
+])
+@pytest.mark.parametrize('stim_channel', (None, False, 'auto'))
+def test_edf_others(fname, stim_channel):
+    """Test EDF with various sampling rates and overlapping annotations."""
+    _test_raw_reader(
+        read_raw_edf, input_fname=fname, stim_channel=stim_channel,
+        verbose='error')
 
 
-def test_edf_data():
+def test_edf_data_broken(tmpdir):
     """Test edf files."""
     raw = _test_raw_reader(read_raw_edf, input_fname=edf_path,
                            exclude=['Ergo-Left', 'H10'], verbose='error')
-    raw_py = read_raw_edf(edf_path, preload=True)
-
+    raw_py = read_raw_edf(edf_path)
+    data = raw_py.get_data()
     assert_equal(len(raw.ch_names) + 2, len(raw_py.ch_names))
 
     # Test with number of records not in header (-1).
-    tempdir = _TempDir()
-    broken_fname = op.join(tempdir, 'broken.edf')
+    broken_fname = op.join(tmpdir, 'broken.edf')
     with open(edf_path, 'rb') as fid_in:
         fid_in.seek(0, 2)
         n_bytes = fid_in.tell()
         fid_in.seek(0, 0)
-        rbytes = fid_in.read(int(n_bytes * 0.4))
+        rbytes = fid_in.read()
     with open(broken_fname, 'wb') as fid_out:
         fid_out.write(rbytes[:236])
         fid_out.write(b'-1      ')
-        fid_out.write(rbytes[244:])
+        fid_out.write(rbytes[244:244 + int(n_bytes * 0.4)])
     with pytest.warns(RuntimeWarning,
                       match='records .* not match the file size'):
         raw = read_raw_edf(broken_fname, preload=True)
         read_raw_edf(broken_fname, exclude=raw.ch_names[:132], preload=True)
 
+    # Test with \x00's in the data
+    with open(broken_fname, 'wb') as fid_out:
+        fid_out.write(rbytes[:184])
+        assert rbytes[184:192] == b'36096   '
+        fid_out.write(rbytes[184:192].replace(b' ', b'\x00'))
+        fid_out.write(rbytes[192:])
+    raw_py = read_raw_edf(broken_fname)
+    data_new = raw_py.get_data()
+    assert_allclose(data, data_new)
+
 
 def test_duplicate_channel_labels_edf():
     """Test reading edf file with duplicate channel names."""
@@ -162,17 +203,20 @@ def test_parse_annotation(tmpdir):
                              dtype=np.int64)
 
     with open(str(annot_file), 'rb') as fid:
-        # ch_data = np.fromfile(fid, dtype=np.int16, count=len(annot))
-        tal_channel_B = _read_ch(fid, subtype='EDF', dtype=np.int16,
+        # ch_data = np.fromfile(fid, dtype='<i2', count=len(annot))
+        tal_channel_B = _read_ch(fid, subtype='EDF', dtype='<i2',
                                  samp=(len(annot) - 1) // 2,
                                  dtype_byte='This_parameter_is_not_used')
 
+    want_onset, want_duration, want_description = zip(
+        *[[180., 0., 'Lights off'], [180., 0., 'Close door'],
+          [180., 0., 'Lights off'], [180., 0., 'Close door'],
+          [3.14, 4.2, 'nothing'], [1800.2, 25.5, 'Apnea']])
     for tal_channel in [tal_channel_A, tal_channel_B]:
         onset, duration, description = _read_annotations_edf([tal_channel])
-        assert_equal(np.column_stack((onset, duration, description)),
-                     [[180., 0., 'Lights off'], [180., 0., 'Close door'],
-                      [180., 0., 'Lights off'], [180., 0., 'Close door'],
-                      [3.14, 4.2, 'nothing'], [1800.2, 25.5, 'Apnea']])
+        assert_allclose(onset, want_onset)
+        assert_allclose(duration, want_duration)
+        assert description == want_description
 
 
 def test_find_events_backward_compatibility():
@@ -195,18 +239,18 @@ def test_find_events_backward_compatibility():
 @pytest.mark.parametrize('fname', [edf_path, bdf_path])
 def test_to_data_frame(fname):
     """Test EDF/BDF Raw Pandas exporter."""
-    ext = op.splitext(fname)[1][1:].lower()
+    ext = op.splitext(fname)[1].lstrip('.').lower()
     if ext == 'edf':
         raw = read_raw_edf(fname, preload=True, verbose='error')
     elif ext == 'bdf':
         raw = read_raw_bdf(fname, preload=True, verbose='error')
     _, times = raw[0, :10]
-    df = raw.to_data_frame()
+    df = raw.to_data_frame(index='time')
     assert (df.columns == raw.ch_names).all()
     assert_array_equal(np.round(times * 1e3), df.index.values[:10])
     df = raw.to_data_frame(index=None, scalings={'eeg': 1e13})
-    assert 'time' in df.index.names
-    assert_array_equal(df.values[:, 0], raw._data[0] * 1e13)
+    assert 'time' in df.columns
+    assert_array_equal(df.values[:, 1], raw._data[0] * 1e13)
 
 
 def test_read_raw_edf_stim_channel_input_parameters():
@@ -260,7 +304,7 @@ def test_read_annot(tmpdir):
 
     # Now test when reading from buffer of data
     with open(str(annot_file), 'rb') as fid:
-        ch_data = np.fromfile(fid, dtype=np.int16, count=len(annot))
+        ch_data = np.fromfile(fid, dtype='<i2', count=len(annot))
     onset, duration, desc = _read_annotations_edf([ch_data])
     annotation = Annotations(onset=onset, duration=duration, description=desc,
                              orig_time=None)
@@ -275,6 +319,29 @@ def test_read_annotations(fname, recwarn):
     assert len(annot.onset) == 2
 
 
+def test_edf_prefilter_parse():
+    """Test prefilter strings from header are parsed correctly."""
+    prefilter_basic = ["HP: 0Hz LP: 0Hz"]
+    highpass, lowpass = _parse_prefilter_string(prefilter_basic)
+    assert_array_equal(highpass, ["0"])
+    assert_array_equal(lowpass, ["0"])
+
+    prefilter_normal_multi_ch = ["HP: 1Hz LP: 30Hz"] * 10
+    highpass, lowpass = _parse_prefilter_string(prefilter_normal_multi_ch)
+    assert_array_equal(highpass, ["1"] * 10)
+    assert_array_equal(lowpass, ["30"] * 10)
+
+    prefilter_unfiltered_ch = prefilter_normal_multi_ch + [""]
+    highpass, lowpass = _parse_prefilter_string(prefilter_unfiltered_ch)
+    assert_array_equal(highpass, ["1"] * 10)
+    assert_array_equal(lowpass, ["30"] * 10)
+
+    prefilter_edf_specs_doc = ["HP:0.1Hz LP:75Hz N:50Hz"]
+    highpass, lowpass = _parse_prefilter_string(prefilter_edf_specs_doc)
+    assert_array_equal(highpass, ["0.1"])
+    assert_array_equal(lowpass, ["75"])
+
+
 @testing.requires_testing_data
 @pytest.mark.parametrize('fname', [test_generator_edf, test_generator_bdf])
 def test_load_generator(fname, recwarn):
@@ -314,7 +381,10 @@ def test_load_generator(fname, recwarn):
                  [0, 1], id='int list')])
 def test_edf_stim_ch_pick_up(test_input, EXPECTED):
     """Test stim_channel."""
-    TYPE_LUT = {v[0]: k for k, v in _KIND_DICT.items()}
+    # This is fragile for EEG/EEG-CSD, so just omit csd
+    KIND_DICT = get_channel_type_constants()
+    TYPE_LUT = {v['kind']: k for k, v in KIND_DICT.items() if k not in
+                ('csd', 'chpi')}  # chpi not needed, and unhashable (a list)
     fname = op.join(data_dir, 'test_stim_channel.edf')
 
     raw = read_raw_edf(fname, stim_channel=test_input)
@@ -322,4 +392,123 @@ def test_edf_stim_ch_pick_up(test_input, EXPECTED):
     assert ch_types == EXPECTED
 
 
-run_tests_if_main()
+@testing.requires_testing_data
+def test_bdf_multiple_annotation_channels():
+    """Test BDF with multiple annotation channels."""
+    raw = read_raw_bdf(bdf_multiple_annotations_path)
+    assert len(raw.annotations) == 10
+    descriptions = np.array(['signal_start', 'EEG-check#1', 'TestStim#1',
+                             'TestStim#2', 'TestStim#3', 'TestStim#4',
+                             'TestStim#5', 'TestStim#6', 'TestStim#7',
+                             'Ligths-Off#1'], dtype='<U12')
+    assert_array_equal(descriptions, raw.annotations.description)
+
+
+@testing.requires_testing_data
+def test_edf_lowpass_zero():
+    """Test if a lowpass filter of 0Hz is mapped to the Nyquist frequency."""
+    raw = read_raw_edf(edf_stim_resamp_path)
+    assert raw.ch_names[100] == 'EEG LDAMT_01-REF'
+    assert len(raw.ch_names[100]) > 15
+    assert_allclose(raw.info["lowpass"], raw.info["sfreq"] / 2)
+
+
+@testing.requires_testing_data
+def test_edf_annot_sub_s_onset():
+    """Test reading of sub-second annotation onsets."""
+    raw = read_raw_edf(edf_annot_sub_s_path)
+    assert_allclose(raw.annotations.onset, [1.951172, 3.492188])
+
+
+def test_invalid_date(tmpdir):
+    """Test handling of invalid date in EDF header."""
+    with open(edf_path, 'rb') as f:  # read valid test file
+        edf = bytearray(f.read())
+
+    # original date in header is 29.04.14 (2014-04-29) at pos 168:176
+    # but we also use Startdate if available,
+    # which starts at byte 88 and is b'Startdate 29-APR-2014 X X X'
+    # create invalid date 29.02.14 (2014 is not a leap year)
+
+    # one wrong: no warning
+    edf[101:104] = b'FEB'
+    assert edf[172] == ord('4')
+    fname = op.join(str(tmpdir), "temp.edf")
+    with open(fname, "wb") as f:
+        f.write(edf)
+    read_raw_edf(fname)
+
+    # other wrong: no warning
+    edf[101:104] = b'APR'
+    edf[172] = ord('2')
+    with open(fname, "wb") as f:
+        f.write(edf)
+    read_raw_edf(fname)
+
+    # both wrong: warning
+    edf[101:104] = b'FEB'
+    edf[172] = ord('2')
+    with open(fname, "wb") as f:
+        f.write(edf)
+    with pytest.warns(RuntimeWarning, match='Invalid date'):
+        read_raw_edf(fname)
+
+    # another invalid date 29.00.14 (0 is not a month)
+    assert edf[101:104] == b'FEB'
+    edf[172] = ord('0')
+    with open(fname, "wb") as f:
+        f.write(edf)
+    with pytest.warns(RuntimeWarning, match='Invalid date'):
+        read_raw_edf(fname)
+
+
+def test_empty_chars():
+    """Test blank char support."""
+    assert int(_edf_str(b'1819\x00 ')) == 1819
+
+
+def _hp_lp_rev(*args, **kwargs):
+    out, orig_units = _read_edf_header(*args, **kwargs)
+    out['lowpass'], out['highpass'] = out['highpass'], out['lowpass']
+    # this will happen for test_edf_stim_resamp.edf
+    if len(out['lowpass']) and out['lowpass'][0] == '0.000' and \
+            len(out['highpass']) and out['highpass'][0] == '0.0':
+        out['highpass'][0] = '10.0'
+    return out, orig_units
+
+
+@pytest.mark.filterwarnings('ignore:.*too long.*:RuntimeWarning')
+@pytest.mark.parametrize('fname, lo, hi, warns', [
+    (edf_path, 256, 0, False),
+    (edf_uneven_path, 50, 0, False),
+    (edf_stim_channel_path, 64, 0, False),
+    pytest.param(edf_overlap_annot_path, 64, 0, False, marks=td_mark),
+    pytest.param(edf_reduced, 256, 0, False, marks=td_mark),
+    pytest.param(test_generator_edf, 100, 0, False, marks=td_mark),
+    pytest.param(edf_stim_resamp_path, 256, 0, True, marks=td_mark),
+])
+def test_hp_lp_reversed(fname, lo, hi, warns, monkeypatch):
+    """Test HP/LP reversed (gh-8584)."""
+    fname = str(fname)
+    raw = read_raw_edf(fname)
+    assert raw.info['lowpass'] == lo
+    assert raw.info['highpass'] == hi
+    monkeypatch.setattr(edf.edf, '_read_edf_header', _hp_lp_rev)
+    if warns:
+        ctx = pytest.warns(RuntimeWarning, match='greater than lowpass')
+        new_lo, new_hi = raw.info['sfreq'] / 2., 0.
+    else:
+        ctx = nullcontext()
+        new_lo, new_hi = lo, hi
+    with ctx:
+        raw = read_raw_edf(fname)
+    assert raw.info['lowpass'] == new_lo
+    assert raw.info['highpass'] == new_hi
+
+
+def test_degenerate():
+    """Test checking of some bad inputs."""
+    for func in (read_raw_edf, read_raw_bdf, read_raw_gdf,
+                 partial(_read_header, exclude=())):
+        with pytest.raises(NotImplementedError, match='Only.*txt.*'):
+            func(edf_txt_stim_channel_path)
