@@ -3,7 +3,9 @@
 #
 # License: BSD (3-clause)
 
+from datetime import datetime, timezone, timedelta
 import os.path as op
+import shutil
 
 import pytest
 from numpy.testing import (assert_array_almost_equal, assert_array_equal,
@@ -13,7 +15,6 @@ import scipy.io as sio
 
 from mne.datasets import testing
 from mne.io import read_raw_gdf
-from mne.io.meas_info import DATE_NONE
 from mne.io.tests.test_raw import _test_raw_reader
 from mne.utils import run_tests_if_main
 from mne import pick_types, find_events, events_from_annotations
@@ -54,13 +55,40 @@ def test_gdf_data():
     assert len(raw.annotations.duration == 963)
 
     # gh-5604
-    assert raw.info['meas_date'] == DATE_NONE
+    assert raw.info['meas_date'] is None
+
+
+@testing.requires_testing_data
+def test_gdf2_birthday(tmpdir):
+    """Test reading raw GDF 2.x files."""
+    new_fname = str(tmpdir.join('temp.gdf'))
+    shutil.copyfile(gdf2_path + '.gdf', new_fname)
+    # go back 44.5 years so the subject should show up as 44
+    offset_edf = (  # to their ref
+        datetime.now(tz=timezone.utc) -
+        datetime(1, 1, 1, tzinfo=timezone.utc)
+    )
+    offset_44_yr = offset_edf - timedelta(days=int(365 * 44.5))  # 44.5 yr ago
+    offset_44_yr_days = offset_44_yr.total_seconds() / (24 * 60 * 60)  # days
+    d = (int(offset_44_yr_days) + 367) * 2 ** 32  # with their conversion
+    with open(new_fname, 'r+b') as fid:
+        fid.seek(176, 0)
+        assert np.fromfile(fid, np.uint64, 1)[0] == 0
+        fid.seek(176, 0)
+        fid.write(np.array([d], np.uint64).tobytes())
+        fid.seek(176, 0)
+        assert np.fromfile(fid, np.uint64, 1)[0] == d
+    raw = read_raw_gdf(new_fname, eog=None, misc=None, preload=True)
+    assert raw._raw_extras[0]['subject_info']['age'] == 44
+    # XXX this is a bug, it should be populated...
+    assert raw.info['subject_info'] is None
 
 
 @testing.requires_testing_data
 def test_gdf2_data():
     """Test reading raw GDF 2.x files."""
     raw = read_raw_gdf(gdf2_path + '.gdf', eog=None, misc=None, preload=True)
+    assert raw._raw_extras[0]['subject_info']['age'] is None
 
     picks = pick_types(raw.info, meg=False, eeg=True, exclude='bads')
     data, _ = raw[picks]
@@ -80,9 +108,11 @@ def test_gdf2_data():
     assert_array_equal(events[:, 2], [20, 28])
 
     # gh-5604
-    assert raw.info['meas_date'] == DATE_NONE
+    assert raw.info['meas_date'] is None
     _test_raw_reader(read_raw_gdf, input_fname=gdf2_path + '.gdf',
-                     eog=None, misc=None)
+                     eog=None, misc=None,
+                     test_scaling=False,  # XXX this should be True
+                     )
 
 
 @testing.requires_testing_data
@@ -94,4 +124,15 @@ def test_one_channel_gdf():
     assert 150.0 == ecg.info['sfreq']
 
 
+@testing.requires_testing_data
+def test_gdf_exclude_channels():
+    """Test reading GDF data with excluded channels."""
+    raw = read_raw_gdf(gdf1_path + '.gdf', exclude=('FP1', 'O1'))
+    assert 'FP1' not in raw.ch_names
+    assert 'O1' not in raw.ch_names
+    raw = read_raw_gdf(gdf2_path + '.gdf', exclude=('Fp1', 'O1'))
+    assert 'Fp1' not in raw.ch_names
+    assert 'O1' not in raw.ch_names
+
+
 run_tests_if_main()
