@@ -20,7 +20,7 @@ from mne import write_events, read_epochs_eeglab
 from mne.io import read_raw_eeglab
 from mne.io.tests.test_raw import _test_raw_reader
 from mne.datasets import testing
-from mne.utils import requires_h5py, run_tests_if_main
+from mne.utils import check_version
 from mne.annotations import events_from_annotations, read_annotations
 from mne.io.eeglab.tests._utils import _read_eeglab_montage
 
@@ -33,7 +33,8 @@ epochs_fname_mat = op.join(base_dir, 'test_epochs.set')
 epochs_fname_onefile_mat = op.join(base_dir, 'test_epochs_onefile.set')
 raw_mat_fnames = [raw_fname_mat, raw_fname_onefile_mat]
 epochs_mat_fnames = [epochs_fname_mat, epochs_fname_onefile_mat]
-
+raw_fname_chanloc = op.join(base_dir, 'test_raw_chanloc.set')
+raw_fname_2021 = op.join(base_dir, 'test_raw_2021.set')
 raw_fname_h5 = op.join(base_dir, 'test_raw_h5.set')
 raw_fname_onefile_h5 = op.join(base_dir, 'test_raw_onefile_h5.set')
 epochs_fname_h5 = op.join(base_dir, 'test_epochs_h5.set')
@@ -41,24 +42,18 @@ epochs_fname_onefile_h5 = op.join(base_dir, 'test_epochs_onefile_h5.set')
 raw_h5_fnames = [raw_fname_h5, raw_fname_onefile_h5]
 epochs_h5_fnames = [epochs_fname_h5, epochs_fname_onefile_h5]
 
-raw_fnames = [raw_fname_mat, raw_fname_onefile_mat,
-              raw_fname_h5, raw_fname_onefile_h5]
 montage_path = op.join(base_dir, 'test_chans.locs')
 
 
-def _check_h5(fname):
-    if fname.endswith('_h5.set'):
-        try:
-            import h5py  # noqa, analysis:ignore
-        except Exception:
-            raise SkipTest('h5py module required')
+needs_h5 = pytest.mark.skipif(not check_version('h5py'), reason='Needs h5py')
 
 
-@requires_h5py
 @testing.requires_testing_data
-@pytest.mark.parametrize(
-    'fname', [raw_fname_mat, raw_fname_h5], ids=op.basename
-)
+@pytest.mark.parametrize('fname', [
+    raw_fname_mat,
+    pytest.param(raw_fname_h5, marks=needs_h5),
+    raw_fname_chanloc,
+], ids=op.basename)
 def test_io_set_raw(fname):
     """Test importing EEGLAB .set files."""
     montage = _read_eeglab_montage(montage_path)
@@ -66,28 +61,53 @@ def test_io_set_raw(fname):
         'EEG {0:03d}'.format(ii) for ii in range(len(montage.ch_names))
     ]
 
-    _test_raw_reader(read_raw_eeglab, input_fname=fname)
+    kws = dict(reader=read_raw_eeglab, input_fname=fname)
+    if fname.endswith('test_raw_chanloc.set'):
+        with pytest.warns(RuntimeWarning,
+                          match="The data contains 'boundary' events"):
+            raw0 = _test_raw_reader(**kws)
+    elif '_h5' in fname:  # should be safe enough, and much faster
+        raw0 = read_raw_eeglab(fname, preload=True)
+    else:
+        raw0 = _test_raw_reader(**kws)
+
     # test that preloading works
-    raw0 = read_raw_eeglab(input_fname=fname, preload=True)
-    raw0.set_montage(montage)
-    raw0.filter(1, None, l_trans_bandwidth='auto', filter_length='auto',
-                phase='zero')
+    if fname.endswith('test_raw_chanloc.set'):
+        raw0.set_montage(montage, on_missing='ignore')
+        # crop to check if the data has been properly preloaded; we cannot
+        # filter as the snippet of raw data is very short
+        raw0.crop(0, 1)
+    else:
+        raw0.set_montage(montage)
+        raw0.filter(1, None, l_trans_bandwidth='auto', filter_length='auto',
+                    phase='zero')
 
     # test that using uint16_codec does not break stuff
-    raw0 = read_raw_eeglab(input_fname=fname,
-                           preload=False, uint16_codec='ascii')
-    raw0.set_montage(montage, update_ch_names=True)
+    read_raw_kws = dict(input_fname=fname, preload=False, uint16_codec='ascii')
+    if fname.endswith('test_raw_chanloc.set'):
+        with pytest.warns(RuntimeWarning,
+                          match="The data contains 'boundary' events"):
+            raw0 = read_raw_eeglab(**read_raw_kws)
+            raw0.set_montage(montage, on_missing='ignore')
+    else:
+        raw0 = read_raw_eeglab(**read_raw_kws)
+        raw0.set_montage(montage)
+
+    # Annotations
+    if fname != raw_fname_chanloc:
+        assert len(raw0.annotations) == 154
+        assert set(raw0.annotations.description) == {'rt', 'square'}
+        assert_array_equal(raw0.annotations.duration, 0.)
 
 
 @testing.requires_testing_data
 def test_io_set_raw_more(tmpdir):
     """Test importing EEGLAB .set files."""
     tmpdir = str(tmpdir)
-    # test reading file with one event (read old version)
     eeg = io.loadmat(raw_fname_mat, struct_as_record=False,
                      squeeze_me=True)['EEG']
 
-    # test negative event latencies
+    # test reading file with one event (read old version)
     negative_latency_fname = op.join(tmpdir, 'test_negative_latency.set')
     evnts = deepcopy(eeg.event[0])
     evnts.latency = 0
@@ -103,6 +123,7 @@ def test_io_set_raw_more(tmpdir):
     with pytest.warns(RuntimeWarning, match="has a sample index of -1."):
         read_raw_eeglab(input_fname=negative_latency_fname, preload=True)
 
+    # test negative event latencies
     evnts.latency = -1
     io.savemat(negative_latency_fname,
                {'EEG': {'trials': eeg.trials, 'srate': eeg.srate,
@@ -126,6 +147,48 @@ def test_io_set_raw_more(tmpdir):
                appendmat=False, oned_as='row')
     shutil.copyfile(op.join(base_dir, 'test_raw.fdt'),
                     overlap_fname.replace('.set', '.fdt'))
+    read_raw_eeglab(input_fname=overlap_fname, preload=True)
+
+    # test reading file with empty event durations
+    empty_dur_fname = op.join(tmpdir, 'test_empty_durations.set')
+    evnts = deepcopy(eeg.event)
+    for ev in evnts:
+        ev.duration = np.array([], dtype='float')
+
+    io.savemat(empty_dur_fname,
+               {'EEG': {'trials': eeg.trials, 'srate': eeg.srate,
+                        'nbchan': eeg.nbchan,
+                        'data': 'test_negative_latency.fdt',
+                        'epoch': eeg.epoch, 'event': evnts,
+                        'chanlocs': eeg.chanlocs, 'pnts': eeg.pnts}},
+               appendmat=False, oned_as='row')
+    shutil.copyfile(op.join(base_dir, 'test_raw.fdt'),
+                    empty_dur_fname.replace('.set', '.fdt'))
+    raw = read_raw_eeglab(input_fname=empty_dur_fname, preload=True)
+    assert (raw.annotations.duration == 0).all()
+
+    # test reading file when the EEG.data name is wrong
+    io.savemat(overlap_fname,
+               {'EEG': {'trials': eeg.trials, 'srate': eeg.srate,
+                        'nbchan': eeg.nbchan, 'data': 'test_overla_event.fdt',
+                        'epoch': eeg.epoch,
+                        'event': [eeg.event[0], eeg.event[0]],
+                        'chanlocs': eeg.chanlocs, 'pnts': eeg.pnts}},
+               appendmat=False, oned_as='row')
+    with pytest.warns(RuntimeWarning, match="must have changed on disk"):
+        read_raw_eeglab(input_fname=overlap_fname, preload=True)
+
+    # raise error when both EEG.data and fdt name from set are wrong
+    overlap_fname = op.join(tmpdir, 'test_ovrlap_event.set')
+    io.savemat(overlap_fname,
+               {'EEG': {'trials': eeg.trials, 'srate': eeg.srate,
+                        'nbchan': eeg.nbchan, 'data': 'test_overla_event.fdt',
+                        'epoch': eeg.epoch,
+                        'event': [eeg.event[0], eeg.event[0]],
+                        'chanlocs': eeg.chanlocs, 'pnts': eeg.pnts}},
+               appendmat=False, oned_as='row')
+    with pytest.raises(FileNotFoundError, match="not find the .fdt data file"):
+        read_raw_eeglab(input_fname=overlap_fname, preload=True)
 
     # test reading file with one channel
     one_chan_fname = op.join(tmpdir, 'test_one_channel.set')
@@ -181,9 +244,11 @@ def test_io_set_raw_more(tmpdir):
 
 
 @pytest.mark.timeout(60)  # ~60 sec on Travis OSX
-@requires_h5py
 @testing.requires_testing_data
-@pytest.mark.parametrize('fnames', [epochs_mat_fnames, epochs_h5_fnames])
+@pytest.mark.parametrize('fnames', [
+    epochs_mat_fnames,
+    pytest.param(epochs_h5_fnames, marks=[needs_h5, pytest.mark.slowtest]),
+])
 def test_io_set_epochs(fnames):
     """Test importing EEGLAB .set epochs files."""
     epochs_fname, epochs_fname_onefile = fnames
@@ -201,7 +266,7 @@ def test_io_set_epochs_events(tmpdir):
     """Test different combinations of events and event_ids."""
     tmpdir = str(tmpdir)
     out_fname = op.join(tmpdir, 'test-eve.fif')
-    events = np.array([[4, 0, 1], [12, 0, 2], [20, 0, 3], [26, 0,  3]])
+    events = np.array([[4, 0, 1], [12, 0, 2], [20, 0, 3], [26, 0, 3]])
     write_events(out_fname, events)
     event_id = {'S255/S8': 1, 'S8': 2, 'S255/S9': 3}
     out_fname = op.join(tmpdir, 'test-eve.fif')
@@ -238,11 +303,16 @@ def test_degenerate(tmpdir):
                       bad_epochs_fname)
 
 
-@pytest.mark.parametrize("fname", raw_fnames)
+@pytest.mark.parametrize("fname", [
+    raw_fname_mat,
+    raw_fname_onefile_mat,
+    # We don't test the h5 varaints here because they are implicitly tested
+    # in test_io_set_raw
+])
+@pytest.mark.filterwarnings('ignore: Complex objects')
 @testing.requires_testing_data
 def test_eeglab_annotations(fname):
     """Test reading annotations in EEGLAB files."""
-    _check_h5(fname)
     annotations = read_annotations(fname)
     assert len(annotations) == 154
     assert set(annotations.description) == {'rt', 'square'}
@@ -305,8 +375,8 @@ def one_chanpos_fname(tmpdir_factory):
         'srate': 128, 'times': np.array([0., 0.1, 0.2]),
         'data': np.empty([3, 3]),
         'chanlocs': np.array(
-            [(b'F3',  1.,  4.,  7.),
-             (b'unknown',  2.,  5.,  8.),
+            [(b'F3', 1., 4., 7.),
+             (b'unknown', 2., 5., 8.),
              (b'FPz', np.nan, np.nan, np.nan)],
             dtype=[('labels', 'S10'), ('X', 'f8'), ('Y', 'f8'), ('Z', 'f8')]
         )
@@ -323,23 +393,22 @@ def test_position_information(one_chanpos_fname):
     """Test reading file with 3 channels - one without position information."""
     nan = np.nan
     EXPECTED_LOCATIONS_FROM_FILE = np.array([
-        [-4.,  1.,  7.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.],
-        [-5.,  2.,  8.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.],
+        [-4.,  1.,  7.,  0.,  0.,  0., nan, nan, nan, nan, nan, nan],
+        [-5.,  2.,  8.,  0.,  0.,  0., nan, nan, nan, nan, nan, nan],
         [nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan],
     ])
 
     EXPECTED_LOCATIONS_FROM_MONTAGE = np.array([
-        [-0.56705965, 0.67706631, 0.46906776, 0, 0, 0, 0, 0, 0, 0, 0, 0],
         [nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan],
-        [0, 0.99977915, -0.02101571, 0, 0, 0, 0, 0, 0, 0, 0, 0],
+        [nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan],
+        [nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan],
     ])
 
-    montage = _read_eeglab_montage(montage_path)
     raw = read_raw_eeglab(input_fname=one_chanpos_fname, preload=True)
     assert_array_equal(np.array([ch['loc'] for ch in raw.info['chs']]),
                        EXPECTED_LOCATIONS_FROM_FILE)
 
-    # To acomodate the new behavior so that:
+    # To accommodate the new behavior so that:
     # read_raw_eeglab(.. montage=montage) and raw.set_montage(montage)
     # behaves the same we need to flush the montage. otherwise we get
     # a mix of what is in montage and in the file
@@ -348,17 +417,23 @@ def test_position_information(one_chanpos_fname):
         preload=True,
     ).set_montage(None)  # Flush the montage builtin within input_fname
 
-    with pytest.raises(ValueError):
-        raw.set_montage(montage, update_ch_names=False)
+    _assert_array_allclose_nan(np.array([ch['loc'] for ch in raw.info['chs']]),
+                               EXPECTED_LOCATIONS_FROM_MONTAGE)
 
-    _msg = (
-        'DigMontage is a only a subset of info. '
-        'Did not set 1 channel positions:\nunknown'
-    )
-    with pytest.warns(RuntimeWarning, match=_msg):
-        raw.set_montage(montage, update_ch_names=False, raise_if_subset=False)
     _assert_array_allclose_nan(np.array([ch['loc'] for ch in raw.info['chs']]),
                                EXPECTED_LOCATIONS_FROM_MONTAGE)
 
 
-run_tests_if_main()
+@testing.requires_testing_data
+def test_io_set_raw_2021():
+    """Test reading new default file format (no EEG struct)."""
+    assert "EEG" not in io.loadmat(raw_fname_2021)
+    _test_raw_reader(reader=read_raw_eeglab, input_fname=raw_fname_2021,
+                     test_preloading=False, preload=True)
+
+
+@testing.requires_testing_data
+def test_read_single_epoch():
+    """Test reading raw set file as an Epochs instance."""
+    with pytest.raises(ValueError, match='trials less than 2'):
+        read_epochs_eeglab(raw_fname_mat)
