@@ -7,8 +7,9 @@
 import numpy as np
 
 from .utils import _create_info, _set_tmin, _create_events, \
-    _create_event_metadata
-from .. import RawArray
+    _create_event_metadata, _validate_ft_struct
+from ...utils import _check_fname
+from ..array.array import RawArray
 from ...epochs import EpochsArray
 from ...evoked import EvokedArray
 
@@ -28,23 +29,23 @@ def read_raw_fieldtrip(fname, info, data_name='data'):
 
     Parameters
     ----------
-    fname: str
+    fname : str
         Path and filename of the .mat file containing the data.
-    info: dict or None
+    info : dict or None
         The info dict of the raw data file corresponding to the data to import.
         If this is set to None, limited information is extracted from the
         FieldTrip structure.
-    data_name: str
+    data_name : str
         Name of heading dict/ variable name under which the data was originally
         saved in MATLAB.
 
     Returns
     -------
-    raw: instance of RawArray
+    raw : instance of RawArray
         A Raw Object containing the loaded data.
-
     """
-    from ...externals.pymatreader.pymatreader import read_mat
+    from ...externals.pymatreader import read_mat
+    fname = _check_fname(fname, overwrite='read', must_exist=True)
 
     ft_struct = read_mat(fname,
                          ignore_fields=['previous'],
@@ -53,6 +54,8 @@ def read_raw_fieldtrip(fname, info, data_name='data'):
     # load data and set ft_struct to the heading dictionary
     ft_struct = ft_struct[data_name]
 
+    _validate_ft_struct(ft_struct)
+
     info = _create_info(ft_struct, info)  # create info structure
     data = np.array(ft_struct['trial'])  # create the main data array
 
@@ -89,26 +92,24 @@ def read_epochs_fieldtrip(fname, info, data_name='data',
 
     Parameters
     ----------
-    fname: str
+    fname : str
         Path and filename of the .mat file containing the data.
-    info: dict or None
+    info : dict or None
         The info dict of the raw data file corresponding to the data to import.
         If this is set to None, limited information is extracted from the
         FieldTrip structure.
-    data_name: str
+    data_name : str
         Name of heading dict/ variable name under which the data was originally
         saved in MATLAB.
-    trialinfo_column: int
-        Column of the trialinfo matrix to use for the event codes
+    trialinfo_column : int
+        Column of the trialinfo matrix to use for the event codes.
 
     Returns
     -------
-    epochs: instance of EpochsArray
+    epochs : instance of EpochsArray
         An EpochsArray containing the loaded data.
-
-
     """
-    from ...externals.pymatreader.pymatreader import read_mat
+    from ...externals.pymatreader import read_mat
     ft_struct = read_mat(fname,
                          ignore_fields=['previous'],
                          variable_names=[data_name])
@@ -116,10 +117,15 @@ def read_epochs_fieldtrip(fname, info, data_name='data',
     # load data and set ft_struct to the heading dictionary
     ft_struct = ft_struct[data_name]
 
+    _validate_ft_struct(ft_struct)
+
     info = _create_info(ft_struct, info)  # create info structure
     data = np.array(ft_struct['trial'])  # create the epochs data array
     events = _create_events(ft_struct, trialinfo_column)
-    metadata = _create_event_metadata(ft_struct)
+    if events is not None:
+        metadata = _create_event_metadata(ft_struct)
+    else:
+        metadata = None
     tmin = _set_tmin(ft_struct)  # create start time
 
     epochs = EpochsArray(data=data, info=info, tmin=tmin,
@@ -143,30 +149,31 @@ def read_evoked_fieldtrip(fname, info, comment=None,
 
     Parameters
     ----------
-    fname: str
+    fname : str
         Path and filename of the .mat file containing the data.
-    info: dict or None
+    info : dict or None
         The info dict of the raw data file corresponding to the data to import.
         If this is set to None, limited information is extracted from the
         FieldTrip structure.
-    comment: str
+    comment : str
         Comment on dataset. Can be the condition.
-    data_name: str
+    data_name : str
         Name of heading dict/ variable name under which the data was originally
         saved in MATLAB.
 
     Returns
     -------
-    evoked: instance of EvokedArray
+    evoked : instance of EvokedArray
         An EvokedArray containing the loaded data.
-
     """
-    from ...externals.pymatreader.pymatreader import read_mat
+    from ...externals.pymatreader import read_mat
     ft_struct = read_mat(fname,
                          ignore_fields=['previous'],
                          variable_names=[data_name])
     ft_struct = ft_struct[data_name]
 
+    _validate_ft_struct(ft_struct)
+
     info = _create_info(ft_struct, info)  # create info structure
     data_evoked = ft_struct['avg']  # create evoked data
 
