@@ -8,7 +8,6 @@ import os
 import types
 
 import numpy as np
-import pytest
 
 import mne
 
@@ -26,8 +25,7 @@ ch_ignore_fields = ('logno', 'cal', 'range', 'scanno', 'coil_type', 'kind',
 info_long_fields = ('hpi_meas', )
 
 system_to_reader_fn_dict = {'neuromag306': mne.io.read_raw_fif,
-                            'CNT': partial(mne.io.read_raw_cnt,
-                                           stim_channel=False),
+                            'CNT': partial(mne.io.read_raw_cnt),
                             'CTF': partial(mne.io.read_raw_ctf,
                                            clean_names=True),
                             'BTI': partial(mne.io.read_raw_bti,
@@ -123,11 +121,6 @@ def get_raw_data(system, drop_extra_chs=False):
     if system == 'eximia':
         crop -= 0.5 * (1.0 / raw_data.info['sfreq'])
     raw_data.crop(0, crop)
-    if any([type_ in raw_data for type_ in ['eeg', 'ecog', 'seeg']]):
-        raw_data.set_eeg_reference([])
-    else:
-        with pytest.raises(ValueError, match='No EEG, ECoG or sEEG channels'):
-            raw_data.set_eeg_reference([])
     raw_data.del_proj('all')
     raw_data.info['comps'] = []
     raw_data.drop_channels(cfg_local['removed_chan_names'])
