@@ -9,29 +9,28 @@ import calendar
 
 from ...utils import logger, fill_doc
 from ..utils import _read_segments_file, _find_channels, _create_chs
-from ..utils import _deprecate_montage
 from ..base import BaseRaw
 from ..meas_info import _empty_info
 from ..constants import FIFF
 
 
 @fill_doc
-def read_raw_nicolet(input_fname, ch_type, montage='deprecated', eog=(),
+def read_raw_nicolet(input_fname, ch_type, eog=(),
                      ecg=(), emg=(), misc=(), preload=False, verbose=None):
     """Read Nicolet data as raw object.
 
-    Note: This reader takes data files with the extension ``.data`` as an
-    input. The header file with the same file name stem and an extension
-    ``.head`` is expected to be found in the same directory.
+    ..note:: This reader takes data files with the extension ``.data`` as an
+             input. The header file with the same file name stem and an
+             extension ``.head`` is expected to be found in the same
+             directory.
 
     Parameters
     ----------
     input_fname : str
-        Path to the data file.
+        Path to the data file (ending with ``.data`` not ``.head``).
     ch_type : str
         Channel type to designate to the data channels. Supported data types
-        include 'eeg', 'seeg'.
-    %(montage_deprecated)s
+        include 'eeg', 'dbs'.
     eog : list | tuple | 'auto'
         Names of channels or list of indices that should be designated
         EOG channels. If 'auto', the channel names beginning with
@@ -59,13 +58,19 @@ def read_raw_nicolet(input_fname, ch_type, montage='deprecated', eog=(),
     --------
     mne.io.Raw : Documentation of attribute and methods.
     """
-    return RawNicolet(input_fname, ch_type, montage=montage, eog=eog, ecg=ecg,
+    return RawNicolet(input_fname, ch_type, eog=eog, ecg=ecg,
                       emg=emg, misc=misc, preload=preload, verbose=verbose)
 
 
 def _get_nicolet_info(fname, ch_type, eog, ecg, emg, misc):
     """Extract info from Nicolet header files."""
-    fname = path.splitext(fname)[0]
+    fname, extension = path.splitext(fname)
+
+    if extension != '.data':
+        raise ValueError(
+            f'File name should end with .data not "{extension}".'
+        )
+
     header = fname + '.head'
 
     logger.info('Reading header...')
@@ -77,8 +82,11 @@ def _get_nicolet_info(fname, ch_type, eog, ecg, emg, misc):
                 value = value[1:-2].split(',')  # strip brackets
             elif var == 'conversion_factor':
                 value = float(value)
-            elif var != 'start_ts':
+            elif var in ['num_channels', 'rec_id', 'adm_id', 'pat_id',
+                         'num_samples']:
                 value = int(value)
+            elif var != 'start_ts':
+                value = float(value)
             header_info[var] = value
 
     ch_names = header_info['elec_names']
@@ -126,7 +134,6 @@ class RawNicolet(BaseRaw):
     ch_type : str
         Channel type to designate to the data channels. Supported data types
         include 'eeg', 'seeg'.
-    %(montage_deprecated)s
     eog : list | tuple | 'auto'
         Names of channels or list of indices that should be designated
         EOG channels. If 'auto', the channel names beginning with
@@ -150,7 +157,7 @@ class RawNicolet(BaseRaw):
     mne.io.Raw : Documentation of attribute and methods.
     """
 
-    def __init__(self, input_fname, ch_type, montage='deprecated', eog=(),
+    def __init__(self, input_fname, ch_type, eog=(),
                  ecg=(), emg=(), misc=(), preload=False,
                  verbose=None):  # noqa: D102
         input_fname = path.abspath(input_fname)
@@ -162,8 +169,7 @@ class RawNicolet(BaseRaw):
             last_samps=last_samps, orig_format='int',
             verbose=verbose)
 
-        _deprecate_montage(self, "read_raw_nicolet", montage)
-
     def _read_segment_file(self, data, idx, fi, start, stop, cals, mult):
         """Read a chunk of raw data."""
-        _read_segments_file(self, data, idx, fi, start, stop, cals, mult)
+        _read_segments_file(
+            self, data, idx, fi, start, stop, cals, mult, dtype='<i2')
