@@ -4,18 +4,19 @@
 #
 # License: BSD (3-clause)
 
-from copy import deepcopy
 import numpy as np
 
 from .constants import FIFF
+from .meas_info import _check_ch_keys
 from .proj import _has_eeg_average_ref_proj, make_eeg_average_ref_proj
 from .proj import setup_proj
-from .pick import pick_types, pick_channels
+from .pick import pick_types, pick_channels, pick_channels_forward
 from .base import BaseRaw
 from ..evoked import Evoked
 from ..epochs import BaseEpochs
+from ..fixes import pinv
 from ..utils import (logger, warn, verbose, _validate_type, _check_preload,
-                     _check_option)
+                     _check_option, fill_doc)
 from ..defaults import DEFAULTS
 
 
@@ -46,56 +47,14 @@ def _copy_channel(inst, ch_name, new_ch_name):
     return inst
 
 
-def _apply_reference(inst, ref_from, ref_to=None):
-    """Apply a custom EEG referencing scheme.
-
-    This function modifies the instance in-place.
-
-    Calculates a reference signal by taking the mean of a set of channels and
-    applies the reference to another set of channels. Input data can be in the
-    form of Raw, Epochs or Evoked.
-
-    Parameters
-    ----------
-    inst : instance of Raw | Epochs | Evoked
-        Data containing the EEG channels and reference channel(s).
-    ref_from : list of str
-        The names of the channels to use to construct the reference. If an
-        empty list is specified, the data is assumed to already have a proper
-        reference and MNE will not attempt any re-referencing of the data.
-    ref_to : list of str | None
-        The names of the channels to apply the reference to. If None (which is
-        the default), all EEG channels are chosen.
-
-    Returns
-    -------
-    inst : instance of Raw | Epochs | Evoked
-        The data with EEG channels re-referenced.
-    ref_data : array, shape (n_times,)
-        Array of reference data subtracted from EEG channels.
-
-    Notes
-    -----
-    This function operates in-place.
-
-    1. If the reference is applied to any EEG channels, this function removes
-       any pre-existing average reference projections.
-
-    2. During source localization, the EEG signal should have an average
-       reference.
-
-    3. The data must be preloaded.
-
-    See Also
-    --------
-    set_eeg_reference : Convenience function for creating an EEG reference.
-    set_bipolar_reference : Convenience function for creating a bipolar
-                            reference.
-    """
+def _check_before_reference(inst, ref_from, ref_to, ch_type):
+    """Prepare instance for referencing."""
     # Check to see that data is preloaded
     _check_preload(inst, "Applying a reference")
 
-    eeg_idx = pick_types(inst.info, eeg=True, meg=False, ref_meg=False)
+    ch_type = _get_ch_type(inst, ch_type)
+    ch_dict = {ch_type: True, 'meg': False, 'ref_meg': False}
+    eeg_idx = pick_types(inst.info, **ch_dict)
 
     if ref_to is None:
         ref_to = [inst.ch_names[i] for i in eeg_idx]
@@ -137,12 +96,27 @@ def _apply_reference(inst, ref_from, ref_to=None):
     inst._projector, _ = \
         setup_proj(inst.info, add_eeg_ref=False, activate=False)
 
+    # If the reference touches EEG/ECoG/sEEG/DBS electrodes, note in the
+    # info that a non-CAR has been applied.
+    ref_to_channels = pick_channels(inst.ch_names, ref_to, ordered=True)
+    if len(np.intersect1d(ref_to_channels, eeg_idx)) > 0:
+        inst.info['custom_ref_applied'] = FIFF.FIFFV_MNE_CUSTOM_REF_ON
+
+    return ref_to
+
+
+def _apply_reference(inst, ref_from, ref_to=None, forward=None,
+                     ch_type='auto'):
+    """Apply a custom EEG referencing scheme."""
+    ref_to = _check_before_reference(inst, ref_from, ref_to, ch_type)
+
     # Compute reference
     if len(ref_from) > 0:
         # this is guaranteed below, but we should avoid the crazy pick_channels
         # behavior that [] gives all. Also use ordered=True just to make sure
         # that all supplied channels actually exist.
         assert len(ref_to) > 0
+        ref_names = ref_from
         ref_from = pick_channels(inst.ch_names, ref_from, ordered=True)
         ref_to = pick_channels(inst.ch_names, ref_to, ordered=True)
 
@@ -151,16 +125,29 @@ def _apply_reference(inst, ref_from, ref_to=None):
         data[..., ref_to, :] -= ref_data
         ref_data = ref_data[..., 0, :]
 
-        # If the reference touches EEG electrodes, note in the info that a
-        # non-CAR has been applied.
-        if len(np.intersect1d(ref_to, eeg_idx)) > 0:
-            inst.info['custom_ref_applied'] = True
+        # REST
+        if forward is not None:
+            # use ch_sel and the given forward
+            forward = pick_channels_forward(forward, ref_names, ordered=True)
+            # 1-3. Compute a forward (G) and avg-ref'ed data (done above)
+            G = forward['sol']['data']
+            assert G.shape[0] == len(ref_names)
+            # 4. Compute the forward (G) and average-reference it (Ga):
+            Ga = G - np.mean(G, axis=0, keepdims=True)
+            # 5. Compute the Ga_inv by SVD
+            Ga_inv = pinv(Ga, rtol=1e-6)
+            # 6. Compute Ra = (G @ Ga_inv) in eq (8) from G and Ga_inv
+            Ra = G @ Ga_inv
+            # 7-8. Compute Vp = Ra @ Va; then Vpa=average(Vp)
+            Vpa = np.mean(Ra @ data[..., ref_from, :], axis=-2, keepdims=True)
+            data[..., ref_to, :] += Vpa
     else:
         ref_data = None
 
     return inst, ref_data
 
 
+@fill_doc
 def add_reference_channels(inst, ref_channels, copy=True):
     """Add reference channels to data that consists of all zeros.
 
@@ -172,10 +159,7 @@ def add_reference_channels(inst, ref_channels, copy=True):
     ----------
     inst : instance of Raw | Epochs | Evoked
         Instance of Raw or Epochs with EEG channels and reference channel(s).
-    ref_channels : str | list of str
-        Name of the electrode(s) which served as the reference in the
-        recording. If a name is provided, a corresponding channel is added
-        and its data is set to 0. This is useful for later re-referencing.
+    %(ref_channels)s
     copy : bool
         Specifies whether the data will be copied (True) or modified in-place
         (False). Defaults to True.
@@ -186,13 +170,10 @@ def add_reference_channels(inst, ref_channels, copy=True):
         Data with added EEG reference channels.
     """
     # Check to see that data is preloaded
-    if not inst.preload:
-        raise RuntimeError('Data needs to be preloaded.')
+    _check_preload(inst, 'add_reference_channels')
+    _validate_type(ref_channels, (list, tuple, str), 'ref_channels')
     if isinstance(ref_channels, str):
         ref_channels = [ref_channels]
-    elif not isinstance(ref_channels, list):
-        raise ValueError("`ref_channels` should be either str or list of str. "
-                         "%s was provided." % type(ref_channels))
     for ch in ref_channels:
         if ch in inst.info['ch_names']:
             raise ValueError("Channel %s already specified in inst." % ch)
@@ -261,75 +242,54 @@ def add_reference_channels(inst, ref_channels, copy=True):
         inst.info._update_redundant()
     if isinstance(inst, BaseRaw):
         inst._cals = np.hstack((inst._cals, [1] * len(ref_channels)))
+        range_ = np.arange(1, len(ref_channels) + 1)
+        for pi, picks in enumerate(inst._read_picks):
+            inst._read_picks[pi] = np.concatenate(
+                [picks, np.max(picks) + range_])
     inst.info._check_consistency()
-    set_eeg_reference(inst, ref_channels=ref_channels, copy=False)
+    set_eeg_reference(inst, ref_channels=ref_channels, copy=False,
+                      verbose=False)
     return inst
 
 
+_ref_dict = {
+    FIFF.FIFFV_MNE_CUSTOM_REF_ON: 'on',
+    FIFF.FIFFV_MNE_CUSTOM_REF_OFF: 'off',
+    FIFF.FIFFV_MNE_CUSTOM_REF_CSD: 'CSD',
+}
+
+
+def _check_can_reref(inst):
+    _validate_type(inst, (BaseRaw, BaseEpochs, Evoked), "Instance")
+    current_custom = inst.info['custom_ref_applied']
+    if current_custom not in (FIFF.FIFFV_MNE_CUSTOM_REF_ON,
+                              FIFF.FIFFV_MNE_CUSTOM_REF_OFF):
+        raise RuntimeError('Cannot set new reference on data with custom '
+                           'reference type %r' % (_ref_dict[current_custom],))
+
+
 @verbose
 def set_eeg_reference(inst, ref_channels='average', copy=True,
-                      projection=False, ch_type='auto', verbose=None):
+                      projection=False, ch_type='auto', forward=None,
+                      verbose=None):
     """Specify which reference to use for EEG data.
 
-    By default, MNE-Python will automatically re-reference the EEG signal to
-    use an average reference (see below). Use this function to explicitly
-    specify the desired reference for EEG. This can be either an existing
-    electrode or a new virtual channel. This function will re-reference the
-    data according to the desired reference and prevent MNE-Python from
-    automatically adding an average reference projection.
-
-    Some common referencing schemes and the corresponding value for the
-    ``ref_channels`` parameter:
-
-    No re-referencing:
-        If the EEG data is already using the proper reference, set
-        ``ref_channels=[]``. This will prevent MNE-Python from automatically
-        adding an average reference projection.
-
-    Average reference:
-        A new virtual reference electrode is created by averaging the current
-        EEG signal by setting ``ref_channels='average'``. Bad EEG channels are
-        automatically excluded if they are properly set in ``info['bads']``.
-
-    A single electrode:
-        Set ``ref_channels`` to a list containing the name of the channel that
-        will act as the new reference, for example ``ref_channels=['Cz']``.
-
-    The mean of multiple electrodes:
-        A new virtual reference electrode is created by computing the average
-        of the current EEG signal recorded from two or more selected channels.
-        Set ``ref_channels`` to a list of channel names, indicating which
-        channels to use. For example, to apply an average mastoid reference,
-        when using the 10-20 naming scheme, set ``ref_channels=['M1', 'M2']``.
+    Use this function to explicitly specify the desired reference for EEG.
+    This can be either an existing electrode or a new virtual channel.
+    This function will re-reference the data according to the desired
+    reference.
 
     Parameters
     ----------
     inst : instance of Raw | Epochs | Evoked
         Instance of Raw or Epochs with EEG channels and reference channel(s).
-    ref_channels : list of str | str
-        The name(s) of the channel(s) used to construct the reference. To apply
-        an average reference, specify ``'average'`` here (default). If an empty
-        list is specified, the data is assumed to already have a proper
-        reference and MNE will not attempt any re-referencing of the data.
-        Defaults to an average reference.
+    %(set_eeg_reference_ref_channels)s
     copy : bool
         Specifies whether the data will be copied (True) or modified in-place
         (False). Defaults to True.
-    projection : bool
-        If ``ref_channels='average'`` this argument specifies if the average
-        reference should be computed as a projection (True) or not (False;
-        default). If ``projection=True``, the average reference is added as a
-        projection and is not applied to the data (it can be applied
-        afterwards with the ``apply_proj`` method). If ``projection=False``,
-        the average reference is directly applied to the data.
-        If ``ref_channels`` is not ``'average'``, ``projection`` must be set to
-        ``False`` (the default in this case).
-    ch_type : 'auto' | 'eeg' | 'ecog' | 'seeg'
-        The name of the channel type to apply the reference to. If 'auto', the
-        first channel type of eeg, ecog or seeg that is found (in that order)
-        will be selected.
-
-        .. versionadded:: 0.19
+    %(set_eeg_reference_projection)s
+    %(set_eeg_reference_ch_type)s
+    %(set_eeg_reference_forward)s
     %(verbose)s
 
     Returns
@@ -340,30 +300,11 @@ def set_eeg_reference(inst, ref_channels='average', copy=True,
         re-referencing the data.
     ref_data : array
         Array of reference data subtracted from EEG channels. This will be
-        ``None`` if ``ref_channels='average'`` and ``projection=True``.
-
-    Notes
-    -----
-    1. If a reference is requested that is not the average reference, this
-       function removes any pre-existing average reference projections.
-
-    2. During source localization, the EEG signal should have an average
-       reference.
-
-    3. In order to apply a reference, the data must be preloaded. This is not
-       necessary if ``ref_channels='average'`` and ``projection=True``.
-
-    4. For an average reference, bad EEG channels are automatically excluded if
-       they are properly set in ``info['bads']``.
-
-    .. versionadded:: 0.9.0
-
-    See Also
-    --------
-    set_bipolar_reference : Convenience function for creating bipolar
-                            references.
+        ``None`` if ``projection=True`` or ``ref_channels='REST'``.
+    %(set_eeg_reference_see_also_notes)s
     """
-    _validate_type(inst, (BaseRaw, BaseEpochs, Evoked), "Instance")
+    from ..forward import Forward
+    _check_can_reref(inst)
 
     if projection:  # average reference projector
         if ref_channels != 'average':
@@ -373,34 +314,58 @@ def set_eeg_reference(inst, ref_channels='average', copy=True,
         if _has_eeg_average_ref_proj(inst.info['projs']):
             warn('An average reference projection was already added. The data '
                  'has been left untouched.')
-            return inst, None
         else:
             # Creating an average reference may fail. In this case, make
             # sure that the custom_ref_applied flag is left untouched.
             custom_ref_applied = inst.info['custom_ref_applied']
             try:
-                inst.info['custom_ref_applied'] = False
+                inst.info['custom_ref_applied'] = FIFF.FIFFV_MNE_CUSTOM_REF_OFF
                 inst.add_proj(make_eeg_average_ref_proj(inst.info,
                               activate=False))
-                # If the data has been preloaded, projections will no
-                # longer be automatically applied.
-                if inst.preload:
-                    logger.info('Average reference projection was added, '
-                                'but has not been applied yet. Use the '
-                                'apply_proj method to apply it.')
             except Exception:
                 inst.info['custom_ref_applied'] = custom_ref_applied
                 raise
-            return inst, None
+            # If the data has been preloaded, projections will no
+            # longer be automatically applied.
+            if inst.preload:
+                logger.info('Average reference projection was added, '
+                            'but has not been applied yet. Use the '
+                            'apply_proj method to apply it.')
+        return inst, None
+    del projection  # not used anymore
 
     inst = inst.copy() if copy else inst
+    ch_type = _get_ch_type(inst, ch_type)
+    ch_dict = {ch_type: True, 'meg': False, 'ref_meg': False}
+    eeg_idx = pick_types(inst.info, **ch_dict)
+    ch_sel = [inst.ch_names[i] for i in eeg_idx]
 
-    _check_option('ch_type', ch_type, ('auto', 'eeg', 'ecog', 'seeg'))
+    if ref_channels == 'REST':
+        _validate_type(forward, Forward, 'forward when ref_channels="REST"')
+    else:
+        forward = None  # signal to _apply_reference not to do REST
+
+    if ref_channels in ('average', 'REST'):
+        logger.info(f'Applying {ref_channels} reference.')
+        ref_channels = ch_sel
+
+    if ref_channels == []:
+        logger.info('EEG data marked as already having the desired reference.')
+    else:
+        logger.info('Applying a custom %s '
+                    'reference.' % DEFAULTS['titles'][ch_type])
+
+    return _apply_reference(inst, ref_channels, ch_sel, forward,
+                            ch_type=ch_type)
+
+
+def _get_ch_type(inst, ch_type):
+    _validate_type(ch_type, str, 'ch_type')
+    _check_option('ch_type', ch_type, ('auto', 'eeg', 'ecog', 'seeg', 'dbs'))
     # if ch_type is 'auto', search through list to find first reasonable
     # reference-able channel type.
-    possible_types = ['eeg', 'ecog', 'seeg']
     if ch_type == 'auto':
-        for type_ in possible_types:
+        for type_ in ['eeg', 'ecog', 'seeg', 'dbs']:
             if type_ in inst:
                 ch_type = type_
                 logger.info('%s channel type selected for '
@@ -408,26 +373,9 @@ def set_eeg_reference(inst, ref_channels='average', copy=True,
                 break
         # if auto comes up empty, or the user specifies a bad ch_type.
         else:
-            raise ValueError('No EEG, ECoG or sEEG channels found '
+            raise ValueError('No EEG, ECoG, sEEG or DBS channels found '
                              'to rereference.')
-
-    ch_dict = {ch_type: True, 'meg': False, 'ref_meg': False}
-    eeg_idx = pick_types(inst.info, **ch_dict)
-    ch_sel = [inst.ch_names[i] for i in eeg_idx]
-
-    if ref_channels == 'average' and not projection:  # apply average reference
-        logger.info('Applying average reference.')
-        ref_channels = ch_sel
-
-    if ref_channels == []:
-        logger.info('EEG data marked as already having the desired reference. '
-                    'Preventing automatic future re-referencing to an average '
-                    'reference.')
-    else:
-        logger.info('Applying a custom %s '
-                    'reference.' % DEFAULTS['titles'][type_])
-
-    return _apply_reference(inst, ref_channels, ch_sel)
+    return ch_type
 
 
 @verbose
@@ -437,15 +385,14 @@ def set_bipolar_reference(inst, anode, cathode, ch_name=None, ch_info=None,
 
     A bipolar reference takes the difference between two channels (the anode
     minus the cathode) and adds it as a new virtual channel. The original
-    channels will be dropped.
+    channels will be dropped by default.
 
     Multiple anodes and cathodes can be specified, in which case multiple
-    virtual channels will be created. The 1st anode will be subtracted from the
-    1st cathode, the 2nd anode from the 2nd cathode, etc.
+    virtual channels will be created. The 1st cathode will be subtracted
+    from the 1st anode, the 2nd cathode from the 2nd anode, etc.
 
-    By default, the virtual channels will be annotated with channel info of
-    the anodes, their locations set to (0, 0, 0) and coil types set to
-    EEG_BIPOLAR.
+    By default, the virtual channels will be annotated with channel-info and
+    -location of the anodes and coil types will be set to EEG_BIPOLAR.
 
     Parameters
     ----------
@@ -476,6 +423,10 @@ def set_bipolar_reference(inst, anode, cathode, ch_name=None, ch_info=None,
     inst : instance of Raw | Epochs | Evoked
         Data with the specified channels re-referenced.
 
+    See Also
+    --------
+    set_eeg_reference : Convenience function for creating an EEG reference.
+
     Notes
     -----
     1. If the anodes contain any EEG channels, this function removes
@@ -487,11 +438,13 @@ def set_bipolar_reference(inst, anode, cathode, ch_name=None, ch_info=None,
     3. The data must be preloaded.
 
     .. versionadded:: 0.9.0
-
-    See Also
-    --------
-    set_eeg_reference : Convenience function for creating an EEG reference.
     """
+    from .meas_info import create_info
+    from ..io import RawArray
+    from ..epochs import EpochsArray
+    from ..evoked import EvokedArray
+
+    _check_can_reref(inst)
     if not isinstance(anode, list):
         anode = [anode]
 
@@ -503,7 +456,7 @@ def set_bipolar_reference(inst, anode, cathode, ch_name=None, ch_info=None,
                          'of cathodes (got %d).' % (len(anode), len(cathode)))
 
     if ch_name is None:
-        ch_name = ['%s-%s' % ac for ac in zip(anode, cathode)]
+        ch_name = [f'{a}-{c}' for (a, c) in zip(anode, cathode)]
     elif not isinstance(ch_name, list):
         ch_name = [ch_name]
     if len(ch_name) != len(anode):
@@ -526,35 +479,64 @@ def set_bipolar_reference(inst, anode, cathode, ch_name=None, ch_info=None,
         raise ValueError('Number of channel info dictionaries must equal the '
                          'number of anodes/cathodes.')
 
-    # Merge specified and anode channel information dictionaries
-    new_chs = []
-    for an, ci in zip(anode, ch_info):
-        an_idx = inst.ch_names.index(an)
-        this_chs = deepcopy(inst.info['chs'][an_idx])
-
-        # Set channel location and coil type
-        this_chs['loc'] = np.zeros(12)
-        this_chs['coil_type'] = FIFF.FIFFV_COIL_EEG_BIPOLAR
-
-        this_chs.update(ci)
-        new_chs.append(this_chs)
-
     if copy:
         inst = inst.copy()
 
-    for i, (an, ca, name, chs) in enumerate(
-            zip(anode, cathode, ch_name, new_chs)):
-        if an in anode[i + 1:] or an in cathode[i + 1:] or not drop_refs:
-            # Make a copy of the channel if it's still needed later
-            # otherwise it's modified inplace
-            _copy_channel(inst, an, 'TMP')
-            an = 'TMP'
-        _apply_reference(inst, [ca], [an])
+    anode = _check_before_reference(inst, ref_from=cathode,
+                                    ref_to=anode, ch_type='auto')
+
+    # Create bipolar reference channels by multiplying the data
+    # (channels x time) with a matrix (n_virtual_channels x channels)
+    # and add them to the instance.
+    multiplier = np.zeros((len(anode), len(inst.ch_names)))
+    for idx, (a, c) in enumerate(zip(anode, cathode)):
+        multiplier[idx, inst.ch_names.index(a)] = 1
+        multiplier[idx, inst.ch_names.index(c)] = -1
+
+    ref_info = create_info(ch_names=ch_name, sfreq=inst.info['sfreq'],
+                           ch_types=inst.get_channel_types(picks=anode))
+
+    # Update "chs" in Reference-Info.
+    for ch_idx, (an, info) in enumerate(zip(anode, ch_info)):
+        _check_ch_keys(info, ch_idx, name='ch_info', check_min=False)
         an_idx = inst.ch_names.index(an)
-        inst.info['chs'][an_idx] = chs
-        inst.info['chs'][an_idx]['ch_name'] = name
-        logger.info('Bipolar channel added as "%s".' % name)
-        inst.info._update_redundant()
+        # Copy everything from anode (except ch_name).
+        an_chs = {k: v for k, v in inst.info['chs'][an_idx].items()
+                  if k != 'ch_name'}
+        ref_info['chs'][ch_idx].update(an_chs)
+        # Set coil-type to bipolar.
+        ref_info['chs'][ch_idx]['coil_type'] = FIFF.FIFFV_COIL_EEG_BIPOLAR
+        # Update with info from ch_info-parameter.
+        ref_info['chs'][ch_idx].update(info)
+
+    # Set other info-keys from original instance.
+    pick_info = {k: v for k, v in inst.info.items() if k not in
+                 ['chs', 'ch_names', 'bads', 'nchan', 'sfreq']}
+    ref_info.update(pick_info)
+
+    # Rereferencing of data.
+    ref_data = multiplier @ inst._data
+
+    if isinstance(inst, BaseRaw):
+        ref_inst = RawArray(ref_data, ref_info, first_samp=inst.first_samp,
+                            copy=None)
+    elif isinstance(inst, BaseEpochs):
+        ref_inst = EpochsArray(ref_data, ref_info, events=inst.events,
+                               tmin=inst.tmin, event_id=inst.event_id,
+                               metadata=inst.metadata)
+    else:
+        ref_inst = EvokedArray(ref_data, ref_info, tmin=inst.tmin,
+                               comment=inst.comment, nave=inst.nave,
+                               kind='average')
+
+    # Add referenced instance to original instance.
+    inst.add_channels([ref_inst], force_update_info=True)
+
+    added_channels = ', '.join([name for name in ch_name])
+    logger.info(f'Added the following bipolar channels:\n{added_channels}')
+
+    for attr_name in ['picks', '_projector']:
+        setattr(inst, attr_name, None)
 
     # Drop remaining channels.
     if drop_refs:
