@@ -4,7 +4,9 @@ from .inverse import (InverseOperator, read_inverse_operator, apply_inverse,
                       apply_inverse_raw, make_inverse_operator,
                       apply_inverse_epochs, write_inverse_operator,
                       compute_rank_inverse, prepare_inverse_operator,
-                      estimate_snr)
-from .psf_ctf import point_spread_function, cross_talk_function
+                      estimate_snr, apply_inverse_cov, INVERSE_METHODS)
 from .time_frequency import (source_band_induced_power, source_induced_power,
                              compute_source_psd, compute_source_psd_epochs)
+from .resolution_matrix import (make_inverse_resolution_matrix,
+                                get_point_spread, get_cross_talk)
+from .spatial_resolution import resolution_metrics
