@@ -9,6 +9,7 @@ import os
 
 from . import get_config
 from .utils import logger, verbose, warn, ProgressBar
+from .utils.check import int_like
 from .fixes import _get_args
 
 if 'MNE_FORCE_SERIAL' in os.environ:
@@ -26,17 +27,17 @@ def parallel_func(func, n_jobs, max_nbytes='auto', pre_dispatch='n_jobs',
 
     Parameters
     ----------
-    func: callable
-        A function
-    n_jobs: int
-        Number of jobs to run in parallel
+    func : callable
+        A function.
+    n_jobs : int
+        Number of jobs to run in parallel.
     max_nbytes : int, str, or None
         Threshold on the minimum size of arrays passed to the workers that
         triggers automated memory mapping. Can be an int in Bytes,
         or a human-readable string, e.g., '1M' for 1 megabyte.
         Use None to disable memmaping of large arrays. Use 'auto' to
         use the value set using mne.set_memmap_min_size.
-    pre_dispatch : int, or string, optional
+    pre_dispatch : int, or str, optional
         See :class:`joblib.Parallel`.
     total : int | None
         If int, use a progress bar to display the progress of dispatched
@@ -54,11 +55,11 @@ def parallel_func(func, n_jobs, max_nbytes='auto', pre_dispatch='n_jobs',
     Returns
     -------
     parallel: instance of joblib.Parallel or list
-        The parallel object
+        The parallel object.
     my_func: callable
-        func if not parallel or delayed(func)
+        ``func`` if not parallel or delayed(func).
     n_jobs: int
-        Number of jobs >= 0
+        Number of jobs >= 0.
     """
     should_print = (logger.level <= logging.INFO)
     # for a single job, we don't need joblib
@@ -114,8 +115,7 @@ def parallel_func(func, n_jobs, max_nbytes='auto', pre_dispatch='n_jobs',
 
     if total is not None:
         def parallel_progress(op_iter):
-            pb = ProgressBar(total, verbose_bool=should_print)
-            return parallel(pb(op_iter))
+            return parallel(ProgressBar(iterable=op_iter, max_value=total))
         parallel_out = parallel_progress
     else:
         parallel_out = parallel
@@ -152,9 +152,9 @@ def check_n_jobs(n_jobs, allow_cuda=False):
     -------
     n_jobs : int
         The checked number of jobs. Always positive (or 'cuda' if
-        applicable.)
+        applicable).
     """
-    if not isinstance(n_jobs, int):
+    if not isinstance(n_jobs, int_like):
         if not allow_cuda:
             raise ValueError('n_jobs must be an integer')
         elif not isinstance(n_jobs, str) or n_jobs != 'cuda':
