@@ -2,29 +2,38 @@
 #
 # License: BSD (3-clause)
 
+from contextlib import contextmanager
 import os.path as op
-import numpy as np
+import pathlib
+import re
 
+import numpy as np
 from numpy.testing import assert_allclose, assert_array_equal
 import pytest
 from scipy import sparse
 from scipy.special import sph_harm
 
 import mne
-from mne import compute_raw_covariance, pick_types, concatenate_raws
+from mne import compute_raw_covariance, pick_types, concatenate_raws, pick_info
 from mne.annotations import _annotations_starts_stops
 from mne.chpi import read_head_pos, filter_chpi
 from mne.forward import _prep_meg_channels
 from mne.datasets import testing
 from mne.forward import use_coil_def
-from mne.io import read_raw_fif, read_info, read_raw_bti, read_raw_kit, BaseRaw
+from mne.io import (read_raw_fif, read_info, read_raw_bti, read_raw_kit,
+                    BaseRaw, read_raw_ctf)
+from mne.io.constants import FIFF
 from mne.preprocessing.maxwell import (
     maxwell_filter, _get_n_moments, _sss_basis_basic, _sh_complex_to_real,
     _sh_real_to_complex, _sh_negate, _bases_complex_to_real, _trans_sss_basis,
-    _bases_real_to_complex, _prep_mf_coils)
-from mne.rank import _get_rank_sss, _compute_rank_int
-from mne.utils import (assert_meg_snr, run_tests_if_main, catch_logging,
-                       requires_version, object_diff, buggy_mkl_svd)
+    _bases_real_to_complex, _prep_mf_coils, find_bad_channels_maxwell,
+    compute_maxwell_basis)
+from mne.rank import _get_rank_sss, _compute_rank_int, compute_rank
+from mne.utils import (assert_meg_snr, catch_logging,
+                       object_diff, buggy_mkl_svd, use_log_level)
+
+io_path = op.join(op.dirname(__file__), '..', '..', 'io', 'tests', 'data')
+raw_small_fname = op.join(io_path, 'test_raw.fif')
 
 data_path = testing.data_path(download=False)
 sss_path = op.join(data_path, 'SSS')
@@ -57,7 +66,8 @@ sss_erm_st1FineCalCrossTalk_fname = pre + 'erm_st1FineCalCrossTalk_raw_sss.fif'
 sss_erm_st1FineCalCrossTalkRegIn_fname = \
     pre + 'erm_st1FineCalCrossTalkRegIn_raw_sss.fif'
 
-sample_fname = op.join(data_path, 'MEG', 'sample_audvis_trunc_raw.fif')
+sample_fname = op.join(
+    data_path, 'MEG', 'sample', 'sample_audvis_trunc_raw.fif')
 sss_samp_reg_in_fname = op.join(data_path, 'SSS',
                                 'sample_audvis_trunc_regIn_raw_sss.fif')
 sss_samp_fname = op.join(data_path, 'SSS', 'sample_audvis_trunc_raw_sss.fif')
@@ -71,9 +81,6 @@ ctc_fname = op.join(sss_path, 'ct_sparse.fif')
 fine_cal_mgh_fname = op.join(sss_path, 'sss_cal_mgh.dat')
 ctc_mgh_fname = op.join(sss_path, 'ct_sparse_mgh.fif')
 
-sample_fname = op.join(data_path, 'MEG', 'sample',
-                       'sample_audvis_trunc_raw.fif')
-
 triux_path = op.join(data_path, 'SSS', 'TRIUX')
 tri_fname = op.join(triux_path, 'triux_bmlhus_erm_raw.fif')
 tri_sss_fname = op.join(triux_path, 'triux_bmlhus_erm_raw_sss.fif')
@@ -90,6 +97,7 @@ tri_cal_fname = op.join(triux_path, 'sss_cal_BMLHUS.dat')
 
 io_dir = op.join(op.dirname(__file__), '..', '..', 'io')
 fname_ctf_raw = op.join(io_dir, 'tests', 'data', 'test_ctf_comp_raw.fif')
+ctf_fname_continuous = op.join(data_path, 'CTF', 'testdata_ctf.ds')
 
 # In some of the tests, use identical coil defs to what is used in
 # MaxFilter
@@ -100,9 +108,6 @@ int_order, ext_order = 8, 3
 mf_head_origin = (0., 0., 0.04)
 mf_meg_origin = (0., 0.013, -0.006)
 
-# otherwise we can get SVD error
-requires_svd_convergence = requires_version('scipy', '0.12')
-
 # 30 random bad MEG channels (20 grad, 10 mag) that were used in generation
 bads = ['MEG0912', 'MEG1722', 'MEG2213', 'MEG0132', 'MEG1312', 'MEG0432',
         'MEG2433', 'MEG1022', 'MEG0442', 'MEG2332', 'MEG0633', 'MEG1043',
@@ -119,6 +124,13 @@ def _assert_n_free(raw_sss, lower, upper=None):
         'nfree fail: %s <= %s <= %s' % (lower, n_free, upper)
 
 
+def _assert_mag_coil_type(info, coil_type):
+    __tracebackhide__ = True
+    picks = pick_types(info, meg='mag', exclude=())
+    coil_types = set(info['chs'][pick]['coil_type'] for pick in picks)
+    assert coil_types == {coil_type}
+
+
 def read_crop(fname, lims=(0, None)):
     """Read and crop."""
     return read_raw_fif(fname, allow_maxshield='yes').crop(*lims)
@@ -136,8 +148,12 @@ def test_movement_compensation(tmpdir):
     #
     # Movement compensation, no regularization, no tSSS
     #
+    _assert_mag_coil_type(raw.info, FIFF.FIFFV_COIL_VV_MAG_T3)
+    assert_allclose(raw.info['chs'][2]['cal'], 4.14e-11, rtol=1e-6)
+    raw.info['chs'][2]['coil_type'] = FIFF.FIFFV_COIL_VV_MAG_T2
     raw_sss = maxwell_filter(raw, head_pos=head_pos, origin=mf_head_origin,
                              regularize=None, bad_condition='ignore')
+    _assert_mag_coil_type(raw_sss.info, FIFF.FIFFV_COIL_VV_MAG_T3)
     assert_meg_snr(raw_sss, read_crop(sss_movecomp_fname, lims),
                    4.6, 12.4, chpi_med_tol=58)
     # IO
@@ -157,7 +173,7 @@ def test_movement_compensation(tmpdir):
     #
     # Movement compensation,    regularization,    tSSS at the end
     #
-    raw_nohpi = filter_chpi(raw.copy())
+    raw_nohpi = filter_chpi(raw.copy(), t_window=0.2)
     with pytest.warns(RuntimeWarning, match='untested'):
         raw_sss_mv = maxwell_filter(raw_nohpi, head_pos=head_pos,
                                     st_duration=4., origin=mf_head_origin,
@@ -182,15 +198,19 @@ def test_movement_compensation(tmpdir):
 
     # some degenerate cases
     raw_erm = read_crop(erm_fname)
-    pytest.raises(ValueError, maxwell_filter, raw_erm, coord_frame='meg',
-                  head_pos=head_pos)  # can't do ERM file
-    pytest.raises(ValueError, maxwell_filter, raw,
-                  head_pos=head_pos[:, :9])  # bad shape
-    pytest.raises(TypeError, maxwell_filter, raw, head_pos='foo')  # bad type
-    pytest.raises(ValueError, maxwell_filter, raw, head_pos=head_pos[::-1])
+    with pytest.raises(ValueError, match='positions can only be used'):
+        maxwell_filter(raw_erm, coord_frame='meg',
+                       head_pos=head_pos)
+    with pytest.raises(ValueError, match=r'of shape \(N, 10\)'):
+        maxwell_filter(raw, head_pos=head_pos[:, :9])
+    with pytest.raises(TypeError, match='instance of ndarray'):
+        maxwell_filter(raw, head_pos='foo')
+    with pytest.raises(ValueError, match='ascending'):
+        maxwell_filter(raw, head_pos=head_pos[::-1])
     head_pos_bad = head_pos.copy()
     head_pos_bad[0, 0] = raw._first_time - 1e-2
-    pytest.raises(ValueError, maxwell_filter, raw, head_pos=head_pos_bad)
+    with pytest.raises(ValueError, match='greater than'):
+        maxwell_filter(raw, head_pos=head_pos_bad)
 
     head_pos_bad = head_pos.copy()
     head_pos_bad[0, 4] = 1.  # off by more than 1 m
@@ -276,9 +296,11 @@ def test_other_systems():
     # CTF
     raw_ctf = read_crop(fname_ctf_raw)
     assert raw_ctf.compensation_grade == 3
-    pytest.raises(RuntimeError, maxwell_filter, raw_ctf)  # compensated
+    with pytest.raises(RuntimeError, match='compensated'):
+        maxwell_filter(raw_ctf)
     raw_ctf.apply_gradient_compensation(0)
-    pytest.raises(ValueError, maxwell_filter, raw_ctf)  # cannot fit headshape
+    with pytest.raises(ValueError, match='digitization points'):
+        maxwell_filter(raw_ctf)
     raw_sss = maxwell_filter(raw_ctf, origin=(0., 0., 0.04))
     _assert_n_free(raw_sss, 68)
     _assert_shielding(raw_sss, raw_ctf, 1.8)
@@ -383,9 +405,12 @@ def test_basic():
     raw = read_crop(raw_fname, (0., 1.))
     raw_err = read_crop(raw_fname).apply_proj()
     raw_erm = read_crop(erm_fname)
-    pytest.raises(RuntimeError, maxwell_filter, raw_err)
-    pytest.raises(TypeError, maxwell_filter, 1.)  # not a raw
-    pytest.raises(ValueError, maxwell_filter, raw, int_order=20)  # too many
+    with pytest.raises(RuntimeError, match='cannot be applied'):
+        maxwell_filter(raw_err)
+    with pytest.raises(TypeError, match='instance of BaseRaw'):
+        maxwell_filter(1.)
+    with pytest.raises(ValueError, match='Number of requested bases'):
+        maxwell_filter(raw, int_order=20)  # too many
 
     n_int_bases = int_order ** 2 + 2 * int_order
     n_ext_bases = ext_order ** 2 + 2 * ext_order
@@ -408,7 +433,8 @@ def test_basic():
     assert len(py_ctc) == 0
     py_st = raw_sss.info['proc_history'][0]['max_info']['max_st']
     assert len(py_st) == 0
-    pytest.raises(RuntimeError, maxwell_filter, raw_sss)
+    with pytest.raises(RuntimeError, match='cannot reapply'):
+        maxwell_filter(raw_sss)
 
     # Test SSS computation at non-standard head origin
     with use_coil_def(elekta_def_fname):
@@ -439,10 +465,14 @@ def test_basic():
     assert _get_n_moments(int_order) == _get_rank_sss(raw_sss)
 
     # Degenerate cases
-    pytest.raises(ValueError, maxwell_filter, raw, coord_frame='foo')
-    pytest.raises(ValueError, maxwell_filter, raw, origin='foo')
-    pytest.raises(ValueError, maxwell_filter, raw, origin=[0] * 4)
-    pytest.raises(ValueError, maxwell_filter, raw, mag_scale='foo')
+    with pytest.raises(ValueError, match='Invalid value'):
+        maxwell_filter(raw, coord_frame='foo')
+    with pytest.raises(ValueError, match='numerical array'):
+        maxwell_filter(raw, origin='foo')
+    with pytest.raises(ValueError, match='3-element array'):
+        maxwell_filter(raw, origin=[0] * 4)
+    with pytest.raises(ValueError, match='must be a float'):
+        maxwell_filter(raw, mag_scale='foo')
     raw_missing = raw.copy().load_data()
     raw_missing.info['bads'] = ['MEG0111']
     raw_missing.pick_types(meg=True)  # will be missing the bad
@@ -510,7 +540,6 @@ def test_bads_reconstruction():
 
 
 @buggy_mkl_svd
-@requires_svd_convergence
 @testing.requires_testing_data
 def test_spatiotemporal():
     """Test Maxwell filter (tSSS) spatiotemporal processing."""
@@ -518,7 +547,7 @@ def test_spatiotemporal():
     raw = read_crop(raw_fname)
 
     # Test that window is less than length of data
-    with pytest.raises(ValueError, match='duration'):
+    with pytest.raises(ValueError, match='must be'):
         maxwell_filter(raw, st_duration=1000.)
 
     # We could check both 4 and 10 seconds because Elekta handles them
@@ -552,12 +581,11 @@ def test_spatiotemporal():
         assert py_st['subspcorr'] == 0.98
 
     # Degenerate cases
-    pytest.raises(ValueError, maxwell_filter, raw, st_duration=10.,
-                  st_correlation=0.)
+    with pytest.raises(ValueError, match='Need 0 < st_correlation'):
+        maxwell_filter(raw, st_duration=10., st_correlation=0.)
 
 
 @pytest.mark.slowtest
-@requires_svd_convergence
 @testing.requires_testing_data
 def test_spatiotemporal_only():
     """Test tSSS-only processing."""
@@ -619,9 +647,13 @@ def test_fine_calibration():
 
     # Test 1D SSS fine calibration
     with use_coil_def(elekta_def_fname):
-        raw_sss = maxwell_filter(raw, calibration=fine_cal_fname,
-                                 origin=mf_head_origin, regularize=None,
-                                 bad_condition='ignore')
+        with catch_logging() as log:
+            raw_sss = maxwell_filter(raw, calibration=fine_cal_fname,
+                                     origin=mf_head_origin, regularize=None,
+                                     bad_condition='ignore', verbose=True)
+    log = log.getvalue()
+    assert 'Using fine calibration' in log
+    assert op.basename(fine_cal_fname) in log
     assert_meg_snr(raw_sss, sss_fine_cal, 82, 611)
     py_cal = raw_sss.info['proc_history'][0]['max_info']['sss_cal']
     assert (py_cal is not None)
@@ -639,7 +671,7 @@ def test_fine_calibration():
     raw_sss_bad = maxwell_filter(
         raw_missing, calibration=fine_cal_fname, origin=mf_head_origin,
         regularize=None, bad_condition='ignore')
-    raw_missing.pick_types()  # actually remove bads
+    raw_missing.pick_types(meg=True)  # actually remove bads
     raw_sss_bad.pick_channels(raw_missing.ch_names)  # remove them here, too
     with pytest.warns(RuntimeWarning, match='cal channels not in data'):
         raw_sss_missing = maxwell_filter(
@@ -708,13 +740,14 @@ def test_cross_talk(tmpdir):
     raw.info['bads'] = bads
     sss_ctc = read_crop(sss_ctc_fname)
     with use_coil_def(elekta_def_fname):
-        raw_sss = maxwell_filter(raw, cross_talk=ctc_fname,
+        raw_sss = maxwell_filter(raw, cross_talk=pathlib.Path(ctc_fname),
                                  origin=mf_head_origin, regularize=None,
                                  bad_condition='ignore')
     assert_meg_snr(raw_sss, sss_ctc, 275.)
     py_ctc = raw_sss.info['proc_history'][0]['max_info']['sss_ctc']
     assert (len(py_ctc) > 0)
-    pytest.raises(ValueError, maxwell_filter, raw, cross_talk=raw)
+    with pytest.raises(TypeError, match='path-like'):
+        maxwell_filter(raw, cross_talk=raw)
     pytest.raises(ValueError, maxwell_filter, raw, cross_talk=raw_fname)
     mf_ctc = sss_ctc.info['proc_history'][0]['max_info']['sss_ctc']
     del mf_ctc['block_id']  # we don't write this
@@ -733,7 +766,6 @@ def test_cross_talk(tmpdir):
                        mf_ctc['decoupler'].toarray())
     assert object_diff(py_ctc, mf_ctc) == ''
     raw_ctf = read_crop(fname_ctf_raw).apply_gradient_compensation(0)
-    pytest.raises(ValueError, maxwell_filter, raw_ctf)  # cannot fit headshape
     raw_sss = maxwell_filter(raw_ctf, origin=(0., 0., 0.04))
     _assert_n_free(raw_sss, 68)
     raw_sss = maxwell_filter(raw_ctf, origin=(0., 0., 0.04), ignore_ref=True)
@@ -786,8 +818,10 @@ def test_head_translation():
 # that calculates the localization error:
 # http://ieeexplore.ieee.org/xpl/articleDetails.jsp?arnumber=1495874
 
-def _assert_shielding(raw_sss, erm_power, shielding_factor, meg='mag'):
+def _assert_shielding(raw_sss, erm_power, min_factor, max_factor=np.inf,
+                      meg='mag'):
     """Assert a minimum shielding factor using empty-room power."""
+    __tracebackhide__ = True
     picks = pick_types(raw_sss.info, meg=meg, ref_meg=False)
     if isinstance(erm_power, BaseRaw):
         picks_erm = pick_types(raw_sss.info, meg=meg, ref_meg=False)
@@ -796,13 +830,77 @@ def _assert_shielding(raw_sss, erm_power, shielding_factor, meg='mag'):
     sss_power = raw_sss[picks][0].ravel()
     sss_power = np.sqrt(np.sum(sss_power * sss_power))
     factor = erm_power / sss_power
-    assert factor >= shielding_factor, \
-        'Shielding factor %0.3f < %0.3f' % (factor, shielding_factor)
+    assert min_factor <= factor < max_factor, (
+        'Shielding factor not %0.3f <= %0.3f < %0.3f'
+        % (min_factor, factor, max_factor))
+
+
+@buggy_mkl_svd
+@testing.requires_testing_data
+@pytest.mark.parametrize('regularize', ('in', None))
+@pytest.mark.parametrize('bads', ([], ['MEG0111']))
+def test_esss(regularize, bads):
+    """Test extended-basis SSS."""
+    # Make some fake "projectors" that actually contain external SSS bases
+    raw_erm = read_crop(erm_fname).load_data().pick_types(meg=True)
+    raw_erm.info['bads'] = bads
+    proj_sss = mne.compute_proj_raw(raw_erm, meg='combined', verbose='error',
+                                    n_mag=15, n_grad=15)
+    good_info = pick_info(raw_erm.info, pick_types(raw_erm.info, meg=True))
+    S_tot = _trans_sss_basis(
+        dict(int_order=0, ext_order=3, origin=(0., 0., 0.)),
+        all_coils=_prep_mf_coils(good_info), coil_scale=1., trans=None)
+    assert S_tot.shape[-1] == len(proj_sss)
+    for a, b in zip(proj_sss, S_tot.T):
+        a['data']['data'][:] = b
+    with catch_logging() as log:
+        raw_sss = maxwell_filter(raw_erm, coord_frame='meg',
+                                 regularize=regularize, verbose=True)
+    log = log.getvalue()
+    assert 'xtend' not in log
+    with catch_logging() as log:
+        raw_sss_2 = maxwell_filter(raw_erm, coord_frame='meg',
+                                   regularize=regularize, ext_order=0,
+                                   extended_proj=proj_sss, verbose=True)
+    log = log.getvalue()
+    assert 'Extending external SSS basis using 15 projection' in log
+    assert_allclose(raw_sss_2._data, raw_sss._data, atol=1e-20)
+
+    # This should work, as the projectors should be a superset
+    raw_erm.info['bads'] = raw_erm.info['bads'] + ['MEG0112']
+    maxwell_filter(raw_erm, coord_frame='meg', extended_proj=proj_sss)
+
+    # Degenerate condititons
+    proj_sss = proj_sss[:2]
+    proj_sss[0]['data']['col_names'] = proj_sss[0]['data']['col_names'][:-1]
+    with pytest.raises(ValueError, match='were missing'):
+        maxwell_filter(raw_erm, coord_frame='meg', extended_proj=proj_sss)
+    proj_sss[0] = 1.
+    with pytest.raises(TypeError, match=r'extended_proj\[0\] must be an inst'):
+        maxwell_filter(raw_erm, coord_frame='meg', extended_proj=proj_sss)
+    with pytest.raises(TypeError, match='extended_proj must be an inst'):
+        maxwell_filter(raw_erm, coord_frame='meg', extended_proj=1.)
+
+
+@contextmanager
+def get_n_projected():
+    """Get the number of projected tSSS components from the log."""
+    count = list()
+    with use_log_level(True):
+        with catch_logging() as log:
+            yield count
+    log = log.getvalue()
+    assert 'Processing data using tSSS' in log
+    log = log.splitlines()
+    reg = re.compile(r'\s+Projecting\s+([0-9])+\s+intersecting tSSS .*')
+    for line in log:
+        m = reg.match(line)
+        if m:
+            count.append(int(m.group(1)))
 
 
 @buggy_mkl_svd
 @pytest.mark.slowtest
-@requires_svd_convergence
 @testing.requires_testing_data
 def test_shielding_factor(tmpdir):
     """Test Maxwell filter shielding factor using empty room."""
@@ -815,95 +913,176 @@ def test_shielding_factor(tmpdir):
     # Vanilla SSS (second value would be for meg=True instead of meg='mag')
     _assert_shielding(read_crop(sss_erm_std_fname), erm_power, 10)  # 1.5)
     raw_sss = maxwell_filter(raw_erm, coord_frame='meg', regularize=None)
-    _assert_shielding(raw_sss, erm_power, 12)  # 1.5)
-    _assert_shielding(raw_sss, erm_power_grad, 0.45, 'grad')  # 1.5)
+    _assert_shielding(raw_sss, erm_power, 12, 13)  # 1.5)
+    _assert_shielding(raw_sss, erm_power_grad, 0.45, 0.55, 'grad')  # 1.5)
 
-    # Using different mag_scale values
+    # No external basis
+    raw_sss_0 = maxwell_filter(raw_erm, coord_frame='meg', regularize=None,
+                               ext_order=0)
+    _assert_shielding(raw_sss_0, erm_power, 1.0, 1.1)
+    del raw_sss_0
+
+    # Regularization
+    _assert_shielding(read_crop(sss_erm_std_fname), erm_power, 10)  # 1.5)
+    raw_sss = maxwell_filter(raw_erm, coord_frame='meg')
+    _assert_shielding(raw_sss, erm_power, 14.5, 15.5)
+
+    #
+    # Extended (eSSS)
+    #
+
+    # Show that using empty-room projectors increase shielding factor
+    proj = mne.compute_proj_raw(raw_erm, meg='combined', verbose='error',
+                                n_mag=15, n_grad=15)
+    raw_sss = maxwell_filter(raw_erm, coord_frame='meg', regularize=None,
+                             extended_proj=proj[:3])
+    _assert_shielding(raw_sss, erm_power, 38, 39)
+    raw_sss = maxwell_filter(raw_erm, coord_frame='meg', regularize=None,
+                             extended_proj=proj)
+    _assert_shielding(raw_sss, erm_power, 49, 51)
+    # Now with reg
+    raw_sss = maxwell_filter(raw_erm, coord_frame='meg',
+                             extended_proj=proj[:3])
+    _assert_shielding(raw_sss, erm_power, 42, 44)
+    raw_sss = maxwell_filter(raw_erm, coord_frame='meg',
+                             extended_proj=proj)
+    _assert_shielding(raw_sss, erm_power, 59, 61)
+
+    #
+    # Different mag_scale values
+    #
     raw_sss = maxwell_filter(raw_erm, coord_frame='meg', regularize=None,
                              mag_scale='auto')
-    _assert_shielding(raw_sss, erm_power, 12)
-    _assert_shielding(raw_sss, erm_power_grad, 0.48, 'grad')
+    _assert_shielding(raw_sss, erm_power, 12, 13)
+    _assert_shielding(raw_sss, erm_power_grad, 0.48, 0.58, 'grad')
     raw_sss = maxwell_filter(raw_erm, coord_frame='meg', regularize=None,
                              mag_scale=1.)  # not a good choice
-    _assert_shielding(raw_sss, erm_power, 7.3)
-    _assert_shielding(raw_sss, erm_power_grad, 0.2, 'grad')
+    _assert_shielding(raw_sss, erm_power, 7.3, 8.)
+    _assert_shielding(raw_sss, erm_power_grad, 0.2, 0.3, 'grad')
     raw_sss = maxwell_filter(raw_erm, coord_frame='meg', regularize=None,
                              mag_scale=1000., bad_condition='ignore')
-    _assert_shielding(raw_sss, erm_power, 4.0)
-    _assert_shielding(raw_sss, erm_power_grad, 0.1, 'grad')
+    _assert_shielding(raw_sss, erm_power, 4.0, 5.0)
+    _assert_shielding(raw_sss, erm_power_grad, 0.1, 0.2, 'grad')
 
+    #
     # Fine cal
+    #
     _assert_shielding(read_crop(sss_erm_fine_cal_fname), erm_power, 12)  # 2.0)
     raw_sss = maxwell_filter(raw_erm, coord_frame='meg', regularize=None,
                              origin=mf_meg_origin,
-                             calibration=fine_cal_fname)
-    _assert_shielding(raw_sss, erm_power, 12)  # 2.0)
+                             calibration=pathlib.Path(fine_cal_fname))
+    _assert_shielding(raw_sss, erm_power, 12, 13)  # 2.0)
 
+    #
     # Crosstalk
+    #
     _assert_shielding(read_crop(sss_erm_ctc_fname), erm_power, 12)  # 2.1)
     raw_sss = maxwell_filter(raw_erm, coord_frame='meg', regularize=None,
                              origin=mf_meg_origin,
                              cross_talk=ctc_fname)
-    _assert_shielding(raw_sss, erm_power, 12)  # 2.1)
+    _assert_shielding(raw_sss, erm_power, 12, 13)  # 2.1)
 
     # Fine cal + Crosstalk
     raw_sss = maxwell_filter(raw_erm, coord_frame='meg', regularize=None,
                              calibration=fine_cal_fname,
                              origin=mf_meg_origin,
                              cross_talk=ctc_fname)
-    _assert_shielding(raw_sss, erm_power, 13)  # 2.2)
+    _assert_shielding(raw_sss, erm_power, 13, 14)  # 2.2)
+    # Fine cal + Crosstalk + Extended
+    raw_sss = maxwell_filter(raw_erm, coord_frame='meg', regularize=None,
+                             calibration=fine_cal_fname,
+                             origin=mf_meg_origin,
+                             cross_talk=ctc_fname, extended_proj=proj)
+    _assert_shielding(raw_sss, erm_power, 28, 30)
+    raw_sss = maxwell_filter(raw_erm, coord_frame='meg', regularize=None,
+                             calibration=fine_cal_fname,
+                             origin=mf_meg_origin,
+                             cross_talk=ctc_fname, extended_proj=proj[:3])
+    _assert_shielding(raw_sss, erm_power, 25, 27)
 
     # tSSS
     _assert_shielding(read_crop(sss_erm_st_fname), erm_power, 37)  # 5.8)
     raw_sss = maxwell_filter(raw_erm, coord_frame='meg', regularize=None,
                              origin=mf_meg_origin, st_duration=1.)
-    _assert_shielding(raw_sss, erm_power, 37)  # 5.8)
+    _assert_shielding(raw_sss, erm_power, 37, 38)  # 5.8)
 
     # Crosstalk + tSSS
-    raw_sss = maxwell_filter(raw_erm, coord_frame='meg', regularize=None,
-                             cross_talk=ctc_fname, origin=mf_meg_origin,
-                             st_duration=1.)
-    _assert_shielding(raw_sss, erm_power, 38)  # 5.91)
+    with get_n_projected() as counts:
+        raw_sss = maxwell_filter(raw_erm, coord_frame='meg', regularize=None,
+                                 cross_talk=ctc_fname, origin=mf_meg_origin,
+                                 st_duration=1.)
+    _assert_shielding(raw_sss, erm_power, 38, 39)  # 5.91)
+    assert counts[0] == 4
 
     # Fine cal + tSSS
-    raw_sss = maxwell_filter(raw_erm, coord_frame='meg', regularize=None,
-                             calibration=fine_cal_fname,
-                             origin=mf_meg_origin, st_duration=1.)
-    _assert_shielding(raw_sss, erm_power, 38)  # 5.98)
+    with get_n_projected() as counts:
+        raw_sss = maxwell_filter(raw_erm, coord_frame='meg', regularize=None,
+                                 calibration=fine_cal_fname,
+                                 origin=mf_meg_origin, st_duration=1.)
+    _assert_shielding(raw_sss, erm_power, 38, 39)  # 5.98)
+    assert counts[0] == 4
+
+    # Extended + tSSS
+    with get_n_projected() as counts:
+        raw_sss = maxwell_filter(raw_erm, coord_frame='meg', regularize=None,
+                                 origin=mf_meg_origin, st_duration=1.,
+                                 extended_proj=proj)
+    _assert_shielding(raw_sss, erm_power, 40, 42)
+    assert counts[0] == 0
+    with get_n_projected() as counts:
+        raw_sss = maxwell_filter(raw_erm, coord_frame='meg', regularize=None,
+                                 origin=mf_meg_origin, st_duration=1.,
+                                 extended_proj=proj[:3])
+    _assert_shielding(raw_sss, erm_power, 35, 37)
+    assert counts[0] == 1
 
     # Fine cal + Crosstalk + tSSS
     _assert_shielding(read_crop(sss_erm_st1FineCalCrossTalk_fname),
-                      erm_power, 39)  # 6.07)
+                      erm_power, 39, 40)  # 6.07)
     raw_sss = maxwell_filter(raw_erm, coord_frame='meg', regularize=None,
                              calibration=fine_cal_fname, origin=mf_meg_origin,
                              cross_talk=ctc_fname, st_duration=1.)
-    _assert_shielding(raw_sss, erm_power, 39)  # 6.05)
+    _assert_shielding(raw_sss, erm_power, 39, 40)  # 6.05)
+
+    # Fine cal + Crosstalk + tSSS + Extended (a bit worse)
+    _assert_shielding(read_crop(sss_erm_st1FineCalCrossTalk_fname),
+                      erm_power, 39, 40)  # 6.07)
+    raw_sss = maxwell_filter(raw_erm, coord_frame='meg', regularize=None,
+                             calibration=fine_cal_fname, origin=mf_meg_origin,
+                             cross_talk=ctc_fname, st_duration=1.,
+                             extended_proj=proj[:3])
+    _assert_shielding(raw_sss, erm_power, 34, 36)
 
     # Fine cal + Crosstalk + tSSS + Reg-in
     _assert_shielding(read_crop(sss_erm_st1FineCalCrossTalkRegIn_fname),
-                      erm_power, 57)  # 6.97)
+                      erm_power, 57, 58)  # 6.97)
     raw_sss = maxwell_filter(raw_erm, calibration=fine_cal_fname,
                              cross_talk=ctc_fname, st_duration=1.,
                              origin=mf_meg_origin,
                              coord_frame='meg', regularize='in')
-    _assert_shielding(raw_sss, erm_power, 53)  # 6.64)
-    raw_sss = maxwell_filter(raw_erm, calibration=fine_cal_fname,
-                             cross_talk=ctc_fname, st_duration=1.,
-                             coord_frame='meg', regularize='in')
-    _assert_shielding(raw_sss, erm_power, 58)  # 7.0)
-    _assert_shielding(raw_sss, erm_power_grad, 1.6, 'grad')
-    raw_sss = maxwell_filter(raw_erm, calibration=fine_cal_fname,
-                             cross_talk=ctc_fname, st_duration=1.,
-                             coord_frame='meg', regularize='in',
-                             mag_scale='auto')
-    _assert_shielding(raw_sss, erm_power, 51)
-    _assert_shielding(raw_sss, erm_power_grad, 1.5, 'grad')
-    raw_sss = maxwell_filter(raw_erm, calibration=fine_cal_fname_3d,
-                             cross_talk=ctc_fname, st_duration=1.,
-                             coord_frame='meg', regularize='in')
-
+    _assert_shielding(raw_sss, erm_power, 53, 54)  # 6.64)
+    with get_n_projected() as counts:
+        raw_sss = maxwell_filter(raw_erm, calibration=fine_cal_fname,
+                                 cross_talk=ctc_fname, st_duration=1.,
+                                 coord_frame='meg', regularize='in')
+    _assert_shielding(raw_sss, erm_power, 58, 59)  # 7.0)
+    _assert_shielding(raw_sss, erm_power_grad, 1.6, 1.7, 'grad')
+    assert counts[0] == 4
+    with get_n_projected() as counts:
+        raw_sss = maxwell_filter(raw_erm, calibration=fine_cal_fname,
+                                 cross_talk=ctc_fname, st_duration=1.,
+                                 coord_frame='meg', regularize='in',
+                                 mag_scale='auto')
+    _assert_shielding(raw_sss, erm_power, 51, 52)
+    _assert_shielding(raw_sss, erm_power_grad, 1.5, 1.6, 'grad')
+    assert counts[0] == 3
+    with get_n_projected() as counts:
+        raw_sss = maxwell_filter(raw_erm, calibration=fine_cal_fname_3d,
+                                 cross_talk=ctc_fname, st_duration=1.,
+                                 coord_frame='meg', regularize='in')
     # Our 3D cal has worse defaults for this ERM than the 1D file
-    _assert_shielding(raw_sss, erm_power, 54)
+    _assert_shielding(raw_sss, erm_power, 57, 58)
+    assert counts[0] == 3
     # Show it by rewriting the 3D as 1D and testing it
     temp_dir = str(tmpdir)
     temp_fname = op.join(temp_dir, 'test_cal.dat')
@@ -911,15 +1090,26 @@ def test_shielding_factor(tmpdir):
         with open(temp_fname, 'w') as fid_out:
             for line in fid:
                 fid_out.write(' '.join(line.strip().split(' ')[:14]) + '\n')
-    raw_sss = maxwell_filter(raw_erm, calibration=temp_fname,
-                             cross_talk=ctc_fname, st_duration=1.,
-                             coord_frame='meg', regularize='in')
+    with get_n_projected() as counts:
+        with pytest.warns(None):  # SVD convergence sometimes
+            raw_sss = maxwell_filter(raw_erm, calibration=temp_fname,
+                                     cross_talk=ctc_fname, st_duration=1.,
+                                     coord_frame='meg', regularize='in')
     # Our 3D cal has worse defaults for this ERM than the 1D file
-    _assert_shielding(raw_sss, erm_power, 44)
+    _assert_shielding(raw_sss, erm_power, 44, 45)
+    assert counts[0] == 3
+
+    # Fine cal + Crosstalk + tSSS + Reg-in + Extended
+    with get_n_projected() as counts:
+        raw_sss = maxwell_filter(raw_erm, calibration=fine_cal_fname,
+                                 cross_talk=ctc_fname, st_duration=1.,
+                                 coord_frame='meg', regularize='in',
+                                 extended_proj=proj[:3])
+    _assert_shielding(raw_sss, erm_power, 48, 50)
+    assert counts[0] == 1
 
 
 @pytest.mark.slowtest
-@requires_svd_convergence
 @testing.requires_testing_data
 def test_all():
     """Test maxwell filter using all options."""
@@ -954,15 +1144,16 @@ def test_all():
 
 
 @pytest.mark.slowtest
-@requires_svd_convergence
 @testing.requires_testing_data
 def test_triux():
     """Test TRIUX system support."""
     raw = read_crop(tri_fname, (0, 0.999))
-    raw.fix_mag_coil_types()
+    _assert_mag_coil_type(raw.info, FIFF.FIFFV_COIL_VV_MAG_T1)
+    assert_allclose(raw.info['chs'][2]['cal'], 1.33e-10, rtol=1e-6)
     # standard
     with use_coil_def(elekta_def_fname):
         sss_py = maxwell_filter(raw, coord_frame='meg', regularize=None)
+    _assert_mag_coil_type(sss_py.info, FIFF.FIFFV_COIL_VV_MAG_T3)
     assert_meg_snr(sss_py, read_crop(tri_sss_fname), 37, 700)
     # cross-talk
     sss_py = maxwell_filter(raw, coord_frame='meg', regularize=None,
@@ -1021,7 +1212,7 @@ def test_mf_skips():
         # skips decrease acceptable duration
         maxwell_filter(raw, st_duration=17., **kwargs)
     onsets, ends = _annotations_starts_stops(
-        raw, ('edge', 'bad_acq_skip'), 'skip_by_annotation', invert=True)
+        raw, ('edge', 'bad_acq_skip'), invert=True)
     assert (ends - onsets).min() / raw.info['sfreq'] == 2.
     assert (ends - onsets).max() / raw.info['sfreq'] == 3.
     for st_duration in (2., 3.):
@@ -1053,4 +1244,152 @@ def test_mf_skips():
     assert_allclose(data_sc, data_cs, atol=1e-20)
 
 
-run_tests_if_main()
+@testing.requires_testing_data
+@pytest.mark.parametrize(
+    ('fname', 'bads', 'annot', 'add_ch', 'ignore_ref', 'want_bads',
+     'return_scores', 'h_freq'), [
+        # Neuromag data tested against MF
+        (sample_fname, [], False, False, False, ['MEG 2443'], False, None),
+        # add 0111 to test picking, add annot to test it, and prepend chs for
+        # idx
+        (sample_fname, ['MEG 0111'], True, True, False, ['MEG 2443'], False,
+         None),
+        # CTF data seems to be sensitive to linalg lib (?) because some
+        # channels are very close to the limit, so we just check that one shows
+        # up
+        (ctf_fname_continuous, [], False, False, False, {'BR1-4304'}, False,
+         None),
+        # faked
+        (ctf_fname_continuous, [], False, False, True, ['MLC24-4304'], False,
+         None),
+        # For `return_scores=True`
+        (sample_fname, ['MEG 0111'], True, True, False, ['MEG 2443'], True,
+         50)
+    ])
+def test_find_bad_channels_maxwell(fname, bads, annot, add_ch, ignore_ref,
+                                   want_bads, return_scores, h_freq):
+    """Test automatic bad channel detection."""
+    if fname.endswith('.ds'):
+        raw = read_raw_ctf(fname).load_data()
+        flat_idx = 33
+    else:
+        raw = read_raw_fif(fname)
+        raw.fix_mag_coil_types().load_data().pick_types(meg=True, exclude=())
+        flat_idx = 1
+    raw.filter(None, 40)
+    raw.info['bads'] = bads
+    raw._data[flat_idx] = 0  # MaxFilter didn't have this but doesn't affect it
+    want_flats = [raw.ch_names[flat_idx]]
+    raw.apply_gradient_compensation(0)
+
+    min_count = 5
+
+    if add_ch:
+        raw_eeg = read_raw_fif(fname)
+        raw_eeg.pick_types(meg=False, eeg=True, exclude=()).load_data()
+        raw_eeg.info['lowpass'] = 40.
+        raw = raw_eeg.add_channels([raw])  # prepend the EEG channels
+        assert 0 in pick_types(raw.info, meg=False, eeg=True)
+    if ignore_ref:
+        # Fake a bad one, otherwise we don't find any
+        assert 42 in pick_types(raw.info, meg=True, ref_meg=False)
+        assert raw.ch_names[42:43] == want_bads
+        raw._data[42] += np.random.RandomState(0).randn(len(raw.times))
+    # maxfilter -autobad on -v -f test_raw.fif -force -cal off -ctc off -regularize off -list -o test_raw.fif -f ~/mne_data/MNE-testing-data/MEG/sample/sample_audvis_trunc_raw.fif  # noqa: E501
+    if annot:
+        # do a problematic one (gh-7741): exactly one "step" unit
+        step = int(round(raw.info['sfreq'] * 5.))
+        dt = 1. / raw.info['sfreq']
+        assert step == 1502
+        raw.annotations.append(step * dt + raw._first_time, dt, 'BAD')
+    with catch_logging() as log:
+        return_vals = find_bad_channels_maxwell(
+            raw, origin=(0., 0., 0.04), regularize=None,
+            bad_condition='ignore', skip_by_annotation='BAD', verbose=True,
+            ignore_ref=ignore_ref, min_count=min_count,
+            return_scores=return_scores, h_freq=h_freq)
+
+    if return_scores:
+        assert len(return_vals) == 3
+        got_bads, got_flats, got_scores = return_vals
+    else:
+        assert len(return_vals) == 2
+        got_bads, got_flats = return_vals
+
+    if isinstance(want_bads, list):
+        assert got_bads == want_bads  # from MaxFilter
+    else:
+        assert want_bads.intersection(set(got_bads))
+    assert got_flats == want_flats
+    log = log.getvalue()
+    assert 'Interval   1:    0.00' in log
+    assert 'Interval   2:    5.00' in log
+
+    if h_freq is not None and h_freq > raw.info['lowpass']:
+        assert 'data has already been low-pass filtered' in log
+
+    if return_scores:
+        meg_chs = raw.copy().pick_types(meg=True, exclude=[]).ch_names
+        ch_types = raw.get_channel_types(meg_chs)
+
+        assert list(got_scores['ch_names']) == meg_chs
+        assert list(got_scores['ch_types']) == ch_types
+        # Check that time is monotonically increasing.
+        assert (np.diff(got_scores['bins'].flatten()) >= 0).all()
+
+        assert (got_scores['scores_flat'].shape ==
+                got_scores['scores_noisy'].shape ==
+                (len(meg_chs), len(got_scores['bins'])))
+
+        assert (got_scores['limits_flat'].shape ==
+                got_scores['limits_noisy'].shape ==
+                (len(meg_chs), 1))
+
+        # Check "flat" scores.
+        scores_flat = got_scores['scores_flat']
+        limits_flat = got_scores['limits_flat']
+        # Deal with NaN's in the scores (can't use np.less directly because of
+        # https://github.com/numpy/numpy/issues/17198)
+        scores_flat[np.isnan(scores_flat)] = np.inf
+        limits_flat[np.isnan(limits_flat)] = -np.inf
+        n_segments_below_limit = (scores_flat < limits_flat).sum(-1)
+        ch_idx = np.where(n_segments_below_limit >=
+                          min(min_count, len(got_scores['bins'])))
+        flats = set(got_scores['ch_names'][ch_idx])
+        assert flats == set(want_flats)
+
+        # Check "noisy" scores.
+        scores_noisy = got_scores['scores_noisy']
+        limits_noisy = got_scores['limits_noisy']
+        scores_noisy[np.isnan(scores_noisy)] = -np.inf
+        limits_noisy[np.isnan(limits_noisy)] = np.inf
+        n_segments_beyond_limit = (scores_noisy > limits_noisy).sum(-1)
+        ch_idx = np.where(n_segments_beyond_limit >=
+                          min(min_count, len(got_scores['bins'])))
+        bads = set(got_scores['ch_names'][ch_idx])
+        assert bads == set(want_bads)
+
+
+@pytest.mark.parametrize('regularize, n', [
+    (None, 80),
+    ('in', 71),
+])
+def test_compute_maxwell_basis(regularize, n):
+    """Test compute_maxwell_basis."""
+    raw = read_raw_fif(raw_small_fname).crop(0, 2)
+    assert raw.info['bads'] == []
+    raw.del_proj()
+    rank = compute_rank(raw)['meg']
+    assert rank == 306
+    raw.info['bads'] = ['MEG 2443']
+    kwargs = dict(regularize=regularize, verbose=True)
+    raw_sss = maxwell_filter(raw, **kwargs)
+    want = raw_sss.get_data('meg')
+    rank = compute_rank(raw_sss)['meg']
+    assert rank == n
+    S, pS, reg_moments, n_use_in = compute_maxwell_basis(raw.info, **kwargs)
+    assert n_use_in == n
+    assert n_use_in == len(reg_moments) - 15  # no externals removed
+    xform = S[:, :n_use_in] @ pS[:n_use_in]
+    got = xform @ raw.pick_types(meg=True, exclude='bads').get_data()
+    assert_allclose(got, want)
