@@ -5,16 +5,17 @@
 
 import numpy as np
 import os.path as op
-import sys
 
-from numpy.testing import assert_array_equal, assert_array_almost_equal
+from numpy.testing import (assert_array_equal, assert_array_almost_equal,
+                           assert_allclose)
 import pytest
+from scipy import linalg, stats
 
 from mne import (Epochs, read_events, pick_types, compute_raw_covariance,
                  create_info, EpochsArray)
+from mne.decoding import Vectorizer
 from mne.io import read_raw_fif
-from mne.utils import (requires_sklearn, run_tests_if_main, check_version,
-                       _get_numpy_libs)
+from mne.utils import requires_sklearn
 from mne.preprocessing.xdawn import Xdawn, _XdawnTransformer
 
 base_dir = op.join(op.dirname(__file__), '..', '..', 'io', 'tests', 'data')
@@ -41,7 +42,7 @@ def test_xdawn():
     # Init xdawn with good parameters
     Xdawn(n_components=2, correct_overlap='auto', signal_cov=None, reg=None)
     # Init xdawn with bad parameters
-    pytest.raises(ValueError, Xdawn, correct_overlap=42)
+    pytest.raises(ValueError, Xdawn, correct_overlap='foo')
 
 
 def test_xdawn_picks():
@@ -61,6 +62,7 @@ def test_xdawn_fit():
     """Test Xdawn fit."""
     # Get data
     raw, events, picks = _get_data()
+    raw.del_proj()
     epochs = Epochs(raw, events, event_id, tmin, tmax, picks=picks,
                     preload=True, baseline=None, verbose=False)
     # =========== Basic Fit test =================
@@ -73,6 +75,8 @@ def test_xdawn_fit():
     evoked = epochs['cond2'].average()
     assert_array_equal(evoked.data, xd.evokeds_['cond2'].data)
 
+    assert_allclose(np.linalg.norm(xd.filters_['cond2'], axis=1), 1)
+
     # ========== with signal cov provided ====================
     # Provide covariance object
     signal_cov = compute_raw_covariance(raw, picks=picks)
@@ -189,14 +193,10 @@ def test_xdawn_regularization():
     xd.fit(epochs)
     xd = Xdawn(correct_overlap=False, reg='diagonal_fixed')
     xd.fit(epochs)
-    bad_eig = (sys.platform.startswith('win') and
-               check_version('numpy', '1.16.5') and
-               'mkl_rt' in _get_numpy_libs())  # some problem with MKL on Win
-    if bad_eig:
-        pytest.skip('Unknown MKL+Windows error fails for eig check')
-    xd = Xdawn(correct_overlap=False, reg=None)
-    with pytest.raises(ValueError, match='Could not compute eigenvalues'):
-        xd.fit(epochs)
+    # XXX in principle this should maybe raise an error due to deficiency?
+    # xd = Xdawn(correct_overlap=False, reg=None)
+    # with pytest.raises(ValueError, match='Could not compute eigenvalues'):
+    #     xd.fit(epochs)
 
 
 @requires_sklearn
@@ -242,8 +242,8 @@ def test_XdawnTransformer():
 
     xdt = _XdawnTransformer()
     xdt.fit(X, y)
-    assert_array_almost_equal(xd.filters_['cond2'][:, :2],
-                              xdt.filters_.reshape(2, 2, 8)[0].T)
+    assert_array_almost_equal(xd.filters_['cond2'][:2, :],
+                              xdt.filters_.reshape(2, 2, 8)[0])
 
     # Transform testing
     xdt.transform(X[1:, ...])  # different number of epochs
@@ -262,4 +262,90 @@ def test_XdawnTransformer():
     pytest.raises(ValueError, xdt.inverse_transform, 42)
 
 
-run_tests_if_main()
+def _simulate_erplike_mixed_data(n_epochs=100, n_channels=10):
+    rng = np.random.RandomState(42)
+    tmin, tmax = 0., 1.
+    sfreq = 100.
+    informative_ch_idx = 0
+
+    y = rng.randint(0, 2, n_epochs)
+    n_times = int((tmax - tmin) * sfreq)
+    epoch_times = np.linspace(tmin, tmax, n_times)
+
+    target_template = 1e-6 * (epoch_times - tmax) * np.sin(
+        2 * np.pi * epoch_times)
+    nontarget_template = 0.7e-6 * (epoch_times - tmax) * np.sin(
+        2 * np.pi * (epoch_times - 0.1))
+
+    epoch_data = rng.randn(n_epochs, n_channels, n_times) * 5e-7
+    epoch_data[y == 0, informative_ch_idx, :] += nontarget_template
+    epoch_data[y == 1, informative_ch_idx, :] += target_template
+
+    mixing_mat = linalg.svd(rng.randn(n_channels, n_channels))[0]
+    mixed_epoch_data = np.dot(mixing_mat.T, epoch_data).transpose((1, 0, 2))
+
+    events = np.zeros((n_epochs, 3), dtype=int)
+    events[:, 0] = np.arange(0, n_epochs * n_times, n_times)
+    events[:, 2] = y
+
+    info = create_info(
+        ch_names=['C{:02d}'.format(i) for i in range(n_channels)],
+        ch_types=['eeg'] * n_channels,
+        sfreq=sfreq)
+    epochs = EpochsArray(mixed_epoch_data, info, events,
+                         tmin=tmin,
+                         event_id={'nt': 0, 't': 1})
+
+    return epochs, mixing_mat
+
+
+@requires_sklearn
+def test_xdawn_decoding_performance():
+    """Test decoding performance and extracted pattern on synthetic data."""
+    from sklearn.model_selection import KFold
+    from sklearn.pipeline import make_pipeline
+    from sklearn.linear_model import LogisticRegression
+    from sklearn.preprocessing import MinMaxScaler
+    from sklearn.metrics import accuracy_score
+
+    n_xdawn_comps = 3
+    expected_accuracy = 0.98
+
+    epochs, mixing_mat = _simulate_erplike_mixed_data(n_epochs=100)
+    y = epochs.events[:, 2]
+
+    # results of Xdawn and _XdawnTransformer should match
+    xdawn_pipe = make_pipeline(
+        Xdawn(n_components=n_xdawn_comps),
+        Vectorizer(),
+        MinMaxScaler(),
+        LogisticRegression(solver='liblinear'))
+    xdawn_trans_pipe = make_pipeline(
+        _XdawnTransformer(n_components=n_xdawn_comps),
+        Vectorizer(),
+        MinMaxScaler(),
+        LogisticRegression(solver='liblinear'))
+
+    cv = KFold(n_splits=3, shuffle=False)
+    for pipe, X in (
+            (xdawn_pipe, epochs),
+            (xdawn_trans_pipe, epochs.get_data())):
+        predictions = np.empty_like(y, dtype=float)
+        for train, test in cv.split(X, y):
+            pipe.fit(X[train], y[train])
+            predictions[test] = pipe.predict(X[test])
+
+        cv_accuracy_xdawn = accuracy_score(y, predictions)
+        assert_allclose(cv_accuracy_xdawn, expected_accuracy, atol=0.01)
+
+        # for both event types, the first component should "match" the mixing
+        fitted_xdawn = pipe.steps[0][1]
+        if isinstance(fitted_xdawn, Xdawn):
+            relev_patterns = np.concatenate(
+                [comps[[0]] for comps in fitted_xdawn.patterns_.values()])
+        else:
+            relev_patterns = fitted_xdawn.patterns_[::n_xdawn_comps]
+
+        for i in range(len(relev_patterns)):
+            r, _ = stats.pearsonr(relev_patterns[i, :], mixing_mat[0, :])
+            assert np.abs(r) > 0.99
