@@ -3,30 +3,32 @@
 # License: BSD (3-clause)
 
 import numpy as np
-from scipy import linalg
 
-from . import io, Epochs
+from .epochs import Epochs
 from .utils import check_fname, logger, verbose, _check_option
+from .io.open import fiff_open
 from .io.pick import pick_types, pick_types_forward
-from .io.proj import Projection, _has_eeg_average_ref_proj
+from .io.proj import (Projection, _has_eeg_average_ref_proj, _read_proj,
+                      make_projector, make_eeg_average_ref_proj, _write_proj)
+from .io.write import start_file, end_file
 from .event import make_fixed_length_events
 from .parallel import parallel_func
 from .cov import _check_n_samples
 from .forward import (is_fixed_orient, _subject_from_forward,
                       convert_forward_solution)
-from .source_estimate import SourceEstimate, VolSourceEstimate
-from .io.proj import make_projector, make_eeg_average_ref_proj
-from .rank import _get_rank_sss
+from .source_estimate import _make_stc
 
 
-def read_proj(fname):
+@verbose
+def read_proj(fname, verbose=None):
     """Read projections from a FIF file.
 
     Parameters
     ----------
-    fname : string
+    fname : str
         The name of file containing the projections vectors. It should end with
         -proj.fif or -proj.fif.gz.
+    %(verbose)s
 
     Returns
     -------
@@ -40,9 +42,9 @@ def read_proj(fname):
     check_fname(fname, 'projection', ('-proj.fif', '-proj.fif.gz',
                                       '_proj.fif', '_proj.fif.gz'))
 
-    ff, tree, _ = io.fiff_open(fname)
+    ff, tree, _ = fiff_open(fname)
     with ff as fid:
-        projs = io.proj._read_proj(fid, tree)
+        projs = _read_proj(fid, tree)
     return projs
 
 
@@ -51,7 +53,7 @@ def write_proj(fname, projs):
 
     Parameters
     ----------
-    fname : string
+    fname : str
         The name of file containing the projections vectors. It should end with
         -proj.fif or -proj.fif.gz.
 
@@ -65,14 +67,15 @@ def write_proj(fname, projs):
     check_fname(fname, 'projection', ('-proj.fif', '-proj.fif.gz',
                                       '_proj.fif', '_proj.fif.gz'))
 
-    fid = io.write.start_file(fname)
-    io.proj._write_proj(fid, projs)
-    io.write.end_file(fid)
+    with start_file(fname) as fid:
+        _write_proj(fid, projs)
+        end_file(fid)
 
 
 @verbose
 def _compute_proj(data, info, n_grad, n_mag, n_eeg, desc_prefix,
                   meg='separate', verbose=None):
+    from scipy import linalg
     grad_ind = pick_types(info, meg='grad', ref_meg=False, exclude='bads')
     mag_ind = pick_types(info, meg='mag', ref_meg=False, exclude='bads')
     eeg_ind = pick_types(info, meg=False, eeg=True, ref_meg=False,
@@ -80,8 +83,6 @@ def _compute_proj(data, info, n_grad, n_mag, n_eeg, desc_prefix,
 
     _check_option('meg', meg, ['separate', 'combined'])
     if meg == 'combined':
-        _get_rank_sss(info, msg='meg="combined" can only be used with '
-                      'Maxfiltered data', verbose=False)
         if n_grad != n_mag:
             raise ValueError('n_grad (%d) must be equal to n_mag (%d) when '
                              'using meg="combined"')
@@ -119,8 +120,7 @@ def _compute_proj(data, info, n_grad, n_mag, n_eeg, desc_prefix,
             continue
         data_ind = data[ind][:, ind]
         # data is the covariance matrix: U * S**2 * Ut
-        U, Sexp2, _ = linalg.svd(data_ind, full_matrices=False,
-                                 overwrite_a=True)
+        U, Sexp2, _ = linalg.svd(data_ind, full_matrices=False)
         U = U[:, :n]
         exp_var = Sexp2 / Sexp2.sum()
         exp_var = exp_var[:n]
@@ -139,20 +139,22 @@ def _compute_proj(data, info, n_grad, n_mag, n_eeg, desc_prefix,
 @verbose
 def compute_proj_epochs(epochs, n_grad=2, n_mag=2, n_eeg=2, n_jobs=1,
                         desc_prefix=None, meg='separate', verbose=None):
-    """Compute SSP (spatial space projection) vectors on Epochs.
+    """Compute SSP (signal-space projection) vectors on epoched data.
+
+    %(compute_ssp)s
 
     Parameters
     ----------
     epochs : instance of Epochs
-        The epochs containing the artifact
+        The epochs containing the artifact.
     n_grad : int
-        Number of vectors for gradiometers
+        Number of vectors for gradiometers.
     n_mag : int
-        Number of vectors for magnetometers
+        Number of vectors for magnetometers.
     n_eeg : int
-        Number of vectors for EEG channels
+        Number of vectors for EEG channels.
     %(n_jobs)s
-        Number of jobs to use to compute covariance
+        Number of jobs to use to compute covariance.
     desc_prefix : str | None
         The description prefix to use. If None, one will be created based on
         the event_id, tmin, and tmax.
@@ -168,7 +170,7 @@ def compute_proj_epochs(epochs, n_grad=2, n_mag=2, n_eeg=2, n_jobs=1,
     Returns
     -------
     projs: list
-        List of projection vectors
+        List of projection vectors.
 
     See Also
     --------
@@ -206,18 +208,20 @@ def _compute_cov_epochs(epochs, n_jobs):
 @verbose
 def compute_proj_evoked(evoked, n_grad=2, n_mag=2, n_eeg=2, desc_prefix=None,
                         meg='separate', verbose=None):
-    """Compute SSP (spatial space projection) vectors on Evoked.
+    """Compute SSP (signal-space projection) vectors on evoked data.
+
+    %(compute_ssp)s
 
     Parameters
     ----------
     evoked : instance of Evoked
-        The Evoked obtained by averaging the artifact
+        The Evoked obtained by averaging the artifact.
     n_grad : int
-        Number of vectors for gradiometers
+        Number of vectors for gradiometers.
     n_mag : int
-        Number of vectors for magnetometers
+        Number of vectors for magnetometers.
     n_eeg : int
-        Number of vectors for EEG channels
+        Number of vectors for EEG channels.
     desc_prefix : str | None
         The description prefix to use. If None, one will be created based on
         tmin and tmax.
@@ -235,7 +239,7 @@ def compute_proj_evoked(evoked, n_grad=2, n_mag=2, n_eeg=2, desc_prefix=None,
     Returns
     -------
     projs : list
-        List of projection vectors
+        List of projection vectors.
 
     See Also
     --------
@@ -252,7 +256,9 @@ def compute_proj_evoked(evoked, n_grad=2, n_mag=2, n_eeg=2, desc_prefix=None,
 def compute_proj_raw(raw, start=0, stop=None, duration=1, n_grad=2, n_mag=2,
                      n_eeg=0, reject=None, flat=None, n_jobs=1, meg='separate',
                      verbose=None):
-    """Compute SSP (spatial space projection) vectors on Raw.
+    """Compute SSP (signal-space projection) vectors on continuous data.
+
+    %(compute_ssp)s
 
     Parameters
     ----------
@@ -290,7 +296,7 @@ def compute_proj_raw(raw, start=0, stop=None, duration=1, n_grad=2, n_mag=2,
     Returns
     -------
     projs: list
-        List of projection vectors
+        List of projection vectors.
 
     See Also
     --------
@@ -349,7 +355,7 @@ def sensitivity_map(fwd, projs=None, ch_type='grad', mode='fixed', exclude=[],
         'fixed', 'ratio', 'radiality', 'angle', 'remaining', or 'dampening'
         corresponding to the argument --map 1, 2, 3, 4, 5, 6 and 7 of the
         command mne_sensitivity_map.
-    exclude : list of string | str
+    exclude : list of str | str
         List of channels to exclude. If empty do not exclude any (default).
         If 'bads', exclude channels in fwd['info']['bads'].
     %(verbose)s
@@ -360,6 +366,7 @@ def sensitivity_map(fwd, projs=None, ch_type='grad', mode='fixed', exclude=[],
         The sensitivity map as a SourceEstimate or VolSourceEstimate instance
         for visualization.
     """
+    from scipy import linalg
     # check strings
     _check_option('ch_type', ch_type, ['eeg', 'grad', 'mag'])
     _check_option('mode', mode, ['free', 'fixed', 'ratio', 'radiality',
@@ -442,12 +449,6 @@ def sensitivity_map(fwd, projs=None, ch_type='grad', mode='fixed', exclude=[],
         sensitivity_map /= np.max(sensitivity_map)
 
     subject = _subject_from_forward(fwd)
-    if fwd['src'][0]['type'] == 'vol':  # volume source space
-        vertices = fwd['src'][0]['vertno']
-        SEClass = VolSourceEstimate
-    else:
-        vertices = [fwd['src'][0]['vertno'], fwd['src'][1]['vertno']]
-        SEClass = SourceEstimate
-    stc = SEClass(sensitivity_map[:, np.newaxis], vertices=vertices, tmin=0,
-                  tstep=1, subject=subject)
-    return stc
+    vertices = [s['vertno'] for s in fwd['src']]
+    return _make_stc(sensitivity_map[:, np.newaxis], vertices, fwd['src'].kind,
+                     tmin=0., tstep=1., subject=subject)
