@@ -11,7 +11,7 @@ from ..cov import compute_whitener
 from ..io.pick import pick_info
 from ..forward import apply_forward
 from ..utils import (logger, verbose, check_random_state, _check_preload,
-                     deprecated, _validate_type)
+                     _validate_type)
 
 
 @verbose
@@ -27,8 +27,8 @@ def simulate_evoked(fwd, stc, info, cov, nave=30, iir_filter=None,
 
     Parameters
     ----------
-    fwd : Forward
-        a forward solution.
+    fwd : instance of Forward
+        A forward solution.
     stc : SourceEstimate object
         The source time courses.
     info : dict
@@ -42,9 +42,7 @@ def simulate_evoked(fwd, stc, info, cov, nave=30, iir_filter=None,
     iir_filter : None | array
         IIR filter coefficients (denominator) e.g. [1, -1, 0.2].
     %(random_state)s
-    use_cps : bool (default True)
-        Whether to use cortical patch statistics to define normal
-        orientations when converting to fixed orientation (if necessary).
+    %(use_cps)s
 
         .. versionadded:: 0.15
     %(verbose)s
@@ -52,7 +50,7 @@ def simulate_evoked(fwd, stc, info, cov, nave=30, iir_filter=None,
     Returns
     -------
     evoked : Evoked object
-        The simulated evoked data
+        The simulated evoked data.
 
     See Also
     --------
@@ -75,42 +73,12 @@ def simulate_evoked(fwd, stc, info, cov, nave=30, iir_filter=None,
     if nave < np.inf:
         noise = _simulate_noise_evoked(evoked, cov, iir_filter, random_state)
         evoked.data += noise.data / math.sqrt(nave)
-        evoked.nave = np.int(nave)
+        evoked.nave = np.int64(nave)
     if cov is not None and cov.get('projs', None):
         evoked.add_proj(cov['projs']).apply_proj()
     return evoked
 
 
-@deprecated('simulate_noise_evoked is deprecated in 0.18 and will be removed '
-            'in 0.19, use add_noise instead')
-def simulate_noise_evoked(evoked, cov, iir_filter=None, random_state=None):
-    """Create noise as a multivariate Gaussian.
-
-    The spatial covariance of the noise is given from the cov matrix.
-
-    Parameters
-    ----------
-    evoked : evoked object
-        an instance of evoked used as template
-    cov : Covariance object
-        The noise covariance
-    iir_filter : None | array
-        IIR filter coefficients (denominator)
-    random_state : None | int | ~numpy.random.RandomState
-        To specify the random generator state.
-
-    Returns
-    -------
-    noise : evoked object
-        an instance of evoked
-
-    Notes
-    -----
-    .. versionadded:: 0.10.0
-    """
-    return _simulate_noise_evoked(evoked, cov, iir_filter, random_state)
-
-
 def _simulate_noise_evoked(evoked, cov, iir_filter, random_state):
     noise = evoked.copy()
     noise.data[:] = 0
@@ -170,6 +138,7 @@ def _add_noise(inst, cov, iir_filter, random_state, allow_subselection=True):
         data = data[np.newaxis]
     # Subselect if necessary
     info = inst.info
+    info._check_consistency()
     picks = gen_picks = slice(None)
     if allow_subselection:
         use_chs = list(set(info['ch_names']) & set(cov['names']))
@@ -177,6 +146,8 @@ def _add_noise(inst, cov, iir_filter, random_state, allow_subselection=True):
         logger.info('Adding noise to %d/%d channels (%d channels in cov)'
                     % (len(picks), len(info['chs']), len(cov['names'])))
         info = pick_info(inst.info, picks)
+        info._check_consistency()
+
         gen_picks = np.arange(info['nchan'])
     for epoch in data:
         epoch[picks] += _generate_noise(info, cov, iir_filter, random_state,
