@@ -10,7 +10,6 @@ from inspect import isgenerator
 from collections import namedtuple
 
 import numpy as np
-from scipy import linalg, sparse
 
 from ..source_estimate import SourceEstimate
 from ..epochs import BaseEpochs
@@ -20,7 +19,7 @@ from ..io.pick import pick_types, pick_info, _picks_to_idx
 
 
 def linear_regression(inst, design_matrix, names=None):
-    """Fit Ordinary Least Squares regression (OLS).
+    """Fit Ordinary Least Squares (OLS) regression.
 
     Parameters
     ----------
@@ -33,28 +32,30 @@ def linear_regression(inst, design_matrix, names=None):
         the first dimension of the data. The first column of this matrix will
         typically consist of ones (intercept column).
     names : array-like | None
-        Optional parameter to name the regressors. If provided, the length must
-        correspond to the number of columns present in regressors
-        (including the intercept, if present).
-        Otherwise the default names are x0, x1, x2...xn for n regressors.
+        Optional parameter to name the regressors (i.e., the columns in the
+        design matrix). If provided, the length must correspond to the number
+        of columns present in design matrix (including the intercept, if
+        present). Otherwise, the default names are ``'x0'``, ``'x1'``,
+        ``'x2', …, 'x(n-1)'`` for ``n`` regressors.
 
     Returns
     -------
     results : dict of namedtuple
-        For each regressor (key) a namedtuple is provided with the
+        For each regressor (key), a namedtuple is provided with the
         following attributes:
 
-            beta : regression coefficients
-            stderr : standard error of regression coefficients
-            t_val : t statistics (beta / stderr)
-            p_val : two-sided p-value of t statistic under the t distribution
-            mlog10_p_val : -log10 transformed p-value.
+            - ``beta`` : regression coefficients
+            - ``stderr`` : standard error of regression coefficients
+            - ``t_val`` : t statistics (``beta`` / ``stderr``)
+            - ``p_val`` : two-sided p-value of t statistic under the t
+              distribution
+            - ``mlog10_p_val`` : -log₁₀-transformed p-value.
 
         The tuple members are numpy arrays. The shape of each numpy array is
         the shape of the data minus the first dimension; e.g., if the shape of
-        the original data was (n_observations, n_channels, n_timepoints),
+        the original data was ``(n_observations, n_channels, n_timepoints)``,
         then the shape of each of the arrays will be
-        (n_channels, n_timepoints).
+        ``(n_channels, n_timepoints)``.
     """
     if names is None:
         names = ['x%i' % i for i in range(design_matrix.shape[1])]
@@ -100,7 +101,7 @@ def linear_regression(inst, design_matrix, names=None):
 
 def _fit_lm(data, design_matrix, names):
     """Aux function."""
-    from scipy import stats
+    from scipy import stats, linalg
     n_samples = len(data)
     n_features = np.product(data.shape[1:])
     if design_matrix.ndim != 2:
@@ -159,7 +160,7 @@ def linear_regression_raw(raw, events, event_id=None, tmin=-.1, tmax=1,
     Internally, this constructs a predictor matrix X of size
     n_samples * (n_conds * window length), solving the linear system
     ``Y = bX`` and returning ``b`` as evoked-like time series split by
-    condition. See [1]_.
+    condition. See :footcite:`SmithKutas2015`.
 
     Parameters
     ----------
@@ -190,9 +191,9 @@ def linear_regression_raw(raw, events, event_id=None, tmin=-.1, tmax=1,
         used.
     covariates : dict-like | None
         If dict-like (e.g., a pandas DataFrame), values have to be array-like
-        and of the same length as the rows in ```events```. Keys correspond
+        and of the same length as the rows in ``events``. Keys correspond
         to additional event types/conditions to be estimated and are matched
-        with the time points given by the first column of ```events```. If
+        with the time points given by the first column of ``events``. If
         None, only binary events (from event_id) are used.
     reject : None | dict
         For cleaning raw data before the regression is performed: set up
@@ -208,7 +209,7 @@ def linear_regression_raw(raw, events, event_id=None, tmin=-.1, tmax=1,
                           eog=250e-5 # V (EOG channels))
 
     flat : None | dict
-        or cleaning raw data before the regression is performed: set up
+        For cleaning raw data before the regression is performed: set up
         rejection parameters based on flatness of the signal. If None, no
         rejection is done. If a dict, keys are ('grad' | 'mag' |
         'eeg' | 'eog' | 'ecg') and values are minimal peak-to-peak values to
@@ -238,10 +239,9 @@ def linear_regression_raw(raw, events, event_id=None, tmin=-.1, tmax=1,
 
     References
     ----------
-    .. [1] Smith, N. J., & Kutas, M. (2015). Regression-based estimation of ERP
-           waveforms: II. Non-linear effects, overlap correction, and practical
-           considerations. Psychophysiology, 52(2), 169-189.
+    .. footbibliography::
     """
+    from scipy import linalg
     if isinstance(solver, str):
         if solver not in {"cholesky"}:
             raise ValueError("No such solver: {}".format(solver))
@@ -312,6 +312,7 @@ def _prepare_rerp_data(raw, events, picks=None, decim=1):
 def _prepare_rerp_preds(n_samples, sfreq, events, event_id=None, tmin=-.1,
                         tmax=1, covariates=None):
     """Build predictor matrix and metadata (e.g. condition time windows)."""
+    from scipy import sparse
     conds = list(event_id)
     if covariates is not None:
         conds += list(covariates)
@@ -352,8 +353,8 @@ def _prepare_rerp_preds(n_samples, sfreq, events, event_id=None, tmin=-.1,
         else:  # for predictors from covariates, e.g. continuous ones
             covs = covariates[cond]
             if len(covs) != len(events):
-                error = ("Condition {0} from ```covariates``` is "
-                         "not the same length as ```events```").format(cond)
+                error = ("Condition {0} from ``covariates`` is "
+                         "not the same length as ``events``").format(cond)
                 raise ValueError(error)
             onsets = -(events[np.where(covs != 0), 0] + tmin_)[0]
             v = np.asarray(covs)[np.nonzero(covs)].astype(float)
