@@ -1,20 +1,23 @@
-# Authors: Alexandre Gramfort <alexandre.gramfort@inria.fr>
-#          Matti Hamalainen <msh@nmr.mgh.harvard.edu>
+# Authors: Matti Hämäläinen <msh@nmr.mgh.harvard.edu>
+#          Alexandre Gramfort <alexandre.gramfort@inria.fr>
+#          Matti Hämäläinen <msh@nmr.mgh.harvard.edu>
 #          Denis A. Engemann <denis.engemann@gmail.com>
 #
 # License: BSD (3-clause)
 
+# Many of the computations in this code were derived from Matti Hämäläinen's
+# C code.
+
 from copy import deepcopy
 from distutils.version import LooseVersion
 from glob import glob
 from functools import partial
 import os
 from os import path as op
-import sys
+import warnings
 from struct import pack
 
 import numpy as np
-from scipy.sparse import coo_matrix, csr_matrix, eye as speye
 
 from .io.constants import FIFF
 from .io.open import fiff_open
@@ -27,9 +30,11 @@ from .channels.channels import _get_meg_system
 from .parallel import parallel_func
 from .transforms import (transform_surface_to, _pol_to_cart, _cart_to_sph,
                          _get_trans, apply_trans, Transform)
-from .utils import logger, verbose, get_subjects_dir, warn, _check_fname
+from .utils import (logger, verbose, get_subjects_dir, warn, _check_fname,
+                    _check_option, _ensure_int, _TempDir, run_subprocess,
+                    _check_freesurfer_home)
 from .fixes import (_serialize_volume_info, _get_read_geometry, einsum, jit,
-                    prange)
+                    prange, bincount)
 
 
 ###############################################################################
@@ -45,9 +50,9 @@ def get_head_surf(subject, source=('bem', 'head'), subjects_dir=None,
     subject : str
         Subject name.
     source : str | list of str
-        Type to load. Common choices would be `'bem'` or `'head'`. We first
-        try loading `'$SUBJECTS_DIR/$SUBJECT/bem/$SUBJECT-$SOURCE.fif'`, and
-        then look for `'$SUBJECT*$SOURCE.fif'` in the same directory by going
+        Type to load. Common choices would be ``'bem'`` or ``'head'``. We first
+        try loading ``'$SUBJECTS_DIR/$SUBJECT/bem/$SUBJECT-$SOURCE.fif'``, and
+        then look for ``'$SUBJECT*$SOURCE.fif'`` in the same directory by going
         through all files matching the pattern. The head surface will be read
         from the first file containing a head surface. Can also be a list
         to try multiple strings.
@@ -138,7 +143,7 @@ def get_meg_helmet_surf(info, trans=None, verbose=None):
     will be approximated based on the sensor locations.
     """
     from scipy.spatial import ConvexHull, Delaunay
-    from .bem import read_bem_surfaces
+    from .bem import read_bem_surfaces, _fit_sphere
     system, have_helmet = _get_meg_system(info)
     if have_helmet:
         logger.info('Getting helmet for system %s' % system)
@@ -152,10 +157,19 @@ def get_meg_helmet_surf(info, trans=None, verbose=None):
                                               exclude=())])
         logger.info('Getting helmet for system %s (derived from %d MEG '
                     'channel locations)' % (system, len(rr)))
-        rr = rr[np.unique(ConvexHull(rr).simplices)]
-        com = rr.mean(axis=0)
-        xy = _pol_to_cart(_cart_to_sph(rr - com)[:, 1:][:, ::-1])
-        tris = _reorder_ccw(rr, Delaunay(xy).simplices)
+        hull = ConvexHull(rr)
+        rr = rr[np.unique(hull.simplices)]
+        R, center = _fit_sphere(rr, disp=False)
+        sph = _cart_to_sph(rr - center)[:, 1:]
+        # add a point at the front of the helmet (where the face should be):
+        # 90 deg az and maximal el (down from Z/up axis)
+        front_sph = [[np.pi / 2., sph[:, 1].max()]]
+        sph = np.concatenate((sph, front_sph))
+        xy = _pol_to_cart(sph[:, ::-1])
+        tris = Delaunay(xy).simplices
+        # remove the frontal point we added from the simplices
+        tris = tris[(tris != len(sph) - 1).all(-1)]
+        tris = _reorder_ccw(rr, tris)
 
         surf = dict(rr=rr, tris=tris)
         complete_surface_info(surf, copy=False, verbose=False)
@@ -214,17 +228,23 @@ def fast_cross_3d(x, y):
     assert y.shape[-1] == 3
     if max(x.size, y.size) >= 500:
         out = np.empty(np.broadcast(x, y).shape)
-        np.multiply(x[..., 1], y[..., 2], out=out[..., 0])
-        out[..., 0] -= x[..., 2] * y[..., 1]
-        np.multiply(x[..., 2], y[..., 0], out=out[..., 1])
-        out[..., 1] -= x[..., 0] * y[..., 2]
-        np.multiply(x[..., 0], y[..., 1], out=out[..., 2])
-        out[..., 2] -= x[..., 1] * y[..., 0]
+        _jit_cross(out, x, y)
         return out
     else:
         return np.cross(x, y)
 
 
+@jit()
+def _jit_cross(out, x, y):
+    out[..., 0] = x[..., 1] * y[..., 2]
+    out[..., 0] -= x[..., 2] * y[..., 1]
+    out[..., 1] = x[..., 2] * y[..., 0]
+    out[..., 1] -= x[..., 0] * y[..., 2]
+    out[..., 2] = x[..., 0] * y[..., 1]
+    out[..., 2] -= x[..., 1] * y[..., 0]
+
+
+@jit()
 def _fast_cross_nd_sum(a, b, c):
     """Fast cross and sum."""
     return ((a[..., 1] * b[..., 2] - a[..., 2] * b[..., 1]) * c[..., 0] +
@@ -232,6 +252,7 @@ def _fast_cross_nd_sum(a, b, c):
             (a[..., 0] * b[..., 1] - a[..., 1] * b[..., 0]) * c[..., 2])
 
 
+@jit()
 def _accumulate_normals(tris, tri_nn, npts):
     """Efficiently accumulate triangle normals."""
     # this code replaces the following, but is faster (vectorized):
@@ -242,10 +263,11 @@ def _accumulate_normals(tris, tri_nn, npts):
     #     this['nn'][verts, :] += this['tri_nn'][p, :]
     #
     nn = np.zeros((npts, 3))
-    for verts in tris.T:  # note this only loops 3x (number of verts per tri)
+    for vi in range(3):
+        verts = tris[:, vi]
         for idx in range(3):  # x, y, z
-            nn[:, idx] += np.bincount(verts, weights=tri_nn[:, idx],
-                                      minlength=npts)
+            nn[:, idx] += bincount(verts, weights=tri_nn[:, idx],
+                                   minlength=npts)
     return nn
 
 
@@ -256,6 +278,7 @@ def _triangle_neighbors(tris, npts):
     # for ti, tri in enumerate(tris):
     #     for t in tri:
     #         neighbor_tri[t].append(ti)
+    from scipy.sparse import coo_matrix
     rows = tris.ravel()
     cols = np.repeat(np.arange(len(tris)), 3)
     data = np.ones(len(cols))
@@ -289,8 +312,8 @@ def _triangle_coords(r, best, r1, nn, r12, r13, a, b, c):  # pragma: no cover
 def _project_onto_surface(rrs, surf, project_rrs=False, return_nn=False,
                           method='accurate'):
     """Project points onto (scalp) surface."""
-    surf_geom = _get_tri_supp_geom(surf)
     if method == 'accurate':
+        surf_geom = _get_tri_supp_geom(surf)
         pt_tris = np.empty((0,), int)
         pt_lens = np.zeros(len(rrs) + 1, int)
         out = _find_nearest_tri_pts(rrs, pt_tris, pt_lens,
@@ -305,12 +328,25 @@ def _project_onto_surface(rrs, surf, project_rrs=False, return_nn=False,
         idx = _compute_nearest(surf['rr'], rrs)
         out = (None, None, surf['rr'][idx])
         if return_nn:
-            nn = _accumulate_normals(surf['tris'], surf_geom['nn'],
+            nn = _accumulate_normals(surf['tris'].astype(int), surf_geom['nn'],
                                      len(surf['rr']))
             out += (nn[idx],)
     return out
 
 
+def _normal_orth(nn):
+    """Compute orthogonal basis given a normal."""
+    assert nn.shape[-1:] == (3,)
+    prod = np.einsum('...i,...j->...ij', nn, nn)
+    _, u = np.linalg.eigh(np.eye(3) - prod)
+    u = u[..., ::-1]
+    #  Make sure that ez is in the direction of nn
+    signs = np.sign(np.matmul(nn[..., np.newaxis, :], u[..., -1:]))
+    signs[signs == 0] = 1
+    u *= signs
+    return u.swapaxes(-1, -2)
+
+
 @verbose
 def complete_surface_info(surf, do_neighbor_vert=False, copy=True,
                           verbose=None):
@@ -354,7 +390,8 @@ def complete_surface_info(surf, do_neighbor_vert=False, copy=True,
     #    Find neighboring triangles, accumulate vertex normals, normalize
     logger.info('    Triangle neighbors and vertex normals...')
     surf['neighbor_tri'] = _triangle_neighbors(surf['tris'], surf['np'])
-    surf['nn'] = _accumulate_normals(surf['tris'], surf['tri_nn'], surf['np'])
+    surf['nn'] = _accumulate_normals(surf['tris'].astype(int),
+                                     surf['tri_nn'], surf['np'])
     _normalize_vectors(surf['nn'])
 
     #   Check for topological defects
@@ -589,12 +626,25 @@ def _fread3(fobj):
 def _fread3_many(fobj, n):
     """Read 3-byte ints from an open binary file object."""
     b1, b2, b3 = np.fromfile(fobj, ">u1",
-                             3 * n).reshape(-1, 3).astype(np.int).T
+                             3 * n).reshape(-1, 3).astype(np.int64).T
     return (b1 << 16) + (b2 << 8) + b3
 
 
-def read_curvature(filepath):
-    """Load in curavature values from the ?h.curv file."""
+def read_curvature(filepath, binary=True):
+    """Load in curvature values from the ?h.curv file.
+
+    Parameters
+    ----------
+    filepath : str
+        Input path to the .curv file.
+    binary : bool
+        Specify if the output array is to hold binary values. Defaults to True.
+
+    Returns
+    -------
+    curv : array, shape=(n_vertices,)
+        The curvature values loaded from the user given file.
+    """
     with open(filepath, "rb") as fobj:
         magic = _fread3(fobj)
         if magic == 16777215:
@@ -604,12 +654,15 @@ def read_curvature(filepath):
             vnum = magic
             _fread3(fobj)
             curv = np.fromfile(fobj, ">i2", vnum) / 100
-        bin_curv = 1 - np.array(curv != 0, np.int)
-    return bin_curv
+    if binary:
+        return 1 - np.array(curv != 0, np.int64)
+    else:
+        return curv
 
 
 @verbose
-def read_surface(fname, read_metadata=False, return_dict=False, verbose=None):
+def read_surface(fname, read_metadata=False, return_dict=False,
+                 file_format='auto', verbose=None):
     """Load a Freesurfer surface mesh in triangular format.
 
     Parameters
@@ -617,7 +670,9 @@ def read_surface(fname, read_metadata=False, return_dict=False, verbose=None):
     fname : str
         The name of the file containing the surface.
     read_metadata : bool
-        Read metadata as key-value pairs.
+        Read metadata as key-value pairs. Only works when reading a FreeSurfer
+        surface file. For .obj files this dictionary will be empty.
+
         Valid keys:
 
             * 'head' : array of int
@@ -634,6 +689,13 @@ def read_surface(fname, read_metadata=False, return_dict=False, verbose=None):
 
     return_dict : bool
         If True, a dictionary with surface parameters is returned.
+    file_format : 'auto' | 'freesurfer' | 'obj'
+        File format to use. Can be 'freesurfer' to read a FreeSurfer surface
+        file, or 'obj' to read a Wavefront .obj file (common format for
+        importing in other software), or 'auto' to attempt to infer from the
+        file name. Defaults to 'auto'.
+
+        .. versionadded:: 0.21.0
     %(verbose)s
 
     Returns
@@ -654,13 +716,117 @@ def read_surface(fname, read_metadata=False, return_dict=False, verbose=None):
     read_tri
     """
     fname = _check_fname(fname, 'read', True)
-    ret = _get_read_geometry()(fname, read_metadata=read_metadata)
+    _check_option('file_format', file_format, ['auto', 'freesurfer', 'obj'])
+
+    if file_format == 'auto':
+        _, ext = op.splitext(fname)
+        if ext.lower() == '.obj':
+            file_format = 'obj'
+        else:
+            file_format = 'freesurfer'
+
+    if file_format == 'freesurfer':
+        ret = _get_read_geometry()(fname, read_metadata=read_metadata)
+    elif file_format == 'obj':
+        ret = _read_wavefront_obj(fname)
+        if read_metadata:
+            ret += (dict(),)
+
     if return_dict:
         ret += (dict(rr=ret[0], tris=ret[1], ntri=len(ret[1]), use_tris=ret[1],
                      np=len(ret[0])),)
     return ret
 
 
+def _read_mri_surface(fname):
+    surf = read_surface(fname, return_dict=True)[2]
+    surf['rr'] /= 1000.
+    surf.update(coord_frame=FIFF.FIFFV_COORD_MRI)
+    return surf
+
+
+def _read_wavefront_obj(fname):
+    """Read a surface form a Wavefront .obj file.
+
+    Parameters
+    ----------
+    fname : str
+        Name of the .obj file to read.
+
+    Returns
+    -------
+    coords : ndarray, shape (n_points, 3)
+        The XYZ coordinates of each vertex.
+    faces : ndarray, shape (n_faces, 3)
+        For each face of the mesh, the integer indices of the vertices that
+        make up the face.
+    """
+    coords = []
+    faces = []
+    with open(fname) as f:
+        for line in f:
+            line = line.strip()
+            if len(line) == 0 or line[0] == "#":
+                continue
+            split = line.split()
+            if split[0] == "v":  # vertex
+                coords.append([float(item) for item in split[1:]])
+            elif split[0] == "f":  # face
+                dat = [int(item.split("/")[0]) for item in split[1:]]
+                if len(dat) != 3:
+                    raise RuntimeError('Only triangle faces allowed.')
+                # In .obj files, indexing starts at 1
+                faces.append([d - 1 for d in dat])
+    return np.array(coords), np.array(faces)
+
+
+def _read_patch(fname):
+    """Load a FreeSurfer binary patch file.
+
+    Parameters
+    ----------
+    fname : str
+        The filename.
+
+    Returns
+    -------
+    rrs : ndarray, shape (n_vertices, 3)
+        The points.
+    tris : ndarray, shape (n_tris, 3)
+        The patches. Not all vertices will be present.
+    """
+    # This is adapted from PySurfer PR #269, Bruce Fischl's read_patch.m,
+    # and PyCortex (BSD)
+    patch = dict()
+    with open(fname, 'r') as fid:
+        ver = np.fromfile(fid, dtype='>i4', count=1)[0]
+        if ver != -1:
+            raise RuntimeError(f'incorrect version # {ver} (not -1) found')
+        npts = np.fromfile(fid, dtype='>i4', count=1)[0]
+        dtype = np.dtype(
+            [('vertno', '>i4'), ('x', '>f'), ('y', '>f'), ('z', '>f')])
+        recs = np.fromfile(fid, dtype=dtype, count=npts)
+    # numpy to dict
+    patch = {key: recs[key] for key in dtype.fields.keys()}
+    patch['vertno'] -= 1
+
+    # read surrogate surface
+    rrs, tris = read_surface(
+        op.join(op.dirname(fname), op.basename(fname)[:3] + 'sphere'))
+    orig_tris = tris
+    is_vert = patch['vertno'] > 0  # negative are edges, ignored for now
+    verts = patch['vertno'][is_vert]
+
+    # eliminate invalid tris and zero out unused rrs
+    mask = np.zeros((len(rrs),), dtype=bool)
+    mask[verts] = True
+    rrs[~mask] = 0.
+    tris = tris[mask[tris].all(1)]
+    for ii, key in enumerate(['x', 'y', 'z']):
+        rrs[verts, ii] = patch[key][is_vert]
+    return rrs, tris, orig_tris
+
+
 ##############################################################################
 # SURFACE CREATION
 
@@ -873,8 +1039,9 @@ def _decimate_surface_spacing(surf, spacing):
     return surf
 
 
+@verbose
 def write_surface(fname, coords, faces, create_stamp='', volume_info=None,
-                  overwrite=False):
+                  file_format='auto', overwrite=False, *, verbose=None):
     """Write a triangular Freesurfer surface mesh.
 
     Accepts the same data format as is returned by read_surface().
@@ -906,8 +1073,15 @@ def write_surface(fname, coords, faces, create_stamp='', volume_info=None,
             * 'cras' : array of float, shape (3,)
 
         .. versionadded:: 0.13.0
-    overwrite : bool
-        If True, overwrite the file if it exists.
+    file_format : 'auto' | 'freesurfer' | 'obj'
+        File format to use. Can be 'freesurfer' to write a FreeSurfer surface
+        file, or 'obj' to write a Wavefront .obj file (common format for
+        importing in other software), or 'auto' to attempt to infer from the
+        file name. Defaults to 'auto'.
+
+        .. versionadded:: 0.21.0
+    %(overwrite)s
+    %(verbose)s
 
     See Also
     --------
@@ -915,71 +1089,153 @@ def write_surface(fname, coords, faces, create_stamp='', volume_info=None,
     read_tri
     """
     fname = _check_fname(fname, overwrite=overwrite)
-    try:
-        import nibabel as nib
-        has_nibabel = True
-    except ImportError:
-        has_nibabel = False
-    if has_nibabel and LooseVersion(nib.__version__) > LooseVersion('2.1.0'):
-        nib.freesurfer.io.write_geometry(fname, coords, faces,
-                                         create_stamp=create_stamp,
-                                         volume_info=volume_info)
-        return
-    if len(create_stamp.splitlines()) > 1:
-        raise ValueError("create_stamp can only contain one line")
+    _check_option('file_format', file_format, ['auto', 'freesurfer', 'obj'])
 
-    with open(fname, 'wb') as fid:
-        fid.write(pack('>3B', 255, 255, 254))
-        strs = ['%s\n' % create_stamp, '\n']
-        strs = [s.encode('utf-8') for s in strs]
-        fid.writelines(strs)
-        vnum = len(coords)
-        fnum = len(faces)
-        fid.write(pack('>2i', vnum, fnum))
-        fid.write(np.array(coords, dtype='>f4').tostring())
-        fid.write(np.array(faces, dtype='>i4').tostring())
+    if file_format == 'auto':
+        _, ext = op.splitext(fname)
+        if ext.lower() == '.obj':
+            file_format = 'obj'
+        else:
+            file_format = 'freesurfer'
 
-        # Add volume info, if given
-        if volume_info is not None and len(volume_info) > 0:
-            fid.write(_serialize_volume_info(volume_info))
+    if file_format == 'freesurfer':
+        try:
+            import nibabel as nib
+            has_nibabel = LooseVersion(nib.__version__) > LooseVersion('2.1.0')
+        except ImportError:
+            has_nibabel = False
+        if has_nibabel:
+            nib.freesurfer.io.write_geometry(fname, coords, faces,
+                                             create_stamp=create_stamp,
+                                             volume_info=volume_info)
+            return
+        if len(create_stamp.splitlines()) > 1:
+            raise ValueError("create_stamp can only contain one line")
+
+        with open(fname, 'wb') as fid:
+            fid.write(pack('>3B', 255, 255, 254))
+            strs = ['%s\n' % create_stamp, '\n']
+            strs = [s.encode('utf-8') for s in strs]
+            fid.writelines(strs)
+            vnum = len(coords)
+            fnum = len(faces)
+            fid.write(pack('>2i', vnum, fnum))
+            fid.write(np.array(coords, dtype='>f4').tobytes())
+            fid.write(np.array(faces, dtype='>i4').tobytes())
+
+            # Add volume info, if given
+            if volume_info is not None and len(volume_info) > 0:
+                fid.write(_serialize_volume_info(volume_info))
+
+    elif file_format == 'obj':
+        with open(fname, 'w') as fid:
+            for line in create_stamp.splitlines():
+                fid.write(f'# {line}\n')
+            for v in coords:
+                fid.write(f'v {v[0]} {v[1]} {v[2]}\n')
+            for f in faces:
+                fid.write(f'f {f[0] + 1} {f[1] + 1} {f[2] + 1}\n')
 
 
 ###############################################################################
 # Decimation
 
-def _decimate_surface(points, triangles, reduction):
+def _decimate_surface_vtk(points, triangles, n_triangles):
     """Aux function."""
-    if 'DISPLAY' not in os.environ and sys.platform != 'win32':
-        os.environ['ETS_TOOLKIT'] = 'null'
     try:
-        from tvtk.api import tvtk
-        from tvtk.common import configure_input
+        from vtk.util.numpy_support import \
+            numpy_to_vtk, numpy_to_vtkIdTypeArray
+        from vtk.numpy_interface.dataset_adapter import WrapDataObject
+        from vtk import \
+            vtkPolyData, vtkQuadricDecimation, vtkPoints, vtkCellArray
     except ImportError:
-        raise ValueError('This function requires the TVTK package to be '
+        raise ValueError('This function requires the VTK package to be '
                          'installed')
     if triangles.max() > len(points) - 1:
         raise ValueError('The triangles refer to undefined points. '
                          'Please check your mesh.')
-    src = tvtk.PolyData(points=points, polys=triangles)
-    decimate = tvtk.QuadricDecimation(target_reduction=reduction)
-    configure_input(decimate, src)
-    decimate.update()
-    out = decimate.output
-    tris = out.polys.to_array()
-    # n-tuples + interleaved n-next -- reshape trick
-    return out.points.to_array(), tris.reshape(tris.size // 4, 4)[:, 1:]
+    src = vtkPolyData()
+    vtkpoints = vtkPoints()
+    with warnings.catch_warnings(record=True):
+        warnings.simplefilter('ignore')
+        vtkpoints.SetData(numpy_to_vtk(points.astype(np.float64)))
+    src.SetPoints(vtkpoints)
+    vtkcells = vtkCellArray()
+    triangles_ = np.pad(
+        triangles, ((0, 0), (1, 0)), 'constant', constant_values=3)
+    with warnings.catch_warnings(record=True):
+        warnings.simplefilter('ignore')
+        idarr = numpy_to_vtkIdTypeArray(triangles_.ravel().astype(np.int64))
+    vtkcells.SetCells(triangles.shape[0], idarr)
+    src.SetPolys(vtkcells)
+    # vtkDecimatePro was not very good, even with SplittingOff and
+    # PreserveTopologyOn
+    decimate = vtkQuadricDecimation()
+    decimate.VolumePreservationOn()
+    decimate.SetInputData(src)
+    reduction = 1 - (float(n_triangles) / len(triangles))
+    decimate.SetTargetReduction(reduction)
+    decimate.Update()
+    out = WrapDataObject(decimate.GetOutput())
+    rrs = out.Points
+    tris = out.Polygons.reshape(-1, 4)[:, 1:]
+    return rrs, tris
 
 
-def decimate_surface(points, triangles, n_triangles):
+def _decimate_surface_sphere(rr, tris, n_triangles):
+    _check_freesurfer_home()
+    map_ = {}
+    ico_levels = [20, 80, 320, 1280, 5120, 20480]
+    map_.update({n_tri: ('ico', ii) for ii, n_tri in enumerate(ico_levels)})
+    oct_levels = 2 ** (2 * np.arange(7) + 3)
+    map_.update({n_tri: ('oct', ii) for ii, n_tri in enumerate(oct_levels, 1)})
+    _check_option('n_triangles', n_triangles, sorted(map_),
+                  extra=' when method="sphere"')
+    func_map = dict(ico=_get_ico_surface, oct=_tessellate_sphere_surf)
+    kind, level = map_[n_triangles]
+    logger.info('Decimating using Freesurfer spherical %s%s downsampling'
+                % (kind, level))
+    ico_surf = func_map[kind](level)
+    assert len(ico_surf['tris']) == n_triangles
+    tempdir = _TempDir()
+    orig = op.join(tempdir, 'lh.temp')
+    write_surface(orig, rr, tris)
+    logger.info('    Extracting main mesh component ...')
+    run_subprocess(
+        ['mris_extract_main_component', orig, orig],
+        verbose='error')
+    logger.info('    Smoothing ...')
+    smooth = orig + '.smooth'
+    run_subprocess(
+        ['mris_smooth', '-nw', orig, smooth],
+        verbose='error')
+    logger.info('    Inflating ...')
+    inflated = orig + '.inflated'
+    run_subprocess(
+        ['mris_inflate', '-no-save-sulc', smooth, inflated],
+        verbose='error')
+    logger.info('    Sphere ...')
+    qsphere = orig + '.qsphere'
+    run_subprocess(
+        ['mris_sphere', '-q', inflated, qsphere], verbose='error')
+    sphere_rr, _ = read_surface(qsphere)
+    norms = np.linalg.norm(sphere_rr, axis=1, keepdims=True)
+    sphere_rr /= norms
+    idx = _compute_nearest(sphere_rr, ico_surf['rr'], method='cKDTree')
+    n_dup = len(idx) - len(np.unique(idx))
+    if n_dup:
+        raise RuntimeError('Could not reduce to %d triangles using ico, '
+                           '%d/%d vertices were duplicates'
+                           % (n_triangles, n_dup, len(idx)))
+    logger.info('[done]')
+    return rr[idx], ico_surf['tris']
+
+
+@verbose
+def decimate_surface(points, triangles, n_triangles, method='quadric',
+                     verbose=None):
     """Decimate surface data.
 
-    .. note:: Requires TVTK to be installed for this to function.
-
-    .. note:: If an if an odd target number was requested,
-              the ``'decimation'`` algorithm used results in the
-              next even number of triangles. For example a reduction request
-              to 30001 triangles may result in 30000 triangles.
-
     Parameters
     ----------
     points : ndarray
@@ -988,6 +1244,13 @@ def decimate_surface(points, triangles, n_triangles):
         The surface to be decimated, a 3 x number of triangles array.
     n_triangles : int
         The desired number of triangles.
+    method : str
+        Can be "quadric" or "sphere". "sphere" will inflate the surface to a
+        sphere using Freesurfer and downsample to an icosahedral or
+        octahedral mesh.
+
+        .. versionadded:: 0.20
+    %(verbose)s
 
     Returns
     -------
@@ -995,9 +1258,36 @@ def decimate_surface(points, triangles, n_triangles):
         The decimated points.
     triangles : ndarray
         The decimated triangles.
+
+    Notes
+    -----
+    **"quadric" mode**
+
+    This requires VTK. If an odd target number was requested,
+    the ``'decimation'`` algorithm used results in the
+    next even number of triangles. For example a reduction request
+    to 30001 triangles may result in 30000 triangles.
+
+    **"sphere" mode**
+
+    This requires Freesurfer to be installed and available in the
+    environment. The destination number of triangles must be one of
+    ``[20, 80, 320, 1280, 5120, 20480]`` for ico (0-5) downsampling or one of
+    ``[8, 32, 128, 512, 2048, 8192, 32768]`` for oct (1-7) downsampling.
+
+    This mode is slower, but could be more suitable for decimating meshes for
+    BEM creation (recommended ``n_triangles=5120``) due to better topological
+    property preservation.
     """
-    reduction = 1 - (float(n_triangles) / len(triangles))
-    return _decimate_surface(points, triangles, reduction)
+    n_triangles = _ensure_int(n_triangles)
+    method_map = dict(quadric=_decimate_surface_vtk,
+                      sphere=_decimate_surface_sphere)
+    _check_option('method', method, sorted(method_map))
+    if n_triangles > len(triangles):
+        raise ValueError('Requested n_triangles (%s) exceeds number of '
+                         'original triangles (%s)'
+                         % (n_triangles, len(triangles)))
+    return method_map[method](points, triangles, n_triangles)
 
 
 ###############################################################################
@@ -1016,11 +1306,11 @@ def read_morph_map(subject_from, subject_to, subjects_dir=None, xhemi=False,
 
     Parameters
     ----------
-    subject_from : string
+    subject_from : str
         Name of the original subject as named in the SUBJECTS_DIR.
-    subject_to : string
+    subject_to : str
         Name of the subject on which to morph as named in the SUBJECTS_DIR.
-    subjects_dir : string
+    subjects_dir : str
         Path to SUBJECTS_DIR is not set in the environment.
     xhemi : bool
         Morph across hemisphere. Currently only implemented for
@@ -1201,6 +1491,7 @@ def _make_morph_map(subject_from, subject_to, subjects_dir, xhemi):
 def _make_morph_map_hemi(subject_from, subject_to, subjects_dir, reg_from,
                          reg_to):
     """Construct morph map for one hemisphere."""
+    from scipy.sparse import csr_matrix, eye as speye
     # add speedy short-circuit for self-maps
     if subject_from == subject_to and reg_from == reg_to:
         fname = op.join(subjects_dir, subject_from, 'surf', reg_from)
@@ -1285,6 +1576,8 @@ def _find_nearest_tri_pts(rrs, pt_triss, pt_lens,
         pqs = np.empty((len(drs), 2))
         dists = np.empty(len(drs))
         dist = np.inf
+        # make life easier for numba var typing
+        p, q, pt = np.float64(0.), np.float64(1.), np.int64(0)
         found = False
         for ii in range(len(drs)):
             pqs[ii] = np.dot(mats[ii], np.dot(r1213s[ii], drs[ii]))
@@ -1372,9 +1665,10 @@ def mesh_edges(tris):
     edges : sparse matrix
         The adjacency matrix.
     """
+    from scipy.sparse import coo_matrix
     if np.max(tris) > len(np.unique(tris)):
-        raise ValueError('Cannot compute connectivity on a selection of '
-                         'triangles.')
+        raise ValueError(
+            'Cannot compute adjacency on a selection of triangles.')
 
     npoints = np.max(tris) + 1
     ones_ntris = np.ones(3 * len(tris))
@@ -1397,15 +1691,16 @@ def mesh_dist(tris, vert):
     Parameters
     ----------
     tris : array (n_tris x 3)
-        Mesh triangulation
+        Mesh triangulation.
     vert : array (n_vert x 3)
-        Vertex locations
+        Vertex locations.
 
     Returns
     -------
     dist_matrix : scipy.sparse.csr_matrix
-        Sparse matrix with distances between adjacent vertices
+        Sparse matrix with distances between adjacent vertices.
     """
+    from scipy.sparse import csr_matrix
     edges = mesh_edges(tris).tocoo()
 
     # Euclidean distances between neighboring vertices
@@ -1435,14 +1730,14 @@ def read_tri(fname_in, swap=False, verbose=None):
         Triangulation (each line contains indices for three points which
         together form a face).
 
-    Notes
-    -----
-    .. versionadded:: 0.13.0
-
     See Also
     --------
     read_surface
     write_surface
+
+    Notes
+    -----
+    .. versionadded:: 0.13.0
     """
     with open(fname_in, "r") as fid:
         lines = fid.readlines()
@@ -1455,10 +1750,10 @@ def read_tri(fname_in, swap=False, verbose=None):
         inds = range(1, 4)
     else:
         raise IOError('Unrecognized format of data.')
-    rr = np.array([np.array([float(v) for v in l.split()])[inds]
-                   for l in lines[1:n_nodes + 1]])
-    tris = np.array([np.array([int(v) for v in l.split()])[inds]
-                     for l in lines[n_nodes + 2:n_nodes + 2 + n_tris]])
+    rr = np.array([np.array([float(v) for v in line.split()])[inds]
+                   for line in lines[1:n_nodes + 1]])
+    tris = np.array([np.array([int(v) for v in line.split()])[inds]
+                     for line in lines[n_nodes + 2:n_nodes + 2 + n_tris]])
     if swap:
         tris[:, [2, 1]] = tris[:, [1, 2]]
     tris -= 1
@@ -1471,38 +1766,27 @@ def read_tri(fname_in, swap=False, verbose=None):
     return (rr, tris)
 
 
+@jit()
 def _get_solids(tri_rrs, fros):
     """Compute _sum_solids_div total angle in chunks."""
     # NOTE: This incorporates the division by 4PI that used to be separate
-    # for tri_rr in tri_rrs:
-    #     v1 = fros - tri_rr[0]
-    #     v2 = fros - tri_rr[1]
-    #     v3 = fros - tri_rr[2]
-    #     triple = np.sum(fast_cross_3d(v1, v2) * v3, axis=1)
-    #     l1 = np.sqrt(np.sum(v1 * v1, axis=1))
-    #     l2 = np.sqrt(np.sum(v2 * v2, axis=1))
-    #     l3 = np.sqrt(np.sum(v3 * v3, axis=1))
-    #     s = (l1 * l2 * l3 +
-    #          np.sum(v1 * v2, axis=1) * l3 +
-    #          np.sum(v1 * v3, axis=1) * l2 +
-    #          np.sum(v2 * v3, axis=1) * l1)
-    #     tot_angle -= np.arctan2(triple, s)
-
-    # This is the vectorized version, but with a slicing heuristic to
-    # prevent memory explosion
     tot_angle = np.zeros((len(fros)))
-    slices = np.r_[np.arange(0, len(fros), 100), [len(fros)]]
-    for i1, i2 in zip(slices[:-1], slices[1:]):
-        # shape (3 verts, n_tri, n_fro, 3 X/Y/Z)
-        vs = (fros[np.newaxis, np.newaxis, i1:i2] -
-              tri_rrs.transpose([1, 0, 2])[:, :, np.newaxis])
-        triples = _fast_cross_nd_sum(vs[0], vs[1], vs[2])
-        ls = np.linalg.norm(vs, axis=3)
-        ss = np.prod(ls, axis=0)
-        ss += einsum('ijk,ijk,ij->ij', vs[0], vs[1], ls[2])
-        ss += einsum('ijk,ijk,ij->ij', vs[0], vs[2], ls[1])
-        ss += einsum('ijk,ijk,ij->ij', vs[1], vs[2], ls[0])
-        tot_angle[i1:i2] = -np.sum(np.arctan2(triples, ss), axis=0)
+    for ti in range(len(tri_rrs)):
+        tri_rr = tri_rrs[ti]
+        v1 = fros - tri_rr[0]
+        v2 = fros - tri_rr[1]
+        v3 = fros - tri_rr[2]
+        v4 = np.empty((v1.shape[0], 3))
+        _jit_cross(v4, v1, v2)
+        triple = np.sum(v4 * v3, axis=1)
+        l1 = np.sqrt(np.sum(v1 * v1, axis=1))
+        l2 = np.sqrt(np.sum(v2 * v2, axis=1))
+        l3 = np.sqrt(np.sum(v3 * v3, axis=1))
+        s = (l1 * l2 * l3 +
+             np.sum(v1 * v2, axis=1) * l3 +
+             np.sum(v1 * v3, axis=1) * l2 +
+             np.sum(v2 * v3, axis=1) * l1)
+        tot_angle -= np.arctan2(triple, s)
     return tot_angle
 
 
