@@ -1,3 +1,4 @@
+import os
 import os.path as op
 
 import numpy as np
@@ -12,7 +13,7 @@ from mne import (read_dipole, read_forward_solution,
                  pick_info, EvokedArray, read_source_spaces, make_ad_hoc_cov,
                  make_forward_solution, Dipole, DipoleFixed, Epochs,
                  make_fixed_length_events, Evoked)
-from mne.dipole import get_phantom_dipoles
+from mne.dipole import get_phantom_dipoles, _BDIP_ERROR_KEYS
 from mne.simulation import simulate_evoked
 from mne.datasets import testing
 from mne.utils import run_tests_if_main, requires_mne, run_subprocess
@@ -24,12 +25,16 @@ from mne.io.constants import FIFF
 from mne.surface import _compute_nearest
 from mne.bem import _bem_find_surface, read_bem_solution
 from mne.transforms import apply_trans, _get_trans
+from mne.source_space import head_to_mni
 
 data_path = testing.data_path(download=False)
 meg_path = op.join(data_path, 'MEG', 'sample')
-fname_dip_xfit = op.join(meg_path, 'sample_audvis-ave_xfit.dip')
+fname_dip_xfit_80 = op.join(meg_path, 'sample_audvis-ave_xfit.dip')
 fname_raw = op.join(meg_path, 'sample_audvis_trunc_raw.fif')
 fname_dip = op.join(meg_path, 'sample_audvis_trunc_set1.dip')
+fname_bdip = op.join(meg_path, 'sample_audvis_trunc_set1.bdip')
+fname_dip_xfit = op.join(meg_path, 'sample_audvis_trunc_xfit.dip')
+fname_bdip_xfit = op.join(meg_path, 'sample_audvis_trunc_xfit.bdip')
 fname_evo = op.join(meg_path, 'sample_audvis_trunc-ave.fif')
 fname_evo_full = op.join(meg_path, 'sample_audvis-ave.fif')
 fname_cov = op.join(meg_path, 'sample_audvis_trunc-cov.fif')
@@ -88,7 +93,7 @@ def test_dipole_fitting_ctf():
     # for now our CTF phantom fitting tutorials will have to do
     # (otherwise we need to add that to the testing dataset, which is
     # a bit too big)
-    fit_dipole(evoked, cov, sphere)
+    fit_dipole(evoked, cov, sphere, rank=dict(meg=len(evoked.data)))
 
 
 @pytest.mark.slowtest
@@ -132,9 +137,17 @@ def test_dipole_fitting(tmpdir):
     # Run mne-python version
     sphere = make_sphere_model(head_radius=0.1)
     with pytest.warns(RuntimeWarning, match='projection'):
-        dip, residual = fit_dipole(evoked, cov, sphere, fname_fwd)
+        dip, residual = fit_dipole(evoked, cov, sphere, fname_fwd,
+                                   rank='info')  # just to test rank support
     assert isinstance(residual, Evoked)
 
+    # Test conversion of dip.pos to MNI coordinates.
+    dip_mni_pos = dip.to_mni('sample', fname_trans,
+                             subjects_dir=subjects_dir)
+    head_to_mni_dip_pos = head_to_mni(dip.pos, 'sample', fwd['mri_head_t'],
+                                      subjects_dir=subjects_dir)
+    assert_allclose(dip_mni_pos, head_to_mni_dip_pos, rtol=1e-3, atol=0)
+
     # Sanity check: do our residuals have less power than orig data?
     data_rms = np.sqrt(np.sum(evoked.data ** 2, axis=0))
     resi_rms = np.sqrt(np.sum(residual.data ** 2, axis=0))
@@ -236,6 +249,9 @@ def test_dipole_fitting_fixed(tmpdir):
     plt.close('all')
     dip_fixed.plot()
     plt.close('all')
+    orig_times = np.array(dip_fixed.times)
+    shift_times = dip_fixed.shift_time(1.).times
+    assert_allclose(shift_times, orig_times + 1)
 
 
 @testing.requires_testing_data
@@ -254,6 +270,7 @@ def test_len_index_dipoles():
     _compare_dipoles(d_mask, dipole[idx])
 
 
+@pytest.mark.slowtest  # slow-ish on Travis OSX
 @testing.requires_testing_data
 def test_min_distance_fit_dipole():
     """Test dipole min_dist to inner_skull."""
@@ -370,7 +387,7 @@ def _check_roundtrip_fixed(dip, tmpdir):
     dip.save(op.join(tempdir, 'test-dip.fif.gz'))
     dip_read = read_dipole(op.join(tempdir, 'test-dip.fif.gz'))
     assert_allclose(dip_read.data, dip_read.data)
-    assert_allclose(dip_read.times, dip.times)
+    assert_allclose(dip_read.times, dip.times, atol=1e-8)
     assert dip_read.info['xplotter_layout'] == dip.info['xplotter_layout']
     assert dip_read.ch_names == dip.ch_names
     for ch_1, ch_2 in zip(dip_read.info['chs'], dip.info['chs']):
@@ -394,7 +411,7 @@ def test_get_phantom_dipoles():
 def test_confidence(tmpdir):
     """Test confidence limits."""
     evoked = read_evokeds(fname_evo_full, 'Left Auditory', baseline=(None, 0))
-    evoked.crop(0.08, 0.08).pick_types()  # MEG-only
+    evoked.crop(0.08, 0.08).pick_types(meg=True)  # MEG-only
     cov = make_ad_hoc_cov(evoked.info)
     sphere = make_sphere_model((0., 0., 0.04), 0.08)
     dip_py = fit_dipole(evoked, cov, sphere)[0]
@@ -402,7 +419,7 @@ def test_confidence(tmpdir):
     dip_py.save(fname_test)
     dip_read = read_dipole(fname_test)
     with pytest.warns(RuntimeWarning, match="'noise/ft/cm', 'prob'"):
-        dip_xfit = read_dipole(fname_dip_xfit)
+        dip_xfit = read_dipole(fname_dip_xfit_80)
     for dip_check in (dip_py, dip_read):
         assert_allclose(dip_check.pos, dip_xfit.pos, atol=5e-4)  # < 0.5 mm
         assert_allclose(dip_check.gof, dip_xfit.gof, atol=5e-1)  # < 0.5%
@@ -414,4 +431,66 @@ def test_confidence(tmpdir):
                             rtol=1.5e-1, err_msg=key)
 
 
+# bdip created with:
+# mne_dipole_fit --meas sample_audvis_trunc-ave.fif --set 1 --meg --tmin 40 --tmax 95 --bmin -200 --bmax 0 --noise sample_audvis_trunc-cov.fif --bem ../../subjects/sample/bem/sample-1280-1280-1280-bem-sol.fif --origin 0\:0\:40 --mri sample_audvis_trunc-trans.fif --bdip sample_audvis_trunc_set1.bdip  # noqa: E501
+# It gives equivalent results to .dip in non-dipole mode.
+# xfit bdip created by taking sample_audvis_trunc-ave.fif, picking MEG
+# channels, writitng to disk (with MNE), then running xfit on 40-95 ms
+# with a 3.3 ms step
+@testing.requires_testing_data
+@pytest.mark.parametrize('fname_dip_, fname_bdip_', [
+    (fname_dip, fname_bdip),
+    (fname_dip_xfit, fname_bdip_xfit),
+])
+def test_bdip(fname_dip_, fname_bdip_, tmpdir):
+    """Test bdip I/O."""
+    # use text as veridical
+    with pytest.warns(None):  # ignored fields
+        dip = read_dipole(fname_dip_)
+    # read binary
+    orig_size = os.stat(fname_bdip_).st_size
+    bdip = read_dipole(fname_bdip_)
+    # test round-trip by writing and reading, too
+    fname = tmpdir.join('test.bdip')
+    bdip.save(fname)
+    bdip_read = read_dipole(fname)
+    write_size = os.stat(str(fname)).st_size
+    assert orig_size == write_size
+    assert len(dip) == len(bdip) == len(bdip_read) == 17
+    dip_has_conf = fname_dip_ == fname_dip_xfit
+    for kind, this_bdip in (('orig', bdip), ('read', bdip_read)):
+        for key, atol in (
+                ('pos', 5e-5),
+                ('ori', 5e-3),
+                ('gof', 0.5e-1),
+                ('times', 5e-5),
+                ('khi2', 1e-2)):
+            d = getattr(dip, key)
+            b = getattr(this_bdip, key)
+            if key == 'khi2' and dip_has_conf:
+                if d is not None:
+                    assert_allclose(d, b, atol=atol,
+                                    err_msg='%s: %s' % (kind, key))
+                else:
+                    assert b is None
+        if dip_has_conf:
+            # conf
+            conf_keys = _BDIP_ERROR_KEYS + ('vol',)
+            assert (set(this_bdip.conf.keys()) ==
+                    set(dip.conf.keys()) ==
+                    set(conf_keys))
+            for key in conf_keys:
+                d = dip.conf[key]
+                b = this_bdip.conf[key]
+                assert_allclose(d, b, rtol=0.12,  # no so great, text I/O
+                                err_msg='%s: %s' % (kind, key))
+        # Not stored
+        assert this_bdip.name is None
+        assert this_bdip.nfree is None
+
+        # Test whether indexing works
+        this_bdip0 = this_bdip[0]
+        _check_dipole(this_bdip0, 1)
+
+
 run_tests_if_main()
