@@ -6,7 +6,7 @@
 import operator
 import numpy as np
 
-from ..fixes import _get_dpss, rfft, irfft, rfftfreq
+from ..fixes import _import_fft
 from ..parallel import parallel_func
 from ..utils import sum_squared, warn, verbose, logger, _check_option
 
@@ -23,14 +23,14 @@ def dpss_windows(N, half_nbw, Kmax, low_bias=True, interp_from=None,
     Parameters
     ----------
     N : int
-        Sequence length
+        Sequence length.
     half_nbw : float
         Standardized half bandwidth corresponding to 2 * half_bw = BW*f0
         = BW*N/dt but with dt taken as 1.
     Kmax : int
-        Number of DPSS windows to return is Kmax (orders 0 through Kmax-1)
+        Number of DPSS windows to return is Kmax (orders 0 through Kmax-1).
     low_bias : bool
-        Keep only tapers with eigenvalues > 0.9
+        Keep only tapers with eigenvalues > 0.9.
     interp_from : int (optional)
         The dpss can be calculated using interpolation from a set of dpss
         with the same NW and Kmax, but shorter N. This is the length of this
@@ -45,12 +45,11 @@ def dpss_windows(N, half_nbw, Kmax, low_bias=True, interp_from=None,
         'zero', 'slinear', 'quadratic, 'cubic') or as an integer specifying the
         order of the spline interpolator to use.
 
-
     Returns
     -------
     v, e : tuple,
-        v is an array of DPSS windows shaped (Kmax, N)
-        e are the eigenvalues
+        The v array contains DPSS windows shaped (Kmax, N).
+        e are the eigenvalues.
 
     Notes
     -----
@@ -61,7 +60,9 @@ def dpss_windows(N, half_nbw, Kmax, low_bias=True, interp_from=None,
     Volume 57 (1978), 1371430
     """
     from scipy import interpolate
+    from scipy.signal.windows import dpss as sp_dpss
     from ..filter import next_fast_len
+    rfft, irfft = _import_fft(('rfft', 'irfft'))
     # This np.int32 business works around a weird Windows bug, see
     # gh-5039 and https://github.com/scipy/scipy/pull/8608
     Kmax = np.int32(operator.index(Kmax))
@@ -93,7 +94,7 @@ def dpss_windows(N, half_nbw, Kmax, low_bias=True, interp_from=None,
         dpss = np.array(dpss)
 
     else:
-        dpss = _get_dpss()(N, half_nbw, Kmax)
+        dpss = sp_dpss(N, half_nbw, Kmax)
 
     # Now find the eigenvalues of the original spectral concentration problem
     # Use the autocorr sequence technique from Percival and Walden, 1993 pg 390
@@ -299,8 +300,9 @@ def _mt_spectra(x, dpss, sfreq, n_fft=None):
     freqs : array
         The frequency points in Hz of the spectra
     """
+    rfft, rfftfreq = _import_fft(('rfft', 'rfftfreq'))
     if n_fft is None:
-        n_fft = x.shape[1]
+        n_fft = x.shape[-1]
 
     # remove mean (do not use in-place subtraction as it may modify input x)
     x = x - np.mean(x, axis=-1, keepdims=True)
@@ -316,9 +318,9 @@ def _mt_spectra(x, dpss, sfreq, n_fft=None):
     for idx, sig in enumerate(x):
         x_mt[idx] = rfft(sig[..., np.newaxis, :] * dpss, n=n_fft)
     # Adjust DC and maybe Nyquist, depending on one-sided transform
-    x_mt[:, :, 0] /= np.sqrt(2.)
+    x_mt[..., 0] /= np.sqrt(2.)
     if x.shape[1] % 2 == 0:
-        x_mt[:, :, -1] /= np.sqrt(2.)
+        x_mt[..., -1] /= np.sqrt(2.)
     return x_mt, freqs
 
 
@@ -410,6 +412,7 @@ def psd_array_multitaper(x, sfreq, fmin=0, fmax=np.inf, bandwidth=None,
     -----
     .. versionadded:: 0.14.0
     """
+    rfftfreq = _import_fft('rfftfreq')
     _check_option('normalization', normalization, ['length', 'full'])
 
     # Reshape data so its 2-D for parallelization
@@ -459,9 +462,10 @@ def tfr_array_multitaper(epoch_data, sfreq, freqs, n_cycles=7.0,
                          zero_mean=True, time_bandwidth=None, use_fft=True,
                          decim=1, output='complex', n_jobs=1,
                          verbose=None):
-    """Compute time-frequency transforms using wavelets and multitaper windows.
+    """Compute Time-Frequency Representation (TFR) using DPSS tapers.
 
-    Uses Morlet wavelets windowed with multiple DPSS tapers.
+    Same computation as `~mne.time_frequency.tfr_multitaper`, but operates on
+    :class:`NumPy arrays <numpy.ndarray>` instead of `~mne.Epochs` objects.
 
     Parameters
     ----------
@@ -472,14 +476,14 @@ def tfr_array_multitaper(epoch_data, sfreq, freqs, n_cycles=7.0,
     freqs : array-like of float, shape (n_freqs,)
         The frequencies.
     n_cycles : float | array of float
-        Number of cycles  in the Morlet wavelet. Fixed number or one per
+        Number of cycles in the wavelet. Fixed number or one per
         frequency. Defaults to 7.0.
     zero_mean : bool
         If True, make sure the wavelets have a mean of zero. Defaults to True.
     time_bandwidth : float
         If None, will be set to 4.0 (3 tapers). Time x (Full) Bandwidth
         product. The number of good tapers (low-bias) is chosen automatically
-        based on this to equal floor(time_bandwidth - 1). Defaults to None
+        based on this to equal floor(time_bandwidth - 1). Defaults to None.
     use_fft : bool
         Use the FFT for convolutions or not. Defaults to True.
     decim : int | slice
@@ -491,7 +495,6 @@ def tfr_array_multitaper(epoch_data, sfreq, freqs, n_cycles=7.0,
         .. note::
             Decimation may create aliasing artifacts, yet decimation
             is done after the convolutions.
-
     output : str, default 'complex'
 
         * 'complex' : single trial complex.
@@ -501,7 +504,6 @@ def tfr_array_multitaper(epoch_data, sfreq, freqs, n_cycles=7.0,
         * 'itc' : inter-trial coherence.
         * 'avg_power_itc' : average of single trial power and inter-trial
           coherence across trials.
-
     %(n_jobs)s
         The number of epochs to process at the same time. The parallelization
         is implemented across channels. Defaults to 1.
@@ -514,7 +516,7 @@ def tfr_array_multitaper(epoch_data, sfreq, freqs, n_cycles=7.0,
         'phase', 'power'], then shape of out is (n_epochs, n_chans, n_freqs,
         n_times), else it is (n_chans, n_freqs, n_times). If output is
         'avg_power_itc', the real values code for 'avg_power' and the
-        imaginary values code for the 'itc': out = avg_power + i * itc
+        imaginary values code for the 'itc': out = avg_power + i * itc.
 
     See Also
     --------
