@@ -7,6 +7,7 @@ import pickle
 from itertools import product
 
 import mne
+from mne.channels import equalize_channels
 from mne.utils import sum_squared, run_tests_if_main, _TempDir, requires_h5py
 from mne.time_frequency import (csd_fourier, csd_multitaper,
                                 csd_morlet, csd_array_fourier,
@@ -57,19 +58,19 @@ def test_csd():
 def test_csd_repr():
     """Test string representation of CrossSpectralDensity."""
     csd = _make_csd()
-    assert str(csd) == ('<CrossSpectralDensity  |  n_channels=3, time=0.0 to '
+    assert str(csd) == ('<CrossSpectralDensity | n_channels=3, time=0.0 to '
                         '1.0 s, frequencies=1.0, 2.0, 3.0, 4.0 Hz.>')
 
-    assert str(csd.mean()) == ('<CrossSpectralDensity  |  n_channels=3, '
+    assert str(csd.mean()) == ('<CrossSpectralDensity | n_channels=3, '
                                'time=0.0 to 1.0 s, frequencies=1.0-4.0 Hz.>')
 
     csd_binned = csd.mean(fmin=[1, 3], fmax=[2, 4])
-    assert str(csd_binned) == ('<CrossSpectralDensity  |  n_channels=3, '
+    assert str(csd_binned) == ('<CrossSpectralDensity | n_channels=3, '
                                'time=0.0 to 1.0 s, frequencies=1.0-2.0, '
                                '3.0-4.0 Hz.>')
 
     csd_binned = csd.mean(fmin=[1, 2], fmax=[1, 4])
-    assert str(csd_binned) == ('<CrossSpectralDensity  |  n_channels=3, '
+    assert str(csd_binned) == ('<CrossSpectralDensity | n_channels=3, '
                                'time=0.0 to 1.0 s, frequencies=1.0, 2.0-4.0 '
                                'Hz.>')
 
@@ -77,7 +78,7 @@ def test_csd_repr():
     csd_no_time.tmin = None
     csd_no_time.tmax = None
     assert str(csd_no_time) == (
-        '<CrossSpectralDensity  |  n_channels=3, time=unknown, '
+        '<CrossSpectralDensity | n_channels=3, time=unknown, '
         'frequencies=1.0, 2.0, 3.0, 4.0 Hz.>'
     )
 
@@ -281,8 +282,8 @@ def test_sym_mat_to_vector():
     # Test complex values: diagonals should be complex conjugates
     comp_vec = np.arange(3) + 1j
     assert_array_equal(_vector_to_sym_mat(comp_vec),
-                       [[0. + 0.j,  1. + 1.j],
-                        [1. - 1.j,  2. + 0.j]])
+                       [[0. + 0.j, 1. + 1.j],
+                        [1. - 1.j, 2. + 0.j]])
 
     # Test preservation of data type
     assert _sym_mat_to_vector(mat.astype(np.int8)).dtype == np.int8
@@ -540,4 +541,14 @@ def test_csd_morlet():
         csd = csd_morlet(epochs_nobase, frequencies=[10], decim=20)
 
 
+def test_equalize_channels():
+    """Test equalization of channels for instances of CrossSpectralDensity."""
+    csd1 = _make_csd()
+    csd2 = csd1.copy().pick_channels(['CH2', 'CH1'], ordered=True)
+    csd1, csd2 = equalize_channels([csd1, csd2])
+
+    assert csd1.ch_names == ['CH1', 'CH2']
+    assert csd2.ch_names == ['CH1', 'CH2']
+
+
 run_tests_if_main()
