@@ -8,7 +8,7 @@ import os.path as op
 import pytest
 import numpy as np
 from numpy.testing import (assert_array_almost_equal, assert_allclose,
-                           assert_equal)
+                           assert_equal, assert_array_less)
 
 from scipy import fftpack
 
@@ -19,7 +19,7 @@ from mne.time_frequency._stockwell import (tfr_stockwell, _st,
                                            _check_input_st,
                                            _st_power_itc)
 
-from mne.time_frequency.tfr import AverageTFR
+from mne.time_frequency import AverageTFR, tfr_array_stockwell
 from mne.utils import run_tests_if_main
 
 base_dir = op.join(op.dirname(__file__), '..', '..', 'io', 'tests', 'data')
@@ -131,6 +131,16 @@ def test_stockwell_api():
     assert (itc.data.max() <= 1.0)
     assert (np.log(power.data.max()) * 20 <= 0.0)
     assert (np.log(power.data.max()) * 20 <= 0.0)
+    with pytest.raises(TypeError, match='ndarray'):
+        tfr_array_stockwell('foo', 1000.)
+    data = np.random.RandomState(0).randn(1, 1024)
+    with pytest.raises(ValueError, match='3D with shape'):
+        tfr_array_stockwell(data, 1000.)
+    data = data[np.newaxis]
+    power, itc, freqs = tfr_array_stockwell(data, 1000., return_itc=True)
+    assert_allclose(itc, np.ones_like(itc))
+    assert power.shape == (1, len(freqs), data.shape[-1])
+    assert_array_less(0, power)
 
 
 run_tests_if_main()
