@@ -12,15 +12,14 @@ import glob
 
 import numpy as np
 from copy import deepcopy
-from scipy import linalg
 
-from .fixes import einsum
+from .fixes import einsum, jit, mean
 from .io.constants import FIFF
 from .io.open import fiff_open
 from .io.tag import read_tag
 from .io.write import start_file, end_file, write_coord_trans
 from .utils import (check_fname, logger, verbose, _ensure_int, _validate_type,
-                    _check_path_like, get_subjects_dir)
+                    _check_path_like, get_subjects_dir, fill_doc, _check_fname)
 
 
 # transformation from anterior/left/superior coordinate system to
@@ -76,12 +75,17 @@ class Transform(dict):
     Parameters
     ----------
     fro : str | int
-        The starting coordinate frame.
+        The starting coordinate frame. See notes for valid coordinate frames.
     to : str | int
-        The ending coordinate frame.
+        The ending coordinate frame. See notes for valid coordinate frames.
     trans : array-like, shape (4, 4) | None
         The transformation matrix. If None, an identity matrix will be
         used.
+
+    Notes
+    -----
+    Valid coordinate frames are 'meg','mri','mri_voxel','head','mri_tal','ras'
+    'fs_tal','ctf_head','ctf_meg','unknown'
     """
 
     def __init__(self, fro, to, trans=None):  # noqa: D102
@@ -98,7 +102,7 @@ class Transform(dict):
         self['trans'] = trans
 
     def __repr__(self):  # noqa: D105
-        return ('<Transform  |  %s->%s>\n%s'
+        return ('<Transform | %s->%s>\n%s'
                 % (_coord_frame_name(self['from']),
                    _coord_frame_name(self['to']), self['trans']))
 
@@ -325,9 +329,9 @@ def rotation3d_align_z_axis(target_z_axis):
 
     # assert that r is a rotation matrix r^t * r = I and det(r) = 1
     assert(np.any((r.dot(r.T) - np.identity(3)) < 1E-12))
-    assert((linalg.det(r) - 1.0) < 1E-12)
+    assert((np.linalg.det(r) - 1.0) < 1E-12)
     # assert that r maps [0 0 1] on the device z axis (target_z_axis)
-    assert(linalg.norm(target_z_axis - r.dot([0, 0, 1])) < 1e-12)
+    assert(np.linalg.norm(target_z_axis - r.dot([0, 0, 1])) < 1e-12)
 
     return r
 
@@ -434,8 +438,12 @@ def _ensure_trans(trans, fro='mri', to='head'):
     return trans
 
 
-def _get_trans(trans, fro='mri', to='head'):
+def _get_trans(trans, fro='mri', to='head', allow_none=True):
     """Get mri_head_t (from=mri, to=head) from mri filename."""
+    types = (Transform, 'path-like')
+    if allow_none:
+        types += (None,)
+    _validate_type(trans, types, 'trans')
     if _check_path_like(trans):
         trans = str(trans)
         if trans == 'fsaverage':
@@ -456,12 +464,10 @@ def _get_trans(trans, fro='mri', to='head'):
     elif isinstance(trans, Transform):
         fro_to_t = trans
         trans = 'instance of Transform'
-    elif trans is None:
+    else:
+        assert trans is None
         fro_to_t = Transform(fro, to)
         trans = 'identity'
-    else:
-        raise ValueError('transform type %s not known, must be str, dict, '
-                         'or None' % type(trans))
     # it's usually a head->MRI transform, so we probably need to invert it
     fro_to_t = _ensure_trans(fro_to_t, fro, to)
     return fro_to_t, trans
@@ -507,7 +513,8 @@ def combine_transforms(t_first, t_second, fro, to):
     return Transform(fro, to, np.dot(t_second['trans'], t_first['trans']))
 
 
-def read_trans(fname, return_all=False):
+@verbose
+def read_trans(fname, return_all=False, verbose=None):
     """Read a -trans.fif file.
 
     Parameters
@@ -519,6 +526,7 @@ def read_trans(fname, return_all=False):
         False (default) will only return the first.
 
         .. versionadded:: 0.15
+    %(verbose)s
 
     Returns
     -------
@@ -578,7 +586,7 @@ def invert_transform(trans):
     inv_trans : dict
         Inverse transform.
     """
-    return Transform(trans['to'], trans['from'], linalg.inv(trans['trans']))
+    return Transform(trans['to'], trans['from'], np.linalg.inv(trans['trans']))
 
 
 def transform_surface_to(surf, dest, trans, copy=False):
@@ -651,12 +659,12 @@ def get_ras_to_neuromag_trans(nasion, lpa, rpa):
                              "arrays of length 3.")
 
     right = rpa - lpa
-    right_unit = right / linalg.norm(right)
+    right_unit = right / np.linalg.norm(right)
 
     origin = lpa + np.dot(nasion - lpa, right_unit) * right_unit
 
     anterior = nasion - origin
-    anterior_unit = anterior / linalg.norm(anterior)
+    anterior_unit = anterior / np.linalg.norm(anterior)
 
     superior_unit = np.cross(right_unit, anterior_unit)
 
@@ -725,7 +733,7 @@ def _sph_to_cart(sph_pts):
 def _get_n_moments(order):
     """Compute the number of multipolar moments (spherical harmonics).
 
-    Equivalent to [1]_ Eq. 32.
+    Equivalent to :footcite:`DarvasEtAl2006` Eq. 32.
 
     .. note:: This count excludes ``degree=0`` (for ``order=0``).
 
@@ -893,16 +901,16 @@ class _TPSWarp(object):
 
     Notes
     -----
-    Adapted from code by `Wang Lin <wanglin193@hotmail.com>`_.
+    Based on the method by :footcite:`Bookstein1989` and
+    adapted from code by Wang Lin (wanglin193@hotmail.com>).
 
     References
     ----------
-    .. [1] Bookstein, F. L. "Principal Warps: Thin Plate Splines and the
-           Decomposition of Deformations." IEEE Trans. Pattern Anal. Mach.
-           Intell. 11, 567-585, 1989.
+    .. footbibliography::
     """
 
     def fit(self, source, destination, reg=1e-3):
+        from scipy import linalg
         from scipy.spatial.distance import cdist
         assert source.shape[1] == destination.shape[1] == 3
         assert source.shape[0] == destination.shape[0]
@@ -971,7 +979,9 @@ class _SphericalSurfaceWarp(object):
     Notes
     -----
     This class can be used to warp data from a source subject to
-    a destination subject, as described in [1]_. The procedure is:
+    a destination subject, as described in :footcite:`DarvasEtAl2006`.
+
+    The procedure is:
 
         1. Perform a spherical harmonic approximation to the source and
            destination surfaces, which smooths them and allows arbitrary
@@ -986,9 +996,7 @@ class _SphericalSurfaceWarp(object):
 
     References
     ----------
-    .. [1] Darvas F, Ermer JJ, Mosher JC, Leahy RM (2006). "Generic head
-           models for atlas-based EEG source analysis."
-           Human Brain Mapping 27:129-143
+    .. footbibliography::
     """
 
     def __repr__(self):
@@ -1031,6 +1039,7 @@ class _SphericalSurfaceWarp(object):
         inst : instance of SphericalSurfaceWarp
             The warping object (for chaining).
         """
+        from scipy import linalg
         from .bem import _fit_sphere
         from .source_space import _check_spacing
         match_rr = _check_spacing(match, verbose=False)[2]['rr']
@@ -1137,13 +1146,14 @@ def _topo_to_sph(topo):
 ###############################################################################
 # Quaternions
 
+@jit()
 def quat_to_rot(quat):
     """Convert a set of quaternions to rotations.
 
     Parameters
     ----------
     quat : array, shape (..., 3)
-        q1, q2, and q3 (x, y, z) parameters of a unit quaternion.
+        The q1, q2, and q3 (x, y, z) parameters of a unit quaternion.
 
     Returns
     -------
@@ -1166,45 +1176,52 @@ def quat_to_rot(quat):
     bc_2 = 2 * b * c
     bd_2 = 2 * b * d
     cd_2 = 2 * c * d
-    rotation = np.array([(aa + bb - cc - dd, bc_2 - ad_2, bd_2 + ac_2),
-                         (bc_2 + ad_2, aa + cc - bb - dd, cd_2 - ab_2),
-                         (bd_2 - ac_2, cd_2 + ab_2, aa + dd - bb - cc),
-                         ])
-    if quat.ndim > 1:
-        rotation = np.rollaxis(np.rollaxis(rotation, 1, quat.ndim + 1),
-                               0, quat.ndim)
+    rotation = np.empty(quat.shape[:-1] + (3, 3))
+    rotation[..., 0, 0] = aa + bb - cc - dd
+    rotation[..., 0, 1] = bc_2 - ad_2
+    rotation[..., 0, 2] = bd_2 + ac_2
+    rotation[..., 1, 0] = bc_2 + ad_2
+    rotation[..., 1, 1] = aa + cc - bb - dd
+    rotation[..., 1, 2] = cd_2 - ab_2
+    rotation[..., 2, 0] = bd_2 - ac_2
+    rotation[..., 2, 1] = cd_2 + ab_2
+    rotation[..., 2, 2] = aa + dd - bb - cc
     return rotation
 
 
+@jit()
 def _one_rot_to_quat(rot):
     """Convert a rotation matrix to quaternions."""
     # see e.g. http://www.euclideanspace.com/maths/geometry/rotations/
     #                 conversions/matrixToQuaternion/
+    det = np.linalg.det(np.reshape(rot, (3, 3)))
+    if np.abs(det - 1.) > 1e-3:
+        raise ValueError('Matrix is not a pure rotation, got determinant != 1')
     t = 1. + rot[0] + rot[4] + rot[8]
     if t > np.finfo(rot.dtype).eps:
         s = np.sqrt(t) * 2.
+        # qw = 0.25 * s
         qx = (rot[7] - rot[5]) / s
         qy = (rot[2] - rot[6]) / s
         qz = (rot[3] - rot[1]) / s
-        # qw = 0.25 * s
     elif rot[0] > rot[4] and rot[0] > rot[8]:
         s = np.sqrt(1. + rot[0] - rot[4] - rot[8]) * 2.
+        # qw = (rot[7] - rot[5]) / s
         qx = 0.25 * s
         qy = (rot[1] + rot[3]) / s
         qz = (rot[2] + rot[6]) / s
-        # qw = (rot[7] - rot[5]) / s
     elif rot[4] > rot[8]:
         s = np.sqrt(1. - rot[0] + rot[4] - rot[8]) * 2
+        # qw = (rot[2] - rot[6]) / s
         qx = (rot[1] + rot[3]) / s
         qy = 0.25 * s
         qz = (rot[5] + rot[7]) / s
-        # qw = (rot[2] - rot[6]) / s
     else:
         s = np.sqrt(1. - rot[0] - rot[4] + rot[8]) * 2.
+        # qw = (rot[3] - rot[1]) / s
         qx = (rot[2] + rot[6]) / s
         qy = (rot[5] + rot[7]) / s
         qz = 0.25 * s
-        # qw = (rot[3] - rot[1]) / s
     return np.array((qx, qy, qz))
 
 
@@ -1230,17 +1247,51 @@ def rot_to_quat(rot):
     return np.apply_along_axis(_one_rot_to_quat, -1, rot)
 
 
+def _quat_to_affine(quat):
+    assert quat.shape == (6,)
+    affine = np.eye(4)
+    affine[:3, :3] = quat_to_rot(quat[:3])
+    affine[:3, 3] = quat[3:]
+    return affine
+
+
 def _angle_between_quats(x, y):
     """Compute the ang between two quaternions w/3-element representations."""
-    # convert to complete quaternion representation
-    # use max() here to be safe in case roundoff errs put us over
-    x0 = np.sqrt(np.maximum(1. - x[..., 0] ** 2 -
-                            x[..., 1] ** 2 - x[..., 2] ** 2, 0.))
-    y0 = np.sqrt(np.maximum(1. - y[..., 0] ** 2 -
-                            y[..., 1] ** 2 - y[..., 2] ** 2, 0.))
-    # the difference z = x * conj(y), and theta = np.arccos(z0)
-    z0 = np.maximum(np.minimum(y0 * x0 + (x * y).sum(axis=-1), 1.), -1)
-    return 2 * np.arccos(z0)
+    # z = conj(x) * y
+    # conjugate just negates all but the first element in a 4-element quat,
+    # so it's just a negative for us
+    z = _quat_mult(-x, y)
+    z0 = _quat_real(z)
+    return 2 * np.arctan2(np.linalg.norm(z, axis=-1), z0)
+
+
+def _quat_real(quat):
+    """Get the real part of our 3-element quat."""
+    assert quat.shape[-1] == 3, quat.shape[-1]
+    return np.sqrt(np.maximum(1. -
+                              quat[..., 0] * quat[..., 0] -
+                              quat[..., 1] * quat[..., 1] -
+                              quat[..., 2] * quat[..., 2], 0.))
+
+
+def _quat_mult(one, two):
+    assert one.shape[-1] == two.shape[-1] == 3
+    w1 = _quat_real(one)
+    w2 = _quat_real(two)
+    out = np.empty(np.broadcast(one, two).shape)
+    # Most mathematical expressions use this sort of notation
+    x1, x2 = one[..., 0], two[..., 0]
+    y1, y2 = one[..., 1], two[..., 1]
+    z1, z2 = one[..., 2], two[..., 2]
+    out[..., 0] = w1 * x2 + x1 * w2 + y1 * z2 - z1 * y2
+    out[..., 1] = w1 * y2 - x1 * z2 + y1 * w2 + z1 * x2
+    out[..., 2] = w1 * z2 + x1 * y2 - y1 * x2 + z1 * w2
+    # only need to compute w because we need signs from it
+    w = w1 * w2 - x1 * x2 - y1 * y2 - z1 * z2
+    signs = np.sign(w)
+    signs = np.where(signs, signs, 1)
+    out *= signs[..., np.newaxis]
+    return out
 
 
 def _skew_symmetric_cross(a):
@@ -1264,8 +1315,75 @@ def _find_vector_rotation(a, b):
     return R
 
 
+@jit()
+def _fit_matched_points(p, x, weights=None, scale=False):
+    """Fit matched points using an analytical formula."""
+    # Follow notation of P.J. Besl and N.D. McKay, A Method for
+    # Registration of 3-D Shapes, IEEE Trans. Patt. Anal. Machine Intell., 14,
+    # 239 - 255, 1992.
+    #
+    # The original method is actually by Horn, Closed-form solution of absolute
+    # orientation using unit quaternions, J Opt. Soc. Amer. A vol 4 no 4
+    # pp 629-642, Apr. 1987. This paper describes how weights can be
+    # easily incorporated, and a uniform scale factor can be computed.
+    #
+    # Caution: This can be dangerous if there are 3 points, or 4 points in
+    #          a symmetric layout, as the geometry can be explained
+    #          equivalently under 180 degree rotations.
+    #
+    # Eventually this can be extended to also handle a uniform scale factor,
+    # as well.
+    assert p.shape == x.shape
+    assert p.ndim == 2
+    assert p.shape[1] == 3
+    # (weighted) centroids
+    if weights is None:
+        mu_p = mean(p, axis=0)  # eq 23
+        mu_x = mean(x, axis=0)
+        dots = np.dot(p.T, x)
+        dots /= p.shape[0]
+    else:
+        weights_ = np.reshape(weights / weights.sum(), (weights.size, 1))
+        mu_p = np.dot(weights_.T, p)[0]
+        mu_x = np.dot(weights_.T, x)[0]
+        dots = np.dot(p.T, weights_ * x)
+    Sigma_px = dots - np.outer(mu_p, mu_x)  # eq 24
+    # x and p should no longer be used
+    A_ij = Sigma_px - Sigma_px.T
+    Delta = np.array([A_ij[1, 2], A_ij[2, 0], A_ij[0, 1]])
+    tr_Sigma_px = np.trace(Sigma_px)
+    # "N" in Horn:
+    Q = np.empty((4, 4))
+    Q[0, 0] = tr_Sigma_px
+    Q[0, 1:] = Delta
+    Q[1:, 0] = Delta
+    Q[1:, 1:] = Sigma_px + Sigma_px.T - tr_Sigma_px * np.eye(3)
+    _, v = np.linalg.eigh(Q)  # sorted ascending
+    quat = np.empty(6)
+    quat[:3] = v[1:, -1]
+    if v[0, -1] != 0:
+        quat[:3] *= np.sign(v[0, -1])
+    rot = quat_to_rot(quat[:3])
+    # scale factor is easy once we know the rotation
+    if scale:  # p is "right" (from), x is "left" (to) in Horn 1987
+        dev_x = x - mu_x
+        dev_p = p - mu_p
+        dev_x *= dev_x
+        dev_p *= dev_p
+        if weights is not None:
+            dev_x *= weights_
+            dev_p *= weights_
+        s = np.sqrt(np.sum(dev_x) / np.sum(dev_p))
+    else:
+        s = 1.
+    # translation is easy once rotation and scale are known
+    quat[3:] = mu_x - s * np.dot(rot, mu_p)
+    return quat, s
+
+
 def _average_quats(quats, weights=None):
     """Average unit quaternions properly."""
+    from scipy import linalg
     assert quats.ndim == 2 and quats.shape[1] in (3, 4)
     if weights is None:
         weights = np.ones(quats.shape[0])
@@ -1286,30 +1404,45 @@ def _average_quats(quats, weights=None):
     #
     # We use unit quats and don't store the last element, so reconstruct it
     # to get our 4-element quaternions:
-    res = np.maximum(1. - np.sum(quats * quats, axis=-1, keepdims=True), 0.)
-    np.sqrt(res, out=res)
-    quats = np.concatenate((quats, res), axis=-1)
+    quats = np.concatenate((_quat_real(quats)[..., np.newaxis], quats), -1)
     quats *= weights[:, np.newaxis]
     A = np.einsum('ij,ik->jk', quats, quats)  # sum of outer product of each q
     avg_quat = linalg.eigh(A)[1][:, -1]  # largest eigenvector is the avg
     # Same as the largest eigenvector from the concatenation of all as
     # linalg.svd(quats, full_matrices=False)[-1][0], but faster.
-    avg_quat = avg_quat[:3]
-    if avg_quat[-1] != 0:
-        avg_quat *= np.sign(avg_quat[-1])
+    #
+    # By local convention we take the real term (which we remove from our
+    # representation) as positive. Since it can be zero, let's just ensure
+    # that the first non-zero element is positive. This shouldn't matter once
+    # we go to a rotation matrix, but it's nice for testing to have
+    # consistency.
+    avg_quat *= np.sign(avg_quat[avg_quat != 0][0])
+    avg_quat = avg_quat[1:]
     return avg_quat
 
 
-def _read_ras_mni_t(subject, subjects_dir=None):
+@fill_doc
+def read_ras_mni_t(subject, subjects_dir=None):
+    """Read a subject's RAS to MNI transform.
+
+    Parameters
+    ----------
+    subject : str
+        The subject.
+    %(subjects_dir)s
+
+    Returns
+    -------
+    ras_mni_t : instance of Transform
+        The transform from RAS to MNI (in mm).
+    """
     subjects_dir = get_subjects_dir(subjects_dir=subjects_dir,
                                     raise_error=True)
     _validate_type(subject, 'str', 'subject')
     fname = op.join(subjects_dir, subject, 'mri', 'transforms',
                     'talairach.xfm')
-    if not op.isfile(fname):
-        raise FileNotFoundError(
-            'FreeSurfer Talairach transformation file not found: %s'
-            % (fname,))
+    fname = _check_fname(
+        fname, 'read', True, 'FreeSurfer Talairach transformation file')
     return Transform('ras', 'mni_tal', _read_fs_xfm(fname)[0])
 
 
@@ -1353,6 +1486,38 @@ def _write_fs_xfm(fname, xfm, kind):
         fid.write((kind + '\n\nTtransform_Type = Linear;\n').encode('ascii'))
         fid.write(u'Linear_Transform =\n'.encode('ascii'))
         for li, line in enumerate(xfm[:-1]):
-            line = ' '.join(['%0.6f' % l for l in line])
+            line = ' '.join(['%0.6f' % part for part in line])
             line += '\n' if li < 2 else ';\n'
             fid.write(line.encode('ascii'))
+
+
+def _quat_to_euler(quat):
+    euler = np.empty(quat.shape)
+    x, y, z = quat[..., 0], quat[..., 1], quat[..., 2]
+    w = _quat_real(quat)
+    np.arctan2(2 * (w * x + y * z), 1 - 2 * (x * x + y * y), out=euler[..., 0])
+    np.arcsin(2 * (w * y - x * z), out=euler[..., 1])
+    np.arctan2(2 * (w * z + x * y), 1 - 2 * (y * y + z * z), out=euler[..., 2])
+    return euler
+
+
+def _euler_to_quat(euler):
+    quat = np.empty(euler.shape)
+    phi, theta, psi = euler[..., 0] / 2, euler[..., 1] / 2, euler[..., 2] / 2
+    cphi, sphi = np.cos(phi), np.sin(phi)
+    del phi
+    ctheta, stheta = np.cos(theta), np.sin(theta)
+    del theta
+    cpsi, spsi = np.cos(psi), np.sin(psi)
+    del psi
+    mult = np.sign(cphi * ctheta * cpsi + sphi * stheta * spsi)
+    if np.isscalar(mult):
+        mult = 1. if mult == 0 else mult
+    else:
+        mult[mult == 0] = 1.
+    mult = mult[..., np.newaxis]
+    quat[..., 0] = sphi * ctheta * cpsi - cphi * stheta * spsi
+    quat[..., 1] = cphi * stheta * cpsi + sphi * ctheta * spsi
+    quat[..., 2] = cphi * ctheta * spsi - sphi * stheta * cpsi
+    quat *= mult
+    return quat
