@@ -12,8 +12,9 @@ import sys
 import logging
 import os.path as op
 import warnings
+from typing import Any, Callable, TypeVar
 
-from ..fixes import _get_args
+from .docs import fill_doc
 from ..externals.decorator import FunctionMaker
 
 
@@ -21,7 +22,40 @@ logger = logging.getLogger('mne')  # one selection here used across mne-python
 logger.propagate = False  # don't propagate (in case of multiple imports)
 
 
-def verbose(function):
+# class to provide frame information (should be low overhead, just on logger
+# calls)
+
+class _FrameFilter(logging.Filter):
+    def __init__(self):
+        self.add_frames = 0
+
+    def filter(self, record):
+        record.frame_info = 'Unknown'
+        if self.add_frames:
+            # 5 is the offset necessary to get out of here and the logging
+            # module, reversal is to put the oldest at the top
+            frame_info = _frame_info(5 + self.add_frames)[5:][::-1]
+            if len(frame_info):
+                frame_info[-1] = (frame_info[-1] + ' :').ljust(30)
+                if len(frame_info) > 1:
+                    frame_info[0] = '┌' + frame_info[0]
+                    frame_info[-1] = '└' + frame_info[-1]
+                for ii, info in enumerate(frame_info[1:-1], 1):
+                    frame_info[ii] = '├' + info
+                record.frame_info = '\n'.join(frame_info)
+        return True
+
+
+_filter = _FrameFilter()
+logger.addFilter(_filter)
+
+
+# Provide help for static type checkers:
+# https://mypy.readthedocs.io/en/stable/generics.html#declaring-decorators
+_FuncT = TypeVar('_FuncT', bound=Callable[..., Any])
+
+
+def verbose(function: _FuncT) -> _FuncT:
     """Verbose decorator to allow functions to override log-level.
 
     Parameters
@@ -32,17 +66,7 @@ def verbose(function):
     Returns
     -------
     dec : callable
-        The decorated function
-
-    Examples
-    --------
-    You can use the ``verbose`` argument to set the verbose level on the fly::
-        >>> import mne
-        >>> cov = mne.compute_raw_covariance(raw, verbose='WARNING')  # doctest: +SKIP
-        >>> cov = mne.compute_raw_covariance(raw, verbose='INFO')  # doctest: +SKIP
-        Using up to 49 segments
-        Number of samples used : 5880
-        [done]
+        The decorated function.
 
     See Also
     --------
@@ -58,40 +82,57 @@ def verbose(function):
     verbosity level for all functions, use :func:`mne.set_log_level`.
 
     This function also serves as a docstring filler.
+
+    Examples
+    --------
+    You can use the ``verbose`` argument to set the verbose level on the fly::
+
+        >>> import mne
+        >>> cov = mne.compute_raw_covariance(raw, verbose='WARNING')  # doctest: +SKIP
+        >>> cov = mne.compute_raw_covariance(raw, verbose='INFO')  # doctest: +SKIP
+        Using up to 49 segments
+        Number of samples used : 5880
+        [done]
     """  # noqa: E501
     # See https://decorator.readthedocs.io/en/latest/tests.documentation.html
     # #dealing-with-third-party-decorators
-    from .docs import fill_doc
-    arg_names = _get_args(function)
     try:
         fill_doc(function)
     except TypeError:  # nothing to add
         pass
 
-    def wrapper(*args, **kwargs):
-        default_level = verbose_level = None
-        if len(arg_names) > 0 and arg_names[0] == 'self':
-            default_level = getattr(args[0], 'verbose', None)
-        if 'verbose' in kwargs:
-            verbose_level = kwargs.pop('verbose')
-        else:
+    # Anything using verbose should either have `verbose=None` in the signature
+    # or have a `self.verbose` attribute (if in a method). This code path
+    # will raise an error if neither is the case.
+    body = """\
+def %(name)s(%(signature)s):\n
+    try:
+        verbose
+    except UnboundLocalError:
+        try:
+            verbose = self.verbose
+        except NameError:
+            raise RuntimeError('Function %%s does not accept verbose parameter'
+                               %% (_function_,))
+        except AttributeError:
+            raise RuntimeError('Method %%s class does not have self.verbose'
+                               %% (_function_,))
+    else:
+        if verbose is None:
             try:
-                verbose_level = args[arg_names.index('verbose')]
-            except (IndexError, ValueError):
+                verbose = self.verbose
+            except (NameError, AttributeError):
                 pass
-
-        # This ensures that object.method(verbose=None) will use object.verbose
-        if verbose_level is None:
-            verbose_level = default_level
-        if verbose_level is not None:
-            # set it back if we get an exception
-            with use_log_level(verbose_level):
-                return function(*args, **kwargs)
-        return function(*args, **kwargs)
-    return FunctionMaker.create(
-        function, 'return decfunc(%(signature)s)',
-        dict(decfunc=wrapper), __wrapped__=function,
-        __qualname__=function.__qualname__)
+    if verbose is not None:
+        with _use_log_level_(verbose):
+            return _function_(%(shortsignature)s)
+    else:
+        return _function_(%(shortsignature)s)"""
+    evaldict = dict(
+        _use_log_level_=use_log_level, _function_=function)
+    fm = FunctionMaker(function, None, None, None, None, function.__module__)
+    attrs = dict(__wrapped__=function, __qualname__=function.__qualname__)
+    return fm.make(body, evaldict, addsource=True, **attrs)
 
 
 class use_log_level(object):
@@ -101,19 +142,24 @@ class use_log_level(object):
     ----------
     level : int
         The level to use.
+    add_frames : int | None
+        Number of stack frames to include.
     """
 
-    def __init__(self, level):  # noqa: D102
+    def __init__(self, level, add_frames=None):  # noqa: D102
         self.level = level
+        self.add_frames = add_frames
+        self.old_frames = _filter.add_frames
 
     def __enter__(self):  # noqa: D105
-        self.old_level = set_log_level(self.level, True)
+        self.old_level = set_log_level(self.level, True, self.add_frames)
 
     def __exit__(self, *args):  # noqa: D105
-        set_log_level(self.old_level)
+        add_frames = self.old_frames if self.add_frames is not None else None
+        set_log_level(self.old_level, add_frames=add_frames)
 
 
-def set_log_level(verbose=None, return_old_level=False):
+def set_log_level(verbose=None, return_old_level=False, add_frames=None):
     """Set the logging level.
 
     Parameters
@@ -127,8 +173,19 @@ def set_log_level(verbose=None, return_old_level=False):
         it doesn't exist, defaults to INFO.
     return_old_level : bool
         If True, return the old verbosity level.
+    add_frames : int | None
+        If int, enable (>=1) or disable (0) the printing of stack frame
+        information using formatting. Default (None) does not change the
+        formatting. This can add overhead so is meant only for debugging.
+
+    Returns
+    -------
+    old_level : int
+        The old level. Only returned if ``return_old_level`` is True.
     """
     from .config import get_config
+    from .check import _check_option, _validate_type
+    _validate_type(verbose, (bool, str, int, None), 'verbose')
     if verbose is None:
         verbose = get_config('MNE_LOGGING_LEVEL', 'INFO')
     elif isinstance(verbose, bool):
@@ -141,12 +198,18 @@ def set_log_level(verbose=None, return_old_level=False):
         logging_types = dict(DEBUG=logging.DEBUG, INFO=logging.INFO,
                              WARNING=logging.WARNING, ERROR=logging.ERROR,
                              CRITICAL=logging.CRITICAL)
-        if verbose not in logging_types:
-            raise ValueError('verbose must be of a valid type')
+        _check_option('verbose', verbose, logging_types, '(when a string)')
         verbose = logging_types[verbose]
-    logger = logging.getLogger('mne')
     old_verbose = logger.level
-    logger.setLevel(verbose)
+    if verbose != old_verbose:
+        logger.setLevel(verbose)
+    if add_frames is not None:
+        _filter.add_frames = int(add_frames)
+        fmt = '%(frame_info)s ' if add_frames else ''
+        fmt += '%(message)s'
+        fmt = logging.Formatter(fmt)
+        for handler in logger.handlers:
+            handler.setFormatter(fmt)
     return (old_verbose if return_old_level else None)
 
 
@@ -170,14 +233,7 @@ def set_log_file(fname=None, output_format='%(message)s', overwrite=None):
         but additionally raises a warning to notify the user that log
         entries will be appended.
     """
-    logger = logging.getLogger('mne')
-    handlers = logger.handlers
-    for h in handlers:
-        # only remove our handlers (get along nicely with nose)
-        if isinstance(h, (logging.FileHandler, logging.StreamHandler)):
-            if isinstance(h, logging.FileHandler):
-                h.close()
-            logger.removeHandler(h)
+    _remove_close_handlers(logger)
     if fname is not None:
         if op.isfile(fname) and overwrite is None:
             # Don't use warn() here because we just want to
@@ -200,6 +256,26 @@ def set_log_file(fname=None, output_format='%(message)s', overwrite=None):
     logger.addHandler(lh)
 
 
+def _remove_close_handlers(logger):
+    for h in list(logger.handlers):
+        # only remove our handlers (get along nicely with nose)
+        if isinstance(h, (logging.FileHandler, logging.StreamHandler)):
+            if isinstance(h, logging.FileHandler):
+                h.close()
+            logger.removeHandler(h)
+
+
+class ClosingStringIO(StringIO):
+    """StringIO that closes after getvalue()."""
+
+    def getvalue(self, close=True):
+        """Get the value."""
+        out = super().getvalue()
+        if close:
+            self.close()
+        return out
+
+
 class catch_logging(object):
     """Store logging.
 
@@ -208,12 +284,11 @@ class catch_logging(object):
     """
 
     def __enter__(self):  # noqa: D105
-        self._data = StringIO()
+        self._data = ClosingStringIO()
         self._lh = logging.StreamHandler(self._data)
         self._lh.setFormatter(logging.Formatter('%(message)s'))
         self._lh._mne_file_like = True  # monkey patch for warn() use
-        for lh in logger.handlers:
-            logger.removeHandler(lh)
+        _remove_close_handlers(logger)
         logger.addHandler(self._lh)
         return self._data
 
@@ -238,7 +313,7 @@ class WrapStdOut(object):
             raise AttributeError("'file' object has not attribute '%s'" % name)
 
 
-_verbose_dec_re = re.compile('^<.*decorator\\.py:decorator-gen.*>$')
+_verbose_dec_re = re.compile('^<decorator-gen-[0-9]+>$')
 
 
 def warn(message, category=RuntimeWarning, module='mne'):
@@ -263,24 +338,18 @@ def warn(message, category=RuntimeWarning, module='mne'):
     root_dir = op.dirname(mne.__file__)
     frame = None
     if logger.level <= logging.WARN:
-        last_fname = ''
         frame = inspect.currentframe()
         while frame:
             fname = frame.f_code.co_filename
             lineno = frame.f_lineno
             # in verbose dec
-            if _verbose_dec_re.match(fname) and \
-                    last_fname == op.basename(__file__):
-                last_fname = fname
-                frame = frame.f_back
-                continue
-            # treat tests as scripts
-            # and don't capture unittest/case.py (assert_raises)
-            if not (fname.startswith(root_dir) or
-                    ('unittest' in fname and 'case' in fname)) or \
-                    op.basename(op.dirname(fname)) == 'tests':
-                break
-            last_fname = op.basename(fname)
+            if not _verbose_dec_re.search(fname):
+                # treat tests as scripts
+                # and don't capture unittest/case.py (assert_raises)
+                if not (fname.startswith(root_dir) or
+                        ('unittest' in fname and 'case' in fname)) or \
+                        op.basename(op.dirname(fname)) == 'tests':
+                    break
             frame = frame.f_back
         del frame
         # We need to use this instead of warn(message, category, stacklevel)
@@ -291,12 +360,25 @@ def warn(message, category=RuntimeWarning, module='mne'):
             globals().get('__warningregistry__', {}))
     # To avoid a duplicate warning print, we only emit the logger.warning if
     # one of the handlers is a FileHandler. See gh-5592
+    # But it's also nice to be able to do:
+    # with mne.utils.use_log_level('warning', add_frames=3):
+    # so also check our add_frames attribute.
     if any(isinstance(h, logging.FileHandler) or getattr(h, '_mne_file_like',
                                                          False)
-           for h in logger.handlers):
+           for h in logger.handlers) or _filter.add_frames:
         logger.warning(message)
 
 
+def _get_call_line():
+    """Get the call line from within a function."""
+    frame = inspect.currentframe().f_back.f_back
+    if _verbose_dec_re.search(frame.f_code.co_filename):
+        frame = frame.f_back
+    context = inspect.getframeinfo(frame).code_context
+    context = 'unknown' if context is None else context[0].strip()
+    return context
+
+
 def filter_out_warnings(warn_record, category=None, match=None):
     r"""Remove particular records from ``warn_record``.
 
@@ -360,14 +442,50 @@ class ETSContext(object):
 
 
 @contextlib.contextmanager
-def wrapped_stdout(indent=''):
-    """Wrap stdout writes to logger.info, with an optional indent prefix."""
+def wrapped_stdout(indent='', cull_newlines=False):
+    """Wrap stdout writes to logger.info, with an optional indent prefix.
+
+    Parameters
+    ----------
+    indent : str
+        The indentation to add.
+    cull_newlines : bool
+        If True, cull any new/blank lines at the end.
+    """
     orig_stdout = sys.stdout
-    my_out = StringIO()
+    my_out = ClosingStringIO()
     sys.stdout = my_out
     try:
         yield
     finally:
         sys.stdout = orig_stdout
+        pending_newlines = 0
         for line in my_out.getvalue().split('\n'):
+            if not line.strip() and cull_newlines:
+                pending_newlines += 1
+                continue
+            for _ in range(pending_newlines):
+                logger.info('\n')
             logger.info(indent + line)
+
+
+def _frame_info(n):
+    frame = inspect.currentframe()
+    try:
+        frame = frame.f_back
+        infos = list()
+        for _ in range(n):
+            try:
+                name = frame.f_globals['__name__']
+            except KeyError:  # in our verbose dec
+                pass
+            else:
+                infos.append(f'{name.lstrip("mne.")}:{frame.f_lineno}')
+            frame = frame.f_back
+            if frame is None:
+                break
+        return infos
+    except Exception:
+        return ['unknown']
+    finally:
+        del frame
