@@ -19,9 +19,8 @@ import warnings
 
 import numpy as np
 from numpy.testing import assert_array_equal, assert_allclose
-from scipy import linalg
 
-from ._logging import warn
+from ._logging import warn, ClosingStringIO
 from .numerics import object_diff
 
 
@@ -65,12 +64,9 @@ class _TempDir(str):
         rmtree(self._path, ignore_errors=True)
 
 
-def requires_nibabel(vox2ras_tkr=False):
-    """Check for nibabel."""
-    import pytest
-    extra = ' with vox2ras_tkr support' if vox2ras_tkr else ''
-    return pytest.mark.skipif(not has_nibabel(vox2ras_tkr),
-                              reason='Requires nibabel%s' % extra)
+def requires_nibabel():
+    """Wrap to requires_module with a function call (fewer lines to change)."""
+    return partial(requires_module, name='nibabel')
 
 
 def requires_dipy():
@@ -125,14 +121,6 @@ if version < '0.8.0':
     raise ImportError
 """
 
-_sklearn_call = """
-required_version = '0.14'
-import sklearn
-version = LooseVersion(sklearn.__version__)
-if version < required_version:
-    raise ImportError
-"""
-
 _mayavi_call = """
 with warnings.catch_warnings(record=True):  # traits
     from mayavi import mlab
@@ -153,25 +141,34 @@ if 'NEUROMAG2FT_ROOT' not in os.environ:
     raise ImportError
 """
 
-_fs_or_ni_call = """
-if not has_nibabel() and not has_freesurfer():
-    raise ImportError
-"""
-
 requires_pandas = partial(requires_module, name='pandas', call=_pandas_call)
 requires_pylsl = partial(requires_module, name='pylsl')
-requires_sklearn = partial(requires_module, name='sklearn', call=_sklearn_call)
+requires_sklearn = partial(requires_module, name='sklearn')
 requires_mayavi = partial(requires_module, name='mayavi', call=_mayavi_call)
 requires_mne = partial(requires_module, name='MNE-C', call=_mne_call)
-requires_freesurfer = partial(requires_module, name='Freesurfer',
-                              call=_fs_call)
+
+
+def requires_freesurfer(arg):
+    """Require Freesurfer."""
+    if isinstance(arg, str):
+        # Calling as  @requires_freesurfer('progname'): return decorator
+        # after checking for progname existence
+        call = """
+from . import run_subprocess
+run_subprocess([%r, '--version'])
+""" % (arg,)
+        return partial(
+            requires_module, name='Freesurfer (%s)' % (arg,), call=call)
+    else:
+        # Calling directly as @requires_freesurfer: return decorated function
+        # and just check env var existence
+        return requires_module(arg, name='Freesurfer', call=_fs_call)
+
+
 requires_neuromag2ft = partial(requires_module, name='neuromag2ft',
                                call=_n2ft_call)
-requires_fs_or_nibabel = partial(requires_module, name='nibabel or Freesurfer',
-                                 call=_fs_or_ni_call)
 
-requires_tvtk = partial(requires_module, name='TVTK',
-                        call='from tvtk.api import tvtk')
+requires_vtk = partial(requires_module, name='vtk')
 requires_pysurfer = partial(requires_module, name='PySurfer',
                             call="""import warnings
 with warnings.catch_warnings(record=True):
@@ -182,7 +179,11 @@ requires_good_network = partial(
          '    raise ImportError')
 requires_nitime = partial(requires_module, name='nitime')
 requires_h5py = partial(requires_module, name='h5py')
-requires_numpydoc = partial(requires_module, name='numpydoc')
+
+
+def requires_numpydoc(func):
+    """Decorate tests that need numpydoc."""
+    return requires_version('numpydoc', '1.0')(func)  # validate needs 1.0
 
 
 def check_version(library, min_version):
@@ -275,8 +276,8 @@ class ArgvSetter(object):
     def __init__(self, args=(), disable_stdout=True,
                  disable_stderr=True):  # noqa: D102
         self.argv = list(('python',) + args)
-        self.stdout = StringIO() if disable_stdout else sys.stdout
-        self.stderr = StringIO() if disable_stderr else sys.stderr
+        self.stdout = ClosingStringIO() if disable_stdout else sys.stdout
+        self.stderr = ClosingStringIO() if disable_stderr else sys.stderr
 
     def __enter__(self):  # noqa: D105
         self.orig_argv = sys.argv
@@ -296,38 +297,34 @@ class ArgvSetter(object):
 class SilenceStdout(object):
     """Silence stdout."""
 
+    def __init__(self, close=True):
+        self.close = close
+
     def __enter__(self):  # noqa: D105
         self.stdout = sys.stdout
         sys.stdout = StringIO()
         return sys.stdout
 
     def __exit__(self, *args):  # noqa: D105
+        if self.close:
+            sys.stdout.close()
         sys.stdout = self.stdout
 
 
-def has_nibabel(vox2ras_tkr=False):
+def has_nibabel():
     """Determine if nibabel is installed.
 
-    Parameters
-    ----------
-    vox2ras_tkr : bool
-        If True, require nibabel has vox2ras_tkr support.
-
     Returns
     -------
     has : bool
         True if the user has nibabel.
     """
     try:
-        import nibabel
-        out = True
-        if vox2ras_tkr:  # we need MGHHeader to have vox2ras_tkr param
-            out = (getattr(getattr(getattr(nibabel, 'MGHImage', 0),
-                                   'header_class', 0),
-                           'get_vox2ras_tkr', None) is not None)
-        return out
+        import nibabel  # noqa
     except ImportError:
         return False
+    else:
+        return True
 
 
 def has_mne_c():
@@ -374,10 +371,14 @@ def assert_object_equal(a, b):
 
 def _raw_annot(meas_date, orig_time):
     from .. import Annotations, create_info
+    from ..annotations import _handle_meas_date
     from ..io import RawArray
     info = create_info(ch_names=10, sfreq=10.)
     raw = RawArray(data=np.empty((10, 10)), info=info, first_samp=10)
+    if meas_date is not None:
+        meas_date = _handle_meas_date(meas_date)
     raw.info['meas_date'] = meas_date
+    raw.info._check_consistency()
     annot = Annotations([.5], [.2], ['dummy'], orig_time)
     raw.set_annotations(annotations=annot)
     return raw
@@ -447,11 +448,23 @@ def assert_meg_snr(actual, desired, min_tol, med_tol=500., chpi_med_tol=500.,
 
 def assert_snr(actual, desired, tol):
     """Assert actual and desired arrays are within some SNR tolerance."""
-    snr = (linalg.norm(desired, ord='fro') /
-           linalg.norm(desired - actual, ord='fro'))
+    from scipy import linalg
+    with np.errstate(divide='ignore'):  # allow infinite
+        snr = (linalg.norm(desired, ord='fro') /
+               linalg.norm(desired - actual, ord='fro'))
     assert snr >= tol, '%f < %f' % (snr, tol)
 
 
+def assert_stcs_equal(stc1, stc2):
+    """Check that two STC are equal."""
+    assert_allclose(stc1.times, stc2.times)
+    assert_allclose(stc1.data, stc2.data)
+    assert_array_equal(stc1.vertices[0], stc2.vertices[0])
+    assert_array_equal(stc1.vertices[1], stc2.vertices[1])
+    assert_allclose(stc1.tmin, stc2.tmin)
+    assert_allclose(stc1.tstep, stc2.tstep)
+
+
 def _dig_sort_key(dig):
     """Sort dig keys."""
     return (dig['kind'], dig['ident'])
@@ -461,17 +474,31 @@ def assert_dig_allclose(info_py, info_bin, limit=None):
     """Assert dig allclose."""
     from ..bem import fit_sphere_to_headshape
     from ..io.constants import FIFF
+    from ..io.meas_info import Info
+    from ..channels.montage import DigMontage
     # test dig positions
-    dig_py = sorted(info_py['dig'], key=_dig_sort_key)
-    dig_bin = sorted(info_bin['dig'], key=_dig_sort_key)
+    dig_py, dig_bin = info_py, info_bin
+    if isinstance(dig_py, Info):
+        assert isinstance(dig_bin, Info)
+        dig_py, dig_bin = dig_py['dig'], dig_bin['dig']
+    else:
+        assert isinstance(dig_bin, DigMontage)
+        assert isinstance(dig_py, DigMontage)
+        dig_py, dig_bin = dig_py.dig, dig_bin.dig
+        info_py = info_bin = None
+    assert isinstance(dig_py, list)
+    assert isinstance(dig_bin, list)
+    dig_py = sorted(dig_py, key=_dig_sort_key)
+    dig_bin = sorted(dig_bin, key=_dig_sort_key)
     assert len(dig_py) == len(dig_bin)
     for ii, (d_py, d_bin) in enumerate(zip(dig_py[:limit], dig_bin[:limit])):
         for key in ('ident', 'kind', 'coord_frame'):
-            assert d_py[key] == d_bin[key]
+            assert d_py[key] == d_bin[key], key
         assert_allclose(d_py['r'], d_bin['r'], rtol=1e-5, atol=1e-5,
                         err_msg='Failure on %s:\n%s\n%s'
                         % (ii, d_py['r'], d_bin['r']))
-    if any(d['kind'] == FIFF.FIFFV_POINT_EXTRA for d in dig_py):
+    if any(d['kind'] == FIFF.FIFFV_POINT_EXTRA for d in dig_py) and \
+            info_py is not None:
         r_bin, o_head_bin, o_dev_bin = fit_sphere_to_headshape(
             info_bin, units='m', verbose='error')
         r_py, o_head_py, o_dev_py = fit_sphere_to_headshape(
@@ -506,3 +533,25 @@ def modified_env(**d):
                 os.environ[key] = val
             elif key in os.environ:
                 del os.environ[key]
+
+
+def _click_ch_name(fig, ch_index=0, button=1):
+    """Click on a channel name in a raw/epochs/ICA browse-style plot."""
+    from ..viz.utils import _fake_click
+    fig.canvas.draw()
+    text = fig.mne.ax_main.get_yticklabels()[ch_index]
+    bbox = text.get_window_extent()
+    x = bbox.intervalx.mean()
+    y = bbox.intervaly.mean()
+    _fake_click(fig, fig.mne.ax_main, (x, y), xform='pix',
+                button=button)
+
+
+def _close_event(fig):
+    """Force calling of the MPL figure close event."""
+    # XXX workaround: plt.close() doesn't spawn close_event on Agg backend
+    # (check MPL github issue #18609; scheduled to be fixed by MPL 3.4)
+    try:
+        fig.canvas.close_event()
+    except ValueError:  # old mpl with Qt
+        pass  # pragma: no cover
