@@ -4,14 +4,19 @@
 #
 # License: BSD (3-clause)
 
+from builtins import input  # no-op here but facilitates testing
+from difflib import get_close_matches
 from distutils.version import LooseVersion
 import operator
 import os
 import os.path as op
 from pathlib import Path
+import sys
+import warnings
 
 import numpy as np
 
+from ..fixes import _median_complex
 from ._logging import warn, logger
 
 
@@ -20,6 +25,10 @@ def _ensure_int(x, name='unknown', must_be='an int'):
     # This is preferred over numbers.Integral, see:
     # https://github.com/scipy/scipy/pull/7351#issuecomment-299713159
     try:
+        # someone passing True/False is much more likely to be an error than
+        # intentional usage
+        if isinstance(x, bool):
+            raise TypeError()
         x = int(operator.index(x))
     except TypeError:
         raise TypeError('%s must be %s, got %s' % (name, must_be, type(x)))
@@ -54,7 +63,7 @@ def check_fname(fname, filetype, endings, endings_err=()):
              % (fname, filetype, print_endings))
 
 
-def check_version(library, min_version):
+def check_version(library, min_version='0.0'):
     r"""Check minimum library version required.
 
     Parameters
@@ -83,6 +92,13 @@ def check_version(library, min_version):
     return ok
 
 
+def _require_version(lib, what, version='0.0'):
+    """Require library for a purpose."""
+    if not check_version(lib, version):
+        extra = f' (version >= {version})' if version != '0.0' else ''
+        raise ImportError(f'The {lib} package{extra} is required to {what}')
+
+
 def _check_mayavi_version(min_version='4.3.0'):
     """Check mayavi version."""
     if not check_version('mayavi', min_version):
@@ -134,35 +150,53 @@ def _check_event_id(event_id, events):
     return event_id
 
 
-def _check_fname(fname, overwrite=False, must_exist=False):
+def _check_fname(fname, overwrite=False, must_exist=False, name='File',
+                 need_dir=False):
     """Check for file existence."""
-    _validate_type(fname, 'path-like', 'fname')
-    if op.isfile(fname):
+    _validate_type(fname, 'path-like', name)
+    if op.exists(fname):
         if not overwrite:
-            raise IOError('Destination file exists. Please use option '
-                          '"overwrite=True" to force overwriting.')
+            raise FileExistsError('Destination file exists. Please use option '
+                                  '"overwrite=True" to force overwriting.')
         elif overwrite != 'read':
             logger.info('Overwriting existing file.')
+        if must_exist:
+            if need_dir:
+                if not op.isdir(fname):
+                    raise IOError(
+                        f'Need a directory for {name} but found a file '
+                        f'at {fname}')
+            else:
+                if not op.isfile(fname):
+                    raise IOError(
+                        f'Need a file for {name} but found a directory '
+                        f'at {fname}')
+            if not os.access(fname, os.R_OK):
+                raise PermissionError(
+                    f'{name} does not have read permissions: {fname}')
     elif must_exist:
-        raise IOError('File "%s" does not exist' % fname)
-    return str(fname)
+        raise FileNotFoundError(f'{name} does not exist: {fname}')
+    return str(op.abspath(fname))
 
 
-def _check_subject(class_subject, input_subject, raise_error=True,
-                   kind='class subject attribute'):
+def _check_subject(first, second, *, raise_error=True,
+                   first_kind='class subject attribute',
+                   second_kind='input subject'):
     """Get subject name from class."""
-    if input_subject is not None:
-        _validate_type(input_subject, 'str', "subject input")
-        if class_subject is not None and input_subject != class_subject:
-            raise ValueError('%s (%r) did not match input subject (%r)'
-                             % (kind, class_subject, input_subject))
-        return input_subject
-    elif class_subject is not None:
-        _validate_type(class_subject, 'str',
-                       "Either subject input or %s" % (kind,))
-        return class_subject
+    if second is not None:
+        _validate_type(second, 'str', "subject input")
+        if first is not None and first != second:
+            raise ValueError(
+                f'{first_kind} ({repr(first)}) did not match '
+                f'{second_kind} ({second})')
+        return second
+    elif first is not None:
+        _validate_type(
+            first, 'str', f"Either {second_kind} subject or {first_kind}")
+        return first
     elif raise_error is True:
-        raise ValueError('Neither subject input nor %s was a string' % (kind,))
+        raise ValueError(f'Neither {second_kind} subject nor {first_kind} '
+                         'was a string')
     return None
 
 
@@ -242,15 +276,50 @@ def _check_pandas_installed(strict=True):
             return False
 
 
-def _check_pandas_index_arguments(index, defaults):
+def _check_eeglabio_installed(strict=True):
+    """Aux function."""
+    try:
+        import eeglabio
+        return eeglabio
+    except ImportError:
+        if strict is True:
+            raise RuntimeError('For this functionality to work, the eeglabio '
+                               'library is required.')
+        else:
+            return False
+
+
+def _check_pandas_index_arguments(index, valid):
     """Check pandas index arguments."""
-    if not any(isinstance(index, k) for k in (list, tuple)):
+    if index is None:
+        return
+    if isinstance(index, str):
         index = [index]
-    invalid_choices = [e for e in index if e not in defaults]
-    if invalid_choices:
-        options = [', '.join(e) for e in [invalid_choices, defaults]]
-        raise ValueError('[%s] is not an valid option. Valid index'
-                         'values are \'None\' or %s' % tuple(options))
+    if not isinstance(index, list):
+        raise TypeError('index must be `None` or a string or list of strings,'
+                        ' got type {}.'.format(type(index)))
+    invalid = set(index) - set(valid)
+    if invalid:
+        plural = ('is not a valid option',
+                  'are not valid options')[int(len(invalid) > 1)]
+        raise ValueError('"{}" {}. Valid index options are `None`, "{}".'
+                         .format('", "'.join(invalid), plural,
+                                 '", "'.join(valid)))
+    return index
+
+
+def _check_time_format(time_format, valid, meas_date=None):
+    """Check time_format argument."""
+    if time_format not in valid and time_format is not None:
+        valid_str = '", "'.join(valid)
+        raise ValueError('"{}" is not a valid time format. Valid options are '
+                         '"{}" and None.'.format(time_format, valid_str))
+    # allow datetime only if meas_date available
+    if time_format == 'datetime' and meas_date is None:
+        warn("Cannot convert to Datetime when raw.info['meas_date'] is "
+             "None. Falling back to Timedelta.")
+        time_format = 'timedelta'
+    return time_format
 
 
 def _check_ch_locs(chs):
@@ -281,14 +350,23 @@ class _IntLike(object):
         else:
             return True
 
+
 int_like = _IntLike()
+path_like = (str, Path)
+
+
+class _Callable(object):
+    @classmethod
+    def __instancecheck__(cls, other):
+        return callable(other)
 
 
 _multi = {
     'str': (str,),
     'numeric': (np.floating, float, int_like),
-    'path-like': (str, Path),
-    'int-like': (int_like,)
+    'path-like': path_like,
+    'int-like': (int_like,),
+    'callable': (_Callable(),),
 }
 try:
     _multi['path-like'] += (os.PathLike,)
@@ -311,7 +389,13 @@ def _validate_type(item, types=None, item_name=None, type_name=None):
         The thing to be checked.
     types : type | str | tuple of types | tuple of str
          The types to be checked against.
-         If str, must be one of {'int', 'str', 'numeric', 'info', 'path-like'}.
+         If str, must be one of {'int', 'str', 'numeric', 'info', 'path-like',
+         'callable'}.
+    item_name : str | None
+        Name of the item to show inside the error message.
+    type_name : str | None
+        Possible types to show inside the error message that the checked item
+        can be.
     """
     if types == "int":
         _ensure_int(item, name=item_name)
@@ -337,8 +421,9 @@ def _validate_type(item, types=None, item_name=None, type_name=None):
             else:
                 type_name[-1] = 'or ' + type_name[-1]
                 type_name = ', '.join(type_name)
-        raise TypeError('%s must be an instance of %s, got %s instead'
-                        % (item_name, type_name, type(item),))
+        _item_name = 'Item' if item_name is None else item_name
+        raise TypeError(f"{_item_name} must be an instance of {type_name}, "
+                        f"got {type(item)} instead")
 
 
 def _check_path_like(item):
@@ -434,60 +519,48 @@ def _check_channels_spatial_filter(ch_names, filters):
 
 
 def _check_rank(rank):
-    """Check rank parameter and deal with deprecation."""
-    err_msg = ('rank must be None, dict, "full", or int, '
-               'got %s (type %s)' % (rank, type(rank)))
+    """Check rank parameter."""
+    _validate_type(rank, (None, dict, str), 'rank')
     if isinstance(rank, str):
-        # XXX we can use rank='' to deprecate to get to None eventually:
-        # if rank == '':
-        #     warn('The rank parameter default in 0.18 of "full" will change '
-        #          'to None in 0.19, set it explicitly to avoid this warning',
-        #          DeprecationWarning)
-        #     rank = 'full'
         if rank not in ['full', 'info']:
             raise ValueError('rank, if str, must be "full" or "info", '
                              'got %s' % (rank,))
-    elif isinstance(rank, bool):
-        raise TypeError(err_msg)
-    elif rank is not None and not isinstance(rank, dict):
-        try:
-            rank = int(operator.index(rank))
-        except TypeError:
-            raise TypeError(err_msg)
-        else:
-            warn('rank as int is deprecated and will be removed in 0.19. '
-                 'use rank=dict(meg=...) instead.', DeprecationWarning)
-            rank = dict(meg=rank)
     return rank
 
 
 def _check_one_ch_type(method, info, forward, data_cov=None, noise_cov=None):
     """Check number of sensor types and presence of noise covariance matrix."""
     from ..cov import make_ad_hoc_cov, Covariance
+    from ..time_frequency.csd import CrossSpectralDensity
     from ..io.pick import pick_info
     from ..channels.channels import _contains_ch_type
-    picks = _check_info_inv(info, forward, data_cov=data_cov,
-                            noise_cov=noise_cov)
-    info_pick = pick_info(info, picks)
+    if isinstance(data_cov, CrossSpectralDensity):
+        _validate_type(noise_cov, [None, CrossSpectralDensity], 'noise_cov')
+        # FIXME
+        picks = list(range(len(data_cov.ch_names)))
+        info_pick = info
+    else:
+        _validate_type(noise_cov, [None, Covariance], 'noise_cov')
+        picks = _check_info_inv(info, forward, data_cov=data_cov,
+                                noise_cov=noise_cov)
+        info_pick = pick_info(info, picks)
     ch_types =\
         [_contains_ch_type(info_pick, tt) for tt in ('mag', 'grad', 'eeg')]
     if sum(ch_types) > 1:
-        if method == 'lcmv' and noise_cov is None:
+        if noise_cov is None:
             raise ValueError('Source reconstruction with several sensor types'
                              ' requires a noise covariance matrix to be '
                              'able to apply whitening.')
-        if method == 'dics':
-            raise RuntimeError(
-                'The use of several sensor types with the DICS beamformer is '
-                'not supported yet.')
     if noise_cov is None:
         noise_cov = make_ad_hoc_cov(info_pick, std=1.)
+        allow_mismatch = True
     else:
         noise_cov = noise_cov.copy()
-        if 'estimator' in noise_cov:
+        if isinstance(noise_cov, Covariance) and 'estimator' in noise_cov:
             del noise_cov['estimator']
-    _validate_type(noise_cov, Covariance, 'noise_cov')
-    return noise_cov, picks
+        allow_mismatch = False
+    _validate_type(noise_cov, (Covariance, CrossSpectralDensity), 'noise_cov')
+    return noise_cov, picks, allow_mismatch
 
 
 def _check_depth(depth, kind='depth_mne'):
@@ -498,10 +571,11 @@ def _check_depth(depth, kind='depth_mne'):
     return _handle_default(kind, depth)
 
 
-def _check_option(parameter, value, allowed_values):
+def _check_option(parameter, value, allowed_values, extra=''):
     """Check the value of a parameter against a list of valid options.
 
-    Raises a ValueError with a readable error message if the value was invalid.
+    Return the value if it is valid, otherwise raise a ValueError with a
+    readable error message.
 
     Parameters
     ----------
@@ -511,26 +585,36 @@ def _check_option(parameter, value, allowed_values):
         The value of the parameter to check.
     allowed_values : list
         The list of allowed values for the parameter.
+    extra : str
+        Extra string to append to the invalid value sentence, e.g.
+        "when using ico mode".
 
     Raises
     ------
     ValueError
-        When the value of the parameter was not one of the valid options.
+        When the value of the parameter is not one of the valid options.
+
+    Returns
+    -------
+    value : any type
+        The value if it is valid.
     """
     if value in allowed_values:
-        return True
+        return value
 
     # Prepare a nice error message for the user
-    msg = ("Invalid value for the '{parameter}' parameter. "
+    extra = ' ' + extra if extra else extra
+    msg = ("Invalid value for the '{parameter}' parameter{extra}. "
            '{options}, but got {value!r} instead.')
+    allowed_values = list(allowed_values)  # e.g., if a dict was given
     if len(allowed_values) == 1:
-        options = 'The only allowed value is %r' % allowed_values[0]
+        options = f'The only allowed value is {repr(allowed_values[0])}'
     else:
         options = 'Allowed values are '
-        options += ', '.join(['%r' % v for v in allowed_values[:-1]])
-        options += ' and %r' % allowed_values[-1]
+        options += ', '.join([f'{repr(v)}' for v in allowed_values[:-1]])
+        options += f', and {repr(allowed_values[-1])}'
     raise ValueError(msg.format(parameter=parameter, options=options,
-                                value=value))
+                                value=value, extra=extra))
 
 
 def _check_all_same_channel_names(instances):
@@ -549,9 +633,9 @@ def _check_combine(mode, valid=('mean', 'median', 'std')):
     elif mode == "std":
         def fun(data):
             return np.std(data, axis=0)
-    elif mode == "median":
+    elif mode == "median" or mode == np.median:
         def fun(data):
-            return np.median(data, axis=0)
+            return _median_complex(data, axis=0)
     elif callable(mode):
         fun = mode
     else:
@@ -568,3 +652,178 @@ def _check_src_normal(pick_ori, src):
         raise RuntimeError('Normal source orientation is supported only for '
                            'surface or discrete SourceSpaces, got type '
                            '%s' % (src.kind,))
+
+
+def _check_stc_units(stc, threshold=1e-7):  # 100 nAm threshold for warning
+    max_cur = np.max(np.abs(stc.data))
+    if max_cur > threshold:
+        warn('The maximum current magnitude is %0.1f nAm, which is very large.'
+             ' Are you trying to apply the forward model to noise-normalized '
+             '(dSPM, sLORETA, or eLORETA) values? The result will only be '
+             'correct if currents (in units of Am) are used.'
+             % (1e9 * max_cur))
+
+
+def _check_pyqt5_version():
+    bad = True
+    try:
+        from PyQt5.Qt import PYQT_VERSION_STR as version
+    except Exception:
+        version = 'unknown'
+    else:
+        if LooseVersion(version) >= LooseVersion('5.10'):
+            bad = False
+    bad &= sys.platform == 'darwin'
+    if bad:
+        warn('macOS users should use PyQt5 >= 5.10 for GUIs, got %s. '
+             'Please upgrade e.g. with:\n\n'
+             '    pip install "PyQt5>=5.10,<5.14"\n'
+             % (version,))
+
+    return version
+
+
+def _check_sphere(sphere, info=None, sphere_units='m'):
+    from ..defaults import HEAD_SIZE_DEFAULT
+    from ..bem import fit_sphere_to_headshape, ConductorModel, get_fitting_dig
+    if sphere is None:
+        sphere = HEAD_SIZE_DEFAULT
+        if info is not None:
+            # Decide if we have enough dig points to do the auto fit
+            try:
+                get_fitting_dig(info, 'extra', verbose='error')
+            except (RuntimeError, ValueError):
+                pass
+            else:
+                sphere = 'auto'
+    if isinstance(sphere, str):
+        if sphere != 'auto':
+            raise ValueError('sphere, if str, must be "auto", got %r'
+                             % (sphere))
+        R, r0, _ = fit_sphere_to_headshape(info, verbose=False, units='m')
+        sphere = tuple(r0) + (R,)
+        sphere_units = 'm'
+    elif isinstance(sphere, ConductorModel):
+        if not sphere['is_sphere'] or len(sphere['layers']) == 0:
+            raise ValueError('sphere, if a ConductorModel, must be spherical '
+                             'with multiple layers, not a BEM or single-layer '
+                             'sphere (got %s)' % (sphere,))
+        sphere = tuple(sphere['r0']) + (sphere['layers'][0]['rad'],)
+        sphere_units = 'm'
+    sphere = np.array(sphere, dtype=float)
+    if sphere.shape == ():
+        sphere = np.concatenate([[0.] * 3, [sphere]])
+    if sphere.shape != (4,):
+        raise ValueError('sphere must be float or 1D array of shape (4,), got '
+                         'array-like of shape %s' % (sphere.shape,))
+    _check_option('sphere_units', sphere_units, ('m', 'mm'))
+    if sphere_units == 'mm':
+        sphere /= 1000.
+
+    sphere = np.array(sphere, float)
+    return sphere
+
+
+def _check_freesurfer_home():
+    from .config import get_config
+    fs_home = get_config('FREESURFER_HOME')
+    if fs_home is None:
+        raise RuntimeError(
+            'The FREESURFER_HOME environment variable is not set.')
+    return fs_home
+
+
+def _suggest(val, options, cutoff=0.66):
+    options = get_close_matches(val, options, cutoff=cutoff)
+    if len(options) == 0:
+        return ''
+    elif len(options) == 1:
+        return ' Did you mean %r?' % (options[0],)
+    else:
+        return ' Did you mean one of %r?' % (options,)
+
+
+def _check_on_missing(on_missing, name='on_missing'):
+    _validate_type(on_missing, str, name)
+    _check_option(name, on_missing, ['raise', 'warn', 'ignore'])
+
+
+def _on_missing(on_missing, msg, name='on_missing', error_klass=None):
+    _check_on_missing(on_missing, name)
+    error_klass = ValueError if error_klass is None else error_klass
+    on_missing = 'raise' if on_missing == 'error' else on_missing
+    on_missing = 'warn' if on_missing == 'warning' else on_missing
+    if on_missing == 'raise':
+        raise error_klass(msg)
+    elif on_missing == 'warn':
+        warn(msg)
+    else:  # Ignore
+        assert on_missing == 'ignore'
+
+
+def _safe_input(msg, *, alt=None, use=None):
+    try:
+        return input(msg)
+    except EOFError:  # MATLAB or other non-stdin
+        if use is not None:
+            return use
+        raise RuntimeError(
+            f'Could not use input() to get a response to:\n{msg}\n'
+            f'You can {alt} to avoid this error.')
+
+
+def _ensure_events(events):
+    events_type = type(events)
+    with warnings.catch_warnings(record=True):
+        warnings.simplefilter('ignore')  # deprecation for object array
+        events = np.asarray(events)
+    if not np.issubdtype(events.dtype, np.integer):
+        raise TypeError('events should be a NumPy array of integers, '
+                        f'got {events_type}')
+    if events.ndim != 2 or events.shape[1] != 3:
+        raise ValueError(
+            f'events must be of shape (N, 3), got {events.shape}')
+    return events
+
+
+def _infer_check_export_fmt(fmt, fname, supported_formats):
+    """Infer export format from filename extension if auto.
+
+    Raises error if fmt is auto and no file extension found,
+    then checks format against supported formats, raises error if format is not
+    supported.
+
+    Parameters
+    ----------
+    fmt : str
+        Format of the export, will only infer the format from filename if fmt
+        is auto.
+    fname : str
+        Name of the target export file, only used when fmt is auto.
+    supported_formats : dict of str : tuple/list
+        Dictionary containing supported formats (as keys) and each format's
+        corresponding file extensions in a tuple/list (e.g. 'eeglab': ('set',))
+    """
+    _validate_type(fmt, str, 'fmt')
+    fmt = fmt.lower()
+    if fmt == "auto":
+        fmt = op.splitext(fname)[1]
+        if fmt:
+            fmt = fmt[1:].lower()
+            # find fmt in supported formats dict's tuples
+            fmt = next((k for k, v in supported_formats.items() if fmt in v),
+                       fmt)  # default to original fmt for raising error later
+        else:
+            raise ValueError(f"Couldn't infer format from filename {fname}"
+                             " (no extension found)")
+
+    if fmt not in supported_formats:
+        supported = []
+        for format, extensions in supported_formats.items():
+            ext_str = ', '.join(f'*.{ext}' for ext in extensions)
+            supported.append(f'{format} ({ext_str})')
+
+        supported_str = ', '.join(supported)
+        raise ValueError(f"Format '{fmt}' is not supported. "
+                         f"Supported formats are {supported_str}.")
+    return fmt
