@@ -4,11 +4,10 @@
 #
 # License: BSD (3-clause)
 
-
+from collections import OrderedDict
 from copy import deepcopy
 import logging
 import json
-from collections import OrderedDict
 
 import numpy as np
 
@@ -93,43 +92,73 @@ class GetEpochsMixin(object):
         -----
         Epochs can be accessed as ``epochs[...]`` in several ways:
 
-            1. ``epochs[idx]``: Return ``Epochs`` object with a subset of
-               epochs (supports single index and python-style slicing).
+        1. **Integer or slice:** ``epochs[idx]`` will return an `~mne.Epochs`
+           object with a subset of epochs chosen by index (supports single
+           index and Python-style slicing).
 
-            2. ``epochs['name']``: Return ``Epochs`` object with a copy of the
-               subset of epochs corresponding to an experimental condition as
-               specified by 'name'.
+        2. **String:** ``epochs['name']`` will return an `~mne.Epochs` object
+           comprising only the epochs labeled ``'name'`` (i.e., epochs created
+           around events with the label ``'name'``).
 
-               If conditions are tagged by names separated by '/' (e.g.
-               'audio/left', 'audio/right'), and 'name' is not in itself an
-               event key, this selects every event whose condition contains
-               the 'name' tag (e.g., 'left' matches 'audio/left' and
-               'visual/left'; but not 'audio_left'). Note that tags selection
-               is insensitive to order: tags like 'auditory/left' and
-               'left/auditory' will be treated the same way when accessed.
+           If there are no epochs labeled ``'name'`` but there are epochs
+           labeled with /-separated tags (e.g. ``'name/left'``,
+           ``'name/right'``), then ``epochs['name']`` will select the epochs
+           with labels that contain that tag (e.g., ``epochs['left']`` selects
+           epochs labeled ``'audio/left'`` and ``'visual/left'``, but not
+           ``'audio_left'``).
 
-            3. ``epochs[['name_1', 'name_2', ... ]]``: Return ``Epochs`` object
-               with a copy of the subset of epochs corresponding to multiple
-               experimental conditions as specified by
-               ``'name_1', 'name_2', ...`` .
+           If multiple tags are provided *as a single string* (e.g.,
+           ``epochs['name_1/name_2']``), this selects epochs containing *all*
+           provided tags. For example, ``epochs['audio/left']`` selects
+           ``'audio/left'`` and ``'audio/quiet/left'``, but not
+           ``'audio/right'``. Note that tag-based selection is insensitive to
+           order: tags like ``'audio/left'`` and ``'left/audio'`` will be
+           treated the same way when selecting via tag.
 
-               If conditions are separated by '/', selects every item
-               containing every list tag (e.g. ['audio', 'left'] selects
-               'audio/left' and 'audio/center/left', but not 'audio/right').
+        3. **List of strings:** ``epochs[['name_1', 'name_2', ... ]]`` will
+           return an `~mne.Epochs` object comprising epochs that match *any* of
+           the provided names (i.e., the list of names is treated as an
+           inclusive-or condition). If *none* of the provided names match any
+           epoch labels, a ``KeyError`` will be raised.
 
-            4. ``epochs['pandas query']``: Return ``Epochs`` object with a
-               copy of the subset of epochs (and matching metadata) that match
-               ``pandas query`` called with ``self.metadata.eval``, e.g.::
+           If epoch labels are /-separated tags, then providing multiple tags
+           *as separate list entries* will likewise act as an inclusive-or
+           filter. For example, ``epochs[['audio', 'left']]`` would select
+           ``'audio/left'``, ``'audio/right'``, and ``'visual/left'``, but not
+           ``'visual/right'``.
 
-                   epochs["col_a > 2 and col_b == 'foo'"]
+        4. **Pandas query:** ``epochs['pandas query']`` will return an
+           `~mne.Epochs` object with a subset of epochs (and matching
+           metadata) selected by the query called with
+           ``self.metadata.eval``, e.g.::
 
-               This is only called if Pandas is installed and ``self.metadata``
-               is a :class:`pandas.DataFrame`.
+               epochs["col_a > 2 and col_b == 'foo'"]
 
-               .. versionadded:: 0.16
+           would return all epochs whose associated ``col_a`` metadata was
+           greater than two, and whose ``col_b`` metadata was the string 'foo'.
+           Query-based indexing only works if Pandas is installed and
+           ``self.metadata`` is a :class:`pandas.DataFrame`.
+
+           .. versionadded:: 0.16
         """
         return self._getitem(item)
 
+    def _item_to_select(self, item):
+        if isinstance(item, str):
+            item = [item]
+
+        # Convert string to indices
+        if isinstance(item, (list, tuple)) and len(item) > 0 and \
+                isinstance(item[0], str):
+            select = self._keys_to_idx(item)
+        elif isinstance(item, slice):
+            select = item
+        else:
+            select = np.atleast_1d(item)
+            if len(select) == 0:
+                select = np.array([], int)
+        return select
+
     def _getitem(self, item, reason='IGNORED', copy=True, drop_event_id=True,
                  select_data=True, return_indices=False):
         """
@@ -163,31 +192,22 @@ class GetEpochsMixin(object):
         self._data = inst._data = data
         del self
 
-        if isinstance(item, str):
-            item = [item]
-
-        # Convert string to indices
-        if isinstance(item, (list, tuple)) and len(item) > 0 and \
-                isinstance(item[0], str):
-            select = inst._keys_to_idx(item)
-        elif isinstance(item, slice):
-            select = item
-        else:
-            select = np.atleast_1d(item)
-            if len(select) == 0:
-                select = np.array([], int)
+        select = inst._item_to_select(item)
         has_selection = hasattr(inst, 'selection')
         if has_selection:
             key_selection = inst.selection[select]
+            drop_log = list(inst.drop_log)
             if reason is not None:
                 for k in np.setdiff1d(inst.selection, key_selection):
-                    inst.drop_log[k] = [reason]
+                    drop_log[k] = (reason,)
+            inst.drop_log = tuple(drop_log)
             inst.selection = key_selection
+            del drop_log
 
         inst.events = np.atleast_2d(inst.events[select])
         if inst.metadata is not None:
             pd = _check_pandas_installed(strict=False)
-            if pd is not False:
+            if pd:
                 metadata = inst.metadata.iloc[select]
                 if has_selection:
                     metadata.index = inst.selection
@@ -227,8 +247,9 @@ class GetEpochsMixin(object):
             msg = str(err.args[0])  # message for KeyError
             pd = _check_pandas_installed(strict=False)
             # See if the query can be done
-            if pd is not False:
-                self._check_metadata()
+            if pd:
+                md = self.metadata if hasattr(self, '_metadata') else None
+                self._check_metadata(metadata=md)
                 try:
                     # Try metadata
                     mask = self.metadata.eval(keys[0], engine='python').values
@@ -264,7 +285,6 @@ class GetEpochsMixin(object):
             43
             >>> len(epochs.events)  # doctest: +SKIP
             43
-
         """
         from ..epochs import BaseEpochs
         if isinstance(self, BaseEpochs) and not self._bad_dropped:
@@ -292,6 +312,7 @@ class GetEpochsMixin(object):
         :meth:`mne.Epochs.next`.
         """
         self._current = 0
+        self._current_detrend_picks = self._detrend_picks
         return self
 
     def __next__(self, return_event_id=False):
@@ -309,18 +330,21 @@ class GetEpochsMixin(object):
         event_id : int
             The event id. Only returned if ``return_event_id`` is ``True``.
         """
+        if not hasattr(self, '_current_detrend_picks'):
+            self.__iter__()  # ensure we're ready to iterate
         if self.preload:
             if self._current >= len(self._data):
-                raise StopIteration  # signal the end
+                self._stop_iter()
             epoch = self._data[self._current]
             self._current += 1
         else:
             is_good = False
             while not is_good:
                 if self._current >= len(self.events):
-                    raise StopIteration  # signal the end properly
+                    self._stop_iter()
                 epoch_noproj = self._get_epoch_from_raw(self._current)
-                epoch_noproj = self._detrend_offset_decim(epoch_noproj)
+                epoch_noproj = self._detrend_offset_decim(
+                    epoch_noproj, self._current_detrend_picks)
                 epoch = self._project_epoch(epoch_noproj)
                 self._current += 1
                 is_good, _ = self._is_good_epoch(epoch)
@@ -333,16 +357,21 @@ class GetEpochsMixin(object):
         else:
             return epoch, self.events[self._current - 1][-1]
 
+    def _stop_iter(self):
+        del self._current
+        del self._current_detrend_picks
+        raise StopIteration  # signal the end
+
     next = __next__  # originally for Python2, now b/c public
 
     def _check_metadata(self, metadata=None, reset_index=False):
         """Check metadata consistency."""
         # reset_index=False will not copy!
-        metadata = self.metadata if hasattr(self, '_metadata') and \
-            metadata is None else metadata
-        if metadata is not None:
+        if metadata is None:
+            return
+        else:
             pd = _check_pandas_installed(strict=False)
-            if pd is not False:
+            if pd:
                 _validate_type(metadata, types=pd.DataFrame,
                                item_name='metadata')
                 if len(metadata) != len(self.events):
@@ -408,7 +437,7 @@ def _prepare_read_metadata(metadata):
         # (which is necessary for round-trip equivalence)
         metadata = json.loads(metadata, object_pairs_hook=OrderedDict)
         assert isinstance(metadata, list)
-        if pd is not False:
+        if pd:
             metadata = pd.DataFrame.from_records(metadata)
             assert isinstance(metadata, pd.DataFrame)
     return metadata
@@ -462,3 +491,48 @@ class _FakeNoPandas(object):  # noqa: D101
         import mne
         mne.epochs._check_pandas_installed = self._old_check
         mne.utils.mixin._check_pandas_installed = self._old_check
+
+
+class ShiftTimeMixin(object):
+    """Class for shift_time method (Epochs, Evoked, and DipoleFixed)."""
+
+    def shift_time(self, tshift, relative=True):
+        """Shift time scale in epoched or evoked data.
+
+        Parameters
+        ----------
+        tshift : float
+            The (absolute or relative) time shift in seconds. If ``relative``
+            is True, positive tshift increases the time value associated with
+            each sample, while negative tshift decreases it.
+        relative : bool
+            If True, increase or decrease time values by ``tshift`` seconds.
+            Otherwise, shift the time values such that the time of the first
+            sample equals ``tshift``.
+
+        Returns
+        -------
+        epochs : instance of Epochs
+            The modified Epochs instance.
+
+        Notes
+        -----
+        This method allows you to shift the *time* values associated with each
+        data sample by an arbitrary amount. It does *not* resample the signal
+        or change the *data* values in any way.
+        """
+        from ..epochs import BaseEpochs
+        _check_preload(self, 'shift_time')
+        start = tshift + (self.times[0] if relative else 0.)
+        new_times = start + np.arange(len(self.times)) / self.info['sfreq']
+        if isinstance(self, BaseEpochs):
+            self._set_times(new_times)
+        else:
+            self.times = new_times
+            self._update_first_last()
+        return self
+
+    def _update_first_last(self):
+        """Update self.first and self.last (sample indices)."""
+        self.first = int(round(self.times[0] * self.info['sfreq']))
+        self.last = len(self.times) + self.first - 1
