@@ -24,7 +24,25 @@ def has_mayavi():
     """Check that mayavi is installed."""
     try:
         with warnings.catch_warnings(record=True):  # traits
-            from mayavi import mlab  # noqa F401
+            from mayavi import mlab  # noqa: F401
+        return True
+    except ImportError:
+        return False
+
+
+def has_pyqt5():
+    """Check if PyQt5 is installed."""
+    try:
+        import PyQt5  # noqa: F401
+        return True
+    except ImportError:
+        return False
+
+
+def has_imageio_ffmpeg():
+    """Check if imageio-ffmpeg is installed."""
+    try:
+        import imageio_ffmpeg  # noqa: F401
         return True
     except ImportError:
         return False
