@@ -6,24 +6,22 @@
 # License: Simplified BSD
 
 import os
+
 import pytest
-import importlib
 import numpy as np
-from mne.viz.backends.renderer import get_3d_backend
+
 from mne.viz.backends.tests._utils import (skips_if_not_mayavi,
                                            skips_if_not_pyvista)
-
-DEFAULT_3D_BACKEND = 'mayavi'  # This should be done with the import
+from mne.viz.backends._utils import ALLOWED_QUIVER_MODES
 
 
 @pytest.fixture
 def backend_mocker():
     """Help to test set up 3d backend."""
     from mne.viz.backends import renderer
-    assert renderer.MNE_3D_BACKEND == DEFAULT_3D_BACKEND  # just double-check
     del renderer.MNE_3D_BACKEND
     yield
-    renderer.MNE_3D_BACKEND = DEFAULT_3D_BACKEND
+    renderer.MNE_3D_BACKEND = None
 
 
 @pytest.mark.parametrize('backend', [
@@ -39,16 +37,30 @@ def test_backend_environment_setup(backend, backend_mocker, monkeypatch):
     # reload the renderer to check if the 3d backend selection by
     # environment variable has been updated correctly
     from mne.viz.backends import renderer
-    importlib.reload(renderer)
+    renderer.set_3d_backend(backend)
     assert renderer.MNE_3D_BACKEND == backend
-    assert get_3d_backend() == backend
+    assert renderer.get_3d_backend() == backend
+
+
+def test_3d_functions(renderer):
+    """Test figure management functions."""
+    fig = renderer.create_3d_figure((300, 300))
+    # Mayavi actually needs something in the display to set the title
+    wrap_renderer = renderer.backend._Renderer(fig=fig)
+    wrap_renderer.sphere(np.array([0., 0., 0.]), 'w', 1.)
+    renderer.backend._check_3d_figure(fig)
+    renderer.backend._set_3d_view(figure=fig, azimuth=None, elevation=None,
+                                  focalpoint=(0., 0., 0.), distance=None)
+    renderer.backend._set_3d_title(figure=fig, title='foo')
+    renderer.backend._take_3d_screenshot(figure=fig)
+    renderer.backend._close_all()
 
 
 def test_3d_backend(renderer):
     """Test default plot."""
     # set data
     win_size = (600, 600)
-    win_color = (0, 0, 0)
+    win_color = 'black'
 
     tet_size = 1.0
     tet_x = np.array([0, tet_size, 0, 0])
@@ -58,10 +70,10 @@ def test_3d_backend(renderer):
                             [0, 1, 3],
                             [0, 2, 3],
                             [1, 2, 3]])
-    tet_color = (1, 1, 1)
+    tet_color = 'white'
 
     sph_center = np.column_stack((tet_x, tet_y, tet_z))
-    sph_color = (1, 0, 0)
+    sph_color = 'red'
     sph_scale = tet_size / 3.0
 
     ct_scalars = np.array([0.0, 0.0, 0.0, 1.0])
@@ -71,8 +83,7 @@ def test_3d_backend(renderer):
         "tris": tet_indices
     }
 
-    qv_mode = "arrow"
-    qv_color = (0, 0, 1)
+    qv_color = 'blue'
     qv_scale = tet_size / 2.0
     qv_center = np.array([np.mean((sph_center[va, :],
                                    sph_center[vb, :],
@@ -86,39 +97,57 @@ def test_3d_backend(renderer):
     txt_x = 0.0
     txt_y = 0.0
     txt_text = "renderer"
-    txt_width = 1.0
+    txt_size = 14
 
     cam_distance = 5 * tet_size
 
     # init scene
-    rend = renderer._Renderer(size=win_size, bgcolor=win_color)
-    rend.set_interactive()
+    rend = renderer.create_3d_figure(
+        size=win_size,
+        bgcolor=win_color,
+        smooth_shading=True,
+        scene=False,
+    )
+    for interaction in ('terrain', 'trackball'):
+        rend.set_interaction(interaction)
 
     # use mesh
-    rend.mesh(x=tet_x, y=tet_y, z=tet_z,
-              triangles=tet_indices,
-              color=tet_color)
+    mesh_data = rend.mesh(
+        x=tet_x,
+        y=tet_y,
+        z=tet_z,
+        triangles=tet_indices,
+        color=tet_color,
+    )
+    rend.remove_mesh(mesh_data)
 
     # use contour
     rend.contour(surface=ct_surface, scalars=ct_scalars,
-                 contours=ct_levels)
+                 contours=ct_levels, kind='line')
+    rend.contour(surface=ct_surface, scalars=ct_scalars,
+                 contours=ct_levels, kind='tube')
 
     # use sphere
     rend.sphere(center=sph_center, color=sph_color,
-                scale=sph_scale)
+                scale=sph_scale, radius=1.0)
 
     # use quiver3d
-    rend.quiver3d(x=qv_center[:, 0],
-                  y=qv_center[:, 1],
-                  z=qv_center[:, 2],
-                  u=qv_dir[:, 0],
-                  v=qv_dir[:, 1],
-                  w=qv_dir[:, 2],
-                  color=qv_color,
-                  scale=qv_scale,
-                  scale_mode=qv_scale_mode,
-                  scalars=qv_scalars,
-                  mode=qv_mode)
+    kwargs = dict(
+        x=qv_center[:, 0],
+        y=qv_center[:, 1],
+        z=qv_center[:, 2],
+        u=qv_dir[:, 0],
+        v=qv_dir[:, 1],
+        w=qv_dir[:, 2],
+        color=qv_color,
+        scale=qv_scale,
+        scale_mode=qv_scale_mode,
+        scalars=qv_scalars,
+    )
+    for mode in ALLOWED_QUIVER_MODES:
+        rend.quiver3d(mode=mode, **kwargs)
+    with pytest.raises(ValueError, match='Invalid value'):
+        rend.quiver3d(mode='foo', **kwargs)
 
     # use tube
     rend.tube(origin=np.array([[0, 0, 0]]),
@@ -128,12 +157,23 @@ def test_3d_backend(renderer):
                      scalars=np.array([[1.0, 1.0]]))
 
     # scalar bar
-    rend.scalarbar(source=tube, title="Scalar Bar")
+    rend.scalarbar(source=tube, title="Scalar Bar",
+                   bgcolor=[1, 1, 1])
 
     # use text
-    rend.text2d(x=txt_x, y=txt_y, text=txt_text, width=txt_width)
+    rend.text2d(x_window=txt_x, y_window=txt_y, text=txt_text,
+                size=txt_size, justification='right')
     rend.text3d(x=0, y=0, z=0, text=txt_text, scale=1.0)
     rend.set_camera(azimuth=180.0, elevation=90.0,
                     distance=cam_distance,
                     focalpoint=center)
+    rend.reset_camera()
     rend.show()
+
+
+def test_get_3d_backend(renderer):
+    """Test get_3d_backend function call for side-effects."""
+    # Test twice to ensure the first call had no side-effect
+    orig_backend = renderer.MNE_3D_BACKEND
+    assert renderer.get_3d_backend() == orig_backend
+    assert renderer.get_3d_backend() == orig_backend
