@@ -3,7 +3,7 @@
 #          Martin Luessi <mluessi@nmr.mgh.harvard.edu>
 #          Eric Larson <larson.eric.d@gmail.com>
 #          Jaakko Leppakangas <jaeilepp@student.jyu.fi>
-#          Daniel McCloy <dan.mccloy@gmail.com>
+#          Daniel McCloy <dan@mccloy.info>
 #
 # License: Simplified BSD
 
@@ -17,7 +17,7 @@ from mne import (read_events, Epochs, pick_types, read_cov, create_info,
                  EpochsArray)
 from mne.channels import read_layout
 from mne.io import read_raw_fif, read_raw_ctf
-from mne.utils import run_tests_if_main
+from mne.utils import run_tests_if_main, _click_ch_name, _close_event
 from mne.viz import plot_drop_log
 from mne.viz.utils import _fake_click
 from mne.datasets import testing
@@ -45,19 +45,32 @@ def _get_epochs(stop=5, meg=True, eeg=False, n_chan=20):
     picks = np.round(np.linspace(0, len(picks) + 1, n_chan)).astype(int)
     with pytest.warns(RuntimeWarning, match='projection'):
         epochs = Epochs(raw, events[:stop], event_id, tmin, tmax, picks=picks,
-                        proj=False)
+                        proj=False, preload=False)
     epochs.info.normalize_proj()  # avoid warnings
     return epochs
 
 
-def test_plot_epochs_basic(capsys):
+@pytest.fixture()
+def epochs():
+    """Get minimal, pre-loaded epochs data suitable for most tests."""
+    return _get_epochs().load_data()
+
+
+def test_plot_epochs_not_preloaded():
+    """Test plotting non-preloaded epochs."""
+    epochs = _get_epochs()
+    assert epochs._data is None
+    epochs.plot()
+    assert epochs._data is None
+
+
+def test_plot_epochs_basic(epochs, capsys):
     """Test epoch plotting."""
-    epochs = _get_epochs().load_data()
     assert len(epochs.events) == 1
     epochs.info['lowpass'] = 10.  # allow heavy decim during plotting
     fig = epochs.plot(scalings=None, title='Epochs')
-    ticks = [x.get_text() for x in fig.axes[0].get_xticklabels()]
-    assert ticks == ['1']
+    ticks = [x.get_text() for x in fig.mne.ax_main.get_xticklabels(minor=True)]
+    assert ticks == ['2']
     plt.close('all')
     # covariance / whitening
     cov = read_cov(cov_fname)
@@ -77,83 +90,136 @@ def test_plot_epochs_basic(capsys):
     with pytest.warns(RuntimeWarning, match='projection'):
         epochs.plot(noise_cov=cov)
     plt.close('all')
-    # have a data channels missing from the covariance
+    # have a data channel missing from the covariance
     cov['names'] = cov['names'][:306]
     cov['data'] = cov['data'][:306][:306]
     with pytest.warns(RuntimeWarning, match='projection'):
         epochs.plot(noise_cov=cov)
     plt.close('all')
-    # check the epochs color plotting
-    epoch_colors = [['r'] * len(epochs.ch_names) for _ in
-                    range(len(epochs.events))]
-    epochs.plot(epoch_colors=epoch_colors)
-    with pytest.raises(TypeError, match='must be an instance of'):
-        epochs.plot(epoch_colors='r')
-    with pytest.raises(ValueError, match='must be list of len'):
-        epochs.plot(epoch_colors=['r'] * len(epochs.ch_names))
-    epoch_colors[0] = None
-    with pytest.raises(TypeError, match='must be an instance of'):
-        epochs.plot(epoch_colors=epoch_colors)
-    epoch_colors[0] = ['r'] * 5
-    with pytest.raises(ValueError, match='epoch_colors for the 0th epoch has'
-                       ' length'):
-        epochs.plot(epoch_colors=epoch_colors)
     # other options
     fig = epochs[0].plot(picks=[0, 2, 3], scalings=None)
     fig.canvas.key_press_event('escape')
-    plt.close('all')
-    keystotest = ['b', 'b', 'left', 'right', 'up', 'down',
-                  'pageup', 'pagedown', '-', '+', '=',
-                  'f11', 'home', '?', 'h', 'o', 'end']
-    fig = epochs.plot()
-    with pytest.warns(None):  # sometimes matplotlib warns about limits
-        for key in keystotest:
-            fig.canvas.key_press_event(key)
-    fig.canvas.scroll_event(0.5, 0.5, -0.5)  # scroll down
-    fig.canvas.scroll_event(0.5, 0.5, 0.5)  # scroll up
-    fig.canvas.resize_event()
-    fig.canvas.close_event()  # closing and epoch dropping
-    plt.close('all')
-    pytest.raises(ValueError, epochs.plot, picks=[])
-    plt.close('all')
-    fig = epochs.plot(events=epochs.events)
-    # test mouse clicks
-    data_ax = fig.get_axes()[0]
-    x = data_ax.get_xlim()[1] / 2
-    y = data_ax.get_ylim()[0] / 2
-    n_epochs = len(epochs)
-    _fake_click(fig, data_ax, [x, y], xform='data')  # mark a bad epoch
-    _fake_click(fig, data_ax, [x, y], xform='data')  # unmark a bad epoch
-    _fake_click(fig, data_ax, [0.5, 0.999])  # click elsewhere in 1st axes
-    _fake_click(fig, data_ax, [-0.1, 0.9])  # click on y-label
-    _fake_click(fig, data_ax, [-0.1, 0.9], button=3)
-    _fake_click(fig, fig.get_axes()[2], [0.5, 0.5])  # change epochs
-    _fake_click(fig, fig.get_axes()[3], [0.5, 0.5])  # change channels
-    fig.canvas.close_event()  # closing and epoch dropping
-    assert(n_epochs - 1 == len(epochs))
-    plt.close('all')
-    epochs.plot_sensors()  # Test plot_sensors
-    plt.close('all')
+    with pytest.raises(ValueError, match='No appropriate channels found'):
+        epochs.plot(picks=[])
     # gh-5906
     epochs = _get_epochs(None).load_data()
     epochs.load_data()
     assert len(epochs) == 7
     epochs.info['bads'] = [epochs.ch_names[0]]
     capsys.readouterr()
+    # test title error handling
+    with pytest.raises(TypeError, match='title must be None or a string, got'):
+        epochs.plot(title=7)
+    # test auto-generated title, and selection mode
+    epochs.plot(group_by='selection', title='')
+
+
+@pytest.mark.parametrize('scalings', (dict(mag=1e-12, grad=1e-11, stim='auto'),
+                                      None, 'auto'))
+def test_plot_epochs_scalings(epochs, scalings):
+    """Test the valid options for scalings."""
+    epochs.plot(scalings=scalings)
+
+
+def test_plot_epochs_colors(epochs):
+    """Test epoch_colors, for compatibility with autoreject."""
+    epoch_colors = [['r'] * len(epochs.ch_names) for _ in
+                    range(len(epochs.events))]
+    epochs.plot(epoch_colors=epoch_colors)
+    with pytest.raises(ValueError, match='length equal to the number of epo'):
+        epochs.plot(epoch_colors=[['r'], ['b']])  # epochs obj has only 1 epoch
+    with pytest.raises(ValueError, match=r'epoch colors for epoch \d+ has'):
+        epochs.plot(epoch_colors=[['r']])  # need 1 color for each channel
+    # also test event_color
+    epochs.plot(event_color='b')
+
+
+def test_plot_epochs_scale_bar(epochs):
+    """Test scale bar for epochs."""
+    fig = epochs.plot()
+    fig.canvas.key_press_event('s')  # default is to not show scalebars
+    ax = fig.mne.ax_main
+    assert len(ax.texts) == 2  # only mag & grad in this instance
+    texts = tuple(t.get_text().strip() for t in ax.texts)
+    wants = ('800.0 fT/cm', '2000.0 fT')
+    assert texts == wants
+
+
+def test_plot_epochs_clicks(epochs, capsys):
+    """Test plot_epochs mouse interaction."""
+    fig = epochs.plot(events=epochs.events)
+    data_ax = fig.mne.ax_main
+    x = fig.mne.traces[0].get_xdata()[3]
+    y = fig.mne.traces[0].get_ydata()[3]
+    n_epochs = len(epochs)
+    epoch_num = fig.mne.inst.selection[0]
+    # test (un)marking bad epochs
+    _fake_click(fig, data_ax, [x, y], xform='data')  # mark a bad epoch
+    assert epoch_num in fig.mne.bad_epochs
+    _fake_click(fig, data_ax, [x, y], xform='data')  # unmark it
+    assert epoch_num not in fig.mne.bad_epochs
+    _fake_click(fig, data_ax, [x, y], xform='data')  # mark it bad again
+    assert epoch_num in fig.mne.bad_epochs
+    # test vline
+    fig.canvas.key_press_event('escape')  # close and drop epochs
+    _close_event(fig)  # XXX workaround, MPL Agg doesn't trigger close event
+    assert(n_epochs - 1 == len(epochs))
+    # test marking bad channels
+    epochs = _get_epochs(None).load_data()  # need more than 1 epoch this time
     fig = epochs.plot(n_epochs=3)
-    data_ax = fig.get_axes()[0]
-    _fake_click(fig, data_ax, [-0.1, 0.9])  # click on y-label
+    data_ax = fig.mne.ax_main
+    first_ch = data_ax.get_yticklabels()[0].get_text()
+    assert first_ch not in fig.mne.info['bads']
+    _click_ch_name(fig, ch_index=0, button=1)  # click ch name to mark bad
+    assert first_ch in fig.mne.info['bads']
+    # test clicking scrollbars
+    _fake_click(fig, fig.mne.ax_vscroll, [0.5, 0.5])
+    _fake_click(fig, fig.mne.ax_hscroll, [0.5, 0.5])
+    # test moving bad epoch offscreen
     fig.canvas.key_press_event('right')  # move right
-    x = fig.get_axes()[0].get_xlim()[1] / 6.
-    y = fig.get_axes()[0].get_ylim()[0] / 2
+    x = fig.mne.traces[0].get_xdata()[-3]
+    y = fig.mne.traces[0].get_ydata()[-3]
     _fake_click(fig, data_ax, [x, y], xform='data')  # mark a bad epoch
     fig.canvas.key_press_event('left')  # move back
     out, err = capsys.readouterr()
     assert 'out of bounds' not in out
     assert 'out of bounds' not in err
-    fig.canvas.close_event()
+    fig.canvas.key_press_event('escape')
+    _close_event(fig)  # XXX workaround, MPL Agg doesn't trigger close event
     assert len(epochs) == 6
-    plt.close('all')
+    # test rightclick → image plot
+    fig = epochs.plot()
+    _click_ch_name(fig, ch_index=0, button=3)  # show image plot
+    assert len(fig.mne.child_figs) == 1
+    # test scroll wheel
+    fig.canvas.scroll_event(0.5, 0.5, -0.5)  # scroll down
+    fig.canvas.scroll_event(0.5, 0.5, 0.5)  # scroll up
+
+
+def test_plot_epochs_keypresses():
+    """Test plot_epochs keypress interaction."""
+    epochs = _get_epochs(stop=15).load_data()  # we need more than 1 epoch
+    epochs.drop_bad(dict(mag=4e-12))  # for histogram plot coverage
+    fig = epochs.plot(n_epochs=3)
+    data_ax = fig.mne.ax_main
+    # make sure green vlines are visible first (for coverage)
+    sample_idx = len(epochs.times) // 2  # halfway through the first epoch
+    x = fig.mne.traces[0].get_xdata()[sample_idx]
+    y = (fig.mne.traces[0].get_ydata()[sample_idx]
+         + fig.mne.traces[1].get_ydata()[sample_idx]) / 2
+    _fake_click(fig, data_ax, [x, y], xform='data')  # click between traces
+    # test keys
+    keys = ('pagedown', 'down', 'up', 'down', 'right', 'left', '-', '+', '=',
+            'd', 'd', 'pageup', 'home', 'shift+right', 'end', 'shift+left',
+            'z', 'z', 's', 's', 'f11', '?', 'h', 'j', 'b')
+    for key in keys * 2:  # test twice → once in normal, once in butterfly view
+        fig.canvas.key_press_event(key)
+    _fake_click(fig, data_ax, [x, y], xform='data', button=3)  # remove vlines
+
+
+def test_epochs_plot_sensors(epochs):
+    """Test sensor plotting."""
+    epochs.plot_sensors()
 
 
 def test_plot_epochs_nodata():
@@ -165,7 +231,7 @@ def test_plot_epochs_nodata():
         epochs.plot()
 
 
-def test_plot_epochs_image():
+def test_plot_epochs_image(epochs):
     """Test plotting of epochs image.
 
     Note that some of these tests that should pass are triggering MPL
@@ -174,9 +240,12 @@ def test_plot_epochs_image():
     `plt.close('all')` just before the offending test seems to prevent this
     warning, though it's unclear why.
     """
-    epochs = _get_epochs()
     figs = epochs.plot_image()
     assert len(figs) == 2  # one fig per ch_type (test data has mag, grad)
+    assert len(plt.get_fignums()) == 2
+    figs = epochs.plot_image()
+    assert len(figs) == 2
+    assert len(plt.get_fignums()) == 4  # should create new figures
     epochs.plot_image(picks='mag', sigma=0.1)
     epochs.plot_image(picks=[0, 1], combine='mean',
                       ts_args=dict(show_sensors=False))
@@ -195,14 +264,11 @@ def test_plot_epochs_image():
     # test order=callable
     epochs.plot_image(picks=[0, 1],
                       order=lambda times, data: np.arange(len(data))[::-1])
-    # test deprecation
-    with pytest.warns(DeprecationWarning, match='group_by="type" is no longe'):
-        epochs.plot_image(group_by='type')
     # test warning
     with pytest.warns(RuntimeWarning, match='Only one channel in group'):
         epochs.plot_image(picks=[1], combine='mean')
     # group_by should be a dict
-    with pytest.raises(AttributeError, match="has no attribute 'items'"):
+    with pytest.raises(TypeError, match="dict or None"):
         epochs.plot_image(group_by='foo')
     # units and scalings keys must match
     with pytest.raises(ValueError, match='Scalings and units must have the'):
@@ -263,61 +329,53 @@ def test_plot_epochs_image():
 
 def test_plot_drop_log():
     """Test plotting a drop log."""
-    epochs = _get_epochs()
-    pytest.raises(ValueError, epochs.plot_drop_log)
+    epochs = _get_epochs()  # not loaded
+    with pytest.raises(ValueError, match='bad epochs have not yet been'):
+        epochs.plot_drop_log()
     epochs.drop_bad()
     epochs.plot_drop_log()
-    plot_drop_log([['One'], [], []])
-    plot_drop_log([['One'], ['Two'], []])
-    plot_drop_log([['One'], ['One', 'Two'], []])
+    plot_drop_log((('One',), (), ()))
+    plot_drop_log((('One',), ('Two',), ()))
+    plot_drop_log((('One',), ('One', 'Two'), ()))
+    for arg in ([], ([],), (1,)):
+        with pytest.raises(TypeError, match='tuple of tuple of str'):
+            plot_drop_log(arg)
     plt.close('all')
 
 
-def test_plot_butterfly():
-    """Test butterfly view in epochs browse window."""
-    rng = np.random.RandomState(0)
-    n_epochs, n_channels, n_times = 50, 30, 20
-    sfreq = 1000.
-    data = np.sin(rng.randn(n_epochs, n_channels, n_times))
-    events = np.array([np.arange(n_epochs), [0] * n_epochs, np.ones([n_epochs],
-                       dtype=np.int)]).T
-    chanlist = ['eeg' if chan < n_channels // 3 else 'ecog'
-                if chan < n_channels // 2 else 'seeg'
-                for chan in range(n_channels)]
-    info = create_info(n_channels, sfreq, chanlist)
-    epochs = EpochsArray(data, info, events)
-    fig = epochs.plot(butterfly=True)
-    keystotest = ['b', 'b', 'left', 'right', 'up', 'down',
-                  'pageup', 'pagedown', '-', '+', '=',
-                  'f11', 'home', '?', 'h', 'o', 'end']
-    for key in keystotest:
-        fig.canvas.key_press_event(key)
-    fig.canvas.scroll_event(0.5, 0.5, -0.5)  # scroll down
-    fig.canvas.scroll_event(0.5, 0.5, 0.5)  # scroll up
-    fig.canvas.resize_event()
-    fig.canvas.close_event()  # closing and epoch dropping
-    plt.close('all')
-
-
-def test_plot_psd_epochs():
+def test_plot_psd_epochs(epochs):
     """Test plotting epochs psd (+topomap)."""
-    epochs = _get_epochs()
-    epochs.load_data()
     epochs.plot_psd(average=True, spatial_colors=False)
     epochs.plot_psd(average=False, spatial_colors=True)
     epochs.plot_psd(average=False, spatial_colors=False)
-    pytest.raises(RuntimeError, epochs.plot_psd_topomap,
-                  bands=[(0, 0.01, 'foo')])  # no freqs in range
-    epochs.plot_psd_topomap()
-
-    # with a flat channel
-    err_str = r'channel\(s\) (%s){1}\.' % epochs.ch_names[2]
+    # test plot_psd_topomap errors
+    with pytest.raises(RuntimeError, match='No frequencies in band'):
+        epochs.plot_psd_topomap(bands=[(0, 0.01, 'foo')])
+    plt.close('all')
+    # test defaults
+    fig = epochs.plot_psd_topomap()
+    assert len(fig.axes) == 10  # default: 5 bands (δ, θ, α, β, γ) + colorbars
+    # test joint vlim
+    fig = epochs.plot_psd_topomap(vlim='joint')
+    vmin_0 = fig.axes[0].images[0].norm.vmin
+    vmax_0 = fig.axes[0].images[0].norm.vmax
+    assert all(vmin_0 == ax.images[0].norm.vmin for ax in fig.axes[1:5])
+    assert all(vmax_0 == ax.images[0].norm.vmax for ax in fig.axes[1:5])
+    # test support for single-bin bands
+    fig = epochs.plot_psd_topomap(bands=[(20, '20 Hz'), (15, 25, '15-25 Hz')])
+    # test with a flat channel
+    err_str = 'for channel %s' % epochs.ch_names[2]
     epochs.get_data()[0, 2, :] = 0
     for dB in [True, False]:
         with pytest.warns(UserWarning, match=err_str):
             epochs.plot_psd(dB=dB)
 
-    plt.close('all')
+
+def test_plot_psdtopo_nirs(fnirs_epochs):
+    """Test plotting of PSD topography for nirs data."""
+    bands = [(0.2, '0.2 Hz'), (0.4, '0.4 Hz'), (0.8, '0.8 Hz')]
+    fig = fnirs_epochs.plot_psd_topomap(bands=bands)
+    assert len(fig.axes) == 6  # 3 band x (plot + cmap)
 
 
 @testing.requires_testing_data
@@ -334,16 +392,15 @@ def test_plot_epochs_ctf():
 
     # test butterfly
     fig = epochs.plot(butterfly=True)
-    keystotest = ['b', 'b', 'left', 'right', 'up', 'down',
-                  'pageup', 'pagedown', '-', '+', '=',
-                  'f11', 'home', '?', 'h', 'o', 'end']
-    for key in keystotest:
+    keys = ('b', 'b', 'pagedown', 'down', 'up', 'down', 'right', 'left', '-',
+            '+', '=', 'd', 'd', 'pageup', 'home', 'end', 'z', 'z', 's', 's',
+            'f11', '?', 'h', 'j')
+    for key in keys:
         fig.canvas.key_press_event(key)
     fig.canvas.scroll_event(0.5, 0.5, -0.5)  # scroll down
     fig.canvas.scroll_event(0.5, 0.5, 0.5)  # scroll up
     fig.canvas.resize_event()
-    fig.canvas.close_event()  # closing and epoch dropping
-    plt.close('all')
+    fig.canvas.key_press_event('escape')  # close and drop epochs
 
 
 @testing.requires_testing_data
@@ -357,13 +414,11 @@ def test_plot_psd_epochs_ctf():
     epochs.plot_psd_topomap()
 
     # EEG060 is flat in this dataset
-    err_str = r'channel\(s\) EEG060\.'
     for dB in [True, False]:
-        with pytest.warns(UserWarning, match=err_str):
+        with pytest.warns(UserWarning, match='for channel EEG060'):
             epochs.plot_psd(dB=dB)
     epochs.drop_channels(['EEG060'])
     epochs.plot_psd(spatial_colors=False, average=False)
-    plt.close('all')
 
 
 run_tests_if_main()
