@@ -12,6 +12,7 @@
 import os.path as op
 
 import numpy as np
+from numpy.testing import assert_allclose
 import pytest
 import matplotlib.pyplot as plt
 from matplotlib import gridspec
@@ -19,10 +20,10 @@ from matplotlib.cm import get_cmap
 
 import mne
 from mne import (read_events, Epochs, read_cov, compute_covariance,
-                 make_fixed_length_events)
+                 make_fixed_length_events, compute_proj_evoked)
 from mne.io import read_raw_fif
-from mne.utils import run_tests_if_main, catch_logging
-from mne.viz.evoked import plot_compare_evokeds
+from mne.utils import catch_logging, requires_version
+from mne.viz import plot_compare_evokeds, plot_evoked_white
 from mne.viz.utils import _fake_click
 from mne.datasets import testing
 from mne.io.constants import FIFF
@@ -38,18 +39,22 @@ event_id, tmin, tmax = 1, -0.1, 0.1
 
 # Use a subset of channels for plotting speed
 # make sure we have a magnetometer and a pair of grad pairs for topomap.
-picks = [0, 1, 2, 3, 4, 6, 7, 61, 122, 183, 244, 305]
-sel = [0, 7]
+default_picks = (0, 1, 2, 3, 4, 6, 7, 61, 122, 183, 244, 305,
+                 315, 316, 317, 318)  # EEG channels
+sel = (0, 7)
 
 
-def _get_epochs():
+def _get_epochs(picks=default_picks):
     """Get epochs."""
     raw = read_raw_fif(raw_fname)
     raw.add_proj([], remove_existing=True)
     events = read_events(event_name)
     epochs = Epochs(raw, events[:5], event_id, tmin, tmax, picks=picks,
                     decim=10, verbose='error')
-    epochs.info['bads'] = [epochs.ch_names[-1]]
+    epochs.info['bads'] = [
+        epochs.ch_names[-5],  # MEG
+        epochs.ch_names[-1]   # EEG
+    ]
     epochs.info.normalize_proj()
     return epochs
 
@@ -60,8 +65,8 @@ def _get_epochs_delayed_ssp():
     events = read_events(event_name)
     reject = dict(mag=4e-12)
     epochs_delayed_ssp = Epochs(raw, events[:10], event_id, tmin, tmax,
-                                picks=picks, proj='delayed', reject=reject,
-                                verbose='error')
+                                picks=default_picks, proj='delayed',
+                                reject=reject, verbose='error')
     epochs_delayed_ssp.info.normalize_proj()
     return epochs_delayed_ssp
 
@@ -71,14 +76,15 @@ def test_plot_evoked_cov():
     evoked = _get_epochs().average()
     cov = read_cov(cov_fname)
     cov['projs'] = []  # avoid warnings
-    evoked.plot(noise_cov=cov, time_unit='s')
+    with pytest.warns(RuntimeWarning, match='No average EEG reference'):
+        evoked.plot(noise_cov=cov, time_unit='s')
     with pytest.raises(TypeError, match='Covariance'):
         evoked.plot(noise_cov=1., time_unit='s')
     with pytest.raises(IOError, match='No such file'):
         evoked.plot(noise_cov='nonexistent-cov.fif', time_unit='s')
     raw = read_raw_fif(raw_sss_fname)
     events = make_fixed_length_events(raw)
-    epochs = Epochs(raw, events, picks=picks)
+    epochs = Epochs(raw, events, picks=default_picks)
     cov = compute_covariance(epochs)
     evoked_sss = epochs.average()
     with pytest.warns(RuntimeWarning, match='relative scaling'):
@@ -89,9 +95,14 @@ def test_plot_evoked_cov():
 @pytest.mark.slowtest
 def test_plot_evoked():
     """Test evoked.plot."""
-    evoked = _get_epochs().average()
+    epochs = _get_epochs()
+    evoked = epochs.average()
+    assert evoked.proj is False
     fig = evoked.plot(proj=True, hline=[1], exclude=[], window_title='foo',
                       time_unit='s')
+    amplitudes = _get_amplitudes(fig)
+    assert len(amplitudes) == len(default_picks)
+    assert evoked.proj is False
     # Test a click
     ax = fig.get_axes()[0]
     line = ax.lines[0]
@@ -116,9 +127,26 @@ def test_plot_evoked():
                   proj='interactive', axes='foo', time_unit='s')
     plt.close('all')
 
-    # test GFP only
-    evoked.plot(gfp='only', time_unit='s')
-    pytest.raises(ValueError, evoked.plot, gfp='foo', time_unit='s')
+    # test `gfp='only'`: GFP (EEG) and RMS (MEG)
+    fig, ax = plt.subplots(3)
+    evoked.plot(gfp='only', time_unit='s', axes=ax)
+
+    assert len(ax[0].lines) == len(ax[1].lines) == len(ax[2].lines) == 1
+
+    assert ax[0].get_title() == 'EEG (3 channels)'
+    assert ax[0].texts[0].get_text() == 'GFP'
+
+    assert ax[1].get_title() == 'Gradiometers (9 channels)'
+    assert ax[1].texts[0].get_text() == 'RMS'
+
+    assert ax[2].get_title() == 'Magnetometers (2 channels)'
+    assert ax[1].texts[0].get_text() == 'RMS'
+
+    plt.close('all')
+
+    # Test invalid `gfp`
+    with pytest.raises(ValueError):
+        evoked.plot(gfp='foo', time_unit='s')
 
     # plot with bad channels excluded, spatial_colors, zorder & pos. layout
     evoked.rename_channels({'MEG 0133': 'MEG 0000'})
@@ -138,6 +166,75 @@ def test_plot_evoked():
     assert 'Need more than one' in log_file.getvalue()
 
 
+@requires_version('matplotlib', '2.2')
+def test_constrained_layout():
+    """Test that we handle constrained layouts correctly."""
+    fig, ax = plt.subplots(1, 1, constrained_layout=True)
+    assert fig.get_constrained_layout()
+    evoked = mne.read_evokeds(evoked_fname)[0]
+    evoked.pick(evoked.ch_names[:2])
+    evoked.plot(axes=ax)  # smoke test that it does not break things
+    assert fig.get_constrained_layout()
+    plt.close('all')
+
+
+def _get_amplitudes(fig):
+    amplitudes = [line.get_ydata() for ax in fig.axes
+                  for line in ax.get_lines()]
+    amplitudes = np.array(
+        [line for line in amplitudes if isinstance(line, np.ndarray)])
+    return amplitudes
+
+
+@pytest.mark.parametrize('picks, rlims, avg_proj', [
+    (default_picks[:-4], (0.59, 0.61), False),  # MEG
+    (np.arange(340, 360), (0.56, 0.57), True),  # EEG
+    (np.arange(340, 360), (0.79, 0.81), False),  # EEG
+])
+def test_plot_evoked_reconstruct(picks, rlims, avg_proj):
+    """Test proj="reconstruct"."""
+    evoked = _get_epochs(picks=picks).average()
+    if avg_proj:
+        evoked.set_eeg_reference(projection=True).apply_proj()
+        assert len(evoked.info['projs']) == 1
+        assert evoked.proj is True
+    else:
+        assert len(evoked.info['projs']) == 0
+        assert evoked.proj is False
+    fig = evoked.plot(proj=True, hline=[1], exclude=[], window_title='foo',
+                      time_unit='s')
+    amplitudes = _get_amplitudes(fig)
+    assert len(amplitudes) == len(picks)
+    assert evoked.proj is avg_proj
+
+    fig = evoked.plot(proj='reconstruct', exclude=[])
+    amplitudes_recon = _get_amplitudes(fig)
+    if avg_proj is False:
+        assert_allclose(amplitudes, amplitudes_recon)
+    proj = compute_proj_evoked(evoked.copy().crop(None, 0).apply_proj())
+    evoked.add_proj(proj)
+    assert len(evoked.info['projs']) == 2 if len(picks) == 3 else 4
+    fig = evoked.plot(proj=True, exclude=[])
+    amplitudes_proj = _get_amplitudes(fig)
+    fig = evoked.plot(proj='reconstruct', exclude=[])
+    amplitudes_recon = _get_amplitudes(fig)
+    assert len(amplitudes_recon) == len(picks)
+    norm = np.linalg.norm(amplitudes)
+    norm_proj = np.linalg.norm(amplitudes_proj)
+    norm_recon = np.linalg.norm(amplitudes_recon)
+    r = np.dot(amplitudes_recon.ravel(), amplitudes.ravel()) / (
+        norm_recon * norm)
+    assert rlims[0] < r < rlims[1]
+    assert 1.05 * norm_proj < norm_recon
+    if not avg_proj:
+        assert norm_proj < norm * 0.9
+
+    cov = read_cov(cov_fname)
+    with pytest.raises(ValueError, match='Cannot use proj="reconstruct"'):
+        evoked.plot(noise_cov=cov, proj='reconstruct')
+    plt.close('all')
+
+
 def test_plot_evoked_image():
     """Test plot_evoked_image."""
     evoked = _get_epochs().average()
@@ -184,6 +281,9 @@ def test_plot_evoked_image():
         pytest.raises(ValueError, evoked.plot_image, group_by=group_by,
                       axes=axes)
 
+    with pytest.raises(ValueError, match='`clim` must be a dict.'):
+        evoked.plot_image(clim=[-4, 4])
+
 
 def test_plot_white():
     """Test plot_white."""
@@ -191,28 +291,36 @@ def test_plot_white():
     cov['method'] = 'empirical'
     cov['projs'] = []  # avoid warnings
     evoked = _get_epochs().average()
+    evoked.set_eeg_reference('average')  # Avoid warnings
+
     # test rank param.
-    evoked.plot_white(cov, rank={'mag': 101, 'grad': 201}, time_unit='s')
-    evoked.plot_white(cov, rank={'mag': 101}, time_unit='s')  # test rank param
-    evoked.plot_white(cov, rank={'grad': 201}, time_unit='s')
-    pytest.raises(
-        ValueError, evoked.plot_white, cov,
-        rank={'mag': 101, 'grad': 201, 'meg': 306}, time_unit='s')
-    pytest.raises(
-        ValueError, evoked.plot_white, cov, rank={'meg': 306}, time_unit='s')
+    with pytest.raises(ValueError, match='exceeds'):
+        evoked.plot_white(cov, rank={'mag': 10})
+    evoked.plot_white(cov, rank={'mag': 1, 'grad': 8, 'eeg': 2}, time_unit='s')
+    fig = evoked.plot_white(cov, rank={'mag': 1}, time_unit='s')  # test rank
+    evoked.plot_white(cov, rank={'grad': 8}, time_unit='s', axes=fig.axes)
+    with pytest.raises(ValueError, match=r'must have shape \(4,\), got \(2,'):
+        evoked.plot_white(cov, axes=fig.axes[:2])
+    with pytest.raises(ValueError, match='When not using SSS'):
+        evoked.plot_white(cov, rank={'meg': 306})
     evoked.plot_white([cov, cov], time_unit='s')
     plt.close('all')
 
+    fig = plot_evoked_white(evoked, [cov, cov])
+    assert len(fig.axes) == 3 * 2
+    axes = np.array(fig.axes).reshape(3, 2)
+    plot_evoked_white(evoked, [cov, cov], axes=axes)
+    with pytest.raises(ValueError, match=r'have shape \(3, 2\), got'):
+        plot_evoked_white(evoked, [cov, cov], axes=axes[:, :1])
+
     # Hack to test plotting of maxfiltered data
-    evoked_sss = evoked.copy()
+    evoked_sss = _get_epochs(picks='meg').average()
     sss = dict(sss_info=dict(in_order=80, components=np.arange(80)))
     evoked_sss.info['proc_history'] = [dict(max_info=sss)]
-    evoked_sss.plot_white(cov, rank={'meg': 64}, time_unit='s')
-    pytest.raises(
-        ValueError, evoked_sss.plot_white, cov, rank={'grad': 201},
-        time_unit='s')
+    evoked_sss.plot_white(cov, rank={'meg': 64})
+    with pytest.raises(ValueError, match='When using SSS'):
+        evoked_sss.plot_white(cov, rank={'grad': 201})
     evoked_sss.plot_white(cov, time_unit='s')
-    plt.close('all')
 
 
 def test_plot_compare_evokeds():
@@ -220,7 +328,7 @@ def test_plot_compare_evokeds():
     evoked = _get_epochs().average()
     # test defaults
     figs = plot_compare_evokeds(evoked)
-    assert len(figs) == 2
+    assert len(figs) == 3
     # test picks, combine, and vlines (1-channel pick also shows sensor inset)
     picks = ['MEG 0113', 'mag'] + 2 * [['MEG 0113', 'MEG 0112']] + [[0, 1]]
     vlines = [[0.1, 0.2], []] + 3 * ['auto']
@@ -277,11 +385,11 @@ def test_plot_compare_evokeds():
     plot_compare_evokeds(evoked_dict, cmap=('cmap title', 'inferno'),
                          linestyles=['-', ':', '--'])
     plt.close('all')
-    # test deprecation
-    with pytest.warns(DeprecationWarning, match='"gfp" is deprecated'):
-        plot_compare_evokeds(evoked, gfp=True)
-    with pytest.warns(DeprecationWarning, match='"max_ticks" changed to '):
-        plot_compare_evokeds(evoked, picks=[0], truncate_yaxis='max_ticks')
+    # test combine
+    match = 'combine must be an instance of None, callable, or str'
+    with pytest.raises(TypeError, match=match):
+        plot_compare_evokeds(evoked, combine=["mean", "gfp"])
+    plt.close('all')
     # test warnings
     with pytest.warns(RuntimeWarning, match='in "picks"; cannot combine'):
         plot_compare_evokeds(evoked, picks=[0], combine='median')
@@ -322,6 +430,12 @@ def test_plot_compare_evokeds():
     figs = plot_compare_evokeds(evoked_dict, axes='topo', legend=True)
     for fig in figs:
         assert len(fig.axes[0].lines) == len(evoked_dict)
+    # test with (fake) CSD data
+    csd = _get_epochs(picks=np.arange(315, 320)).average()  # 5 EEG chs
+    for entry in csd.info['chs']:
+        entry['coil_type'] = FIFF.FIFFV_COIL_EEG_CSD
+        entry['unit'] = FIFF.FIFF_UNIT_V_M2
+    plot_compare_evokeds(csd, picks='csd', axes='topo')
     # old tests
     red.info['chs'][0]['loc'][:2] = 0  # test plotting channel at zero
     plot_compare_evokeds([red, blue], picks=[0],
@@ -402,6 +516,3 @@ def test_plot_ctf():
                       topomap_args={'axes': topo_axes}, title=None)
     midpoints_after = get_axes_midpoints(topo_axes)
     assert (np.linalg.norm(midpoints_before - midpoints_after) < 0.1).all()
-
-
-run_tests_if_main()
